package com.megabras.bluelogg.DT70KV;

import android.app.Activity;
import android.graphics.Typeface;
import android.graphics.pdf.PdfDocument;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Paper;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.bluelogg.md15kvr.Controller;
import com.megabras.modbus.DT70KV.Device;
import com.megabras.modbus.DT70KV.Measurement;
import com.megabras.modbus.DT70KV.TestData;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Locale;


public class ReportPDF extends ReportProvider
{

    private final String
            TAG = "ReportPDF";

    protected com.megabras.modbus.DT70KV.FormatData
            formatData = new com.megabras.modbus.DT70KV.FormatData();

    public ReportPDF(Activity activity, File dir, File tempDir, Handler handler)
    {
        super(activity, dir, tempDir, handler);
    }

    public void generate(TestData testData, Device device, File picture, File map, File[] chartImageArray)
    {
        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();

        PdfDocument document = new PdfDocument();
        PdfDocument.PageInfo pageInfo;

        if ( Locale.getDefault().getLanguage().equals( Locale.ENGLISH.getLanguage() ) )
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.Letter.getWidth(), Paper.Letter.getHeight(), 1).create();
        }
        else
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.A4.getWidth(), Paper.A4.getHeight(), 1).create();
        }

        CURSOR_Y     = 0;
        CURSOR_X     = 0;
        PAPER_WIDTH  = pageInfo.getPageWidth()  - (MARGIN * 2);
        PAPER_HEIGHT = pageInfo.getPageHeight() - (MARGIN * 2);

        PdfDocument.Page page = document.startPage( pageInfo );
        canvas = page.getCanvas();

        moveCursor(MARGIN, MARGIN);

        // BORDER
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(PAPER_HEIGHT);
        this.drawSquare();

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_general_info)).sendToTarget();

        // TITLE
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawTitle( activity.getString(R.string.glove_test).toUpperCase() );

        ArrayList<String> titles = new ArrayList<>();
        ArrayList<String> values = new ArrayList<>();

        titles.add( activity.getString(R.string.md_model       )     );
        titles.add( activity.getString(R.string.md_serial      )  );
        titles.add( activity.getString(R.string.md_test_number )    );
        titles.add( activity.getString(R.string.md_test_date   ) );

        values.add( testData.getModel()                                                                     );
        values.add( testData.getSerial()                                                                    );
        values.add( testData.getTestNumber() + ""                                                           );
        values.add( formatData.dateTime(testData.getDate(), device.getDateFormat(), device.getTimeFormat()) );

        if ( testData.isEquipmentVersionDefinied() )
        {
            titles.add(2, "FIRMWARE"             );
            values.add(2, testData.getEquipmentVersion() );
        }

        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawLineItem(titles, values);

        if ( testData.getGpsAcuraccy()>0 )
        {
            titles = new ArrayList<>();
            values = new ArrayList<>();

            titles.add( activity.getString( R.string.md_latitude     ) );
            titles.add( activity.getString( R.string.md_longitude    ) );
            titles.add( activity.getString( R.string.md_altitude     ) );
            titles.add( activity.getString( R.string.md_gps_accuracy ) );

            values.add( testData.getGpsLatitude() + ""              );
            values.add( testData.getGpsLongitude() + ""             );
            values.add( testData.getGpsAltitude() + " m"            );
            values.add( "\u00B1" + testData.getGpsAcuraccy() + " m" );

            left  (0);
            top   (0);
            right (PAPER_WIDTH);
            bottom(20);
            this.drawLineItem(titles, values);
        }

        titles                = new ArrayList<>();
        ArrayList<File> files = new ArrayList<>();

        titles.add( activity.getString( R.string.picture ) );
        titles.add( activity.getString( R.string.map     ) );

        files.add(picture);
        files.add(map);

        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(200);

        if (picture.exists() && map.exists())
            this.drawLineImage(titles, files);
        else if (picture.exists())
            this.drawLineItem(titles.get(0), picture);
        else if (map.exists())
            this.drawLineItem(titles.get(1), map);

        saveCursor();

        int CHART_W_AREA = (int)(PAPER_WIDTH * 0.75);

        // TEST MODE
        left  (0);
        top   (0);
        right (CHART_W_AREA);
        bottom(20);

        titles = new ArrayList<>();
        values = new ArrayList<>();
        titles.add( activity.getString( R.string.glove_serial ) );
        titles.add( activity.getString( R.string.type         ) );
        titles.add( activity.getString( R.string.result       ) );
        titles.add( activity.getString( R.string.reason       ) );
        String[] gloveClasses = activity.getResources().getStringArray( R.array.glove_class );

        String hex          = com.megabras.modbus.MUtils.intToHex( testData.getTestResult(), 4 );
        boolean[] statusOne = com.megabras.modbus.MUtils.hexToBitArray(hex);

        String reason = " - - - ";
        String result = " - - - ";
        if ( statusOne[0] )
            reason = activity.getString(R.string.spark_detection).toUpperCase();
        else if (statusOne[1])
            reason = activity.getString(R.string.overcurrent).toUpperCase();

        if ( statusOne[2] && statusOne[3] )
            result = activity.getString(R.string.pass);
        else if ( !statusOne[2] && statusOne[3] )
            result = activity.getString(R.string.fail);
        else if (statusOne[2])
            result = activity.getString(R.string.aborted);

        values.add( testData.getGloveSerial().toUpperCase() );
        values.add( gloveClasses[testData.getGloveClass()]  );
        values.add( result                                  );
        values.add( reason                                  );
        this.drawLineItem(titles, values);

        String duration = formatData.elapsed( (short)testData.getMeasurementLast().getElapsedTotal() );

        double V_MAX = -Integer.MAX_VALUE;
        double A_MAX = -Integer.MAX_VALUE;

        for (Measurement m : testData.getMeasurement())
        {
            if (m.getVolt() > V_MAX)
            {
                V_MAX = m.getVolt();
            }
            if (m.getAmpRT() > A_MAX)
            {
                A_MAX = m.getAmpRT();
            }
        }

        String maxV  = formatData.volts( V_MAX  );
        String maxA  = formatData.amps ( A_MAX  );

        titles = new ArrayList<>();
        values = new ArrayList<>();

        titles.add( activity.getString( R.string.test_duration_mmss ) );
        titles.add( activity.getString( R.string.maximum_voltage    ) );
        titles.add( activity.getString( R.string.maximum_current    ) );

        values.add( duration );
        values.add( maxV     );
        values.add( maxA     );

        this.drawLineItem(titles, values);

        // CHARTS
        left  (0);
        top   (0);
        right (CHART_W_AREA);
        bottom((PAPER_HEIGHT - CURSOR_Y) / 2);

        files  = new ArrayList<>();
        if (chartImageArray.length>0)
            files.add(chartImageArray[0]);
        if (chartImageArray.length>1)
            files.add(chartImageArray[1]);

        this.drawLineImage(files);

        files  = new ArrayList<>();
        if (chartImageArray.length>2)
            files.add(chartImageArray[2]);
        else
            files.add(null);

        if (chartImageArray.length>3)
            files.add(chartImageArray[3]);
        else
            files.add(null);

        this.drawLineImage(files);

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_tables)).sendToTarget();

        // TABLE
        restoreCursor();

        moveCursor(CHART_W_AREA, 0);

        left  (0);
        top   (0);
        right (PAPER_WIDTH - CHART_W_AREA);
        bottom(10);

        values = new ArrayList<>();
        values.add( activity.getString( R.string.MMSS    ) );
        values.add( activity.getString( R.string.voltage ) );
        values.add( activity.getString( R.string.current ) );

        this.drawRow(Typeface.DEFAULT_BOLD, values);

        int PAPER_LEFT = PAPER_HEIGHT - CURSOR_Y - BRAND_HEIGHT;
        int ROWS_MAX   = PAPER_LEFT / getBounds().height();

        ArrayList<Measurement> data = testData.getMeasurement();

        if (data.size() > ROWS_MAX)
        {
            int FIRST_HALF = ROWS_MAX / 2;
            int LAST_HALF  = ROWS_MAX - FIRST_HALF;
            for ( int i = 0; i < FIRST_HALF; i++ )
            {
                int    baseSeconds = testData.getMeasurement().get(i).getElapsedTotal();
                double voltage     = testData.getMeasurement().get(i).getVolt();
                double current     = testData.getMeasurement().get(i).getAmpRT();

                String sElapsed = formatData.elapsed( baseSeconds );
                String sVolts   = formatData.volts  ( voltage     );
                String sAmps    = formatData.amps   ( current     );

                values = new ArrayList<>();
                values.add(sElapsed);
                values.add(sVolts);
                values.add(sAmps);

                this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
            }
            for ( int i = (data.size()-1)-LAST_HALF; i < data.size(); i++ )
            {
                int    baseSeconds = testData.getMeasurement().get(i).getElapsedTotal();
                double voltage     = testData.getMeasurement().get(i).getVolt();
                double current     = testData.getMeasurement().get(i).getAmpRT();

                String sElapsed = formatData.elapsed( baseSeconds );
                String sVolts   = formatData.volts  ( voltage     );
                String sAmps    = formatData.amps   ( current     );

                values = new ArrayList<>();
                values.add(sElapsed);
                values.add(sVolts);
                values.add(sAmps);

                this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
            }
        }
        else
        {
            for ( int i = 0; i < testData.getMeasurement().size(); i++ )
            {
                int    baseSeconds = testData.getMeasurement().get(i).getElapsedTotal();
                double voltage     = testData.getMeasurement().get(i).getVolt();
                double current     = testData.getMeasurement().get(i).getAmpRT();

                String sElapsed = formatData.elapsed( baseSeconds );
                String sVolts   = formatData.volts  ( voltage     );
                String sAmps    = formatData.amps   ( current     );

                values = new ArrayList<>();
                values.add(sElapsed);
                values.add(sVolts);
                values.add(sAmps);

                this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
            }
        }

        drawGeneratedBy();

        document.finishPage(page);

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.saving_file)).sendToTarget();

        String docFile = testData.getModel() + "_" + testData.getSerial() + "_" + testData.getID() + "_report.pdf";
        File tmpFile   = new File(dir + "/"+ docFile);

        try
        {
            document.writeTo(new FileOutputStream(tmpFile));
            // close the document
            document.close();

            handler.obtainMessage(Controller.mHandler.REPORT_FINISHED, 1, -1, activity.getString(R.string.finished)).sendToTarget();

            sendFile(tmpFile);

            if ( tempDir != null )
                for ( File temp : tempDir.listFiles() )
                    temp.delete();
        }
        catch (IOException e)
        {
            Log.e(TAG, "error "+e.toString());
            handler.obtainMessage(Controller.mHandler.REPORT_ERROR, 1, -1, activity.getString(R.string.finished)).sendToTarget();
        }
    }
}

