package com.megabras.bluelogg.MDXXKVR;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;

import com.megabras.DeviceInfo;
import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.R;
import com.megabras.modbus.MDXXKVR.Device;
import com.megabras.modbus.MDXXKVR.Measure;
import com.megabras.modbus.MDXXKVR.TestData;
import com.megabras.modbus.common.DielectricDischarge;
import com.megabras.modbus.common.ResultDAI;
import com.megabras.modbus.common.ResultPI;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Objects;

import androidx.annotation.NonNull;

@SuppressWarnings("WeakerAccess")
public class Database extends SQLiteOpenHelper
{
    @SuppressWarnings("FieldCanBeLocal")
    private String
            TAG          = "Database";

    private static final String
            TEXT_TYPE    = " TEXT",
            COMMA_SEP    = ",";

    @SuppressWarnings("FieldCanBeLocal")
    private String
            root         = Environment.getExternalStorageDirectory().toString(),
            picSuffix    = ".jpg",
            audioSuffix  = "_annotation.3gp",
            mapSuffix    = "_map_snapshot.jpg";

    private File
            dir,
            tempDir,
            rootDir;

    public Database( Context context, String model )
    {
        super( context, Entry.getDbName( model ), null, Entry.DB_VERSION );
        createFolder( context, model );
    }

    private String getTableName( TestData testData )
    {
        return testData.getDeviceInfo().getModel().replaceAll("-", "") + "_" + testData.getDeviceInfo().getSerial() + "_" + testData.getTestNumber() + "_" + testData.getDate().getTime();
    }

    public void save( TestData testData )
    {
        String TABLE_NAME = getTableName( testData );
        SQLiteDatabase db = this.getWritableDatabase();

        Cursor c = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + TABLE_NAME + "';", null);
        // TABLE ALREADY EXITS
        int len = c.getCount();
        c.close();
        if ( len > 0 )
            return;

        String FIELD_ENTRIES =
                Entry.DESCRIPTION   + TEXT_TYPE + COMMA_SEP +
                Entry.COL_A         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_B         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_C         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_D         + TEXT_TYPE;

        String SQL_CREATE_TABLE = "CREATE TABLE " + TABLE_NAME + " (" + FIELD_ENTRIES + " )";
        db.execSQL( SQL_CREATE_TABLE );
        db.beginTransaction();

        ContentValues values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.MODEL );
        values.put( Entry.COL_A      , testData.getDeviceInfo().getModel() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SERIAL );
        values.put( Entry.COL_A      , testData.getDeviceInfo().getSerial() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.FIRMWARE );
        values.put( Entry.COL_A      , testData.getDeviceInfo().getFirmware() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.HARDWARE );
        values.put( Entry.COL_A      , testData.getDeviceInfo().getHardware() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.TEST_NUMBER );
        values.put( Entry.COL_A      , testData.getTestNumber() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DATE );
        values.put( Entry.COL_A      , testData.getDate().getTime()+"" );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.OP_MODE );
        values.put( Entry.COL_A      , testData.getMode() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        ResultDAI dai = new ResultDAI();
        ResultPI  pi  = new ResultPI();

        dai.setTimeA( testData.getDAI().getTimeA() );
        dai.setTimeB( testData.getDAI().getTimeB() );
        pi.setTimeA( testData.getPI().getTimeA() );
        pi.setTimeB( testData.getPI().getTimeB() );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DAI_CONFIG );
        values.put( Entry.COL_A      , dai.getTimeA() );
        values.put( Entry.COL_B      , dai.getTimeB() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.PI_CONFIG );
        values.put( Entry.COL_A      , pi.getTimeA() );
        values.put( Entry.COL_B      , pi.getTimeB() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DAI );
        values.put( Entry.COL_A      , testData.getDAI().getResult() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.PI );
        values.put( Entry.COL_A      , testData.getPI().getResult() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.CAPACITANCE );
        values.put( Entry.COL_A      , testData.getCapacitance() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        if (testData.getDD()>=0)
        {
            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.DD);
            values.put( Entry.COL_A, testData.getDD());
            db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );
        }

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SVT );
        values.put( Entry.COL_A      , testData.getSVT() );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ACCURACY );
        values.put( Entry.COL_A      , testData.getGpsAcuraccy()+"" );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
        values.put( Entry.COL_A      , testData.getGpsAltitude()+"" );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LATITUDE );
        values.put( Entry.COL_A      , testData.getGpsLatitude()+"" );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
        values.put( Entry.COL_A      , testData.getGpsLongitude()+"" );
        db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );

        for ( Measure m : testData.getMeasure() )
        {
            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.RESULT );
            values.put( Entry.COL_A      , m.getElapsed() );
            values.put( Entry.COL_B      , m.getOhms() );
            values.put( Entry.COL_C      , m.getVolts() );
            values.put( Entry.COL_D      , m.getAmps() );
            db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );
        }

        if ( testData.getDielectricDischarge() != null )
        {
            for ( DielectricDischarge dd : testData.getDielectricDischarge() )
            {
                values = new ContentValues();
                values.put( Entry.DESCRIPTION, Entry.IDD       );
                values.put( Entry.COL_A      , dd.getElapsed() );
                values.put( Entry.COL_B      , dd.getAmps()    );
                db.insertWithOnConflict( TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE );
            }
        }

        db.setTransactionSuccessful();
        db.endTransaction();
    }

    public void saveOrUpdate( TestData testData )
    {
        try
        {
            String TABLE_NAME = testData.getDeviceInfo().getModel().replaceAll("-", "") + "_" + testData.getDeviceInfo().getSerial() + "_" + testData.getTestNumber() + "_" + testData.getDate().getTime();
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values;

            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.GPS_LATITUDE );
            values.put( Entry.COL_A, testData.getGpsLatitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
            values.put( Entry.COL_A, testData.getGpsLongitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
            values.put( Entry.COL_A, testData.getGpsAltitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.GPS_ACCURACY );
            values.put( Entry.COL_A, testData.getGpsAcuraccy());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            db.setTransactionSuccessful();
            db.endTransaction();
        }
        catch ( Exception e )
        {
            android.util.Log.e( TAG, e.toString() );
        }
    }

    public List<FileDetails> getSavedFilesList( @NonNull Device device )
    {
        try
        {
            List<FileDetails> filesList = new ArrayList<>();
            SQLiteDatabase db = this.getReadableDatabase();


            Cursor c = db.rawQuery( "SELECT name FROM sqlite_master WHERE type='table' AND name LIKE '%" + device.info().getSerial() + "%'", null );

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    String tableName = c.getString( 0 );
                    if ( !tableName.contains( "metadata" ) )
                    {
                        FileDetails file = new FileDetails();
                        String[] temp = tableName.split( "_" );
                        file.FILENAME = tableName;
                        file.MODEL = temp[0];
                        file.SERIAL = temp[1];
                        file.INDEX = temp[2];
                        String seconds = temp[3];
                        long time = Long.parseLong( seconds );
                        Date date = new Date( time );
                        SimpleDateFormat simpleDate = new SimpleDateFormat( device.getDateTimeFormatPattern(), Locale.getDefault() );
                        file.DATE = simpleDate.format( date );
                        filesList.add( file );
                    }
                    c.moveToNext();
                }
            }
            c.close();
            return filesList;
        }
        catch (Exception e)
        {
            return null;
        }
    }

    public TestData getTestData( TestData testData )
    {
        return getTestData( getTableName( testData ) );
    }

    public TestData getTestData( String TABLE_NAME )
    {
        try
        {
            TestData testData = null;
            DeviceInfo deviceInfo = null;
            ResultDAI dai = new ResultDAI();
            ResultPI pi = new ResultPI();

            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery("SELECT * FROM " + TABLE_NAME, null);

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    String colA = c.getString(0);
                    String colB = c.getString(1);
                    String colC = c.getString(2);
                    String colD = c.getString(3);
                    String colE = c.getString(4);

                    switch (colA)
                    {
                        case Entry.MODEL:
                            deviceInfo = new DeviceInfo( colB );
                            break;
                        case Entry.SERIAL:
                            if ( deviceInfo != null )
                            {
                                deviceInfo.setSerial( colB );
                                testData = new TestData( deviceInfo );
                            }
                            break;
                        case Entry.FIRMWARE:
                            if ( testData != null )
                            {
                                testData.getDeviceInfo().setFirmware( Integer.parseInt( colB ) );
                            }
                            break;
                        case Entry.HARDWARE:
                            if ( testData != null )
                            {
                                testData.getDeviceInfo().setHardware( Integer.parseInt( colB ) );
                            }
                            break;
                        case Entry.TEST_NUMBER:
                            if ( testData != null )
                            {
                                testData.setTestNumber( Integer.parseInt( colB ) );
                            }
                            break;
                        case Entry.DATE:
                            if ( testData != null )
                            {
                                Date datef = new Date();
                                datef.setTime( Long.parseLong( colB ) );
                                testData.setDate( datef );
                            }
                            break;
                        case Entry.OP_MODE:
                            if ( testData != null )
                            {
                                testData.setMode( Integer.parseInt( colB ) );
                            }
                            break;
                        case Entry.DAI_CONFIG:
                            dai.setTimeA( Integer.parseInt( colB ) );
                            dai.setTimeB( Integer.parseInt( colC ) );
                            break;
                        case Entry.DAI:
                            dai.setResult( Double.parseDouble( colB ) );
                            if ( testData != null )
                            {
                                testData.setDAI( dai );
                            }
                            break;
                        case Entry.PI_CONFIG:
                            pi.setTimeA( Integer.parseInt( colB ) );
                            pi.setTimeB( Integer.parseInt( colC ) );
                            break;
                        case Entry.PI:
                            pi.setResult(Double.parseDouble(colB));
                            if ( testData != null )
                            {
                                testData.setPI(pi);
                            }
                            break;
                        case Entry.SVT:
                            if ( testData != null )
                            {
                                testData.setSVT(Double.parseDouble(colB));
                            }
                            break;
                        case Entry.CAPACITANCE:
                            if ( testData != null )
                            {
                                testData.setCapacitance( Integer.parseInt( colB ) );
                            }
                            break;
                        case Entry.DD:
                            if ( testData != null )
                            {
                                testData.setDD( Double.parseDouble( colB ) );
                            }
                            break;
                        case Entry.GPS_LATITUDE:
                            if ( testData != null )
                            {
                                testData.setGpsLatitude(Double.parseDouble(colB));
                            }
                            break;
                        case Entry.GPS_LONGITUDE:
                            if ( testData != null )
                            {
                                testData.setGpsLongitude(Double.parseDouble(colB));
                            }
                            break;
                        case Entry.GPS_ALTITUDE:
                            if ( testData != null )
                            {
                                testData.setGpsAltitude(Double.parseDouble(colB));
                            }
                            break;
                        case Entry.GPS_ACCURACY:
                            if ( testData != null )
                            {
                                testData.setGpsAcuraccy( Double.parseDouble( colB ) );
                            }
                            break;
                        case Entry.RESULT:
                            Measure m = new Measure();
                            m.setElapsed( Integer.parseInt( colB ) );
                            m.setOhms( Double.parseDouble( colC ) );
                            m.setVolts( Integer.parseInt( colD ) );
                            m.setAmps( Double.parseDouble( colE ) );
                            if ( testData != null )
                            {
                                testData.addMeasure( m );
                            }
                            break;
                        case Entry.IDD:
                            DielectricDischarge dd = new DielectricDischarge();
                            dd.setElapsed( Integer.parseInt( colB ) );
                            dd.setAmps( Double.parseDouble( colC ) );
                            if ( testData != null )
                            {
                                testData.addDielectricDischarge( dd );
                            }
                            break;
                    }
                    c.moveToNext();
                }
            }

            c.close();
            return testData;
        } catch (Exception e) {
            android.util.Log.e("Database", e.toString());
            return null;
        }
    }

    public void deleteTable( String name )
    {
        SQLiteDatabase db = this.getWritableDatabase();
        String SQL_DELETE_TABLE = "DROP TABLE IF EXISTS " + name;
        db.execSQL( SQL_DELETE_TABLE );

        //MD15KVR_MA4125B_72_1526065135873

        String[] parts = name.split( "_" );

        deleteFiles( parts[1], parts[3] );
    }

    //##########################################################################
    //# Folders
    //##########################################################################

    public boolean isExternalStorageAvaiable()
    {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals( state );
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public boolean createFolder( Context context, String model )
    {
        if ( !this.isExternalStorageAvaiable() )
        {
            return false;
        }

        rootDir     = new File(root + "/" + context.getResources().getString(R.string.company));
        if(!rootDir.exists())
            rootDir.mkdir();

        dir     = new File(root + "/" + context.getResources().getString(R.string.company)+ "/" + model);
        tempDir = new File(dir + "/temp");

        if(!dir.exists())
            dir.mkdir();

        if(!tempDir.exists())
            tempDir.mkdir();

        return dir.exists();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getFolder()
    {
        if(!rootDir.exists())
            rootDir.mkdir();

        if(!dir.exists())
            dir.mkdir();

        return dir;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getTemporaryFolder()
    {
        if(!rootDir.exists())
            rootDir.mkdir();

        if(!dir.exists())
            dir.mkdir();

        if(!tempDir.exists())
            tempDir.mkdir();

        return tempDir;
    }

    public File getPictureFile( String serial, long timeStamp )
    {
        String img = serial + "_" + timeStamp + picSuffix;
        return new File( dir, img );
    }

    public File getAudioFile( String serial, long timeStamp )
    {
        String audio = serial + "_" + timeStamp + audioSuffix;
        return new File( dir, audio );
    }

    public File getMapSnapshotFile( String serial, long timeStamp )
    {
        String map = serial + "_" + timeStamp + mapSuffix;
        return new File( dir, map );
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public void deleteFiles( String serial, String timeStamp )
    {
        if ( dir != null && dir.listFiles() != null )
        {
            for ( File child : Objects.requireNonNull( dir.listFiles() ) )
            {
                if ( child.getName().contains( serial + "_" + timeStamp ) )
                    child.delete();
            }
        }
    }

    @Override
    public void onCreate(SQLiteDatabase db)
    {
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
    {
    }

    public static abstract class Entry
    {
        public static final int
                DB_VERSION = 1;

//        public static String
//                DB_NAME     = "MD15KVR_REGISTERS.db";

        public static final String
                DESCRIPTION = "DESCRIPTION",
                COL_A       = "COL_A",
                COL_B       = "COL_B",
                COL_C       = "COL_C",
                COL_D       = "COL_D";

        public static final String
                MODEL         = "MODEL",
                SERIAL        = "SERIAL",
                FIRMWARE      = "FIRMWARE",
                HARDWARE      = "HARDWARE",
                OP_MODE       = "OP_MODE",
                DAI_CONFIG    = "DAI_CONFIG",
                PI_CONFIG     = "PI_CONFIG",
                DATE          = "DATE",
                TEST_NUMBER   = "TEST_NUMBER",
                DAI           = "DAI",
                PI            = "PI",
                SVT           = "SVT",
                CAPACITANCE   = "CAPACITANCE",
                DD            = "DD",
                IDD           = "IDD",
                RESULT        = "RESULT",
                GPS_LATITUDE  = "GPS_LATITUDE",
                GPS_LONGITUDE = "GPS_LONGITUDE",
                GPS_ALTITUDE  = "GPS_ALTITUDE",
                GPS_ACCURACY  = "GPS_ACCURACY";

        public static String getDbName( String model )
        {
            return model + "_REGISTERS.db";
        }
    }

}

