package com.megabras.bluelogg.MPKXX5R;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import com.megabras.bluelogg.R;

import org.achartengine.chart.DialChart;
import org.achartengine.model.CategorySeries;
import org.achartengine.renderer.DefaultRenderer;
import org.achartengine.renderer.DialRenderer;
import org.achartengine.util.MathHelper;

public class CustomDialChart extends DialChart
{
    @SuppressWarnings("unused")
    private final String
            TAG = "CustomDialChart";

    private static final int
            NEEDLE_RADIUS = 3;

    private DialRenderer
            mRenderer;

    @SuppressWarnings("FieldCanBeLocal")
    private int
            minorTicksRatio = 30,
            majorTicksRatio;


    CustomDialChart( CategorySeries dataset, DialRenderer renderer, int majorTicksRatio )
    {
        super( dataset, renderer );
        mRenderer = renderer;
        this.majorTicksRatio = majorTicksRatio;
    }

    /**
     * The graphical representation of the dial chart.
     *
     * @param canvas the canvas to paint to
     * @param x the top left x value of the view to draw to
     * @param y the top left y value of the view to draw to
     * @param width the width of the view to draw to
     * @param height the height of the view to draw to
     * @param paint the paint
     */

    @Override
    public void draw(Canvas canvas, int x, int y, int width, int height, Paint paint) {
        paint.setAntiAlias(mRenderer.isAntialiasing());
        paint.setStyle(Paint.Style.FILL);
        paint.setTextSize(mRenderer.getLabelsTextSize());
        //int legendSize = getLegendSize(mRenderer, height / 5, 0);
        int legendSize  = 0,
            //left        = x,
            //top         = y,
            right       = x + width,
            sLength     = mDataset.getItemCount();

        String[] titles = new String[sLength];

        for (int i = 0; i < sLength; i++) {
            titles[i] = mDataset.getCategory(i);
        }

        if (mRenderer.isFitLegend()) {
            legendSize = drawLegend(canvas, mRenderer, titles, x, right, y, width, height, legendSize,
                    paint, true);
        }

        drawBackground(mRenderer, canvas, x, y, width, height, paint, false, DefaultRenderer.NO_COLOR);

        int bottom = y + height - legendSize;
        int mRadius = Math.min(Math.abs(right - x), Math.abs( (bottom - y)*2 ));
        int radius = (int) (mRadius * 0.35 * mRenderer.getScale());

        if (mCenterX == NO_VALUE) {
            mCenterX = (x + right) / 2;
        }

        if (mCenterY == NO_VALUE) {
            mCenterY = ((bottom + y) / 2) + (int)(radius/2.6); //CHANGES: this is to center half circle + infinite Tick
        }

        float
            shortRadius = radius * 0.95f,
            longRadius  = radius * 1.1f;

        double
            min         = mRenderer.getMinValue(),
            max         = mRenderer.getMaxValue(),
            angleMin    = mRenderer.getAngleMin(),
            angleMax    = mRenderer.getAngleMax();

        if (!mRenderer.isMinValueSet() || !mRenderer.isMaxValueSet()) {
            int count = mRenderer.getSeriesRendererCount();

            for (int i = 0; i < count; i++) {
                double value = mDataset.getValue(i);
                if (!mRenderer.isMinValueSet()) {
                    min = Math.min(min, value);
                }

                if (!mRenderer.isMaxValueSet()) {
                    max = Math.max(max, value);
                }
            }
        }

        if (min == max) {
            min = min * 0.5;
            max = max * 1.5;
        }

        paint.setColor(mRenderer.getLabelsColor());
        double minorTicks = mRenderer.getMinorTicksSpacing();
        double majorTicks = mRenderer.getMajorTicksSpacing();

        if (minorTicks == MathHelper.NULL_VALUE) {
            minorTicks = (max - min) / minorTicksRatio;
        }

        if (majorTicks == MathHelper.NULL_VALUE) {
            majorTicks = (max - min) / majorTicksRatio;
        }

        paint.setStrokeWidth(1.0f);
        paint.setAlpha(125);

        drawTicks(canvas, min, max, angleMin, angleMax, mCenterX, mCenterY, longRadius, radius,
                minorTicks, paint, false);

        paint.setAlpha(255);
        paint.setStrokeWidth(2.0f);

        drawTicks(canvas, min, max, angleMin, angleMax, mCenterX, mCenterY, longRadius, shortRadius,
                majorTicks, paint, true);

        int count = mRenderer.getSeriesRendererCount();

        drawTitle(canvas, x, y + (height/2)-20, width, paint);

        for (int i = 0; i < count; i++) {
            double angle;
            if(mDataset.getValue(i)<=max)
                angle = getAngleForValue(mDataset.getValue(i), angleMin, angleMax, min, max);
            else
                angle = getAngleForValue(max, angleMin, angleMax, min, max);

            paint.setColor(mRenderer.getSeriesRendererAt(i).getColor());
            boolean type = mRenderer.getVisualTypeForIndex(i) == DialRenderer.Type.ARROW;
            drawNeedle(canvas, angle, mCenterX, mCenterY, shortRadius, type, paint);
        }

        drawLegend(canvas, mRenderer, titles, x, right, y, width, height, legendSize, paint, false);

        //drawTitle(canvas, x, y + (height/2), width, paint);
    }

    public void drawTitle(Canvas canvas, int x, int y, int width, Paint paint) {
        if(this.mRenderer.isShowLabels()) {
            paint.setColor(this.mRenderer.getLabelsColor());
            paint.setTextAlign(Paint.Align.CENTER);
            paint.setTextSize(this.mRenderer.getChartTitleTextSize());
            String[] range = this.mRenderer.getChartTitle().split("\n");
            this.drawString(canvas, range[0], (float)(x + width / 2), (float)y + this.mRenderer.getChartTitleTextSize(), paint);
            if( range.length>1 ){
                int size = (int) ((this.mRenderer.getChartTitleTextSize()/3) * 2);
                paint.setTextSize( size );
                this.drawString(canvas, range[1], (float)(x + width / 2), (float)y + size + this.mRenderer.getChartTitleTextSize(), paint);
            }
        }
    }

    private double getAngleForValue(double value, double minAngle, double maxAngle, double min,
                                    double max) {
        double angleDiff = maxAngle - minAngle;
        double diff = max - min;
        return Math.toRadians(minAngle + (value - min) * angleDiff / diff);
    }

    private Context context;
    public void setContext(Context context){
        this.context = context;
    }

    private void drawTicks(Canvas canvas, double min, double max, double minAngle, double maxAngle,
                           int centerX, int centerY, double longRadius, double shortRadius, double ticks, Paint paint,
                           boolean labels) {
        for (double i = min; i <= max; i += ticks) {
            double angle = getAngleForValue(i, minAngle, maxAngle, min, max);
            double sinValue = Math.sin(angle);
            double cosValue = Math.cos(angle);
            int x1 = Math.round(centerX + (float) (shortRadius * sinValue));
            int y1 = Math.round(centerY + (float) (shortRadius * cosValue));

            //int fontSize = 8;

            int fontSize = (int) context.getResources().getDimension( R.dimen.mpk257_dial_tick_font);

            /* Infinity symbol needs higher font size */
            //if(i>=max)
                //fontSize = 14;

            float val = 20;
            if(context!=null){
                DisplayMetrics metrics = context.getResources().getDisplayMetrics();
                val = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, fontSize, metrics);
            }

            int x3 = Math.round(centerX + (float) ((shortRadius-val) * sinValue));
            int y3 = Math.round(centerY + (float) ((shortRadius-val) * cosValue));
            int x2 = Math.round(centerX + (float) (longRadius * sinValue));
            int y2 = Math.round(centerY + (float) (longRadius * cosValue));

            canvas.drawLine(x1, y1, x2, y2, paint);

            if (labels) {
                paint.setTextSize(val);
                paint.setTextAlign(Paint.Align.CENTER);

                String text = (float)i + "";


                if(max == (int)max){// Number is integer
                    if (Math.round(i) == (long) i) {
                        text = (long) i + "";
                    }
                }

                Rect bounds = new Rect();
                paint.getTextBounds("0", 0, 1, bounds);

                float diff = (float) bounds.height() / 2;

                /* Infinity symbol */
//                if((float)i>=max) {
//                    text = "\u221E";
//                    x3 = (int)(x3+diff);
//                }

                //canvas.drawText(text, x1, y1+diff, paint);
                canvas.drawText(text, x3, y3+diff, paint);
            }
        }
    }

    private void drawNeedle(Canvas canvas, double angle, int centerX, int centerY, double radius,
                            boolean arrow, Paint paint) {
        double diff = Math.toRadians(90);
        int needleSinValue = (int) (NEEDLE_RADIUS * Math.sin(angle - diff));
        int needleCosValue = (int) (NEEDLE_RADIUS * Math.cos(angle - diff));
        int needleX = (int) (radius * Math.sin(angle));
        int needleY = (int) (radius * Math.cos(angle));
        int needleCenterX = centerX + needleX;
        int needleCenterY = centerY + needleY;

        float[] points;

        if (arrow) {
            int arrowBaseX = centerX + (int) (radius * 0.85 * Math.sin(angle));
            int arrowBaseY = centerY + (int) (radius * 0.85 * Math.cos(angle));
            points = new float[] { arrowBaseX - needleSinValue, arrowBaseY - needleCosValue,
                    needleCenterX, needleCenterY, arrowBaseX + needleSinValue, arrowBaseY + needleCosValue };
            float width = paint.getStrokeWidth();
            paint.setStrokeWidth(5);
            canvas.drawLine(centerX, centerY, needleCenterX, needleCenterY, paint);
            paint.setStrokeWidth(width);
        } else {
//            points = new float[] { centerX - needleSinValue, centerY - needleCosValue, needleCenterX,
//                    needleCenterY, centerX + needleSinValue, centerY + needleCosValue };
            points = new float[] { centerX - needleSinValue, centerY - needleCosValue, needleCenterX,
                    needleCenterY, centerX + needleSinValue, centerY + needleCosValue };
            canvas.drawCircle(centerX, centerY, (int)(radius*0.1), paint);
        }

        drawPath(canvas, points, paint, true);
    }

//    public void drawTitle(Canvas canvas, int x, int y, int width, int height, Paint paint) {
//        if (mRenderer.isShowLabels()) {
//            paint.setColor(mRenderer.getLabelsColor());
//            paint.setTextAlign(Align.CENTER);
//            paint.setTextSize(mRenderer.getChartTitleTextSize());
//            drawString(canvas, mRenderer.getChartTitle(), x + width / 2,
//                    y + (height/2) + mRenderer.getChartTitleTextSize(), paint);
//        }
//    }
}

