package com.megabras.bluelogg.MPKXX5R;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MPKXX5R.Measurement;
import com.megabras.modbus.MPKXX5R.RecordData;
import com.megabras.modbus.MPKXX5R.TestData;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

public final class Database extends SQLiteOpenHelper
{
    @SuppressWarnings("FieldCanBeLocal")
    private String
            TAG          = "Database";

    //@SuppressWarnings("unused")
//    private static final String
//            TEXT_TYPE    = " TEXT",
//            INT_TYPE     = " INTEGER",
//            FLOAT_TYPE   = " FLOAT",
//            DOUBLE_TYPE  = " DOUBLE",
//            DATE_TYPE    = " DATETIME",
//            COMMA_SEP    = ",";

    private String
            root         = Environment.getExternalStorageDirectory().toString(),
            serial;

    private File
            dir,
            tempDir,
            rootDir;

    public Database( Context context, String model, String serial )
    {
        super( context, Entry.DB_NAME, null, Entry.DB_VERSION );
        createTables();
        createFolder( context, model );
        this.serial = serial;
    }

    private void createTables()
    {
        SQLiteDatabase db = this.getWritableDatabase();

        String SQL;

//        db.execSQL( "DROP TABLE IF EXISTS " + Entry.TABLE_RECORDS );
//        db.execSQL( "DROP TABLE IF EXISTS " + Entry.TABLE_TESTS );
//        db.execSQL( "DROP TABLE IF EXISTS " + Entry.TABLE_MEASURES );

        SQL = "CREATE TABLE IF NOT EXISTS " + Entry.TABLE_RECORDS + " (\n"
                + "\t" + Entry.RECORD_ID    + " INTEGER PRIMARY KEY AUTOINCREMENT, \n"
                + "\t" + Entry.NAME         + " TEXT   , \n"
                + "\t" + Entry.DEVICE_ID    + " INTEGER, \n"
                + "\t" + Entry.MODEL        + " TEXT   , \n"
                + "\t" + Entry.SERIAL       + " TEXT   , \n"
                + "\t" + Entry.PREFFIX      + " INTEGER, \n"
                + "\t" + Entry.HARDWARE     + " INTEGER, \n"
                + "\t" + Entry.FIRMWARE     + " INTEGER, \n"
                + "\t" + Entry.LAST_CHANGE  + " DATETIME, \n"
                + "\t" + Entry.LONGITUDE    + " REAL, \n"
                + "\t" + Entry.LATITUDE     + " REAL, \n"
                + "\t" + Entry.ALTITUDE     + " REAL, \n"
                + "\t" + Entry.ACCURACY     + " REAL  \n"
                + ");";

        db.execSQL(SQL);

        SQL = "CREATE TABLE IF NOT EXISTS " + Entry.TABLE_TESTS + " (\n"
                + "\t" + Entry.TEST_ID      + " INTEGER PRIMARY KEY AUTOINCREMENT, \n"
                + "\t" + Entry.NUMBER       + " INTEGER, \n"
                + "\t" + Entry.DATE         + " DATETIME, \n"
                + "\t" + Entry.MODE         + " INTEGER, \n"
                + "\t" + Entry.HAS_TIMER    + " INTEGER, \n"
                + "\t" + Entry.TIMER        + " INTEGER, \n"
                + "\t" + Entry.HAS_BSG      + " INTEGER, \n"
                + "\t" + Entry.RECORD_ID    + " INTEGER, \n"
                + "\t" + "FOREIGN KEY (" + Entry.RECORD_ID + ") "
                + "\t" + "REFERENCES " + Entry.TABLE_RECORDS + " ( " + Entry.RECORD_ID + ") ON DELETE CASCADE "
            + ");";

        db.execSQL(SQL);

        SQL = "CREATE TABLE IF NOT EXISTS " + Entry.TABLE_MEASURES + " (\n"
                + "\t" + Entry.MEASURE_ID   + " INTEGER PRIMARY KEY AUTOINCREMENT, \n"
                + "\t" + Entry.ELAPSED      + " INTEGER, \n"
                + "\t" + Entry.OHM          + " REAL, \n"
                + "\t" + Entry.AMP          + " REAL, \n"
                + "\t" + Entry.LOOP         + " REAL, \n"
                + "\t" + Entry.STATUS       + " INTEGER, \n"
                + "\t" + Entry.TEST_ID      + " INTEGER, \n"
                + "\t" + "FOREIGN KEY (" + Entry.TEST_ID + ") "
                + "\t" + "REFERENCES " + Entry.TABLE_TESTS + " ( " + Entry.TEST_ID + ") ON DELETE CASCADE "
                + ");";

        db.execSQL(SQL);

        db.execSQL("PRAGMA foreign_keys=ON");
    }

    void saveOrUpdate( RecordData recordData )
    {
        SQLiteDatabase db = this.getWritableDatabase();

        db.beginTransaction();

        ContentValues values = new ContentValues();

        // RECORD DATA
        if ( recordData.getRecordId() > 0 ) // UPDATE
        {
            values.put( Entry.LAST_CHANGE, new Date().getTime() );
            values.put( Entry.LONGITUDE  , recordData.getGpsLongitude() );
            values.put( Entry.LATITUDE   , recordData.getGpsLatitude() );
            values.put( Entry.ALTITUDE   , recordData.getGpsAltitude() );
            values.put( Entry.ACCURACY   , recordData.getGpsAccuracy() );

            db.update( Entry.TABLE_RECORDS, values, Entry.RECORD_ID + "=?", new String[]{ recordData.getRecordId() + "" } );
            //Log.e(TAG, "UPDATE RECORD: " + recordData.getRecordId() );
        }
        else // INSERT NEW
        {
            values.put( Entry.NAME       , recordData.getRecordName() );
            values.put( Entry.DEVICE_ID  , recordData.getDeviceId() );
            values.put( Entry.MODEL      , recordData.getDeviceModel() );
            values.put( Entry.SERIAL     , recordData.getDeviceSerial() );
            values.put( Entry.PREFFIX    , recordData.getDeviceVersionPreffix() );
            values.put( Entry.HARDWARE   , recordData.getDeviceHardware() );
            values.put( Entry.FIRMWARE   , recordData.getDeviceFirmware() );
            values.put( Entry.LAST_CHANGE, new Date().getTime() );
            values.put( Entry.LONGITUDE  , recordData.getGpsLongitude() );
            values.put( Entry.LATITUDE   , recordData.getGpsLatitude() );
            values.put( Entry.ALTITUDE   , recordData.getGpsAltitude() );
            values.put( Entry.ACCURACY   , recordData.getGpsAccuracy() );

            long id = db.insertWithOnConflict( Entry.TABLE_RECORDS, null, values, SQLiteDatabase.CONFLICT_IGNORE );
            recordData.setRecordId( (int) id );

           // Log.e(TAG, "NEW RECORD: " + recordData.getRecordId() );
        }

        // TEST DATA
        for ( TestData td : recordData.getTestDatas() )
        {
            if ( td.getTestId() < 0 ) // INSERT NEW TEST DATA
            {
                values = new ContentValues();
                values.put( Entry.NUMBER    , td.getTestNumber() );
                values.put( Entry.DATE      , td.getDate().getTime() );
                values.put( Entry.MODE      , td.getMode() );
                values.put( Entry.HAS_TIMER , td.isTimerEnabled() ? 1 : 0 );
                values.put( Entry.TIMER     , td.getTimer() );
                values.put( Entry.HAS_BSG   , td.isBsgEnabled() ? 1 : 0 );
                values.put( Entry.RECORD_ID , recordData.getRecordId() );

                long id = db.insertWithOnConflict( Entry.TABLE_TESTS, null, values, SQLiteDatabase.CONFLICT_IGNORE );
                td.setTestId( (int) id );

                //Log.e(TAG, "NEW TEST: " + td.getTestId() );
            }

            for ( Measurement m : td.getMeasurements() )
            {
                if ( m.getId() < 0 ) // INSERT NEW MEASUREMENT
                {
                    values = new ContentValues();
                    values.put( Entry.ELAPSED , m.getElapsed() );
                    values.put( Entry.OHM     , m.getOhm() );
                    values.put( Entry.AMP     , m.getAmp() );
                    values.put( Entry.LOOP    , m.getClampAmp() );
                    values.put( Entry.STATUS  , m.getStatus() );
                    values.put( Entry.TEST_ID , td.getTestId() );

                    long mId = db.insertWithOnConflict( Entry.TABLE_MEASURES, null, values, SQLiteDatabase.CONFLICT_IGNORE );
                    m.setId( (int) mId );

                    //Log.e(TAG, "NEW MEASURE: " + m.getId() );
                }
            }
        }

        db.setTransactionSuccessful();
        db.endTransaction();
    }

    @SuppressWarnings("unused")
    void checkDatabase()
    {
        // Entry.TABLE_RECORDS
        SQLiteDatabase db         = this.getReadableDatabase();
        Cursor         c          = db.rawQuery( "SELECT * FROM " + Entry.TABLE_RECORDS + ";", null );

        if ( c == null )
        {
            return;
        }

        if ( c.moveToFirst() )
        {
            if ( !c.isAfterLast() ) Log.e( TAG, Entry.TABLE_RECORDS + " EMPTY **********************************************************************************");
            while ( !c.isAfterLast() )
            {
                int i = 0;
                Log.e( TAG, "RecordData ##############################################################");
                Log.e( TAG, "recordId       : " + c.getInt( i++ ) );
                Log.e( TAG, "recordName     : " + c.getString( i++ ) );
                Log.e( TAG, "deviceId       : " + c.getInt( i++ ) );
                Log.e( TAG, "model          : " + c.getString( i++ ) );
                Log.e( TAG, "serial         : " + c.getString( i++ ) );
                Log.e( TAG, "devicePreffix  : " + c.getInt( i++ ) );
                Log.e( TAG, "deviceHardware : " + c.getInt( i++ ) );
                Log.e( TAG, "deviceFirmware : " + c.getInt( i++ ) );
                Log.e( TAG, "lastChanged    : " + new Date( Long.parseLong( c.getString( i++ ) ) ) );
                Log.e( TAG, "longitude      : " + c.getDouble( i++ ) );
                Log.e( TAG, "latitude       : " + c.getDouble( i++ ) );
                Log.e( TAG, "altitude       : " + c.getDouble( i++ ) );
                Log.e( TAG, "accuracy       : " + c.getDouble( i ) );
                Log.e( TAG, "" );
                c.moveToNext();
            }
        }
        c.close();

        c = db.rawQuery( "SELECT * FROM " + Entry.TABLE_TESTS + ";", null );
        if ( c.moveToFirst() )
        {
            if ( !c.isAfterLast() ) Log.e( TAG, Entry.TABLE_TESTS+ " EMPTY **********************************************************************************");
            while ( !c.isAfterLast() )
            {
                int i = 0;
                Log.e( TAG, "TestData ##############################################################");
                Log.e( TAG, "testId         : " + c.getInt( i++ ) );
                Log.e( TAG, "#              : " + c.getInt( i++ ) );
                Log.e( TAG, "date           : " + new Date( Long.parseLong( c.getString( i++ ) ) ) );
                Log.e( TAG, "mode           : " + c.getInt( i++ ) );
                Log.e( TAG, "is Timer       : " + (c.getInt( i++ ) == 1) );
                Log.e( TAG, "Timer          : " + c.getInt( i++ ) );
                Log.e( TAG, "is BSG         : " + (c.getInt( i++ ) == 1) );
                Log.e( TAG, "RecordID       : " + c.getInt( i ) );
                Log.e( TAG, "" );
                c.moveToNext();
            }
        }
        c.close();

        c = db.rawQuery( "SELECT * FROM " + Entry.TABLE_MEASURES + ";", null );
        if ( c.moveToFirst() )
        {
            if ( !c.isAfterLast() ) Log.e( TAG, Entry.TABLE_MEASURES + " EMPTY **********************************************************************************");
            while ( !c.isAfterLast() )
            {
                int i = 0;
                Log.e( TAG, "Measurement ##############################################################");
                Log.e( TAG, "ID       : " + c.getInt( i++ ) );
                Log.e( TAG, "ELAPSED  : " + c.getInt( i++ ) );
                Log.e( TAG, "OHM      : " + c.getDouble( i++ ) );
                Log.e( TAG, "AMP      : " + c.getDouble( i++ ) );
                Log.e( TAG, "LOOP     : " + c.getDouble( i++ ) );
                Log.e( TAG, "STATUS   : " + c.getInt( i++ ));
                Log.e( TAG, "TEST ID  : " + c.getInt( i ) );
                Log.e( TAG, "" );

                c.moveToNext();
            }
        }
        c.close();
    }

    void deleteRecordData( int recordDataId )
    {
        try
        {
            SQLiteDatabase db = this.getWritableDatabase();
            db.delete( Entry.TABLE_RECORDS, Entry.RECORD_ID + "=?", new String[]{recordDataId + ""} );
            deleteFiles( recordDataId );
            //checkDatabase();
        }
        catch ( Exception e )
        {
            android.util.Log.e( TAG, Objects.requireNonNull( e.getMessage() ) );
        }
    }

    List<FileDetails> getSavedList()
    {
        try
        {
            List<FileDetails> filesList = new ArrayList<>();
            SQLiteDatabase    db        = this.getReadableDatabase();
            //Cursor            c         = db.rawQuery( "SELECT " + Entry.RECORD_ID + ", " + Entry.NAME + ", " + Entry.LAST_CHANGE +  " FROM " + Entry.TABLE_RECORDS + " ORDER BY " + Entry.RECORD_ID + " ;", null );
            Cursor            c         = db.query( Entry.TABLE_RECORDS,  new String[] {Entry.RECORD_ID, Entry.NAME, Entry.LAST_CHANGE},null, null, null, null, null);

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    int i = 0;
                    int recordId       = c.getInt( i++ );
                    String recordName  = c.getString( i++ );
                    String sDate       = c.getString( i );

                    long   lDate = Long.parseLong( sDate );
                    Date date = new Date( lDate );

                    FileDetails file = new FileDetails();
                    file.id          = recordId;
                    file.recordName  = recordName;
                    file.lastChanged = date;

//                    file.DATE = formatData.dateTime( new Date( lDate ), dateFormat, timeFormat );

//                    Log.e( TAG, "FileDetails ##############################################################");
//                    Log.e( TAG, "recordId       : " + file.id );
//                    Log.e( TAG, "recordName     : " + file.recordName );
//                    Log.e( TAG, "lastChanged    : " + file.lastChanged );

                    filesList.add( file );

                    //getRecordData( file.id );

                    c.moveToNext();
                }
            }
            c.close();

//            if ( filesList.size() > 0 )
//            {
//                getRecordData( filesList.get( filesList.size() - 1 ).id );
//            }

            return filesList;
        }
        catch ( Exception e )
        {
            return null;
        }
    }


    RecordData getRecordData( int recordId )
    {
        try
        {
            RecordData     recordData = new RecordData();
            SQLiteDatabase db         = this.getReadableDatabase();
            Cursor         c          = db.rawQuery( "SELECT * FROM " + Entry.TABLE_RECORDS + " WHERE " + Entry.RECORD_ID + "=?", new String[]{ recordId + "" } );

            if ( c.moveToFirst() )
            {
                int i = 0;
                recordData.setRecordId( c.getInt( i++ ) );
                recordData.setRecordName( c.getString( i++ ) );
                recordData.setDeviceId( c.getInt( i++ ) );
                recordData.setDeviceModel( c.getString( i++ ) );
                recordData.setDeviceSerial( c.getString( i++ ) );
                recordData.setDeviceVersionPreffix( c.getInt( i++ ) );
                recordData.setDeviceHardware( c.getInt( i++ ) );
                recordData.setDeviceFirmware( c.getInt( i++ ) );

                String sDate = c.getString( i++ );
                long   lDate = Long.parseLong( sDate );

                recordData.setLastChanged( new Date( lDate ) );

                recordData.setGpsLongitude( c.getDouble( i++ ) );
                recordData.setGpsLatitude( c.getDouble( i++ ) );
                recordData.setGpsAltitude( c.getDouble( i++ ) );
                recordData.setGpsAccuracy( c.getDouble( i ) );

//                Log.e( TAG, "RecordData ##############################################################");
//                Log.e( TAG, "recordId       : " + recordData.getRecordId() );
//                Log.e( TAG, "recordName     : " + recordData.getRecordName() );
//                Log.e( TAG, "deviceId       : " + recordData.getDeviceId() );
//                Log.e( TAG, "model          : " + recordData.getDeviceModel() );
//                Log.e( TAG, "serial         : " + recordData.getDeviceSerial() );
//                Log.e( TAG, "devicePreffix  : " + recordData.getDeviceVersionPreffix() );
//                Log.e( TAG, "deviceHardware : " + recordData.getDeviceHardware() );
//                Log.e( TAG, "deviceFirmware : " + recordData.getDeviceFirmware() );
//                Log.e( TAG, "lastChanged    : " + recordData.getLastChanged() );
//                Log.e( TAG, "longitude      : " + recordData.getGpsLongitude() );
//                Log.e( TAG, "latitude       : " + recordData.getGpsLatitude() );
//                Log.e( TAG, "altitude       : " + recordData.getGpsAltitude() );
//                Log.e( TAG, "accuracy       : " + recordData.getGpsAccuracy() );
            }
            c.close();

            recordData.setTestDatas( getTestDatas( recordData.getRecordId() ) );

            return recordData;
        }
        catch ( Exception e )
        {
            return null;
        }
    }

    private ArrayList<TestData> getTestDatas( int recordId )
    {
        try
        {
            ArrayList<TestData> testDatas = new ArrayList<>();
            SQLiteDatabase      db        = this.getReadableDatabase();
            Cursor              c         = db.rawQuery( "SELECT * FROM " + Entry.TABLE_TESTS + " WHERE " + Entry.RECORD_ID + "=?", new String[]{recordId + ""} );
//            Cursor              c         = db.rawQuery( "SELECT * FROM " + Entry.TABLE_TESTS + ";", null );

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    int i = 0;
                    TestData td = new TestData();

                    td.setTestId( c.getInt( i++ ) );
                    td.setTestNumber( c.getInt( i++ ) );

                    String sDate = c.getString( i++ );
                    long   lDate = Long.parseLong( sDate );

                    td.setDate( new Date( lDate ) );
                    td.setMode( c.getInt( i++ ) );
                    td.setTimerEnabled( c.getInt( i++ ) == 1 );
                    td.setTimer( c.getInt( i++ ) );
                    td.setBsgEnabled( c.getInt( i ) == 1 );

//                    Log.e( TAG, "TestData ##############################################################");
//                    Log.e( TAG, "testId         : " + td.getTestId() );
//                    Log.e( TAG, "#              : " + td.getTestNumber() );
//                    Log.e( TAG, "date           : " + td.getDate() );
//                    Log.e( TAG, "mode           : " + td.getMode() );
//                    Log.e( TAG, "is Timer       : " + td.isTimerEnabled() );
//                    Log.e( TAG, "Timer          : " + td.getTimer() );
//                    Log.e( TAG, "is BSG         : " + td.isBsgEnabled() );
//                    Log.e( TAG, "RecordID       : " + c.getInt( i ) );

                    testDatas.add( td );
                    c.moveToNext();
                }
            }
            c.close();

            for ( TestData t : testDatas )
            {
                t.setMeasurements( getMeasurements( t.getTestId() ) );
            }

            return testDatas;
        }
        catch ( Exception e )
        {
            return null;
        }
    }

    private ArrayList<Measurement> getMeasurements( int testId )
    {
        try
        {
            ArrayList<Measurement> measures = new ArrayList<>();
            SQLiteDatabase         db       = this.getReadableDatabase();
            Cursor                 c        = db.rawQuery( "SELECT * FROM " + Entry.TABLE_MEASURES + " WHERE " + Entry.TEST_ID + "=?", new String[]{testId + ""} );

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    int i = 0;
                    Measurement m = new Measurement();

                    m.setId( c.getInt( i++ ) );
                    m.setElapsed( c.getInt( i++ ) );
                    m.setOhm( c.getDouble( i++ ) );
                    m.setAmp( c.getDouble( i++ ) );
                    m.setClampAmp( c.getDouble( i++ ) );
                    m.setStatus( c.getInt( i ) );

//                    Log.e( TAG, "Measurement ##############################################################");
//                    Log.e( TAG, "ID       : " + m.getId() );
//                    Log.e( TAG, "ELAPSED  : " + m.getElapsedString() );
//                    Log.e( TAG, "OHM      : " + m.getOhmString() );
//                    Log.e( TAG, "AMP      : " + m.getAmpString() );
//                    Log.e( TAG, "LOOP     : " + m.getClampAmp() );
//                    Log.e( TAG, "STATUS   : " + m.getStatus() );
//                    Log.e( TAG, "TEST ID  : " + c.getInt( i++ ) );

                    measures.add( m );

                    c.moveToNext();
                }
            }
            c.close();
            return measures;
        }
        catch ( Exception e )
        {
            return null;
        }
    }

    //##########################################################################
    //# Folders
    //##########################################################################

    private boolean isExternalStorageAvaiable()
    {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals( state );
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public boolean createFolder( Context context, String model )
    {
        if ( !this.isExternalStorageAvaiable() )
            return false;

        rootDir = new File( root + "/" + context.getResources().getString( R.string.company ) );
        if ( !rootDir.exists() )
            rootDir.mkdir();

        dir = new File( root + "/" + context.getResources().getString( R.string.company ) + "/" + model );
        tempDir = new File( dir + "/temp" );

        if ( !dir.exists() )
            dir.mkdir();

        if ( !tempDir.exists() )
            tempDir.mkdir();

        return dir.exists();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getFolder()
    {
        if ( !rootDir.exists() )
            rootDir.mkdir();

        if ( !dir.exists() )
            dir.mkdir();

        return dir;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    File getTemporaryFolder()
    {
        if ( !rootDir.exists() )
            rootDir.mkdir();

        if ( !dir.exists() )
            dir.mkdir();

        if ( !tempDir.exists() )
            tempDir.mkdir();

        return tempDir;
    }

    public File getPictureFile( int recordId )
    {
        String img = serial + "_ID" + recordId + "_" + ".jpg";
        return new File(dir, img);
    }

    public File getAudioFile( int recordId )
    {
        String audioSuffix = "_annotation.3gp";
        String audio       = serial + "_ID" + recordId + audioSuffix;
        return new File( dir, audio );
    }

    public File getMapSnapshotFile( int recordId )
    {
        String mapSuffix = "_map_snapshot.jpg";
        String map       = serial + "_ID" + recordId + "_" + mapSuffix;
        return new File( dir, map );
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private void deleteFiles( int recordId )
    {
        if ( dir != null && dir.listFiles() != null )
        {
            for ( File child : Objects.requireNonNull( dir.listFiles() ) )
            {
                if( child.getName().contains( serial + "_ID" + recordId ) )
                {
                    child.delete();
                }
            }
        }
    }

    @Override public void onCreate(SQLiteDatabase db) {}
    @Override public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) { }

    public static abstract class Entry
    {
        static final int
                DB_VERSION = 1;

        static final String
                DB_NAME         = Controller.model + "_" + Controller.serial + "_REGISTERS.db",
                TABLE_RECORDS   = "RECORDS",
                TABLE_TESTS     = "TESTS",
                TABLE_MEASURES  = "MEASURES",

                // RECORDS FIELDS
                RECORD_ID       = "RECORD_ID",
                NAME            = "NAME",
                DEVICE_ID       = "DEVICE_ID",
                MODEL           = "MODEL",
                SERIAL          = "SERIAL",
                PREFFIX         = "PREFFIX",
                HARDWARE        = "HARDWARE",
                FIRMWARE        = "FIRMWARE",
                LAST_CHANGE     = "LAST_CHANGE",
                LONGITUDE       = "LONGITUDE",
                LATITUDE        = "LATITUDE",
                ALTITUDE        = "ALTITUDE",
                ACCURACY        = "ACCURACY",

                // TESTS FIELDS
                TEST_ID         = "TEST_ID",
                NUMBER          = "NUMBER",
                DATE            = "DATE",
                MODE            = "MODE",
                HAS_TIMER       = "HAS_TIMER",
                TIMER           = "TIMER",
                HAS_BSG         = "HAS_BSG",

                // MEASURES FIELDS
                MEASURE_ID      = "MEASURE_ID",
                ELAPSED         = "ELAPSED",
                OHM             = "OHM",
                AMP             = "AMP",
                LOOP            = "LOOP",
                STATUS          = "STATUS";
    }

}

