package com.megabras.bluelogg.MPKXX5R;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MPKXX5R.DeviceOld;
import com.megabras.modbus.MPKXX5R.FormatData;

import org.achartengine.GraphicalView;
import org.achartengine.model.CategorySeries;
import org.achartengine.renderer.DialRenderer;
import org.achartengine.renderer.SimpleSeriesRenderer;

import androidx.fragment.app.Fragment;

public class MainFragment extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG             = "MainFragment",
            CATEGORY_NAME   = "ANALOG";

    @SuppressWarnings("FieldCanBeLocal")
    private int
            layoutID        =  R.layout.mpkxx5r_fgr_main;

    private Context
            context;

    private LinearLayout
            layout;

    private GraphicalView
            graphicalView;

    private DialRenderer
            renderer;

    private CategorySeries
            category;

    private View
            view;

    @SuppressWarnings("NullableProblems")
    @Override
    public View onCreateView( LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState )
    {
        context = inflater.getContext();
        view = inflater.inflate( layoutID, container, false );
        view.setBackgroundColor( context.getResources().getColor( R.color.transparent ) );

        getIV( R.id.ui_main_iv_bsg ).setVisibility( View.INVISIBLE );
        getIV( R.id.ui_main_iv_autosave ).setVisibility( View.INVISIBLE );
        getTV( R.id.ui_main_tv_mode ).setVisibility( View.GONE );
        getTV( R.id.ui_main_tv_mode ).setText( "" );
        getTV( R.id.ui_main_tv_auto_current ).setText( "" );
        getTV( R.id.ui_main_tv_auto_timer ).setText( "" );
        getTV( R.id.ui_main_tv_test_number ).setText( "" );
        getTV( R.id.ui_main_tv_ohm ).setText( "" );
        getTV( R.id.ui_main_tv_amp ).setText( "" );
        getTV( R.id.ui_main_tv_loop ).setText( "" );
        getTV( R.id.ui_main_tv_elapsed ).setText( "" );
        getTV( R.id.ui_main_tv_record_name ).setText( "" );

        return view;
    }

    @Override
    public void onStart()
    {
        super.onStart();
        prepareGauge();
        setCurrentGauge( 110, 1, 11, "A" );
        this.updateGauge( 0 );
    }

    public void disconnected()
    {
        setCurrentGauge( 110, 1, 11, "A" );
        this.updateGauge( 0 );
        getIV( R.id.ui_main_iv_bsg ).setVisibility( View.INVISIBLE );
        getIV( R.id.ui_main_iv_autosave ).setVisibility( View.INVISIBLE );
        getTV( R.id.ui_main_tv_mode ).setVisibility( View.GONE );
        getTV( R.id.ui_main_tv_mode ).setText( "" );
        getTV( R.id.ui_main_tv_auto_current ).setText( "" );
        getTV( R.id.ui_main_tv_auto_timer ).setText( "" );
        getTV( R.id.ui_main_tv_test_number ).setText( "" );
        getTV( R.id.ui_main_tv_ohm ).setText( "" );
        getTV( R.id.ui_main_tv_amp ).setText( "" );
        getTV( R.id.ui_main_tv_loop ).setText( "" );
        getTV( R.id.ui_main_tv_elapsed ).setText( "" );
        getTV( R.id.ui_main_tv_record_name ).setText( "" );
    }

    private void prepareGauge()
    {
        if ( this.getView() != null )
        {
            layout = this.getView().findViewById( R.id.gauge );
        }
        else
        {
            return;
        }

        if ( layout.getChildCount() > 0 )
        {
            layout.removeAllViews();
        }

        category = new CategorySeries( CATEGORY_NAME );
        category.add( CATEGORY_NAME, 0 );

        renderer = new DialRenderer();

        DisplayMetrics metrics = context.getResources().getDisplayMetrics();
        int            fSize   = (int) context.getResources().getDimension( R.dimen.mpk257_dial_title_font );
        float          val     = TypedValue.applyDimension( TypedValue.COMPLEX_UNIT_DIP, fSize, metrics );
        renderer.setChartTitleTextSize( val );

        fSize = (int) context.getResources().getDimension( R.dimen.mpk257_dial_label_font );
        val = TypedValue.applyDimension( TypedValue.COMPLEX_UNIT_DIP, fSize, metrics );
        renderer.setLabelsTextSize( val );
        //@param margins an array containing the margin size values, in this order:
        renderer.setMargins( new int[]{0, 0, 0, 0} );// top, left, bottom, right
        renderer.setInScroll( true );

        SimpleSeriesRenderer r = new SimpleSeriesRenderer();
        r.setColor( Color.RED );
        renderer.addSeriesRenderer( r );
        renderer.setLabelsColor( Color.WHITE );
        renderer.setAngleMin( 270 );
        renderer.setAngleMax( 72 );
        renderer.setShowLabels( true );
        renderer.setLegendHeight( 0 );
        renderer.setShowLegend( false );
        renderer.setVisualTypes( new DialRenderer.Type[]{DialRenderer.Type.NEEDLE} );
        renderer.setMinValue( 0 );
        renderer.setPanEnabled( false );
        renderer.setZoomEnabled( false );
    }

    private GraphicalView getDialChartView( Context context, CategorySeries dataset,
                                            DialRenderer renderer, int majorTicksRatio ) {
        CustomDialChart chart = new CustomDialChart(dataset, renderer, majorTicksRatio);
        chart.setContext(context);
        return new GraphicalView(context, chart);
    }

    private void updateGauge( double value )
    {
        category.set( 0, CATEGORY_NAME, value );
        graphicalView.repaint();
    }

    public void updateMeasurement( DeviceOld device )
    {
        setTVText( R.id.ui_main_tv_test_number  , device.getRTTestNumber() );
        if ( device.getRtMeasurement() != null )
        {
            setTVText( R.id.ui_main_tv_ohm, device.getRtMeasurement().getOhmString() );
            setTVText( R.id.ui_main_tv_amp, device.getRtMeasurement().getAmpString() );
            setTVText( R.id.ui_main_tv_loop, device.getRtMeasurement().getClampAmpString( device.getBSG() ) );
            setTVText( R.id.ui_main_tv_elapsed, device.getRtMeasurement().getElapsedString() );

            updateGauge( device.getRtMeasurement().getAmp() );
        }
    }

    public void updateParameters( DeviceOld device )
    {
        getTV( R.id.ui_main_tv_mode ).setVisibility( View.VISIBLE );
        if ( device.getMode() == DeviceOld.MODE_MANUAL )
        {
            setTVText( R.id.ui_main_tv_mode, getString( R.string.manual ) );
            setTVText( R.id.ui_main_tv_auto_current, "- - -" );
        }
        else
        {
            setTVText( R.id.ui_main_tv_mode, getString( R.string.auto ) );
            setTVText( R.id.ui_main_tv_auto_current, device.getConfAmp() + " A" );
        }

        if ( device.getTimer() )
        {
            setTVText( R.id.ui_main_tv_auto_timer, FormatData.elapsed( device.getConfTimer() ) );
        }
        else
        {
            setTVText( R.id.ui_main_tv_auto_timer, "- - -" );
        }

        if ( device.getBSG() )
        {
            getIV( R.id.ui_main_iv_bsg ).setVisibility( View.VISIBLE );
        }
        else
        {
            getIV( R.id.ui_main_iv_bsg ).setVisibility( View.INVISIBLE );
        }

        if ( device.getAutoSave() )
        {
            getIV( R.id.ui_main_iv_autosave ).setVisibility( View.VISIBLE );
        }
        else
        {
            getIV( R.id.ui_main_iv_autosave ).setVisibility( View.INVISIBLE );
        }

        setTVText( R.id.ui_main_tv_record_name, device.getRecordName() );
    }

    @SuppressWarnings("SameParameterValue")
    private void setCurrentGauge( double range, double minorTickSpacing, int majorTickRatio, String unit )
    {
        if ( layout.getChildCount() > 0 )
        {
            layout.removeAllViews();
        }

        category.clear();
        category = new CategorySeries( CATEGORY_NAME );
        category.add( CATEGORY_NAME, 0 );
        renderer.setMinorTicksSpacing( minorTickSpacing );
        renderer.setChartTitle( unit );
        renderer.setMaxValue( range );

        if ( view != null )
        {
            graphicalView = this.getDialChartView(
                    context,
                    category,
                    renderer,
                    majorTickRatio );

            layout.addView( graphicalView, new TableRow.LayoutParams( TableRow.LayoutParams.WRAP_CONTENT, TableRow.LayoutParams.WRAP_CONTENT ) );
        }
    }

    public void reset()
    {
        getTV(R.id.ui_main_tv_test_number).setText( "" );
        getTV(R.id.ui_main_tv_ohm).setText( "" );
        getTV(R.id.ui_main_tv_amp).setText( "" );
        getTV(R.id.ui_main_tv_loop).setText( "" );
        getTV(R.id.ui_main_tv_elapsed).setText( "" );
    }

    public void resetGauge()
    {
        updateGauge( 0D );
    }

    private TextView getTV( int id )
    {
        return ((TextView) view.findViewById( id ));
    }

    private void setTVText( int id, String text )
    {
        if ( !getTV( id ).getText().equals( text ) )
        {
            getTV( id ).setText( text );
        }
    }

    @SuppressWarnings("SameParameterValue")
    private void setTVText( int id, int number )
    {
        String text = number + "";
        setTVText( id, text );
    }

    @SuppressWarnings("SameParameterValue")
    private ImageView getIV( int id )
    {
        return ((ImageView) view.findViewById( id ));
    }
}

