package com.megabras.bluelogg.MPKXX5R;

import android.app.Activity;
import android.graphics.Typeface;
import android.graphics.pdf.PdfDocument;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Paper;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.bluelogg.md15kvr.Controller;
import com.megabras.modbus.MPKXX5R.Measurement;
import com.megabras.modbus.MPKXX5R.RecordData;
import com.megabras.modbus.MPKXX5R.TestData;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Objects;

@SuppressWarnings("ResultOfMethodCallIgnored")
public class ReportPDF extends ReportProvider
{
    @SuppressWarnings({"unused", "FieldCanBeLocal"})
    private final String
            TAG = "ReportPDF";

    @SuppressWarnings("FieldCanBeLocal")
    private final int
            LINE_HEIGHT      = 20,
            TEST_INFO_HEIGHT = 15,
            TABLE_ROW_HEIGHT = 10,
            PIC_HEIGHT       = 200;

    public ReportPDF( Activity activity, File dir, File tempDir, Handler handler )
    {
        super( activity, dir, tempDir, handler );
    }

    public void generate( RecordData recordData, File picture, File map )
    {
        handler.obtainMessage( Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString( R.string.report_creating_file ) ).sendToTarget();

        PdfDocument document = new PdfDocument();
        PdfDocument.PageInfo pageInfo;

        if ( Locale.getDefault().getLanguage().equals( Locale.ENGLISH.getLanguage() ) )
        {
            pageInfo = new PdfDocument.PageInfo.Builder( Paper.Letter.getWidth(), Paper.Letter.getHeight(), 1 ).create();
        }
        else
        {
            pageInfo = new PdfDocument.PageInfo.Builder( Paper.A4.getWidth(), Paper.A4.getHeight(), 1 ).create();
        }

        CURSOR_Y     = 0;
        CURSOR_X     = 0;
        PAPER_WIDTH  = pageInfo.getPageWidth() - (MARGIN * 2);
        PAPER_HEIGHT = pageInfo.getPageHeight() - (MARGIN * 2);

        // CALC NUM OF PAGES
        ArrayList<ArrayList<TestData>> pages = new ArrayList<>();
        pages.add( new ArrayList<>() ); // PAGE ONE

        int firstPage = 0;
        if ( recordData.getGpsAccuracy() > 0 )
        {
            firstPage += LINE_HEIGHT;
        }
        if ( picture.exists() || map.exists() )
        {
            firstPage += PIC_HEIGHT;
        }

        // ALL PAGES
        int allPages = LINE_HEIGHT * 3; // TITLE + EQUIPMENT INFO LINE + RECORDNAME

        int tdSize = allPages;

        for ( TestData td : recordData.getTestDatas() )
        {
            int subSize = 0;
            subSize += LINE_HEIGHT; // BLANK SPACE
            subSize += TEST_INFO_HEIGHT; // TEST #
            subSize += TABLE_ROW_HEIGHT; // MM:SS
            subSize += TABLE_ROW_HEIGHT * td.getMeasurements().size(); // MEASUREMENTS
            tdSize  += subSize;

            if ( pages.size() > 1 ) // NOT PAGE ONE
            {
                firstPage = 0;
            }

            if ( ( firstPage + tdSize ) > PAPER_HEIGHT  )
            {
                pages.add( new ArrayList<>() ); // NEW PAGE
                tdSize = allPages + subSize;
            }

            pages.get( pages.size() - 1 ).add( td );
        }

        ////////////////////////////////////////////////////////////////////////////////////////////

        handler.obtainMessage( Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString( R.string.report_general_info ) ).sendToTarget();

        for ( int i = 0; i < pages.size(); i++ )
        {
            PdfDocument.Page page = document.startPage( pageInfo );
            canvas = page.getCanvas();

            resetCursor();
            moveCursor( MARGIN, MARGIN );

            // BORDER ##################################################################################
            top( 0 );
            left( 0 );
            right( PAPER_WIDTH );
            bottom( PAPER_HEIGHT );
            this.drawSquare();

            // TITLE ###################################################################################
            top( 0 );
            left( 0 );
            right( PAPER_WIDTH );
            bottom( LINE_HEIGHT );
            this.drawTitle( activity.getString( R.string.test_report ).toUpperCase() );

            // EQUIPMENT ###############################################################################
            ArrayList<String> titles = new ArrayList<>();
            ArrayList<String> values = new ArrayList<>();

            titles.add( activity.getString( R.string.md_model ) );
            titles.add( activity.getString( R.string.md_serial ) );
            titles.add( "FIRMWARE" );

            values.add( recordData.getDeviceModel() );
            values.add( recordData.getDeviceSerial() );
            values.add( recordData.getDeviceVersionPreffix() + "." + recordData.getDeviceHardware() + "." + recordData.getDeviceFirmware() );

            top( 0 );
            left( 0 );
            right( PAPER_WIDTH );
            bottom( LINE_HEIGHT );
            this.drawLineItem( titles, values );

            // RECORD NAME #############################################################################
            titles = new ArrayList<>();
            values = new ArrayList<>();

            titles.add( activity.getString( R.string.record_name ) );

            values.add( recordData.getRecordName() );

            top( 0 );
            left( 0 );
            right( PAPER_WIDTH );
            bottom( LINE_HEIGHT );
            this.drawLineItem( titles, values, ALIGN_LEFT );

            if ( i == 0 ) // PAGE ONE
            {
                // GPS #####################################################################################
                if ( recordData.getGpsAccuracy() > 0 )
                {
                    titles = new ArrayList<>();
                    values = new ArrayList<>();

                    titles.add( activity.getString( R.string.md_latitude ) );
                    titles.add( activity.getString( R.string.md_longitude ) );
                    titles.add( activity.getString( R.string.md_altitude ) );
                    titles.add( activity.getString( R.string.md_gps_accuracy ) );

                    values.add( recordData.getGpsLatitudeAsString() );
                    values.add( recordData.getGpsLongitudeAsString() );
                    values.add( recordData.getGpsAltitudeAsString() );
                    values.add( recordData.getGpsAccuracyAsString() );

                    left( 0 );
                    top( 0 );
                    right( PAPER_WIDTH );
                    bottom( LINE_HEIGHT );
                    this.drawLineItem( titles, values );
                }

                // IMAGES ##################################################################################
                titles = new ArrayList<>();
                ArrayList<File> files = new ArrayList<>();

                titles.add( activity.getString( R.string.picture ) );
                titles.add( activity.getString( R.string.map ) );

                files.add( picture );
                files.add( map );

                left( 0 );
                top( 0 );
                right( PAPER_WIDTH );
                bottom( PIC_HEIGHT );

                if ( picture.exists() && map.exists() )
                {
                    this.drawLineImage( titles, files );
                }
                else if ( picture.exists() )
                {
                    this.drawLineItem( titles.get( 0 ), picture );
                }
                else if ( map.exists() )
                {
                    this.drawLineItem( titles.get( 1 ), map );
                }
            }

            saveCursor();

            int MARGIN_SIDE = (int) (PAPER_WIDTH * 0.32);


            // TEST DATA
            moveCursor( MARGIN_SIDE, 0 );

            left( 0 );
            top( 0 );
            right( PAPER_WIDTH - ( MARGIN_SIDE * 2 )  );
            bottom( LINE_HEIGHT );

            String[] modes = activity.getResources().getStringArray( R.array.mpkxx5r_mode_auto_manual );

            for ( TestData td : pages.get( i ) )
            {
                drawBlankLine( LINE_HEIGHT );

                // TEST INFO
                bottom( TEST_INFO_HEIGHT );
                titles = new ArrayList<>();
                values = new ArrayList<>();

                titles.add( activity.getString( R.string.test_number ) );
                titles.add( activity.getString( R.string.date        ) );
                titles.add( activity.getString( R.string.time        ) );
                titles.add( activity.getString( R.string.md_mode     ) );
                titles.add( activity.getString( R.string.md_timer    ) );

                values.add( td.getTestNumber() + "" );
                String[] dateTime = td.getDateAsString().split( " " );
                values.add( dateTime[0] );
                String time = dateTime[1];
                if ( dateTime.length > 2 )
                {
                    time += " " + dateTime[2];
                }
                values.add( time );
                if ( td.getMode() > -1 && td.getMode() < modes.length )
                {
                    values.add( modes[td.getMode()] );
                }
                else
                {
                    values.add( "- - -" );
                }
                if ( td.isTimerEnabled() )
                {
                    values.add( td.getTimer() + " s" );
                }
                else
                {
                    values.add( "- - -" );
                }
                this.drawLineItem( titles, values, ALIGN_CENTER, Typeface.create( "sans-serif-condensed", Typeface.NORMAL ), 7f );

                // TABLE TITLE
                bottom( TABLE_ROW_HEIGHT );

                titles = new ArrayList<>();
                titles.add( activity.getString( R.string.mmss      ) );
                titles.add( activity.getString( R.string.letter_R  ) );
                titles.add( activity.getString( R.string.letter_It ) );
                titles.add( activity.getString( R.string.letter_Ig ) );

                this.drawRow( Typeface.DEFAULT_BOLD, titles );

                for ( Measurement m : td.getMeasurements() )
                {
                    values = new ArrayList<>();
                    values.add( m.getElapsedString() );
                    values.add( m.getOhmString() );
                    values.add( m.getAmpString() );
                    values.add( m.getClampAmpString( td.isBsgEnabled() ) );
                    this.drawRow( Typeface.create( "sans-serif-condensed", Typeface.NORMAL ), values );
                }
            }

            drawPageNumber( i + 1, pages.size(), ALIGN_LEFT );
            drawGeneratedBy();

            document.finishPage( page );
        }


        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.saving_file)).sendToTarget();

        String docFile = recordData.getDeviceModel().trim() + "_" + recordData.getDeviceSerial().trim() + "_ID" + recordData.getRecordId() + "_report.pdf";
        File tmpFile   = new File(dir + "/"+ docFile);

        try
        {
            document.writeTo( new FileOutputStream( tmpFile ) );
            document.close();

            handler.obtainMessage(Controller.mHandler.REPORT_FINISHED, 1, -1, activity.getString(R.string.finished)).sendToTarget();

            sendFile(tmpFile);

            if ( tempDir != null )
            {
                for ( File temp : Objects.requireNonNull( tempDir.listFiles() ) )
                {
                    temp.delete();
                }
            }
        }
        catch ( IOException e )
        {
            Log.e( TAG, "error " + e.toString() );
            handler.obtainMessage( Controller.mHandler.REPORT_ERROR, 1, -1, activity.getString( R.string.finished ) ).sendToTarget();
        }
    }
}

