package com.megabras.bluelogg.MPKXX5R;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MPKXX5R.FormatData;
import com.megabras.modbus.Modbus;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class TableFilesAdapter extends BaseAdapter
{
    private LayoutInflater
            inflater;

    private List<FileDetails>
            fileDetailsList;

    private int
            openedRecordId = -1,
            dateFormat = Modbus.DATE_YYYY_MM_DD,
            timeFormat = Modbus.TIME_24H;

    TableFilesAdapter( Context context )
    {
        fileDetailsList = new ArrayList<>();
        inflater        = LayoutInflater.from( context );
    }

    static class ViewHolder
    {
        TextView
                ui_txw_record_id,
                ui_txw_record_name,
                ui_txw_last_change;

        LinearLayout
                ui_lyt_opened;
    }

    public void setDateTimeFormat( int dateFormat, int timeFormat )
    {
        this.dateFormat = dateFormat;
        this.timeFormat = timeFormat;
    }

    public int getCount()
    {
        return fileDetailsList.size();
    }

    public FileDetails getItem( int position )
    {
        return fileDetailsList.get( position );
    }

    @Override
    public long getItemId( int position )
    {
        return 0;
    }

    public void add( List<FileDetails> fileDetails )
    {
        //fileDetailsList.clear();
        fileDetailsList = fileDetails;
        notifyDataSetChanged();
    }

    public void add( int id, String recordName, Date date )
    {
        FileDetails f = new FileDetails();
        f.id          = id;
        f.recordName  = recordName;
        f.lastChanged = date;
        fileDetailsList.add( f );
    }

    int setOpenedRecordId( int recId )
    {
        openedRecordId = recId;
        for ( int i = 0; i < fileDetailsList.size(); i++ )
        {
            if ( fileDetailsList.get( i ).id ==  openedRecordId )
            {
                return i;
            }
        }
        return -1;
    }

    public void remove( int position )
    {
        fileDetailsList.remove( position );
    }

    public void clear()
    {
        fileDetailsList.clear();
    }

    public void repaint()
    {
        notifyDataSetChanged();
    }

    private TextView getTXW( int id, View v )
    {
        return ((TextView) v.findViewById( id ));
    }

    public View getView( int position, View vi, ViewGroup parent )
    {
        if ( vi == null )
        {
            ViewHolder h = new ViewHolder();
            vi                   = inflater.inflate( R.layout.mpkxx5r_table_saved_files, parent, false );
            h.ui_txw_record_id   = getTXW( R.id.ui_txw_record_id  , vi );
            h.ui_txw_record_name = getTXW( R.id.ui_txw_record_name, vi );
            h.ui_txw_last_change = getTXW( R.id.ui_txw_last_change, vi );
            h.ui_lyt_opened      = vi.findViewById( R.id.ui_lyt_opened );
            vi.setTag( h );
        }

        ViewHolder h = (ViewHolder) vi.getTag();
        h.ui_txw_record_id.setText( String.format( Locale.US, "%d", fileDetailsList.get( position ).id ) );
        h.ui_txw_record_name.setText( fileDetailsList.get( position ).recordName );
        h.ui_txw_last_change.setText( FormatData.getDateFormatted( fileDetailsList.get( position ).lastChanged, dateFormat, timeFormat ) );
        if ( openedRecordId == fileDetailsList.get( position ).id )
        {
            h.ui_lyt_opened.setVisibility( View.VISIBLE );
        }
        else
        {
            h.ui_lyt_opened.setVisibility( View.INVISIBLE );
        }

        return vi;
    }
}

