package com.megabras.bluelogg.bluetooth;

import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.os.Handler;

import com.megabras.modbus.Message;
import com.megabras.modbus.Modbus;

import java.util.Queue;

public class BluetoothDev
{
    public static final int
            // Message types sent from the BluetoothChatService Handler
            MESSAGE_STATE_CHANGE = 1,
            MESSAGE_READ         = 2,
            MESSAGE_WRITE        = 3,
            // Constants that indicate the current connection state
            STATE_CLOSED         = 0,
            STATE_CONNECTING     = 1,
            STATE_CONNECTED      = 2,
            STATE_FAIL           = 3,
            STATE_NOT_RESPONDING = 4,
            STATE_LOST           = 5;

    protected Context
            context;

    protected BluetoothDevice
            device;

    protected Handler
            mHandler;

    Message
            messageStatus;

    private int
            mState,
            prevState;

    protected Queue<Message>
            FIFO_MESSAGE;

    protected int
            CYCLE_TIME   = 50;

    protected int
            current_time = 0,
            TIME_OUT     = 600;

    protected boolean
            idle         = true;

    protected Runnable
            runFIFO;

    protected String
            rx = "";

    BluetoothDev( Context context, BluetoothDevice device, Handler handler, Message messageStatus )
    {
        this.context = context;
        this.device = device;
        this.messageStatus = messageStatus;
        mState = STATE_CLOSED;
        mHandler = handler;
    }

    protected synchronized void setState( int state )
    {
        prevState = mState;
        mState = state;
        mHandler.obtainMessage( MESSAGE_STATE_CHANGE, state, -1 ).sendToTarget();
    }

    synchronized void setState( int state, String message )
    {
        prevState = mState;
        mState = state;
        mHandler.obtainMessage( MESSAGE_STATE_CHANGE, state, -1, message ).sendToTarget();
    }

    public synchronized int getState()
    {
        return mState;
    }

    public synchronized int getPrevState()
    {
        return prevState;
    }

    public synchronized void connect( boolean secure )
    {
        setState( STATE_CONNECTING );
    }

    public void addMessage( Message message )
    {
        FIFO_MESSAGE.add( message );
    }

    public void setTimeout( int timeout )
    {
        TIME_OUT = timeout;
    }

    public void setCycleTime( int time )
    {
        CYCLE_TIME = time;
    }

    public void priorityMessage( Message message )
    {
        idle         = true;
        current_time = 0;
        FIFO_MESSAGE.clear();
        FIFO_MESSAGE.add( message );
    }

    public synchronized void stop()
    {
        setState( STATE_CLOSED );
    }

    protected boolean checkResponseWithFrames(String response)
    {
        if (response.startsWith( Modbus.START_FRAME+"0190") && response.endsWith(Modbus.END_FRAME))
            return true;
        else
            return response.startsWith(Modbus.START_FRAME)
                    && response.endsWith(Modbus.END_FRAME)
                    && com.megabras.modbus.MUtils.checkSumSafe(response);
    }

    protected boolean checkResponseWithoutFrames(String response){
        return com.megabras.modbus.MUtils.checkSumSafe(response);
    }
}
