package com.megabras.bluelogg.bluetooth;

import android.Manifest;
import android.app.Dialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.SpannableStringBuilder;
import android.text.style.ImageSpan;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.AbsListView;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.megabras.bluelogg.extended.MDialog;
import com.megabras.bluelogg.R;

import java.lang.ref.WeakReference;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Set;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

public class EnableBluetooth extends AppCompatActivity
{
	private final String
			TAG = "EnableBluetooth";

	private ArrayList<Equipment>
			models = new ArrayList<>();

	private BluetoothAdapter
			adapter;

	private DeviceBroadcastReceiver
			broadcastReceiver;

	public static boolean
			permissionToGPSAccepted	= false;

	public static String[]
			permissionsGPS = {android.Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION};

	public static final int
			REQUEST_GPS_PERMISSION = 202;

	private ListDevices
			listDevices;

	private ListView
			listView;

	private int
			STATE = DeviceBroadcastReceiver.FINISHED;

	private Database
			database;

	@Override
	public void onCreate( Bundle savedInstanceState )
	{
		super.onCreate( savedInstanceState );
		setContentView( R.layout.bluetooth_devices );

		findViewById( R.id.progressBar ).setVisibility( View.GONE );
		findViewById( R.id.bt_scan ).setOnClickListener( v -> scan() );
		findViewById( R.id.bt_stop ).setVisibility( View.GONE );

		String msg = getString( R.string.no_device_push_scan );
		msg = msg.replace( "@", getString( R.string.scan ).toUpperCase() );
		((TextView) findViewById( R.id.explanation )).setText( msg );
		findViewById( R.id.explanation ).setVisibility( View.VISIBLE );

		adapter = BluetoothAdapter.getDefaultAdapter();
		listDevices = new ListDevices( this );
		database = new Database( this );
		models = getModelsList();

		listView = findViewById( R.id.list );
		listView.setChoiceMode( AbsListView.CHOICE_MODE_SINGLE );
		listView.setAdapter( listDevices );
		listView.setOnItemClickListener( ( parent, view, position, id ) ->
		{
			try
			{
				listDevices.showProgressBar( position );
				if ( adapter.isDiscovering() )
				{
					adapter.cancelDiscovery();
				}

				MBluetoothDevice mDevice = listDevices.getDevice( position );

				if ( mDevice != null )
				{
					if ( mDevice.getType() == BluetoothDevice.DEVICE_TYPE_LE || mDevice.getDevice().getBondState() == BluetoothDevice.BOND_BONDED )
					{
						if ( mDevice.getType() == BluetoothDevice.DEVICE_TYPE_LE )
						{
							database.insert( mDevice ); // Insert only BLE, thats not require to bound
						}
						listView.setEnabled( false ); // This is to prevent double touch (open to instances)
						mDevice.startActivity( true );
						close();
					}
					else if ( mDevice.getDevice().getBondState() == BluetoothDevice.BOND_NONE )
					{
						mDevice.getDevice().createBond();
					}
				}
			}
			catch (Exception e)
			{
				Log.e(TAG, e.toString());
			}
		});

		findViewById( R.id.bt_cancel ).setOnClickListener( v -> close() );

		if ( database.alreadyExists() )
		{
			ArrayList<SavedDevice> savedDevicesList =  database.selectList();
			if ( savedDevicesList != null )
			{
				for ( SavedDevice sd : savedDevicesList )
				{
					for ( Equipment e : models )
					{
						if ( sd.model.equals( e.getModel() ) )
						{
							BluetoothDevice dev = adapter.getRemoteDevice( sd.mac );
							listDevices.add( new MBluetoothDevice( this, dev, sd.model, sd.serial, e.getTargetClass(), e.getId(), sd.type ) );
							findViewById( R.id.explanation ).setVisibility( View.GONE );
							break;
						}
					}
				}
				listDevices.resetNears();
			}
		}
		else
		{
			database.create();
		}

		checkPermissions();
		register();
		checkBluetooth();
	}

	private ArrayList<Equipment> getModelsList()
	{
		ArrayList<Equipment> models = new ArrayList<>();
		String[] list = getResources().getStringArray( R.array.equipment_list );

		for ( String aList : list )
		{
			aList = aList.replaceAll(" ", "");
			String[] temp = aList.split(":");
			int id;
			try
			{
				id = Integer.parseInt( temp[2] );
			}
			catch (Exception e)
			{
				id = 0;
			}

			models.add( new Equipment( temp[0], temp[1], id ) );
		}
		return models;
	}

	private ArrayList<MBluetoothDevice> getBoundedDevices( BluetoothAdapter adapter, ArrayList<Equipment> models )
	{
		Set<BluetoothDevice> devices = adapter.getBondedDevices();
		ArrayList<MBluetoothDevice> mDevices = new ArrayList<>();

		for ( BluetoothDevice dev : devices )
		{
			for ( Equipment e : models )
			{
				if ( dev.getName().contains( e.getModel() + "-" ) )
				{
					mDevices.add( new MBluetoothDevice( this, dev, e.getTargetClass(), e.getId() ) );
					break;
				}
			}
		}

		if ( mDevices.size() > 0 )
		{
			Collections.sort( mDevices );
		}

		return mDevices;
	}

	private void register()
	{
		try
		{
			if ( permissionToGPSAccepted )
			{
				broadcastReceiver = new DeviceBroadcastReceiver();
				broadcastReceiver.setHandler(new mHandler(this));
				IntentFilter filter = new IntentFilter();
				filter.addAction( BluetoothDevice.ACTION_BOND_STATE_CHANGED );
				filter.addAction( BluetoothDevice.ACTION_FOUND );
				filter.addAction( BluetoothAdapter.ACTION_DISCOVERY_STARTED );
				filter.addAction( BluetoothAdapter.ACTION_DISCOVERY_FINISHED );
				this.registerReceiver( broadcastReceiver, filter );
			}
		}
		catch ( Exception e )
		{
			Log.e( TAG, e.toString() );
		}
	}

	private void scan()
	{
		if ( permissionToGPSAccepted )
		{
			if ( adapter.isDiscovering() )
			{
				adapter.cancelDiscovery();
			}
			if ( listDevices != null )
			{
				listDevices.clear();
				listDevices.add( getBoundedDevices( adapter, models ) );
			}
			if ( listView != null )
			{
				listView.setItemChecked( -1, true );
			}
			adapter.startDiscovery();
		}
		else
		{
			ActivityCompat.requestPermissions( this, permissionsGPS, REQUEST_GPS_PERMISSION );
		}
	}

	private void checkPermissions()
	{
		if ( ContextCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED )
		{
			permissionToGPSAccepted = true;
		}
	}

	private void checkBluetooth()
	{
		try
		{
			if ( !adapter.isEnabled() )
			{
				startActivityForResult( new Intent( BluetoothAdapter.ACTION_REQUEST_ENABLE ), 1 );
			}
			else
			{
				bluetoohIsEnabled();
			}
		}
		catch ( Exception e )
		{
			MDialog.BluetoothAdapterNotFound( this ).show();
		}
	}
	
	protected void onActivityResult(int requestCode, int resultCode, Intent data){
		switch (requestCode) {
			case 1:
				if (resultCode == RESULT_OK)
					bluetoohIsEnabled();
				else if (resultCode == RESULT_CANCELED)
					finish();
				break;
		}
	}

	private void bluetoohIsEnabled()
	{
		ArrayList<MBluetoothDevice> mDevices = getBoundedDevices( adapter, models );
		listDevices.add( mDevices );
		if ( mDevices != null && mDevices.size() > 0 )
		{
			findViewById( R.id.explanation ).setVisibility( View.GONE );
		}
	}

	public void close()
	{
		if ( adapter != null && adapter.isDiscovering() )
		{
			adapter.cancelDiscovery();
		}
		try
		{
			if ( broadcastReceiver != null )
			{
				this.unregisterReceiver( broadcastReceiver );
				broadcastReceiver = null;
			}
		}
		catch ( Exception ignored )
		{
		}
		this.finish();
	}
	
	@Override
	public void onBackPressed()
	{
		close();
	}

	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		try
		{
			if ( broadcastReceiver != null )
			{
				this.unregisterReceiver( broadcastReceiver );
				broadcastReceiver = null;
			}
		}
		catch ( Exception ignored )
		{
		}
	}

	public void stopScan( View v )
	{
		try
		{
			if ( adapter != null && adapter.isDiscovering() )
			{
				adapter.cancelDiscovery();
			}
		}
		catch ( Exception e )
		{
			Log.e( TAG, e.toString() );
		}
	}

	@SuppressWarnings({"JavaReflectionMemberAccess"})
	public void unpair( View v )
	{
		try
		{
			View row = (View) v.getParent();
			if ( listView != null && STATE == DeviceBroadcastReceiver.FINISHED )
			{
				final int index = listView.getPositionForView( row );
				final MBluetoothDevice dev = listDevices.getDevice( index );

				if ( dev == null )
					return;

				final Dialog dialog = new Dialog(this);
				dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
				dialog.setContentView( R.layout.dialog_yes_no );
				dialog.setCanceledOnTouchOutside( false );
				if ( dialog.getWindow() != null )
				{
					dialog.getWindow().getAttributes().width = android.view.ViewGroup.LayoutParams.WRAP_CONTENT;
					dialog.getWindow().getAttributes().height = android.view.ViewGroup.LayoutParams.WRAP_CONTENT;
					dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
				}

				String message = getString(R.string.unpair_confirm);
				message += "\r\n\n";
				message += dev.getName() + " " + dev.getSerial();

				((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.unpair );
				((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_device_gray );
				((TextView) dialog.findViewById( R.id.message )).setText( message );

				dialog.findViewById( R.id.btn_no ).setOnClickListener( vi -> dialog.dismiss() );
				dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( vi -> dialog.dismiss() );

				Button ok = dialog.findViewById(R.id.btn_yes);
				ok.setOnClickListener(v1 ->
				{
					try
					{
						database.delete( dev.getName(), dev.getSerial() );
						if ( dev.getDevice() != null )
						{
							Method m = dev.getDevice().getClass().getMethod( "removeBond", (Class[]) null );
							if ( m != null )
							{
								m.invoke( dev.getDevice(), (Object[]) null );
								listDevices.remove( index );
							}
						}
					}
					catch (Exception ignored)
					{
					}
					finally
					{
						dialog.dismiss();
					}
				});

				// OnBackPressed
				dialog.setOnCancelListener( DialogInterface::dismiss );

				dialog.show();
			}
		}
		catch ( Exception e )
		{
			Log.e( TAG, "unpair: " + e.getMessage() );
		}
	}

	@Override
	public void onRequestPermissionsResult( int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults )
	{
		super.onRequestPermissionsResult( requestCode, permissions, grantResults );
		switch ( requestCode )
		{
			case REQUEST_GPS_PERMISSION:
				if ( grantResults.length > 1 )
				{
					permissionToGPSAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
				}
				else if ( (grantResults.length > 0) )
				{
					permissionToGPSAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED);
				}
				if ( permissionToGPSAccepted )
				{
					register();
					scan();
				}
				break;
		}
	}

	private static class mHandler extends Handler
	{
		private final WeakReference<EnableBluetooth>
				mActivity;

		public mHandler( EnableBluetooth activity )
		{
			mActivity = new WeakReference<>( activity );
		}

		@Override
		public void handleMessage( @NonNull Message msg )
		{
			if ( mActivity.get() == null )
				return;

			final EnableBluetooth atv = mActivity.get();

			atv.STATE = msg.what;

			switch ( msg.what )
			{
				case DeviceBroadcastReceiver.STATE_CHANGED:
					int state = (int) msg.obj;
					if ( state == BluetoothDevice.BOND_BONDING )
					{
						if ( atv.findViewById( R.id.bt_scan ) == null || atv.findViewById( R.id.tv_message ) == null || atv.listView == null )
							return;
						atv.findViewById( R.id.bt_scan ).setEnabled( false );
						((TextView) atv.findViewById( R.id.tv_message )).setText( R.string.pairing );
						atv.listView.setEnabled( false );
						atv.listView.setAlpha( 0.5f );
					}
					else if (state==BluetoothDevice.BOND_BONDED)
					{
						if (atv.listView==null)
							return;
						int position = atv.listView.getCheckedItemPosition();
						if ( atv.listDevices.getDevice( position ) != null )
						{
							atv.listDevices.getDevice( position ).startActivity( true );
							atv.close();
						}
					}
					else
					{
						if (atv.findViewById(R.id.progressBar)!=null)
							atv.findViewById(R.id.progressBar).setVisibility(View.GONE);
						if (atv.findViewById(R.id.tv_message)!=null)
							((TextView)atv.findViewById(R.id.tv_message)).setText(R.string.devices);
						if (atv.findViewById(R.id.bt_scan)!=null)
							atv.findViewById(R.id.bt_scan).setEnabled(true);
						if (atv.listView!=null)
						{
							atv.listView.setEnabled(true);
							atv.listView.setAlpha(1.0f);
							atv.listView.setItemChecked(-1, true);
						}
						atv.STATE = DeviceBroadcastReceiver.FINISHED;
					}
					break;
				case DeviceBroadcastReceiver.STARTED:
					atv.findViewById( R.id.progressBar ).setVisibility( View.VISIBLE );
					atv.findViewById( R.id.explanation ).setVisibility( View.GONE );
					atv.findViewById( R.id.bt_scan ).setVisibility( View.GONE );
					atv.findViewById( R.id.bt_stop ).setVisibility( View.VISIBLE );
					((TextView) atv.findViewById( R.id.tv_message )).setText( R.string.scanning );
					atv.listView.setEnabled( false );
					atv.listView.setAlpha( 0.5f );
					break;
				case DeviceBroadcastReceiver.FOUND:
					if ( msg.obj instanceof BluetoothDevice )
					{
						BluetoothDevice device = (BluetoothDevice) msg.obj;
						for ( Equipment e : atv.models )
						{
							if ( device.getName().contains( e.getModel() + "-" ) )
							{
								int scrollTo = atv.listDevices.add( new MBluetoothDevice( atv, device, e.getTargetClass(), e.getId() ) );
								atv.findViewById( R.id.explanation ).setVisibility( View.GONE );
								atv.listView.smoothScrollToPosition( scrollTo );
								break;
							}
						}
					}
					break;
				case DeviceBroadcastReceiver.FINISHED:
					atv.findViewById( R.id.progressBar ).setVisibility( View.GONE );
					if ( atv.listDevices.getCount() == 0 )
					{
						atv.findViewById( R.id.explanation ).setVisibility( View.VISIBLE );
					}
					((TextView) atv.findViewById( R.id.tv_message )).setText( R.string.devices );
					atv.findViewById( R.id.bt_scan ).setVisibility( View.VISIBLE );
					atv.findViewById( R.id.bt_stop ).setVisibility( View.GONE );
					atv.listView.setEnabled( true );
					atv.listView.setAlpha( 1.0f );
					break;
			}
		}
	}
}