package com.megabras.bluelogg.em4058;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Environment;
import android.util.Log;

import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.tm25r.ListSites;
import com.megabras.bluelogg.tm25r.ListTest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

@SuppressWarnings("ResultOfMethodCallIgnored")
public class EM4058test {
	private String TAG = "EM4058test";
	private Context context;
	private Activity activity;
	private File dir;
	private File dirlib;
	private File dirdata;
	private File dirsite;
	private String fileCompany = "company.xml";
	private String fileTestsites = "testsites.xml";
	private DocumentBuilder documentBuilder;
	private String fileData = "";
	private String testSite = "";
	private String testOperator = "";
	private String model = "";
	private String serial = "";
	private String currentObject = "";
	private String currentTestNumber = "";
	private ArrayList<EM4058values> measuredValues = new ArrayList<EM4058values>();
	private int selectedFrequency = 0;
	private String equipmentModel = "";


	public EM4058test(Activity activity, String equipmentModel){
		this.context = activity.getApplicationContext();
		this.activity = activity;
		this.equipmentModel = equipmentModel;
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();		
		try {
			documentBuilder = documentBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}

	public void addNewMeasuredValues(int testNumber){
		measuredValues.add(new EM4058values(testNumber));
	}

	public EM4058values getLastMeasuredValues(){
		if(measuredValues.size()>0)
			return measuredValues.get(measuredValues.size()-1);
		else
			return null;
	}

//	public ArrayList<EM4058values> getMeasuredValuesList(){
//		return measuredValues;
//	}

	public void setSelectedFrequency(int frequency){
		selectedFrequency = frequency;
	}

	public int getSelectedFrequency(){
		return selectedFrequency;
	}
	
	public void clear(){
		dirsite = null;		
		fileData = "";
		testSite = "";
		testOperator = "";		
		currentObject = "";
		currentTestNumber = "";
	}


//	public boolean isExternalStorageAvaiable() {
//	    String state = Environment.getExternalStorageState();
//		return Environment.MEDIA_MOUNTED.equals(state);
//	}

	public boolean isExternalStorageAvaiable() {
		return Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED);
	}
	
	
	public boolean createFolder(){
		if(!this.isExternalStorageAvaiable()){
			return false;
		}

		String root = Environment.getExternalStorageDirectory().toString();
		String companyName = context.getResources().getString(R.string.company);
		//companyName = companyName.replace(" ", "\ ");
		File dirCompany = new File(root + "/" + companyName);
		if(!dirCompany.exists()){
			dirCompany.mkdir();
		}

	    dir = new File(root + "/" + companyName + "/" + equipmentModel);
	    dirlib = new File(dir + "/lib");
	    dirdata = new File(dir + "/data");
	    	    
	    if(!dir.exists()){
	    	dir.mkdir();
	    }
		if(!dirlib.exists()){
			dirlib.mkdir();
		}
		if(!dirdata.exists()){
			dirdata.mkdir();
		}
	    return dir.exists();		
	}

	public void New(String testSite, String testOperator){		
		this.testSite = testSite;
		this.testOperator = testOperator;
		setDataFile();
	}

	private void setDataFile(){
		if(testSite.equals("")){
			testSite = "testsite";
		}
		
		String name = MUtils.removeSpecialChars(testSite);
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HHmm", Locale.getDefault());
		Date date = new Date();
		String time = dateFormat.format(date);		
		this.fileData = name + "_" + time;	
		this.dirsite = new File(dirdata + "/" + fileData);
		if(!dirsite.exists()){
			dirsite.mkdir();
		}
		Log.e(TAG,"PATH: "+ dirsite.getAbsolutePath());
		this.fileData = this.fileData + ".xml";		
	}
	
	
	public void Save(){		
		try {
			File file = new File(dirsite, fileData);			
			Document doc = documentBuilder.newDocument();			
			Element test = doc.createElement("test");
			doc.appendChild(test);			
			
			if(!testOperator.equals("")){
				String[] op = getOperatorDetails(testOperator);				
				Element operator = doc.createElement("operator");
				test.appendChild(operator);
				operator.appendChild( setNode("opname"		, testOperator	, doc) );
				operator.appendChild( setNode("extension"	, op[0]			, doc) );
				operator.appendChild( setNode("email"		, op[1]			, doc) );				
			} else {				
				Element operator = doc.createElement("operator");
				test.appendChild(operator);
				operator.appendChild( setNode("opname", "", doc) );
				operator.appendChild( setNode("extension", "", doc) );
				operator.appendChild( setNode("email", "", doc) );
			}
			
			if(!model.equals("") || !serial.equals("")){								
				Element equipment = doc.createElement("equipment");
				test.appendChild(equipment);
				equipment.appendChild( setNode("model"		, model		, doc) );
				equipment.appendChild( setNode("serial"		, serial	, doc) );							
			}
			
			if(!testSite.equals("")){			
				String[] st = getSiteDetails(testSite);				
				Element site = doc.createElement("testsite");
				test.appendChild(site);				
				site.appendChild(	setNode( "sitename"	, testSite	, doc )	);
				site.appendChild(	setNode( "contact"	, st[0]		, doc )	);
				site.appendChild(	setNode( "address"	, st[1]		, doc )	);
				site.appendChild(	setNode( "city"		, st[2]		, doc )	);
				site.appendChild(	setNode( "state"	, st[3]		, doc )	);
				site.appendChild(	setNode( "zipcode"	, st[4]		, doc )	);
				site.appendChild(	setNode( "phone"	, st[5]		, doc )	);
				site.appendChild(	setNode( "fax"		, st[6]		, doc )	);			
			}
			
			Element objects = doc.createElement("objects");
			test.appendChild(objects);	
			
			Element comments = doc.createElement("comments");
			test.appendChild(comments);	
						
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
				
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);			
		} catch (Exception e) {
			e.printStackTrace();			
		}		
	}
	
	
	public String[] getOperatorFromFile(String selectedSite){
		File dirsites = new File(dirdata + "/" + selectedSite);	
		String fileDatas = selectedSite + ".xml";	
		File file = new File(dirsites, fileDatas);
		String[] det = new String[3];
		try{
			Document doc = documentBuilder.parse(file);				
			Node opname = doc.getElementsByTagName("opname").item(0);
			Node extension = doc.getElementsByTagName("extension").item(0);
			Node email = doc.getElementsByTagName("email").item(0);		
					
			det[0] = (opname != null) ? opname.getTextContent() : "";			
			det[1] = (extension != null ) ? extension.getTextContent() : "";			
			det[2] = (email != null ) ? email.getTextContent() : "";		
			
			return det;
			
		} catch (Exception e){
			Log.e(TAG, e.getMessage());
			return null;
		}		
	}
	
	
	public String[] getOperatorFromFile(){
		File file = new File(dirsite, fileData);		
		String[] det = new String[3];
		try{
			Document doc = documentBuilder.parse(file);				
			Node opname = doc.getElementsByTagName("opname").item(0);
			Node extension = doc.getElementsByTagName("extension").item(0);
			Node email = doc.getElementsByTagName("email").item(0);		
					
			det[0] = (opname != null) ? opname.getTextContent() : "";			
			det[1] = (extension != null ) ? extension.getTextContent() : "";			
			det[2] = (email != null ) ? email.getTextContent() : "";		
			
			return det;
			
		} catch (Exception e){
			Log.e(TAG, e.getMessage());
			return new String[]{"","",""};
		}		
	}
	
	
	public String[] getTestsiteFromFile(String selectedSite){
		File dirsites = new File(dirdata + "/" + selectedSite);	
		String fileDatas = selectedSite + ".xml";	
		File file = new File(dirsites, fileDatas);
		
		try{
			Document doc = documentBuilder.parse(file);				
			Node testSite = doc.getElementsByTagName("testsite").item(0);
		
			NodeList details = testSite.getChildNodes();
			String[] det = new String[details.getLength()];
			
			for(int i = 0; i < details.getLength(); i++){
				det[i] = details.item(i).getTextContent();				
			}
			
			return det;
			
		} catch (Exception e){
			Log.e(TAG, e.getMessage());
			return null;
		}		
	}
	
	
	public String[] getObjectsFromFile(String selectedSite){
		try{
			File dirsites = new File(dirdata + "/" + selectedSite);	
			String fileDatas = selectedSite + ".xml";	
			File file = new File(dirsites, fileDatas);
					
			Document doc = documentBuilder.parse(file);			
			
			Node objects = doc.getElementsByTagName("objects").item(0);			
			NodeList list = objects.getChildNodes();
			
			String[] obj = new String[list.getLength()];
			
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				obj[i] = node.getFirstChild().getTextContent();			
			}
			
			return obj;
			
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}
	
	
	public ArrayList<ListTest> getTestResultsFromFile(String selectedSite, String objectName){
		try{			
			File dirsites = new File(dirdata + "/" + selectedSite);	
			String fileDatas = selectedSite + ".xml";	
			File file = new File(dirsites, fileDatas);		
			Document doc = documentBuilder.parse(file);			
			ArrayList<ListTest> tests = new ArrayList<ListTest>();			
			Node objects = doc.getElementsByTagName("objects").item(0);			
			NodeList list = objects.getChildNodes();
			
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				String name = node.getFirstChild().getTextContent();				
				if(objectName.equals(name)){
					NodeList testList = node.getLastChild().getChildNodes();
					for (int a = 0; a < testList.getLength(); a++) {
						Node testNode = testList.item(a);
						Node testNum = testNode.getFirstChild();
						Node testDate = testNum.getNextSibling();
						Node testR = testDate.getNextSibling();
						Node testRc = testR.getNextSibling();
						Node testC = testRc.getNextSibling();
						Node testL = testC.getNextSibling();
						
						String audioFileName = "object_" + MUtils.removeSpecialChars(name) + "_test-" + testNum.getTextContent() + "_annotation.3gp";
						
						File audio = new File(dirsite,audioFileName);
						String audioPath = "";
						if(audio.exists()){						
							audioPath = audio.toString();
						}
						ListTest test = new ListTest(	objectName,
													    testNum.getTextContent(),
														testDate.getTextContent(),
														testR.getTextContent(),
														testRc.getTextContent(),
														testC.getTextContent(),
														testL.getTextContent(),
														audioPath );
						tests.add(test);
					}
				}				
			}			
			return tests;
			
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}

	public ArrayList<EM4058values> getEM4058TestResultsFromFile(String selectedSite, String objectName){
		try{
			File dirsites = new File(dirdata + "/" + selectedSite);
			String fileDatas = selectedSite + ".xml";
			File file = new File(dirsites, fileDatas);
			Document doc = documentBuilder.parse(file);
			ArrayList<EM4058values> tests = new ArrayList<EM4058values>();
			Node objects = doc.getElementsByTagName("objects").item(0);
			NodeList list = objects.getChildNodes();

			for (int i = 0; i < list.getLength(); i++) {
				Node node = list.item(i);
				String name = node.getFirstChild().getTextContent();
				if(objectName.equals(name)){
					NodeList testList = node.getLastChild().getChildNodes();
					for (int a = 0; a < testList.getLength(); a++) {
						Node testNode = testList.item(a);
						Node number = testNode.getFirstChild();
						Node date = number.getNextSibling();
						Node type = date.getNextSibling();
						Node distance = type.getNextSibling();
						Node frequency = distance.getNextSibling();
						Node voltage = frequency.getNextSibling();
						Node result = voltage.getNextSibling();
						Node r270Hz = result.getNextSibling();
						Node r570Hz = r270Hz.getNextSibling();
						Node r870Hz = r570Hz.getNextSibling();
						Node r1170Hz = r870Hz.getNextSibling();
						Node r1470Hz = r1170Hz.getNextSibling();

						String audioFileName = "object_" + MUtils.removeSpecialChars(name) + "_test-" + number.getTextContent() + "_annotation.3gp";

						File audio = new File(dirsite,audioFileName);
						String audioPath = "";
						if(audio.exists()){
							audioPath = audio.toString();
						}

						int testNumber = Integer.parseInt(number.getTextContent());

						EM4058values v = new EM4058values(testNumber);

						v.setObjectName(objectName);
						v.setAudioPath(audioPath);
						v.setTestDate(date.getTextContent());
						v.setTestType(type.getTextContent());
						v.setDistance(distance.getTextContent());
						v.setFrequency(frequency.getTextContent());
						v.setVoltage(voltage.getTextContent());
						v.setResult(result.getTextContent());
						v.setResult270Hz(r270Hz.getTextContent());
						v.setResult570Hz(r570Hz.getTextContent());
						v.setResult870Hz(r870Hz.getTextContent());
						v.setResult1170Hz(r1170Hz.getTextContent());
						v.setResult1470Hz(r1470Hz.getTextContent());

						tests.add(v);
					}
				}
			}
			return tests;

		} catch (Exception e){
			e.printStackTrace();
			return null;
		}
	}
	
	
	public void loadDataFile(String selectedSite){
		this.dirsite = new File(dirdata + "/" + selectedSite);	
		this.fileData = selectedSite + ".xml";	
		File file = new File(dirsite, fileData);		
		try{
			Document doc = documentBuilder.parse(file);	
			Node site = doc.getElementsByTagName("sitename").item(0);
			Node op = doc.getElementsByTagName("opname").item(0);
			
			if (site != null){
				testSite = site.getTextContent();
			}				
			if (op != null){
				testOperator = op.getTextContent();
			}
		} catch (Exception e){
			Log.e(TAG, e.getMessage());
		}		
	}
	
	
	public String[] getDataFileDetails(String selectedSite){
		try{
			File dirsites = new File(dirdata, selectedSite);
			File files = new File(dirsites, selectedSite + ".xml");
			int voices = 0;
			int pictures = 0;
			String operator = "";
			String objects = "";
			String tests = "";
			String[] details = new String[6];		
		
			Document doc   =   documentBuilder.parse(files);
			Node op        =   doc.getElementsByTagName("opname").item(0);
			if(op != null){
				operator   =   op.getTextContent();
			}
			
			NodeList ob    =   doc.getElementsByTagName("object");
			if(ob != null){
				objects    =   ob.getLength() + "";
			}
			
			NodeList tt    =   doc.getElementsByTagName("read");
			if(tt != null){
				tests      =   tt.getLength() + "";
			}
			
			long size = 0;
			if (dirsites.isDirectory()){
				for (File child : dirsites.listFiles()){
					if(child.getName().contains(".3gp")){
						voices++;
					}
					if(child.getName().contains(".jpg") && !child.getName().contains("snapshot")){
						pictures++;
					}
					size = size + child.length();
		        }
				details[0] = sizeFormater(size);
				details[1] = String.valueOf(voices);
				details[2] = String.valueOf(pictures);
				details[3] = operator;
				details[4] = objects;
				details[5] = tests;
			}			
			return details;
		} catch (Exception e){
			Log.e("getDataFileDetails", e.getMessage());
			return null;
		}		
	}
	
	
	private String sizeFormater(float bytes){
		DecimalFormat formater = new DecimalFormat("#.##");
		final long KILO = 1024;
		final long MEGA = KILO * 1024;
		final long GIGA = MEGA * 1024;
		float result = 0;
		String formated = "";
		
		if( bytes >= GIGA) {
			result = (float) bytes / GIGA;
			formated = formater.format(result) + " GB";
		} else if( bytes >= MEGA ) {
			result = (float) bytes / MEGA;
			formated = formater.format(result) + " MB";
		} else if( bytes >= KILO ) {
			result = (float) bytes / KILO;
			formated = formater.format(result) + " KB";
		} else {
			result = bytes;
			formated = formater.format(result) + " bytes";
		}		
		return formated;
	}
	
	
	public void deleteDataFile(String selectedSite){
		File dirToDelete = new File(dirdata + "/" + selectedSite);		
		try{			
			if (dirToDelete.isDirectory()){
				for (File child : dirToDelete.listFiles()){
					child.delete();
		        }
			}			
			dirToDelete.delete();
		} catch (Exception e){
			Log.e("deletDataFile", e.getMessage());
		}		
	}
	
	
	public String getCurrentTestSiteName(){
		return testSite;
	}
	
	
	public String getCurrentOperator(){
		if(testOperator.equals("")){
			return "";
		}
		return testOperator;
	}
	
	
	public File getRootFolder(){
		return dir;
	}
	
	
	public File getDataFolder(){
		return dirsite;
	}
	
	
	public File getDataFolder(String selectedSite){
		return new File(dirdata + "/" + selectedSite);
	}	

	
	public boolean isSetTestSite(){
		return !this.currentObject.equals("");
	}
	
	
	public String getCurrentTestNumber(){
		return this.currentTestNumber;
	}
	
	
	public void addTestObject(String name){
		try {
			this.currentObject = name;			
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);
			
			Node objects = doc.getElementsByTagName("objects").item(0);			
			Element obj = doc.createElement("object");
			
			objects.appendChild(obj);
			obj.setAttribute("id", currentObject);
			obj.appendChild(	setNode( "name"	       , currentObject	, doc )	);
			obj.appendChild(	setNode( "longititude" , ""	            , doc )	);
			obj.appendChild(	setNode( "latitude"	   , ""	            , doc )	);
			obj.appendChild(	setNode( "altitude"	   , ""	            , doc )	);
			obj.appendChild(	setNode( "accuracy"	   , ""	            , doc )	);
			obj.appendChild(	setNode( "readings"	   , ""	            , doc )	);
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
				
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);			
		} catch (Exception e) {
			e.printStackTrace();
		}			
	}
	
	
	public void addComments(String msg){
		try {						
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);
			
			Node comments = doc.getElementsByTagName("comments").item(0);			
			
			comments.setTextContent(msg);			
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
				
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);			
		} catch (Exception e) {
			e.printStackTrace();
		}			
	}
	
	
	public String getComments(String selectedSite){
		try {
			
			File dirsites = new File(dirdata + "/" + selectedSite);	
			String fileDatas = selectedSite + ".xml";	
			File file = new File(dirsites, fileDatas);					
			Document doc = documentBuilder.parse(file);
			
			Node comments = doc.getElementsByTagName("comments").item(0);
			return comments.getTextContent();
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}			
	}
	
	
	public String getComments(){
		try {							
			File file = new File(dirsite, fileData);					
			Document doc = documentBuilder.parse(file);
			
			Node comments = doc.getElementsByTagName("comments").item(0);
			return comments.getTextContent();
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}			
	}
	
	
	public void addTestResult(EM4058values v){
		try{
			if(currentObject.equals("")){
				if(this.objectExists("object")){
					currentObject = "object";
				} else {
					addTestObject("object");	
				}				
			}			
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);
			
			Node readings = getObject(doc);			
			Element read = doc.createElement("read");
			readings.appendChild(read);

			read.appendChild(	setNode( "number"	, v.getTestNumber()+""  , doc )	);
			read.appendChild(	setNode( "date"		, v.getTestDate()		, doc )	);
			read.appendChild(	setNode( "type"		, v.getTestType()		, doc )	);
			read.appendChild(	setNode( "distance"	, v.getDistance()		, doc )	);
			read.appendChild(	setNode( "frequency", v.getFrequency()		, doc )	);
			read.appendChild(	setNode( "voltage"	, v.getVoltage()		, doc )	);
			read.appendChild(	setNode( "result"	, v.getResult()			, doc )	);
			read.appendChild(	setNode( "r270Hz"	, v.getResult270Hz()	, doc )	);
			read.appendChild(	setNode( "r570Hz"	, v.getResult570Hz()	, doc )	);
			read.appendChild(	setNode( "r870Hz"	, v.getResult870Hz()	, doc )	);
			read.appendChild(	setNode( "r1170Hz"	, v.getResult1170Hz()	, doc )	);
			read.appendChild(	setNode( "r1470Hz"	, v.getResult1470Hz()	, doc )	);

			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
				
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);	
			
			currentTestNumber = v.getTestNumber()+"";
		} catch (Exception e){
			e.printStackTrace();
		}		
	}
	
	
	public void addGpsData(String[] gps){
		try{			
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);
			
			Node obj = doc.getElementById(gps[0]);
			NodeList objChilds = obj.getChildNodes();
			
			DecimalFormatSymbols separator = new DecimalFormatSymbols(Locale.getDefault());
			separator.setDecimalSeparator('.');			
			DecimalFormat sixDigit = new DecimalFormat("#.000000", separator);
	        sixDigit.setRoundingMode(RoundingMode.DOWN);
			
			Double d = Double.parseDouble(gps[1]);			
			objChilds.item(1).setTextContent( sixDigit.format(d) );
									
			d = Double.parseDouble(gps[2]);		
			objChilds.item(2).setTextContent( sixDigit.format(d) );
			
			if (gps[3].contains("m")){
				objChilds.item(3).setTextContent( gps[3] );
			} else {
				d = Double.parseDouble(gps[3]);
				objChilds.item(3).setTextContent( d.intValue() + " m" );	
			}			
			
			if (gps[4].contains("m")){
				objChilds.item(4).setTextContent( gps[4] );
			} else {
				objChilds.item(4).setTextContent( "\u00B1" + gps[4] + " m");	
			}				
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);				
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);									
		} catch (Exception e){
			e.printStackTrace();
		}		
	}
	
	
	public String[] getGpsData(String objName){
		try{			
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);
			
			Node obj = doc.getElementById(objName);
			NodeList objChilds = obj.getChildNodes();
									
			String[] gps = new String[4];
			if(objChilds.getLength() > 2){
				gps[0] = objChilds.item(1).getTextContent();
				gps[1] = objChilds.item(2).getTextContent();
				gps[2] = objChilds.item(3).getTextContent();
				gps[3] = objChilds.item(4).getTextContent();	
			} else {
				gps[0] = "";
				gps[1] = "";
				gps[2] = "";
				gps[3] = "";
			}			
			
			return gps;						
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}
	
	
	public String[] getGpsDataFromFile(String objName, String selectedSite){
		try{			
			
			File dirsites = new File(dirdata + "/" + selectedSite);	
			String fileDatas = selectedSite + ".xml";	
			
			File file = new File(dirsites, fileDatas);		
			Document doc = documentBuilder.parse(file);
			
			Node obj = doc.getElementById(objName);
			NodeList objChilds = obj.getChildNodes();
									
			String[] gps = new String[4];
			if(objChilds.getLength() > 2){
				gps[0] = objChilds.item(1).getTextContent();
				gps[1] = objChilds.item(2).getTextContent();
				gps[2] = objChilds.item(3).getTextContent();
				gps[3] = objChilds.item(4).getTextContent();	
			} else {
				gps[0] = "";
				gps[1] = "";
				gps[2] = "";
				gps[3] = "";
			}			
			
			return gps;						
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}
	
	
	public String getCurrentObject(){
		return currentObject;
	}
	
	
	public String getCurrentObjectClean(){		
		return MUtils.removeSpecialChars(currentObject);
	}
	
	
	public void setCurrentObject(String currentObject){
		this.currentObject = currentObject;
	}

	
	public String[] getObjects(){
		try{
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);			
			
			Node objects = doc.getElementsByTagName("objects").item(0);			
			NodeList list = objects.getChildNodes();
			
			String[] obj = new String[list.getLength()];
			
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				obj[i] = node.getFirstChild().getTextContent();			
			}
			
			return obj;
			
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}
	
	
	public boolean objectExists(String objName){
		try{
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);			
			Node obj = doc.getElementById(objName);			
			return obj!=null;			
		} catch (Exception e){
			e.printStackTrace();
			return false;
		}		
	}
	
	
	public boolean operatorExists(String opName){
		String[] operators = this.getOperators();
		for (String op : operators) {
			if (op.equals(opName)){
				return true;
			}
		}
		return false;
	}
	
	
	public boolean testsiteExists(String siteName){
		String[] sites = this.getSitesForSpinner();
		for (String s : sites) {
			if (s.equals(siteName)){
				return true;
			}
		}
		return false;
	}
	
	
	public List<String> getObjectsList(){
		List<String> objs = new ArrayList<String>();
		try{			
			
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);			
			Node objects = doc.getElementsByTagName("objects").item(0);			
			NodeList list = objects.getChildNodes();
			
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);				
				objs.add(node.getFirstChild().getTextContent());
			}
			
			return objs;
			
		} catch (Exception e){
			e.printStackTrace();
			objs.add("");
			return objs;
		}		
	}
	
	
	public ArrayList<ListSites> getSavedFiles(){
		try{
			String[] list = dirdata.list();	
			Arrays.sort(list);
			ArrayList<ListSites> sites = new ArrayList<ListSites>();			
			for (int i = 0; i < list.length; i++) {	 
				ListSites site = new ListSites(	list[i] );
				sites.add(site);				
			}		
			return sites;			
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}
	

	public static long getFolderSize(File dir) {
		long size = 0;
		for (File file : dir.listFiles()) {
		    if (file.isFile()) {
		        size += file.length();
		    }
		    else {
		        size += getFolderSize(file);
	        }
		}
		return size;		
	}
	
	
	public File getPictureFile(){		
		String img = MUtils.removeSpecialChars(currentObject) + ".jpg";
		File file = new File(dirsite, img);
		return file;
	}

	
	public File getPictureFile(String name){
		String img = MUtils.removeSpecialChars(name) + ".jpg";
		File file = new File(dirsite, img);
		return file;
	}
	
	
	public File getSnapshotFile(){
		String name = "snapshot";
		String img = MUtils.removeSpecialChars(name) + ".jpg";
		File file = new File(dirsite, img);
		return file;
	}
	
	
	public File getSnapshotFile(String selSite){
		String name = "snapshot";
		String img = MUtils.removeSpecialChars(name) + ".jpg";
		File file = new File(dirdata + "/" + selSite, img);
		return file;
	}
	
	
	public void deleteSnapshotFile(){
		String name = "snapshot";
		String img = MUtils.removeSpecialChars(name) + ".jpg";
		File file = new File(dirsite, img);
		if (file.exists()){
			file.delete();
		}
	}
	
	
	public File getPictureFileFromFile(String selectedSite, String name){
		File dir = new File(dirdata + "/" + selectedSite);
		String img = MUtils.removeSpecialChars(name) + ".jpg";
		File file = new File(dir, img);
		return file;
	}

	
	public ArrayList<EM4058values> getTestResults(String objectName){
		try{			
			File file = new File(dirsite, fileData);		
			Document doc = documentBuilder.parse(file);			
			ArrayList<EM4058values> tests = new ArrayList<EM4058values>();
			Node objects = doc.getElementsByTagName("objects").item(0);			
			NodeList list = objects.getChildNodes();
			
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				String name = node.getFirstChild().getTextContent();				
				if(objectName.equals(name)){
					NodeList testList = node.getLastChild().getChildNodes();
					for (int a = 0; a < testList.getLength(); a++) {
						Node testNode = testList.item(a);
						Node number = testNode.getFirstChild();
						Node date = number.getNextSibling();
						Node type = date.getNextSibling();
						Node distance = type.getNextSibling();
						Node frequency = distance.getNextSibling();
						Node voltage = frequency.getNextSibling();
						Node result = voltage.getNextSibling();
						Node r270Hz = result.getNextSibling();
						Node r570Hz = r270Hz.getNextSibling();
						Node r870Hz = r570Hz.getNextSibling();
						Node r1170Hz = r870Hz.getNextSibling();
						Node r1470Hz = r1170Hz.getNextSibling();

						String audioFileName = "object_" + MUtils.removeSpecialChars(name) + "_test-" + number.getTextContent() + "_annotation.3gp";
						
						File audio = new File(dirsite,audioFileName);
						String audioPath = "";
						if(audio.exists()){						
							audioPath = audio.toString();
						}

						int testNumber = Integer.parseInt(number.getTextContent());

						EM4058values v = new EM4058values(testNumber);

						v.setObjectName(objectName);
						v.setAudioPath(audioPath);
						v.setTestDate(date.getTextContent());
						v.setTestType(type.getTextContent());
						v.setDistance(distance.getTextContent());
						v.setFrequency(frequency.getTextContent());
						v.setVoltage(voltage.getTextContent());
						v.setResult(result.getTextContent());
						v.setResult270Hz(r270Hz.getTextContent());
						v.setResult570Hz(r570Hz.getTextContent());
						v.setResult870Hz(r870Hz.getTextContent());
						v.setResult1170Hz(r1170Hz.getTextContent());
						v.setResult1470Hz(r1470Hz.getTextContent());

						tests.add(v);
					}
				}				
			}			
			return tests;
			
		} catch (Exception e){
			e.printStackTrace();
			return null;
		}		
	}
	

	private Node getObject(Document doc){		
		Node objects = doc.getElementsByTagName("objects").item(0);			
		NodeList list = objects.getChildNodes();		
		for (int i = 0; i < list.getLength(); i++) {	 
			Node node = list.item(i);
			Node subnode = node.getFirstChild();			
			if (currentObject.equals(subnode.getTextContent())) {
				return node.getLastChild();
			}	 
		}		
		return null;
	}


	public void setEquipmentModel(String model, String serial){
		this.model = model;
		this.serial = serial;
	}

	
	public String getEquipmentModel(){
		return model + " - " + serial;		
	}
		
	
	public String getRootPath(){		 
		return context.getFilesDir().toString();		
	}
	
	
	public boolean createCompanyFile(){		
		try {			
			File file = new File(dirlib, fileCompany);
			
			if(file.exists()){
				return true;
			}
						
			Document doc = documentBuilder.newDocument();			
			Element root = doc.createElement("companydata");
			doc.appendChild(root);
			
			Element companyDetails = doc.createElement("companydetails");
			root.appendChild(companyDetails);

			companyDetails.appendChild(setNode("company","",doc));
			companyDetails.appendChild(setNode("address","",doc));
			companyDetails.appendChild(setNode("city","",doc));
			companyDetails.appendChild(setNode("state","",doc));
			companyDetails.appendChild(setNode("zipcode","",doc));
			companyDetails.appendChild(setNode("phone","",doc));
			companyDetails.appendChild(setNode("fax","",doc));
												
			Element operators = doc.createElement("operators");
			root.appendChild(operators);
						
			TransformerFactory factory = TransformerFactory.newInstance();			
			Transformer transformer = factory.newTransformer();
			
			DOMSource domSource = new DOMSource(doc.getDocumentElement());
			OutputStream output = new ByteArrayOutputStream();
			StreamResult result = new StreamResult(output);
			
			transformer.transform(domSource, result);
						
			String string = output.toString();
			FileOutputStream outputStream;
						
			outputStream = new FileOutputStream(file);
			outputStream.write(string.getBytes());
			outputStream.close();
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}		
	}
	
	
	public boolean createSitesFile(){		
		try {			
			File file = new File (dirlib, fileTestsites);
			
			if(file.exists()){
				return true;
			}
			
			Document doc = documentBuilder.newDocument();
			
			Element root = doc.createElement("testsites");
			doc.appendChild(root);
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
						
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);	
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
			Log.e(TAG,e.getMessage());
			Log.e(TAG,e.getStackTrace().toString());
			return false;
		}		
	}
	
	
	public boolean addSite(String sitename, String contact, String address, String city, String state, String zipcode, String phone, String fax){		
		try {			
			File file = new File (dirlib, fileTestsites);
			
			if(!file.exists()){
				return false;
			}
		
			Document doc = documentBuilder.parse(file);

			Node testsites = doc.getElementsByTagName("testsites").item(0);
			NodeList list = testsites.getChildNodes();			

			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();				
				if (sitename.equals(subnode.getNodeValue())) {
					return false;					
				}	 
			}			
			
			Element site = doc.createElement("site");			
			testsites.appendChild(site);
			site.appendChild(setNode("sitename",sitename,doc));
			site.appendChild(setNode("contact",contact,doc));
			site.appendChild(setNode("address",address,doc));
			site.appendChild(setNode("city",city,doc));
			site.appendChild(setNode("state",state,doc));
			site.appendChild(setNode("zipcode",zipcode,doc));
			site.appendChild(setNode("phone",phone,doc));
			site.appendChild(setNode("fax",fax,doc));
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);

			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}		
	}
	
	
	public boolean updateSiteInFile(String contact, String address, String city, String state, String zipcode, String phone, String fax, String opName, String opExten, String opMail){		
		try {
			Document doc = documentBuilder.parse(new File (dirsite, fileData));			
			doc.getElementsByTagName("contact").item(0).setTextContent(contact);
			doc.getElementsByTagName("address").item(0).setTextContent(address);
			doc.getElementsByTagName("city").item(0).setTextContent(city);
			doc.getElementsByTagName("state").item(0).setTextContent(state);
			doc.getElementsByTagName("zipcode").item(0).setTextContent(zipcode);
			doc.getElementsByTagName("phone").item(0).setTextContent(phone);
			doc.getElementsByTagName("fax").item(0).setTextContent(fax);
			doc.getElementsByTagName("opname").item(0).setTextContent(opName);
			doc.getElementsByTagName("extension").item(0).setTextContent(opExten);
			doc.getElementsByTagName("email").item(0).setTextContent(opMail);
						
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			
			File file = new File (dirsite, fileData);
						
			//StreamResult result = new StreamResult(file);
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			return true;			
		} catch (Exception e) {
			Log.e("Error", e.toString());
			e.printStackTrace();
		}		
		return false;
	}
	
	
	public boolean updateCompany(String companyname, String address, String city, String state, String zipcode, String phone, String fax){		
		try {
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));
			
			doc.getElementsByTagName("company").item(0).setTextContent(companyname);
			doc.getElementsByTagName("address").item(0).setTextContent(address);
			doc.getElementsByTagName("city").item(0).setTextContent(city);
			doc.getElementsByTagName("state").item(0).setTextContent(state);
			doc.getElementsByTagName("zipcode").item(0).setTextContent(zipcode);
			doc.getElementsByTagName("phone").item(0).setTextContent(phone);
			doc.getElementsByTagName("fax").item(0).setTextContent(fax);
						
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			
			File file = new File(dirlib, fileCompany);

			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return false;
	}
	
	
	public String[] getCompany(){		
		try {			
			String[] data = new String[7];			
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));			
			data[0] = doc.getElementsByTagName("company").item(0).getTextContent();
			data[1] = doc.getElementsByTagName("address").item(0).getTextContent();
			data[2] = doc.getElementsByTagName("city").item(0).getTextContent();
			data[3] = doc.getElementsByTagName("state").item(0).getTextContent();
			data[4] = doc.getElementsByTagName("zipcode").item(0).getTextContent();
			data[5] = doc.getElementsByTagName("phone").item(0).getTextContent();
			data[6] = doc.getElementsByTagName("fax").item(0).getTextContent();
			return data;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return null;
	}

	
	public boolean addOperator(String name, String extension, String email){		
		try {			
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));
			Node operators = doc.getElementsByTagName("operators").item(0);
			NodeList list = operators.getChildNodes();			

			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();				
				if (name.equals(subnode.getNodeValue())) {
					return false;					
				}	 
			}			
			
			Element operator = doc.createElement("operator");			
			operators.appendChild(operator);
			operator.appendChild(setNode("name",name,doc));
			operator.appendChild(setNode("extension",extension,doc));
			operator.appendChild(setNode("email",email,doc));
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			
			File file = new File(dirlib, fileCompany);

			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return false;
	}
	
	
	public boolean updateSite(String sitename, String contact, String address, String city, String state, String zipcode, String phone, String fax){
		if(!removeSite(sitename)){
			return false;
		}		
		return addSite(sitename, contact, address, city, state, zipcode, phone, fax);	
	}
	
	
	public boolean removeOperator(String name){		
		try {
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));			
			Node operators = doc.getElementsByTagName("operators").item(0);						
			NodeList list = operators.getChildNodes();
			 
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();
				if (name.equals(subnode.getNodeValue())) {
					operators.removeChild(node);					
				}	 
			}
						
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			
			File file = new File(dirlib, fileCompany);

			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return false;
	}
	
	
	public boolean removeObject(String object){		
		try {			
			File file = new File(dirsite, fileData);			
			Document doc = documentBuilder.parse(file);			
			Node objects = doc.getElementsByTagName("objects").item(0);			
			Node obj = doc.getElementById(object);			
			objects.removeChild(obj);			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);						
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			
			File img = this.getPictureFile(object); // Delete picture			
			if (img.exists()) {
				img.delete();
			}
			
			String objName = MUtils.removeSpecialChars( object ); // Delete audio files
			if ( dirsite.isDirectory() ){
				for ( File child : dirsite.listFiles() ){
					String fileName = child.getName(); 
					if( fileName.contains( "object_" + objName ) ){
						File fileToDelete = new File( dirsite, fileName);
						if ( fileToDelete.exists() ){
							fileToDelete.delete();							
						}
					}				
		        }
			}			
			
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return false;
	}
	
	
	public void removeTestAudio(String object, String test){
		String objName  =  MUtils.removeSpecialChars( object );
		String voice    =  "object_" + objName + "_test-" + test +"_annotation.3gp";			
		File audioFile  =  new File( dirsite , voice );		
		if( audioFile.exists() ){
			audioFile.delete();
		}
	}
	
	
	public boolean removeTest(String object, String test){		
		try {			
			removeTestAudio( object , test );
			File file          =  new File( dirsite , fileData );			
			Document doc       =  documentBuilder.parse( file );			
			Node obj           =  doc.getElementById( object );
			Node objReadings   =  obj.getLastChild();			
			NodeList readings  =  objReadings.getChildNodes();
			
			for (int i = 0; i < readings.getLength(); i++){
				Node read = readings.item(i);
				Node testNumber = read.getFirstChild();
				if (test.equals( testNumber.getTextContent() )){					
					objReadings.removeChild(read);
				}
			}			
			
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);						
			//StreamResult result = new StreamResult(file.toURI().toString());
			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return false;
	}
	
	
	public String[] getOperators(){		
		try {
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));			
			Node operators = doc.getElementsByTagName("operators").item(0);						
			NodeList list = operators.getChildNodes();
			String[] op = new String[list.getLength()];
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();
				if(subnode.getNodeValue().equals(this.getDefaultOperator())){
					op[i] = subnode.getNodeValue() + " " + activity.getResources().getString(R.string.defaults); 	
				} else {
					op[i] = subnode.getNodeValue();	
				}
			}			
			return op;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return null;
	}
	
	
	public String[] getOperatorsOrderByDefault(){		
		try {
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));			
			Node operators = doc.getElementsByTagName("operators").item(0);						
			NodeList list = operators.getChildNodes();
			String[] op;
			if(!this.getDefaultOperator().equals("")){
				op = new String[list.getLength()];
				op[0] = this.getDefaultOperator();	
			} else {
				op = new String[list.getLength()+1];
				op[0] = "";
			}
						 
			for (int i = 0; i < list.getLength(); i++) {				
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();
				if(subnode.getNodeValue().equals(this.getDefaultOperator())){					
					//op[i] = subnode.getNodeValue(); 	
				} else {
					op[i+1] = subnode.getNodeValue();	
				}
			}			
			return op;			
		} catch (Exception e) {
			Log.e("getOperatorsOrderByDef", e.toString());
			Log.e("getOperatorsOrderByDef", e.getMessage());
			return new String[]{""};
		}		
	}
	
	
	public List<String> getSites(){		
		try {
			Document doc = documentBuilder.parse(new File (dirlib, fileTestsites));			
			Node sites = doc.getElementsByTagName("testsites").item(0);						
			NodeList list = sites.getChildNodes();
			
			List<String> site = new ArrayList<String>();
			
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();			
					
				site.add(subnode.getNodeValue());
			}			
			return site;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return null;
	}
	
	
	public String[] getSitesForSpinner(){		
		try {
			Document doc = documentBuilder.parse(new File (dirlib, fileTestsites));			
			Node sites = doc.getElementsByTagName("testsites").item(0);						
			NodeList list = sites.getChildNodes();
			String[] site = new String[list.getLength()+1];
			site[0] = "";
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();				
				site[i+1] = subnode.getNodeValue();				
			}			
			return site;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return new String[]{};
	}
	
	
	public String[] getOperatorDetails(String name){		
		try {
			if(name.equals("operator")){
				String[] op = new String[]{"",""};
				return op;
			}
			Document doc = documentBuilder.parse(new File (dirlib, fileCompany));			
			Node operators = doc.getElementsByTagName("operators").item(0);						
			NodeList list = operators.getChildNodes();
			String[] op = new String[2];			 
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();
				if (name.equals(subnode.getNodeValue())) {
					Node operator = doc.getElementsByTagName("operator").item(i);
					NodeList oplist = operator.getChildNodes();
					op[0] = oplist.item(1).getTextContent();
					op[1] = oplist.item(2).getTextContent();
					return op;
				}	 
			}			
		} catch (Exception e) {
			e.printStackTrace();			
		}		
		return new String[]{"","",""};
	}
	
	
	public String[] getSiteDetails(String name){		
		try {			
			if(name.equals("testsite")){
				String[] site = new String[]{"","","","","","",""};
				return site;
			}
			Document doc = documentBuilder.parse(new File (dirlib, fileTestsites));			
			Node sites = doc.getElementsByTagName("testsites").item(0);						
			NodeList list = sites.getChildNodes();
			String[] site = new String[7];
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();
				if (name.equals(subnode.getNodeValue())) {
					Node operator = doc.getElementsByTagName("site").item(i);
					NodeList sitelist = operator.getChildNodes();					
					site[0] = sitelist.item(1).getTextContent();
					site[1] = sitelist.item(2).getTextContent();
					site[2] = sitelist.item(3).getTextContent();
					site[3] = sitelist.item(4).getTextContent();
					site[4] = sitelist.item(5).getTextContent();
					site[5] = sitelist.item(6).getTextContent();
					site[6] = sitelist.item(7).getTextContent();
					return site;
				}	 
			}			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return null;
	}
	
	
	public String[] getFileSiteDetails(){		
		try {			
			File file = new File(dirsite, fileData);
			Document doc = documentBuilder.parse(file);			
			Node sites = doc.getElementsByTagName("testsite").item(0);						
			NodeList list = sites.getChildNodes();
			String[] site = new String[7];																
			site[0] = list.item(1).getTextContent();
			site[1] = list.item(2).getTextContent();
			site[2] = list.item(3).getTextContent();
			site[3] = list.item(4).getTextContent();
			site[4] = list.item(5).getTextContent();
			site[5] = list.item(6).getTextContent();
			site[6] = list.item(7).getTextContent();
			return site;						
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return null;
	}
	
		
	public boolean updateOperator(String name, String extension, String email){
		if(!removeOperator(name)){
			return false;
		}		
		return addOperator(name,extension,email);	
	}
	
	
	public boolean removeSite(String name){		
		try {			
			File file = new File(dirlib, fileTestsites);			
			Document doc = documentBuilder.parse(file);			
			Node testsites = doc.getElementsByTagName("testsites").item(0);						
			NodeList list = testsites.getChildNodes();
			 
			for (int i = 0; i < list.getLength(); i++) {	 
				Node node = list.item(i);
				Node subnode = node.getFirstChild().getFirstChild();				
				if (name.equals(subnode.getNodeValue())) {
					testsites.removeChild(node);					
				}	 
			}
						
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);

			StreamResult result = new StreamResult(new FileOutputStream(file));
			transformer.transform(source, result);
			return true;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return false;
	}
	
	
	private Element setNode(String name, String content, Document doc){
		Element e = doc.createElement(name);				
		e.appendChild(doc.createTextNode(content));
		return e;		
	}
	
	
	public void setDefaultOperator(String operator){
		if(operator.equals(this.getDefaultOperator())){
			operator="";
		}
		SharedPreferences sharedPref = activity.getPreferences(Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = sharedPref.edit();
		editor.putString("default_op", operator);
		editor.commit();
	}
	
	
	public String getDefaultOperator(){
		SharedPreferences sharedPref = activity.getPreferences(Context.MODE_PRIVATE);		
		return sharedPref.getString("default_op", "");		
	}

	public class EM4058values{
		private String objectName ="";
		private String audioPath="";
		private int testNumber;
		private String testDate="";
		private String testType="";
		private String distance="";
		private String frequency="";
		private String voltage="";
		private String result="";
		private String result270Hz="";
		private String result570Hz="";
		private String result870Hz="";
		private String result1170Hz="";
		private String result1470Hz="";

		public EM4058values(int testNumber){
			this.testNumber=testNumber;
		}

		public void setObjectName(String name){
			this.objectName = name;
		}

		public String getObjectName(){
			return this.objectName;
		}

		public void setAudioPath(String path){
			this.audioPath = path;
		}

		public String getAudioPath(){
			return this.audioPath;
		}

		public int getTestNumber(){
			return this.testNumber;
		}

		public void setTestDate(String testDate){
			this.testDate=testDate;
		}

		public String getTestDate(){
			return this.testDate;
		}

		public void setTestType(String testType){
			this.testType=testType;
		}

		public String getTestType(){
			return this.testType;
		}

		public void setDistance(String distance){
			this.distance=distance;
		}

		public String getDistance(){
			return this.distance;
		}

		public void setFrequency(String frequency){
			this.frequency=frequency;
		}

		public String getFrequency(){
			return this.frequency;
		}

		public void setVoltage(String voltage){
			this.voltage=voltage;
		}

		public String getVoltage(){
			return this.voltage;
		}

		public void setResult(String result){
			this.result=result;
		}

		public String getResult(){
			return this.result;
		}

		public void setResult270Hz(String result270Hz){
			this.result270Hz=result270Hz;
		}

		public String getResult270Hz(){
			return this.result270Hz;
		}

		public void setResult570Hz(String result570Hz){
			this.result570Hz=result570Hz;
		}

		public String getResult570Hz(){
			return this.result570Hz;
		}

		public void setResult870Hz(String result870Hz){
			this.result870Hz=result870Hz;
		}

		public String getResult870Hz(){
			return this.result870Hz;
		}

		public void setResult1170Hz(String result1170Hz){
			this.result1170Hz=result1170Hz;
		}

		public String getResult1170Hz(){
			return this.result1170Hz;
		}

		public void setResult1470Hz(String result1470Hz){
			this.result1470Hz=result1470Hz;
		}

		public String getResult1470Hz(){
			return this.result1470Hz;
		}

		@Override
		public String toString(){
			String temp =
			testNumber + " | " +
			testDate + " | " +
			testType + " | " +
			distance + " | " +
			frequency + " | " +
			//voltage + " | " +
			result + " | " +
			result270Hz + " | " +
			result570Hz + " | " +
			result870Hz + " | " +
			result1170Hz + " | " +
			result1470Hz;
			return temp;
		}

		public String[] getAll(){
			return new String[]{
					testNumber+"",
					testDate,
					testType,
					distance,
					frequency,
					//voltage,
					result,
					result270Hz,
					result570Hz,
					result870Hz,
					result1170Hz,
					result1470Hz
			};
		}
	}

}
