package com.megabras.bluelogg.extended;

import android.annotation.SuppressLint;
import android.app.ActionBar.LayoutParams;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

import androidx.core.app.ActivityCompat;

//import android.support.v4.app.ActivityCompat;

public class GPS {
	private final String
			TAG 	= "GPS";

	private Activity
			activity;

	private LocationManager
			locationManager;

	private LocationListener
			locationListener;

	private Runnable
			gpstimeout;

	private int
			GPSTIMEOUT = 1000 * 30;

	private String[]
			gps;

	private boolean
			gpsRunning = false;

	private Handler
			handler;

	private float
			lastAccuracy = Float.MAX_VALUE;

	private GpsInterface
			mInterface;

	private DecimalFormatSymbols
			separator;

	private DecimalFormat
			coordenatesFormat,
			altitudeFormat;

	public static final int
			WAIT 		= 0,
			RESULT 		= 1,
			CANCELLED 	= 2,
			DELETED 	= 3;

	public GPS(Activity activity) {
		this.activity 		= activity;
		handler 			= new Handler();
		mInterface 			= (GpsInterface) activity;
		separator 			= new DecimalFormatSymbols(Locale.getDefault());
		separator.setDecimalSeparator('.');
		coordenatesFormat 	= new DecimalFormat("#.######", separator);
		coordenatesFormat.setRoundingMode(RoundingMode.DOWN);
		altitudeFormat 		= new DecimalFormat("#");
		altitudeFormat.setRoundingMode(RoundingMode.DOWN);

		gpstimeout = new Runnable() {
			public void run() {
				if (lastAccuracy == Float.MAX_VALUE) {
					dlgCancelGPS();
				} else {
					onFoundGpsData();
				}
			}
		};
	}

	public GPS(GpsInterface parent, Activity activity) {
		this.activity 		= activity;
		handler 			= new Handler();
		mInterface 			= parent;
		separator 			= new DecimalFormatSymbols(Locale.getDefault());
		separator.setDecimalSeparator('.');
		coordenatesFormat 	= new DecimalFormat("#.######", separator);
		coordenatesFormat.setRoundingMode(RoundingMode.DOWN);
		altitudeFormat 		= new DecimalFormat("#");
		altitudeFormat.setRoundingMode(RoundingMode.DOWN);

		gpstimeout = new Runnable() {
			public void run() {
				if (lastAccuracy == Float.MAX_VALUE) {
					dlgCancelGPS();
				} else {
					onFoundGpsData();
				}
			}
		};
	}

	private boolean sensorExists() {
		String provider = Settings.Secure.getString(activity.getContentResolver(), Settings.Secure.LOCATION_PROVIDERS_ALLOWED);
		return provider.contains("gps");
	}

	private boolean enableSensor() {
		Intent intent = null;
		if (!sensorExists()) {
			intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
			activity.startActivityForResult(intent, 1);
			return false;
		}
		return true;
	}

	public boolean getDataGPS() {
		try {
			if (ActivityCompat.checkSelfPermission(activity, android.Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(activity, android.Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
				return false;
			}
			if (!enableSensor() || gpsRunning) {
				return false;
			}

			mInterface.GpsData(GPS.WAIT, null);

			gpsRunning = true;
			gps = new String[4];

			locationListener = new LocationListener() {
				public void onLocationChanged(Location location) {
					Double latPoint = location.getLatitude();
					Double lngPoint = location.getLongitude();
					Float accuracy = location.getAccuracy();
					Double altitude = location.getAltitude();

					if (accuracy < lastAccuracy) {
						gps[0] = coordenatesFormat.format(lngPoint);
						gps[1] = coordenatesFormat.format(latPoint);
						gps[2] = altitudeFormat.format(altitude);
						gps[3] = accuracy + "";
					}

					lastAccuracy = accuracy;

					if (!gps[0].equals("") && !gps[1].equals("")) {//First value
						onFoundGpsData();
					}

				}

				public void onProviderEnabled(String provider) {}
				public void onProviderDisabled(String provider) {}
				public void onStatusChanged(String provider, int status, Bundle extras) {}
			};

			locationManager = (LocationManager) activity.getSystemService(Context.LOCATION_SERVICE);
			locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, locationListener);
		  
	    	if (GPSTIMEOUT > 0) {//GPSTIMEOUT = 0 (timeout disabled)
				handler.postDelayed(gpstimeout, GPSTIMEOUT);
	    	}
	    	return true;
		}
		catch (Exception e){
			android.util.Log.e(TAG, "ERROR GPS DATA: " + e.getMessage());
		}
		return true;
    }
	
	private void onFoundGpsData(){
		handler.removeCallbacks(gpstimeout);
		if (ActivityCompat.checkSelfPermission(activity, android.Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(activity, android.Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
			locationManager.removeUpdates(locationListener);
		}

		gpsRunning = false;
		lastAccuracy = Float.MAX_VALUE;
		mInterface.GpsData(GPS.RESULT, gps);
		dialogShowResult();
	}

	private void dlgCancelGPS()
	{
		final Dialog dialog = new Dialog( activity );
		dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
		dialog.setCanceledOnTouchOutside( false );
		if ( dialog.getWindow() != null )
		{
			dialog.getWindow().getAttributes().width = android.view.ViewGroup.LayoutParams.WRAP_CONTENT;
			dialog.getWindow().getAttributes().height = android.view.ViewGroup.LayoutParams.WRAP_CONTENT;
			dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
		}
		dialog.setContentView( R.layout.dialog_yes_no );

		((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.gps_timeout );
		((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

		ImageButton btnClose = dialog.findViewById( R.id.btn_close_dlg );
		if ( btnClose != null )
		{
			btnClose.setOnClickListener( v ->
			{
				mInterface.GpsData( GPS.CANCELLED, null );
				if ( ActivityCompat.checkSelfPermission( activity, android.Manifest.permission.ACCESS_FINE_LOCATION ) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission( activity, android.Manifest.permission.ACCESS_COARSE_LOCATION ) == PackageManager.PERMISSION_GRANTED )
				{
					locationManager.removeUpdates( locationListener );
				}
				gpsRunning = false;
				dialog.dismiss();
			} );
		}

		final TextView message = dialog.findViewById( R.id.message );
		message.setText( R.string.gps_timeout_message );

		final Button btnNo = dialog.findViewById( R.id.btn_no );
		btnNo.setOnClickListener( v ->
		{
			mInterface.GpsData( GPS.CANCELLED, null );
			if ( ActivityCompat.checkSelfPermission( activity, android.Manifest.permission.ACCESS_FINE_LOCATION ) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission( activity, android.Manifest.permission.ACCESS_COARSE_LOCATION ) == PackageManager.PERMISSION_GRANTED )
			{
				locationManager.removeUpdates( locationListener );
			}
			gpsRunning = false;
			dialog.dismiss();
		} );

		final Button btnYes = dialog.findViewById( R.id.btn_yes );
		btnYes.setOnClickListener( v ->
		{
			if ( gpsRunning )
			{
				handler.postDelayed( gpstimeout, GPSTIMEOUT );
			}
			dialog.dismiss();
		} );

		dialog.show();
	}

	public void dialogConfirmReplace( String[] data )
	{
		final Dialog dialog = new Dialog( activity );
		dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
		dialog.setCanceledOnTouchOutside( false );
		dialog.setContentView( R.layout.dialog_gps_details );
		if ( dialog.getWindow() != null )
		{
			dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
		}

		((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.gps_data );
		((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_about_gray );

		((TextView) dialog.findViewById( R.id.longitude )).setText( data[0] );
		((TextView) dialog.findViewById( R.id.latitude )).setText( data[1] );
		String altitude;
		if ( data[2].contains( "m" ) )
		{
			altitude = data[2];
		}
		else
		{
			altitude = data[2] + " m";
		}
		((TextView) dialog.findViewById( R.id.altitude )).setText( altitude );
		String accuracy;
		if ( data[3].contains( "\u00B1" ) )
		{
			accuracy = data[3];
		}
		else
		{
			accuracy = "\u00B1" + data[3] + " m";
		}
		((TextView) dialog.findViewById( R.id.accuracy )).setText( accuracy );

		dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> dialog.dismiss() );

		final Button btnNo = dialog.findViewById( R.id.btn_no );
		btnNo.setText( R.string.close );
		btnNo.setOnClickListener( v -> dialog.dismiss() );

		final Button btnYes = dialog.findViewById( R.id.btn_yes );
		btnYes.setText( R.string.replace );
		btnYes.setOnClickListener( v ->
		{
			getDataGPS();
			dialog.dismiss();
		} );

		dialog.show();
	}

	@SuppressLint("SetTextI18n")
	private void dialogShowResult()
	{
		final Dialog dialog = new Dialog( activity );
		dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
		dialog.setCanceledOnTouchOutside( false );
		dialog.setContentView( R.layout.dialog_gps_details );
		if ( dialog.getWindow() != null )
		{
			dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
		}

		((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.gps_data );
		((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_about_gray );

		((TextView) dialog.findViewById( R.id.longitude )).setText( gps[0] );
		((TextView) dialog.findViewById( R.id.latitude )).setText( gps[1] );
		((TextView) dialog.findViewById( R.id.altitude )).setText( gps[2] + " m" );
		((TextView) dialog.findViewById( R.id.accuracy )).setText( "\u00B1" + gps[3] + " m" );

		dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> dialog.dismiss() );

		final Button btnNo = dialog.findViewById( R.id.btn_no );
		btnNo.setVisibility( View.GONE );

		final Button btnYes = dialog.findViewById( R.id.btn_yes );
		btnYes.setText( "OK" );
		btnYes.setOnClickListener( v -> dialog.dismiss() );
		dialog.show();
	}

	public void cancel()
	{
		if ( locationManager != null )
		{
			if ( ActivityCompat.checkSelfPermission( activity, android.Manifest.permission.ACCESS_FINE_LOCATION ) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission( activity, android.Manifest.permission.ACCESS_COARSE_LOCATION ) == PackageManager.PERMISSION_GRANTED )
			{
				locationManager.removeUpdates( locationListener );
			}
		}
		if ( handler != null )
		{
			handler.removeCallbacks( gpstimeout );
		}
	}

	public interface GpsInterface
	{
		void GpsData( int which, String[] data );
	}
}


