package com.megabras.bluelogg.hp60kv;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Environment;
import android.text.InputFilter;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.InputFilterMinMax;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Protocol;

import java.io.File;
import java.io.FileOutputStream;
import java.text.DecimalFormat;

public class Helper
{
    @SuppressWarnings("unused")
    private String
            TAG = "HELPER";

    private Activity
            activity;

    private DecimalFormat
            format02 = new DecimalFormat("00"),
            format2 = new DecimalFormat("0.00");

    private Dialog
            warnDialog;

    private Dialog
            internalErrorDialog;

    Helper(Activity activity)
    {
        this.activity = activity;
    }

    File saveChartImage(Charts chart, File dir)
    {
        String type = "";
        switch ( chart.getChartType() )
        {
            case Charts.VOLTAGE_CHART:
                type = "voltage";
                break;
            case Charts.CURRENT_CHART:
                type = "current";
                break;
            case Charts.RESISTANCE_CHART:
                type = "resistance";
                break;
        }

        Bitmap bitmap = chart.getChartBitmap();

        File f = null;
        if ( Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED) )
        {
            String location = dir.getAbsolutePath();
            String fileName = type + ".png";
            f = new File(location + File.separator + fileName);
        }

        try
        {
            if ( f!=null )
            {
                FileOutputStream ostream = new FileOutputStream(f);
                bitmap.compress(Bitmap.CompressFormat.PNG, 10, ostream);
                ostream.close();
            }
            return f;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return null;
        }
    }

    void pickVoltage(final Controller controller, final Protocol modbus, final Device device, final int min, final int max, final int step)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnity  = dialog.findViewById(R.id.tv_unit);

        String unity = "V";
        message.setText( activity.getString(R.string.voltage) );
        tvUnity.setText(unity);
        String volt = device.getTestVoltage()+"";
        etNumber.setText( volt );

        etNumber.setSelection(0, etNumber.getText().length());
        etNumber.requestFocus();
        final InputMethodManager imm = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
        if ( imm != null )
        {
            imm.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
        }

        String warningMessage =
                activity.getString(R.string.range)
                        + ": " + min + " " + unity + " - "
                        + max + " " + unity
                        + " ("+activity.getString(R.string.step, step+" " + unity)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumber.getText().toString().equals("") )
                return;

            String sNumber =  etNumber.getText().toString()
                                .replace(unity,"")
                                .replace(",",".")
                                .trim();
            int number;

            if ( sNumber.contains(".") )
                number = (int) Double.parseDouble( sNumber );
            else
                number  = Integer.parseInt( sNumber );


            number      = (number/step)*step;

            if( number >= min && number <= max )
            {
                device.setTestVoltage(number);
                if ( imm != null )
                {
                    imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
                }
                controller.sendMessage( modbus.write.voltage( device.getTestVoltage() ) );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            if ( imm != null )
            {
                imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
            }
            dialog.dismiss();
        });

        dialog.show();
    }

    void pickNumber(final TextView tv, int title, final String unity, final int min, final int max, final int step)
    {
        final Dialog dialog = new Dialog( activity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.main_dialog_number );
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById( R.id.et_number  );
        final TextView message  = dialog.findViewById( R.id.tv_message );
        final TextView warning  = dialog.findViewById( R.id.tv_warning );
        final TextView tvUnity  = dialog.findViewById( R.id.tv_unit    );

        message.setText ( activity.getString( title ) );
        tvUnity.setText ( unity );
        etNumber.setText( tv.getText().toString().replace(unity, "").replace(",", ".").trim() );
        etNumber.setSelection(0, etNumber.getText().length());
        etNumber.requestFocus();
        final InputMethodManager imm = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
        if ( imm != null )
        {
            imm.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
        }

        String warningMessage =
                activity.getString(R.string.range)
                        + ": " + min + " " + unity + " - "
                        + max + " " + unity
                        + " ("+activity.getString(R.string.step, step+" " + unity)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( etNumber.getText().toString().equals("") )
                return;

            String sNumber = etNumber.getText().toString()
                                .replace(unity,"")
                                .replace(",",".")
                                .trim();

            int number;

            if ( sNumber.contains(".") )
                number = (int) Double.parseDouble( sNumber );
            else
                number  = Integer.parseInt( sNumber );
            number      = (number/step)*step;

            if( number >= min && number <= max )
            {
                String n = number + " " + unity;
                tv.setText( n );
                if ( imm != null )
                {
                    imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
                }
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            if ( imm != null )
            {
                imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
            }
            dialog.dismiss();
        });

        dialog.show();
    }

    @SuppressWarnings("SameParameterValue")
    void pickNumberFloat( final TextView tv, int title, final String unity, final double min, final double max )
    {
        final Dialog dialog = new Dialog( activity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.main_dialog_number_float );
        if (dialog.getWindow()!=null)
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById( R.id.et_number  );
        final TextView message  = dialog.findViewById( R.id.tv_message );
        final TextView warning  = dialog.findViewById( R.id.tv_warning );
        final TextView tvUnity  = dialog.findViewById( R.id.tv_unit    );

        message.setText ( activity.getString( title ) );
        tvUnity.setText ( unity );
        etNumber.setText( tv.getText().toString().replace(unity, "").trim() );
        etNumber.setSelection(0, etNumber.getText().length());
        etNumber.requestFocus();
        final InputMethodManager imm = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
        if ( imm != null )
        {
            imm.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
        }

        String warningMessage =
                activity.getString(R.string.range)
                        + ": " + format2.format( min ) + " " + unity + " - "
                        + format2.format( max ) + " " + unity;

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById( R.id.bt_save );
        buttonSave.setOnClickListener(v ->
        {
            if( etNumber.getText().toString().equals("") )
                return;

            String sNumber = etNumber.getText().toString()
                                .replace( unity    ,""   )
                                .replace(",", "." )
                                .trim();
            double number  = Double.parseDouble( sNumber );

            if( number >= min && number <= max )
            {
                String n = format2.format( number ) + " " + unity;
                tv.setText( n );
                if ( imm != null )
                {
                    imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
                }
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            if ( imm != null )
            {
                imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
            }
            dialog.dismiss();
        });

        dialog.show();
    }

    Spinner getSpinner(int spinnerID, int arrayID, View inflatedView)
    {
        ArrayAdapter<CharSequence> filter =
                ArrayAdapter.createFromResource(activity, arrayID, android.R.layout.simple_spinner_item);
        filter.setDropDownViewResource(android.R.layout.simple_list_item_activated_1);

        final Spinner spinner = inflatedView.findViewById(spinnerID);
        spinner.setAdapter(filter);
        return spinner;
    }

    @SuppressWarnings("SameParameterValue")
    Spinner getSpinner(int spinnerID, String[] values, View inflatedView)
    {
        final Spinner spinner = inflatedView.findViewById(spinnerID);
        ArrayAdapter<CharSequence> filter = new ArrayAdapter<>(activity,
                android.R.layout.simple_spinner_item,
                values);
        filter.setDropDownViewResource(android.R.layout.simple_list_item_activated_1);
        spinner.setAdapter(filter);
        return spinner;
    }

    void closeOverlay(FrameLayout v)
    {
        v.removeAllViews();
        v.setVisibility(View.GONE);
    }

    void pickTimerWithStep(final TextView tv, int min, int max, int step)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_timer);
        if (dialog.getWindow()!=null)
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumberMM = dialog.findViewById(R.id.et_number_mm);
        final EditText etNumberSS = dialog.findViewById(R.id.et_number_ss);
        final TextView message    = dialog.findViewById(R.id.tv_message);
        final TextView warning    = dialog.findViewById(R.id.tv_warning);


        max = max / 60;
//        final int
//                min  = Protocol.DEVICE_TIMER_MIN,
//                max  = Protocol.DEVICE_TIMER_MAX/60,
//                step = Protocol.DEVICE_STEP_TIMER;

        message.setText( activity.getString(R.string.test_duration) );

        etNumberMM.setFilters(new InputFilter[]{ new InputFilterMinMax(0, 20)});
        etNumberSS.setFilters(new InputFilter[]{ new InputFilterMinMax(0, 59)});

        String[] parts = tv.getText().toString().split(":");
        String num = Integer.parseInt(parts[0])+"";
        etNumberMM.setText( num );
        etNumberSS.setText( parts[1] );

        etNumberMM.setSelectAllOnFocus(true);
        etNumberSS.setSelectAllOnFocus(true);

        View.OnFocusChangeListener onFocus = new View.OnFocusChangeListener() {
            String temp = "";
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                EditText et = (EditText) v;
                if(hasFocus)
                {
                    temp = et.getText().toString();
                    et.setText("");
                }
                else
                {
                    if (et.getText().toString().equals(""))
                        et.setText(temp);
                }
            }
        };

        etNumberMM.setOnFocusChangeListener(onFocus);
        etNumberSS.setOnFocusChangeListener(onFocus);

//        final int
//                min  = Protocol.DEVICE_TIMER_MIN,
//                max  = Protocol.DEVICE_TIMER_MAX/60,
//                step = Protocol.DEVICE_STEP_TIMER;

        final int max2  = max;

        String warningMessage =
                activity.getString(R.string.range) + ": " + min + " s - " + max + " m ("+activity.getString(R.string.step, step+" s)");

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumberMM.getText().toString().equals("") || etNumberSS.getText().toString().equals("") )
                return;

            int minutes = Integer.parseInt(etNumberMM.getText().toString());
            int seconds = Integer.parseInt(etNumberSS.getText().toString());
            seconds     = (seconds/step)*step;
            int time    = (minutes*60) + seconds;

            if( time >= (min) && time <= (max2*60) )
            {
                String elapsed = format02.format(minutes) + ":" + format02.format(seconds);
                tv.setText( elapsed );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    void warnInternalError()
    {
        if ( internalErrorDialog == null )
        {
            internalErrorDialog = new Dialog( activity );
            internalErrorDialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
            internalErrorDialog.setCanceledOnTouchOutside( false );
            internalErrorDialog.setCancelable( false );
            internalErrorDialog.setContentView( R.layout.dialog_message_white );
            if ( internalErrorDialog.getWindow() != null )
            {
                internalErrorDialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            }
            ((TextView) internalErrorDialog.findViewById( R.id.dlg_title )).setText( R.string.internal_error );
            ((ImageView) internalErrorDialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) internalErrorDialog.findViewById( R.id.observation )).setText( "" );
            internalErrorDialog.findViewById( R.id.ui_iv_warning ).setVisibility( View.GONE );
            internalErrorDialog.findViewById( R.id.btn_close_dlg ).setVisibility( View.GONE );
            ((Button) internalErrorDialog.findViewById( R.id.btn_yes )).setText( R.string.md_ok );
            internalErrorDialog.findViewById( R.id.btn_yes ).setOnClickListener( v -> internalErrorDialog.dismiss() );
            ((TextView) internalErrorDialog.findViewById( R.id.message )).setText( activity.getResources().getString( R.string.internal_error_message ) );
        }

        internalErrorDialog.show();
    }

    void warnAboutLock( boolean spark )
    {
        if ( warnDialog == null )
        {
            warnDialog = new Dialog( activity );
            warnDialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
            warnDialog.setCanceledOnTouchOutside( false );
            warnDialog.setCancelable( false );
            warnDialog.setContentView( R.layout.dialog_message_white );
            if ( warnDialog.getWindow() != null )
            {
                warnDialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            }
            ((TextView) warnDialog.findViewById( R.id.dlg_title )).setText( R.string.md_remote_control_locked );
            ((ImageView) warnDialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) warnDialog.findViewById( R.id.observation )).setText( "" );
            warnDialog.findViewById( R.id.ui_iv_warning ).setVisibility( View.GONE );
            warnDialog.findViewById( R.id.btn_close_dlg ).setVisibility( View.GONE );
            ((Button) warnDialog.findViewById( R.id.btn_yes )).setText( R.string.md_ok );
            warnDialog.findViewById( R.id.btn_yes ).setOnClickListener( v -> warnDialog.dismiss() );
        }

        if ( spark )
            ((TextView) warnDialog.findViewById( R.id.message )).setText( activity.getResources().getString( R.string.remote_control_locked_spark ) );
        else
            ((TextView) warnDialog.findViewById( R.id.message )).setText( activity.getResources().getString( R.string.remote_control_locked ) );

        warnDialog.show();
    }

//    @SuppressLint("ClickableViewAccessibility")
//    void warnAboutLock(boolean spark)
//    {
//        final View
//                inflatedView = View.inflate(activity, R.layout.md10kvx_fgr_msg, null);
//
//        final FrameLayout
//                overlay = MUtils.getOverlay(activity, Controller.OVERLAY_ID, inflatedView);
//
//        //This is to prevent the ontouch propagate to others views
//        RelativeLayout overlayBase = inflatedView.findViewById(R.id.overlay_base);
//        overlayBase.requestDisallowInterceptTouchEvent(true);
//
//        overlayBase.setOnTouchListener((v, event) -> true);
//
//        TextView title = inflatedView.findViewById(R.id.tx_title);
//        title.setText(activity.getResources().getString(R.string.md_remote_control_locked));
//
//        TextView message = inflatedView.findViewById(R.id.tx_message);
//        if (spark)
//            message.setText(activity.getResources().getString(R.string.remote_control_locked_spark));
//        else
//            message.setText(activity.getResources().getString(R.string.md_remote_control_locked_explained));
//
//        Button btClose = inflatedView.findViewById(R.id.bt_confirm);
//        btClose.setVisibility(View.GONE);
//
//        Button btCancel = inflatedView.findViewById(R.id.bt_cancel);
//        btCancel.setText(activity.getResources().getString(R.string.md_ok));
//        btCancel.setOnClickListener(v -> closeOverlay(overlay));
//    }
}
