package com.megabras.bluelogg.hp60kv;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TableLayout;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Measurement;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

//import android.support.annotation.NonNull;
//import android.support.v4.app.Fragment;


public class MainScreen extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG             = "MainScreen",
            NONE            = " - - - ";

    @SuppressWarnings("FieldCanBeLocal")
    private int
            layoutID        =  R.layout.hp60kv_fgr_main;

    private Context
            context;

    private TextView
            result_pass,
            result_fail,
            result_pi,
            result_lock,
            result_spark,
            result_aborted,
            result_overcurrent,
            result_emergency,
            result_protection,
            tvTestNumber,
            def_mode,
            def_voltage,
            def_max_voltage,
            def_max_current,
            def_timer,
            def_ramp_up,
            def_ramp_down,
            tvRampElapsed,
            tvElapsed,
            tvVoltage,
            ui_tv_resistance,
            tvCurrent;

    private ImageView
            ic_ramp,
            ic_ramp_fix,
            ic_ramp_up,
            ic_ramp_down;

    private TableLayout
            colResults;

    private String[]
            modes;

    private com.megabras.modbus.HP60KV.FormatData
            formatData;

    public MainScreen()
    {
        formatData = new com.megabras.modbus.HP60KV.FormatData();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        context = inflater.getContext();
        View view = inflater.inflate(layoutID, container, false);
        view.setBackgroundColor(context.getResources().getColor(R.color.transparent));
        return view;
    }

    @Override
    public void onStart()
    {
        super.onStart();
        if (this.getView()==null)
            return;
        modes              = context.getResources().getStringArray(R.array.hp60kv_op_modes);
        result_pass        = this.getView().findViewById(R.id.result_pass);
        result_fail        = this.getView().findViewById(R.id.result_fail);
        result_pi          = this.getView().findViewById(R.id.result_pi);
        result_aborted     = this.getView().findViewById(R.id.result_aborted);
        result_lock        = this.getView().findViewById(R.id.result_lock);
        result_protection  = this.getView().findViewById(R.id.result_protection);
        result_overcurrent = this.getView().findViewById(R.id.result_overcurrent);
        result_emergency   = this.getView().findViewById(R.id.result_emergency);
        result_spark       = this.getView().findViewById(R.id.result_spark);
        tvTestNumber       = this.getView().findViewById(R.id.lb_test_number);
        def_mode           = this.getView().findViewById(R.id.lb_mode);
        def_voltage        = this.getView().findViewById(R.id.lb_defined_voltage);
        def_max_voltage    = this.getView().findViewById(R.id.lb_defined_max_voltage);
        def_max_current    = this.getView().findViewById(R.id.lb_max_current);
        def_timer          = this.getView().findViewById(R.id.lb_timer);
        def_ramp_up        = this.getView().findViewById(R.id.lb_ramp_up);
        def_ramp_down      = this.getView().findViewById(R.id.lb_ramp_down);
        tvElapsed          = this.getView().findViewById(R.id.lb_time_elapsed);
        tvRampElapsed      = this.getView().findViewById(R.id.lb_time_ramp);
        tvVoltage          = this.getView().findViewById(R.id.lb_voltage);
        ui_tv_resistance   = this.getView().findViewById(R.id.lb_resistance);
        tvCurrent          = this.getView().findViewById(R.id.lb_current);
        colResults         = this.getView().findViewById(R.id.col_results);
        ic_ramp            = this.getView().findViewById(R.id.ic_ramp);
        ic_ramp_fix        = this.getView().findViewById(R.id.ic_ramp_fix);
        ic_ramp_up         = this.getView().findViewById(R.id.ic_ramp_up);
        ic_ramp_down       = this.getView().findViewById(R.id.ic_ramp_down);
        tvRampElapsed.setVisibility(View.GONE);
        ui_tv_resistance.setVisibility(View.GONE);
    }

    public void setVisibility(int id, int visibility)
    {
        if (getView()!=null && getView().findViewById(id)!=null)
        {
            getView().findViewById(id).setVisibility(visibility);
        }
    }

    void setDeviceID( int id )
    {
        formatData.setDeviceId( id );
    }

    public void setPI(String pi)
    {
        result_pi.setText(pi);
        updateVisibility(result_pi, View.VISIBLE);
    }

    void hidePI()
    {
        if (result_pi!=null && result_pi.getVisibility()==View.VISIBLE)
        {
            updateVisibility(result_pi, View.GONE);
        }
    }

    public void setTestNumber(int number)
    {
        String testNumber;
        if(number<=0)
            testNumber = NONE;
        else
            testNumber = number +"";
        tvTestNumber.setText(testNumber);
    }

    void setTestParameters( Device device, int firmware )
    {
        def_mode.setText( modes[device.getOperationMode()] );

        switch (device.getOperationMode())
        {
            case Device.MODE_MANUAL:
                if (device.getParamManual()==null)
                    return;
                def_max_voltage.setVisibility(View.VISIBLE);
                def_voltage.setVisibility(View.VISIBLE);
                def_timer      .setVisibility(View.GONE);
                def_ramp_up    .setVisibility(View.GONE);
                def_ramp_down  .setVisibility(View.GONE);
                colResults     .setVisibility(View.GONE);
                def_voltage    .setText( formatData.formatVoltage(device.getTestVoltage()) );
                def_max_voltage.setText( formatData.formatVoltage(device.getParamManual().getMaxVoltage()) );
                setMaxCurrent(device.getParamManual().getMaxCurrent(), firmware);
                ic_ramp        .setVisibility(View.GONE);
                ic_ramp_fix    .setVisibility(View.GONE);
                ic_ramp_up     .setVisibility(View.GONE);
                ic_ramp_down   .setVisibility(View.GONE);
                //tvRampElapsed  .setVisibility(View.GONE);
                break;
            case Device.MODE_AUTO:
                if (device.getParamSVT()==null)
                    return;
                def_max_voltage.setVisibility(View.VISIBLE);
                def_voltage.setVisibility(View.GONE);
                def_timer      .setVisibility(View.VISIBLE);
                def_ramp_up    .setVisibility(View.GONE);
                def_ramp_down  .setVisibility(View.GONE);
                colResults     .setVisibility(View.GONE);
                def_max_voltage.setText( formatData.formatVoltage(device.getParamSVT().getVoltage()) );
                setMaxCurrent(device.getParamSVT().getMaxCurrent(), firmware);
                def_timer      .setText( formatData.formatElapsedTime(device.getParamSVT().getDuration()) );
                ic_ramp        .setVisibility(View.GONE);
                ic_ramp_fix    .setVisibility(View.GONE);
                ic_ramp_up     .setVisibility(View.GONE);
                ic_ramp_down   .setVisibility(View.GONE);
               // tvRampElapsed  .setVisibility(View.GONE);
                break;
            case Device.MODE_RAMP:
                if (device.getParamRamp()==null)
                    return;
                def_max_voltage.setVisibility(View.VISIBLE);
                def_voltage.setVisibility(View.GONE);
                def_timer      .setVisibility(View.VISIBLE);
                def_ramp_up    .setVisibility(View.VISIBLE);
                def_ramp_down  .setVisibility(View.VISIBLE);
                colResults     .setVisibility(View.VISIBLE);
                def_max_voltage.setText( formatData.formatVoltage(device.getParamRamp().getVoltage()) );
                setMaxCurrent(device.getParamRamp().getMaxCurrent(), firmware);
                def_timer      .setText( formatData.formatElapsedTime(device.getParamRamp().getDuration()) );
                def_ramp_up    .setText( String.format("%s V/s", device.getParamRamp().getRampUpVoltage()) );
                def_ramp_down  .setText( String.format("%s V/s", device.getParamRamp().getRampDownVoltage()) );
                ic_ramp     .setVisibility( View.VISIBLE );
                ic_ramp_fix .setVisibility( View.GONE    );
                ic_ramp_up  .setVisibility( View.GONE    );
                ic_ramp_down.setVisibility( View.GONE    );
                break;
        }
    }

    private void setMaxCurrent(int maxCurrent, int firmware)
    {
        if ( maxCurrent > 0 )
            def_max_current.setText( formatData.formatCurrentConfig( maxCurrent, firmware) );
        else
            def_max_current.setText(NONE);
    }

    private void updateTextView(TextView tv, String text)
    {
        if (tv==null)
            return;
        if (!tv.getText().equals(text))
            tv.setText(text);
    }

    private void updateVisibility(View v, int visibility)
    {
        if (v==null)
            return;
        if (v.getVisibility()!=visibility)
            v.setVisibility(visibility);
    }

    void waitonDisplay()
    {
        tvVoltage.setText(R.string.wait);
        tvCurrent.setText(R.string.wait);
        ui_tv_resistance.setText(R.string.wait);
    }

    public void update(Measurement m, Device device, int firmware)
    {
        //updateTextView(tvElapsed       , formatData.formatElapsedTime(m.getElapsed()));
        updateTextView(tvCurrent       , formatData.formatCurrentRT ( m.getCurrent()   , firmware)     );
        updateTextView(tvVoltage       , formatData.formatVoltage   ( m.getVoltage())                  );
        updateTextView(ui_tv_resistance, formatData.formatResistance( m.getResistance(), m.getStats()) );

        if( device.getOperationMode()==Device.MODE_RAMP )
        {
            updateTextView(tvElapsed       , formatData.formatElapsedTime( m.getElapsedTotal() ));
            if ( !device.getStatusTwo(0) &&
                 !device.getStatusTwo(1) &&
                 !device.getStatusTwo(2) )
            {
                updateVisibility(ic_ramp     , View.VISIBLE);
                updateVisibility(ic_ramp_fix , View.GONE);
                updateVisibility(ic_ramp_up  , View.GONE);
                updateVisibility(ic_ramp_down, View.GONE);
            }
            else if (  device.getStatusTwo(0) &&
                      !device.getStatusTwo(1) &&
                      !device.getStatusTwo(2) )
            {
                updateVisibility(ic_ramp     , View.GONE);
                updateVisibility(ic_ramp_fix , View.GONE);
                updateVisibility(ic_ramp_up  , View.VISIBLE);
                updateVisibility(ic_ramp_down, View.GONE);
            }
            else if ( !device.getStatusTwo(0) &&
                       device.getStatusTwo(1) &&
                      !device.getStatusTwo(2) )
            {
                updateVisibility(ic_ramp     , View.GONE);
                updateVisibility(ic_ramp_fix , View.VISIBLE);
                updateVisibility(ic_ramp_up  , View.GONE);
                updateVisibility(ic_ramp_down, View.GONE);
            }
            else if (  device.getStatusTwo(0) &&
                       device.getStatusTwo(1) &&
                      !device.getStatusTwo(2) )
            {
                updateVisibility(ic_ramp     , View.GONE);
                updateVisibility(ic_ramp_fix , View.GONE);
                updateVisibility(ic_ramp_up  , View.GONE);
                updateVisibility(ic_ramp_down, View.VISIBLE);
            }
            else if ( !device.getStatusTwo(0) &&
                      !device.getStatusTwo(1) &&
                       device.getStatusTwo(2) )
            {
                updateVisibility(ic_ramp     , View.VISIBLE);
                updateVisibility(ic_ramp_fix , View.GONE);
                updateVisibility(ic_ramp_up  , View.GONE);
                updateVisibility(ic_ramp_down, View.GONE);
            }
        }
        else
        {
            updateTextView(tvElapsed     , formatData.formatElapsedTime(m.getElapsed()));
            updateVisibility(ic_ramp     , View.GONE);
            updateVisibility(ic_ramp_fix , View.GONE);
            updateVisibility(ic_ramp_up  , View.GONE);
            updateVisibility(ic_ramp_down, View.GONE);
        }
    }

    void updateStatus( boolean[] statusOne )
    {
        updateVisibility(result_lock       , (statusOne[Device.ST_ONE_GENERATION_LOCK]     ? View.VISIBLE : View.GONE));
        updateVisibility(result_emergency  , (statusOne[Device.ST_ONE_EMERGENCY_STOP]      ? View.VISIBLE : View.GONE));
        updateVisibility(result_spark      , (statusOne[Device.ST_ONE_SPARK_DETECTED]      ? View.VISIBLE : View.GONE));
        updateVisibility(result_overcurrent, (statusOne[Device.ST_ONE_OVERCURRENT]         ? View.VISIBLE : View.GONE));
        updateVisibility(result_protection , (statusOne[Device.ST_ONE_INTERNAL_PROTECTION] ? View.VISIBLE : View.GONE));
        updateVisibility(result_pi         , (statusOne[Device.ST_ONE_IP_CALCULATION]      ? View.VISIBLE : View.GONE));

        if ( statusOne[6] && statusOne[7] )
        {
            updateVisibility(result_pass   , View.VISIBLE);
            updateVisibility(result_fail   , View.GONE);
            updateVisibility(result_aborted, View.GONE);
        }
        else if ( !statusOne[6] && statusOne[7] )
        {
            updateVisibility(result_pass   , View.GONE);
            updateVisibility(result_fail   , View.VISIBLE);
            updateVisibility(result_aborted, View.GONE);
        }
        else if ( statusOne[6] )
        {
            updateVisibility(result_pass   , View.GONE);
            updateVisibility(result_fail   , View.GONE);
            updateVisibility(result_aborted, View.VISIBLE);
        }
        else
        {
            updateVisibility(result_pass   , View.GONE);
            updateVisibility(result_fail   , View.GONE);
            updateVisibility(result_aborted, View.GONE);
        }
    }

    public void reset()
    {
        updateTextView( tvTestNumber, NONE );
        updateTextView( def_mode, NONE );
        updateTextView( def_voltage, NONE );
        updateTextView( def_max_voltage, NONE );
        updateTextView( def_max_current, NONE );
        updateTextView( def_timer, NONE );
        updateTextView( tvElapsed, NONE );
        updateTextView( tvRampElapsed, NONE );
        updateTextView( tvVoltage, NONE );
        updateTextView( ui_tv_resistance, NONE );
        updateTextView( tvCurrent, NONE );
        updateTextView( def_ramp_up, NONE );
        updateTextView( def_ramp_down, NONE );
        updateTextView( result_pi, NONE );
        updateVisibility( result_pi, View.GONE );
        updateVisibility( result_pass, View.GONE );
        updateVisibility( result_fail, View.GONE );
        updateVisibility( result_aborted, View.GONE );
        updateVisibility( result_lock, View.GONE );
        updateVisibility( result_overcurrent, View.GONE );
        updateVisibility( result_emergency, View.GONE );
        updateVisibility( result_spark, View.GONE );
    }
}

