package com.megabras.bluelogg.md10kvx;

import android.app.Activity;
import android.graphics.Typeface;
import android.graphics.pdf.PdfDocument;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Paper;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.bluelogg.extended.Units;
import com.megabras.bluelogg.md10kvx.MD10KVx.mHandler;
import com.megabras.bluelogg.md10kvx.MD10KVxTest.Result;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Locale;

@SuppressWarnings("WeakerAccess")
public class ReportPDF extends ReportProvider
{

    @SuppressWarnings("FieldCanBeLocal")
    private final String
            TAG = "ReportPDF";

    public ReportPDF( Activity activity, File dir, File tempDir, mHandler handler)
    {
        super(activity, dir, tempDir, handler);
    }

    public void generate( MD10KVx parent, MD10KVxTest test, File picture, File map, File[] chartImageArray )
    {
        handler.obtainMessage( mHandler.REPORT_UPDATE, 1, -1, activity.getString( R.string.report_creating_file ) ).sendToTarget();

        PdfDocument document = new PdfDocument();
        PdfDocument.PageInfo pageInfo;

        if ( Locale.getDefault().getLanguage().equals( Locale.ENGLISH.getLanguage() ) )
        {
            pageInfo = new PdfDocument.PageInfo.Builder( Paper.Letter.getWidth(), Paper.Letter.getHeight(), 1 ).create();
        }
        else
        {
            pageInfo = new PdfDocument.PageInfo.Builder( Paper.A4.getWidth(), Paper.A4.getHeight(), 1 ).create();
        }

        CURSOR_Y     = 0;
        CURSOR_X     = 0;
        PAPER_WIDTH  = pageInfo.getPageWidth() - (MARGIN * 2);
        PAPER_HEIGHT = pageInfo.getPageHeight() - (MARGIN * 2);

        PdfDocument.Page page = document.startPage(pageInfo);
        canvas = page.getCanvas();

        moveCursor( MARGIN, MARGIN );

        // BORDER
        top( 0 );
        left( 0 );
        right( PAPER_WIDTH );
        bottom( PAPER_HEIGHT );
        this.drawSquare();

        handler.obtainMessage( mHandler.REPORT_UPDATE, 1, -1, activity.getString( R.string.report_general_info ) ).sendToTarget();

        // TITLE
        top( 0 );
        left( 0 );
        right( PAPER_WIDTH );
        bottom( 20 );
        this.drawTitle( activity.getString( R.string.md_insulation_test_report ) );

        ArrayList<String> titles = new ArrayList<>();
        ArrayList<String> values = new ArrayList<>();

        titles.add( activity.getString(R.string.md_model       ) );
        titles.add( activity.getString(R.string.md_serial      ) );
        titles.add( activity.getString(R.string.md_test_number ) );
        titles.add( activity.getString(R.string.md_test_date   ) );

        values.add( test.getModel() );
        values.add( test.getSerial() );
        values.add( test.getTestNum() + "" );
        values.add( test.getTestDateAsString() );

        top( 0 );
        left( 0 );
        right( PAPER_WIDTH );
        bottom( 20 );
        this.drawLineItem( titles, values );

        if ( !test.getAccuracy().equals( "" ) )
        {
            titles = new ArrayList<>();
            values = new ArrayList<>();

            titles.add( activity.getString( R.string.md_latitude     ) );
            titles.add( activity.getString( R.string.md_longitude    ) );
            titles.add( activity.getString( R.string.md_altitude     ) );
            titles.add( activity.getString( R.string.md_gps_accuracy ) );

            values.add( test.getLatitude() );
            values.add( test.getLongitude() );
            values.add( test.getAltitudeAsString() );
            values.add( test.getAccuracyAsString() );

            left( 0 );
            top( 0 );
            right( PAPER_WIDTH );
            bottom( 20 );
            this.drawLineItem( titles, values );
        }

        titles                = new ArrayList<>();
        ArrayList<File> files = new ArrayList<>();

        titles.add( activity.getString( R.string.picture ) );
        titles.add( activity.getString( R.string.map     ) );

        files.add( picture );
        files.add( map );

        left( 0 );
        top( 0 );
        right( PAPER_WIDTH );
        bottom( 200 );

        if ( picture != null && picture.exists() && map != null && map.exists() )
        {
            this.drawLineImage( titles, files );
        }
        else if ( picture != null && picture.exists() )
        {
            this.drawLineItem( titles.get( 0 ), picture );
        }
        else if ( map != null && map.exists() )
        {
            this.drawLineItem( titles.get( 1 ), map );
        }

        saveCursor();

        int CHART_W_AREA = (int) (PAPER_WIDTH * 0.75);

        // TEST DETAILS
        left( 0 );
        top( 0 );
        right( CHART_W_AREA );
        bottom( 20 );

        int size = test.getResults().size() - 1;

        int ss = Integer.parseInt( test.getResults().get( size ).getSeconds() );
        float oo = Float.parseFloat( test.getResults().get( size ).getResistance() );

        String duration = String.format( Locale.US, "%02d", ( ss / 1000 ) / 60 ) + ":" + String.format( Locale.US, "%02d", ( ss / 1000 ) % 60 );
        String lastOhm  = parent.resolutionR( oo );
        double OHM_MIN  = +Double.MAX_VALUE;
        double OHM_MAX  = -Double.MAX_VALUE;
        String sOHM_MIN = "";
        String sOHM_MAX = "";

        for ( Result r : test.getResults() )
        {
            float ohm  = Float.valueOf( r.getResistance() );

            if ( ohm < OHM_MIN )
            {
                OHM_MIN = ohm;
                sOHM_MIN = parent.resolutionR( ohm );
            }
            if ( ohm > OHM_MAX )
            {
                OHM_MAX = ohm;
                sOHM_MAX = parent.resolutionR( ohm );
            }
        }

        titles = new ArrayList<>();
        values = new ArrayList<>();

        titles.add( activity.getString( R.string.test_duration_mmss ) );
        titles.add( activity.getString( R.string.resistance         ) );
        titles.add( activity.getString( R.string.minimum_resistance ) );
        titles.add( activity.getString( R.string.maximum_resistance ) );

        values.add( duration );
        values.add( lastOhm  );
        values.add( sOHM_MIN );
        values.add( sOHM_MAX );

        this.drawLineItem( titles, values );

        titles = new ArrayList<>();
        values = new ArrayList<>();

        // DAI
        if ( !test.getDAI().equals( "" ) )
        {
            titles.add( activity.getString( R.string.DAI ) );
            values.add( test.getDAI() );
        }
        else
        {
            titles.add( activity.getString(R.string.DAI)    );
            values.add( activity.getString(R.string.dashes) );
        }

        // PI
        if ( !test.getPI().equals( "" ) )
        {
            titles.add( activity.getString( R.string.PI ) );
            values.add( test.getPI() );
        }
        else
        {
            titles.add( activity.getString(R.string.PI)     );
            values.add( activity.getString(R.string.dashes) );
        }

        // SVT
        if ( !test.getSVT().equals( "" ) )
        {
            titles.add( activity.getString( R.string.SVT ) );
            values.add( test.getSVT() );
        }
        else
        {
            titles.add( activity.getString( R.string.SVT ) );
            values.add( activity.getString( R.string.dashes ) );
        }

        // CAP
        if ( !test.getCapacitance().equals( "" ) )
        {
            titles.add( activity.getString( R.string.md_capacitance ).toUpperCase() );
            values.add( test.getCapacitance() );
        }
        else
        {
            titles.add( activity.getString( R.string.md_capacitance ) );
            values.add( activity.getString( R.string.dashes ) );
        }

        this.drawLineItem(titles, values);

        // CHARTS
        left( 0 );
        top( 0 );
        right( CHART_W_AREA );
        bottom( (PAPER_HEIGHT - CURSOR_Y) / 2 );

        files = new ArrayList<>();
        if ( chartImageArray.length > 0 )
        {
            files.add( chartImageArray[0] );
        }
        if ( chartImageArray.length > 1 )
        {
            files.add( chartImageArray[1] );
        }

        this.drawLineImage( files );

        files = new ArrayList<>();
        if ( chartImageArray.length > 2 )
        {
            files.add( chartImageArray[2] );
        }
        else
        {
            files.add( null );
        }

        if ( chartImageArray.length > 3 )
        {
            files.add( chartImageArray[3] );
        }
        else
        {
            files.add( null );
        }

        this.drawLineImage( files );

        handler.obtainMessage( mHandler.REPORT_UPDATE, 1, -1, activity.getString( R.string.report_tables ) ).sendToTarget();

        // TABLE
        restoreCursor();

        moveCursor( CHART_W_AREA, 0 );

        left( 0 );
        top( 0 );
        right( PAPER_WIDTH - CHART_W_AREA );
        bottom( 10 );

        values = new ArrayList<>();
        values.add( activity.getString( R.string.MMSS ) );
        values.add( activity.getString( R.string.V    ) );
        values.add( activity.getString( R.string.ohm  ) );
        values.add( activity.getString( R.string.A    ) );

        this.drawRow( Typeface.DEFAULT_BOLD, values );

        int PAPER_LEFT = PAPER_HEIGHT - CURSOR_Y - BRAND_HEIGHT;
        int ROWS_MAX   = PAPER_LEFT / getBounds().height();

        ArrayList<Result> data = test.getResults();

        if ( data.size() > ROWS_MAX )
        {
            int FIRST_HALF = ROWS_MAX / 2;
            int LAST_HALF  = ROWS_MAX - FIRST_HALF;
            for ( int i = 0; i < FIRST_HALF; i++ )
            {
                int   sec  = Integer.parseInt( data.get( i ).getSeconds() );
                float ohm  = Float.valueOf( data.get( i ).getResistance() );
                float volt = Float.valueOf( data.get( i ).getVoltage() );
                float amp  = parent.getSimpleVoltage( volt ) / ohm;

                final boolean overrangeR = ohm >= (10 * Units.TERA);
                final boolean overrangeA = amp < (1 / Units.PICO);

                String formattedResistance   = parent.resolutionR( ohm );
                String formattedMilliseconds = String.format( Locale.US, "%02d", ( sec / 1000 ) / 60 ) + ":" + String.format( Locale.US, "%02d", ( sec / 1000 ) % 60 );
                String formattedVoltage      = parent.resolutionV( volt / Units.KILO )  + " kV";
                String calculatedCurrent     = parent.resolutionA( amp );

                if ( overrangeR || overrangeA )
                {
                    calculatedCurrent = "0nA";
                    formattedResistance = ">10T\u03A9";
                }

                values = new ArrayList<>();
                values.add( formattedMilliseconds );
                values.add( formattedVoltage );
                values.add( formattedResistance );
                values.add( calculatedCurrent );

                this.drawRow( Typeface.create( "sans-serif-condensed", Typeface.NORMAL ), values );
            }
            for ( int i = (data.size()-1)-LAST_HALF; i < data.size(); i++ )
            {
                int   sec  = Integer.parseInt( data.get( i ).getSeconds() );
                float ohm  = Float.valueOf( data.get( i ).getResistance() );
                float volt = Float.valueOf( data.get( i ).getVoltage() );
                float amp  = parent.getSimpleVoltage( volt ) / ohm;

                final boolean overrangeR = ohm >= (10 * Units.TERA);
                final boolean overrangeA = amp < (1 / Units.PICO);

                String formattedResistance   = parent.resolutionR( ohm );
                String formattedMilliseconds = String.format( Locale.US, "%02d", ( sec / 1000 ) / 60 ) + ":" + String.format( Locale.US, "%02d", ( sec / 1000 ) % 60 );
                String formattedVoltage      = parent.resolutionV( volt / Units.KILO )  + " kV";
                String calculatedCurrent     = parent.resolutionA( amp );

                if ( overrangeR || overrangeA )
                {
                    calculatedCurrent = "0nA";
                    formattedResistance = ">10T\u03A9";
                }

                values = new ArrayList<>();
                values.add( formattedMilliseconds );
                values.add( formattedVoltage );
                values.add( formattedResistance );
                values.add( calculatedCurrent );

                this.drawRow( Typeface.create( "sans-serif-condensed", Typeface.NORMAL ), values );
            }
        }
        else
        {
            for ( int i = 0; i < data.size(); i++ )
            {
                int   sec  = Integer.parseInt( data.get( i ).getSeconds() );
                float ohm  = Float.valueOf( data.get( i ).getResistance() );
                float volt = Float.valueOf( data.get( i ).getVoltage() );
                float amp  = parent.getSimpleVoltage( volt ) / ohm;

                final boolean overrangeR = ohm >= (10 * Units.TERA);
                final boolean overrangeA = amp < (1 / Units.PICO);

                String formattedResistance   = parent.resolutionR( ohm );
                String formattedMilliseconds = String.format( Locale.US, "%02d", ( sec / 1000 ) / 60 ) + ":" + String.format( Locale.US, "%02d", ( sec / 1000 ) % 60 );
                String formattedVoltage      = parent.resolutionV( volt / Units.KILO )  + " kV";
                String calculatedCurrent     = parent.resolutionA( amp );

                if ( overrangeR || overrangeA )
                {
                    calculatedCurrent = "0nA";
                    formattedResistance = ">10T\u03A9";
                }

                values = new ArrayList<>();
                values.add( formattedMilliseconds );
                values.add( formattedVoltage );
                values.add( formattedResistance );
                values.add( calculatedCurrent );

                this.drawRow( Typeface.create( "sans-serif-condensed", Typeface.NORMAL ), values );
            }
        }

        drawGeneratedBy();

        // finish the page
        document.finishPage( page );

        handler.obtainMessage( mHandler.REPORT_UPDATE, 1, -1, activity.getString( R.string.saving_file ) ).sendToTarget();

        String docFile = "Report_" +
                test.getModel() + "_" +
                test.getSerial() + "_" +
                test.getTestNum() + "_" +
                test.getTestDate() + ".pdf";

        File tmpFile   = new File(dir + "/"+ docFile);

        try
        {
            document.writeTo(new FileOutputStream(tmpFile));
            // close the document
            document.close();

            handler.obtainMessage( mHandler.MESSAGE_REPORT_DONE, 1, -1, activity.getString(R.string.finished)).sendToTarget();

            sendFile(tmpFile);
        }
        catch (IOException e)
        {
            Log.e(TAG, "error "+e.toString());
            handler.obtainMessage( mHandler.MESSAGE_REPORT_DONE, 1, -1, activity.getString( R.string.finished ) ).sendToTarget();
        }
    }
}

