package com.megabras.bluelogg.md15kvr;

import android.annotation.SuppressLint;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.TimePicker;

import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.BooleanObservable;
import com.megabras.bluelogg.extended.CountDown;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.IntegerObservable;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.DielectricDischarge;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.ParamManual;
import com.megabras.modbus.MD15KVR.ParamRamp;
import com.megabras.modbus.MD15KVR.ParamSVT;
import com.megabras.modbus.MD15KVR.ParamTimer;
import com.megabras.modbus.MD15KVR.Protocol;
import com.megabras.modbus.MD15KVR.TestData;
import com.megabras.modbus.Modbus;

import java.io.File;
import java.lang.ref.WeakReference;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;

@SuppressWarnings("FieldCanBeLocal, unused")
@SuppressLint("SetTextI18n")
public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Observer, Picture.PictureInterface
{
    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    protected String
            TAG = "md15kvr.Controller";

    public static String
            model,
            serial;

    @SuppressWarnings("WeakerAccess")
    public static int
            firmware,
            HARDWARE_EQUIPMENT = -1,
            FIRMWARE_EQUIPMENT = -1,
            FIRMWARE_DD        = 1,
            FIRMWARE_IDD       = 11;

    protected Helper
            helper;

    private FragmentActivity
            activity;

    private Menu
            menu;

    private MenuItem
            menuOptions;

    private MainScreen
            mainScreen;

    private Charts
            chartResistance,
            chartVoltage,
            chartCurrent,
            chartIDD,
            table;

    private ViewPager
            viewPager;

    private SlidingTabLayout
            tabLayout;

    private View
            pageTestSetup,
            pageGeneral;

    @SuppressWarnings("WeakerAccess")
    public int
            controlButtonsID = R.layout.md15kvr_control_buttons,
            //overlayID,
            optionsID,
            buzzerParam      = 200,
            timeFormat       = 0,
            selectedIndex    = -1,
            OVERRANGE        = 15;

    private LinearLayout
            toolBar;

    private com.megabras.modbus.MD15KVR.Protocol
            modbus = new com.megabras.modbus.MD15KVR.Protocol();

    private Device
            device = new Device();

    private Button
            buttonStart,
            buttonStop,
            buttonVoltage;

    private ImageButton
            buttonPrinter,
            buttonFilter,
            buttonSetup,
            buttonSave,
            buttonManage,
            buttonGPS,
            buttonPicture,
            buttonVoice,
            buttonMap;

    private boolean
            initialConfig  = true,
            resetAfter     = false,
            WAITING_CAP    = false,
            WAITING_DD     = false,
            WAITING_IDD    = false,
            showDialogEnd  = false;

    private CommonUI
            mCommonUI;

    private TestData
            testData;

    protected Database
            database;

    private ReportPDF
            reportPDF;

    private Picture
            picture;

    private GPS
            mGPS;

    private com.megabras.bluelogg.extended.Audio
            audio;

    private GoogleMaps
            map;

    protected mHandler
            handler;

    private Dialog
            dlgProgress,
            dialogFinished;

    private com.megabras.bluelogg.Main
            parent;

    private BooleanObservable
            isTestRunning    = new BooleanObservable(1, true),
            isConnected      = new BooleanObservable(2, true),
            isPrinterEnabled = new BooleanObservable(3, true),
            isTestStarting   = new BooleanObservable(4, false),
            isAutoPilot      = new BooleanObservable(5, false),
            isFilterEnabled  = new BooleanObservable(6, true);

    private IntegerObservable
            messageID        = new IntegerObservable(1, 0);

    protected com.megabras.modbus.MD15KVR.FormatData
            formatData = new com.megabras.modbus.MD15KVR.FormatData();

    private PagerAdapter
            pagerAdapter;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    public Controller(){}

    @SuppressLint("CommitPrefEdits")
    public Controller(FragmentActivity fragmentActivity, CommonUI ui, String model, String serial)
    {
        com.megabras.modbus.MD15KVR.Protocol.setRevisionMap(4);//TEMP
        firmware          = -1;
        helper            = new Helper(fragmentActivity);
        activity          = fragmentActivity;
        parent            = (com.megabras.bluelogg.Main)activity;
        pageGeneral       = View.inflate(activity, R.layout.main_fgr_general_setup, null);
        pageTestSetup     = View.inflate(activity, R.layout.md15kvr_fgr_test_setup, null);
        Controller.model  = model;
        Controller.serial = serial.trim();
        handler           = new mHandler(this);
        mCommonUI         = ui;
        picture           = Picture.getInstance();
        picture.initialize(fragmentActivity, this);
        mGPS              = new GPS(this, fragmentActivity);
        audio             = new Audio(fragmentActivity);
        map               = new GoogleMaps(fragmentActivity);
        isTestRunning.addObserver(this);
        isConnected.addObserver(this);
        isPrinterEnabled.addObserver(this);
        isFilterEnabled.addObserver(this);
        isTestStarting.addObserver(this);
        isAutoPilot.addObserver(this);
        messageID.addObserver(this);
        device.addVoltageObserver(this);
        database  = new Database(activity, model, Database.Entry.DB_NAME);
        reportPDF = new ReportPDF(activity, database.getFolder(), database.getTemporaryFolder(), handler);

        LogCat.LEVEL = LogCat.LEVEL_NONE;
    }

    /***********************************************************************************************
     * REGION On Destroy
     ***********************************************************************************************/

    public void onClose(){
        if (mGPS != null) {
            mGPS.cancel();
        }
        if (picture != null) {
            picture.close();
        }
//        if (shutter != null){
//            shutter.release();
//        }
    }

    /***********************************************************************************************
     * REGION Setters
     ***********************************************************************************************/

    public void setDateTimeFormat(int dateFormat, int timeFormat)
    {
        device.setDateFormat(dateFormat);
        device.setTimeFormat(timeFormat);
    }

    public void setToolBarId(int toolBarId){
        toolBar = activity.findViewById(toolBarId);
        toolBar.findViewById(R.id.bt_export).setVisibility(View.GONE);
        toolBar.findViewById(R.id.bt_details).setVisibility(View.GONE);
        buttonMap     = toolBar.findViewById(R.id.btn_map);
        buttonSave    = toolBar.findViewById(R.id.bt_save);
        buttonManage  = toolBar.findViewById(R.id.bt_manager);
        buttonGPS     = toolBar.findViewById(R.id.bt_gps);
        buttonPicture = toolBar.findViewById(R.id.bt_picture);
        buttonVoice   = toolBar.findViewById(R.id.bt_record);

        buttonSave.setOnClickListener(v -> {
            if (testData!=null)
            {
                database.save(testData);
                enableToolBar(true);
                mCommonUI.enableButton(buttonSave, false);
            }
        });

        buttonManage.setOnClickListener(v -> pageFilesManager());

        buttonPicture.findViewById(R.id.bt_picture).setOnClickListener(v -> {
            if (Main.permissionToPictureAccepted)
                takePicture();
            else
                ((Main)activity).requestPermission(Main.REQUEST_PICTURE_PERMISSION);
        });

        buttonGPS.setOnClickListener(v -> {
            if (Main.permissionToGPSAccepted)
                getGPS();
            else
                ((Main)activity).requestPermission(Main.REQUEST_GPS_PERMISSION);
        });

        buttonVoice.setOnClickListener(v -> {
            if (Main.permissionToRecordAccepted)
                recordAudio();
            else
                ((Main)activity).requestPermission(Main.REQUEST_RECORD_AUDIO_PERMISSION);
        });

        buttonMap.setOnClickListener(v -> {
            if(Main.permissionToMemoryAccepted)
            {
                map();
            }
            else
            {
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_MAP_PERMISSION);
            }
        });

        enableToolBar(false);
        mCommonUI.enableButton(buttonManage, true);
    }

    private void map(){
        if ( testData==null || testData.getGpsAcuraccy()==0  )
            return;
        MapData mapData = new MapData(
                testData.getTestNumber()+"",
                testData.getGpsLongitude(),
                testData.getGpsLatitude(),
                testData.getGpsAltitude()+"",
                testData.getGpsAcuraccy()+""
        );
        map.showMap(database.getMapSnapshotFile(testData.getSerial(),testData.getDate().getTime()+""), mapData);
    }

    public void setAutoConn(boolean enable){
        device.setAutoConnection(enable);
    }

    public boolean getAutoConn(){
        return device.getAutoConnection();
    }

    public void setTemperatureUnity(int unity){}

    /***********************************************************************************************
     * REGION Pages
     ***********************************************************************************************/

    private int pageTestSetupMODE;
    private void pageTestSetup()
    {
        final FrameLayout
                overlay = MUtils.getOverlay( activity, OVERLAY_ID, pageTestSetup );

        pageTestSetupMODE = device.getOperationMode();

        final String[] modes = new String[]
        {
            activity.getString(R.string.Manual),
            activity.getString(R.string.Temporized),
            activity.getString(R.string.SVT),
            activity.getString(R.string.Ramp)
        };

        final Spinner
                spFilter = helper.getSpinner(R.id.spFilter, R.array.enable_disable, pageTestSetup),
                spMode   = helper.getSpinner(R.id.spMode  , modes                 , pageTestSetup);

        final int filterSelection = ( device.getStatus(Device.ST_FILTER) ) ? 0 : 1;
        spFilter.setSelection( filterSelection );

        final TextView tvMaxVoltage    = pageTestSetup.findViewById(R.id.tvMaxVoltage);
        final TextView tvManualVoltage = pageTestSetup.findViewById(R.id.tvManualVoltage);

        // MAX VOLTAGE
        helper.changeMaxVoltageSelector(device.getOperationMode(), tvManualVoltage, tvMaxVoltage, device);
        ImageButton buttonSetMaxVoltage = pageTestSetup.findViewById(R.id.bt_set_max_voltage);
        buttonSetMaxVoltage.setOnClickListener(v -> helper.pagePickNumberMaxVoltage( pageTestSetupMODE, tvManualVoltage, tvMaxVoltage ));

        // MANUAL / TIMER VOLTAGE
        final LinearLayout lineVoltage = pageTestSetup.findViewById(R.id.lineManual);
        if (device.getOperationMode()==Protocol.MODE_MANUAL || device.getOperationMode()==Protocol.MODE_TIMER)
            lineVoltage.setVisibility(View.VISIBLE);
        else
            lineVoltage.setVisibility(View.GONE);
        tvManualVoltage.setText( device.getTestVoltage()+" V" );
        ImageButton buttonSetVoltage = pageTestSetup.findViewById(R.id.bt_set_voltage);
        buttonSetVoltage.setOnClickListener(v -> helper.pagePickNumber( tvMaxVoltage, tvManualVoltage ));

        // TIMER
        final LinearLayout lineTimer = pageTestSetup.findViewById(R.id.lineTimer);
        final TextView tvTimer = pageTestSetup.findViewById(R.id.tv_timer);
        helper.changeTimerSelector(device.getOperationMode(), tvTimer, lineTimer, device);
        ImageButton buttonSetTimer = pageTestSetup.findViewById(R.id.ibt_timer);
        buttonSetTimer.setOnClickListener(v -> helper.pagePickNumberTimer( tvTimer ));

        //CAPACITANCE / DIELECTRIC DISCHARGE
        final LinearLayout lineCapacitance = pageTestSetup.findViewById(R.id.lineCapacitance);
        final LinearLayout lineDielectric  = pageTestSetup.findViewById(R.id.lineDielectric);
        final Spinner      spCapacitance   = helper.getSpinner(R.id.spCapacitance, R.array.enable_disable, pageTestSetup);
        final Spinner      spDielectric    = helper.getSpinner(R.id.spDielectric , R.array.enable_disable, pageTestSetup);

        spCapacitance.setSelection(device.getCapacitanceEnabled() ? 0 : 1);
        spDielectric.setSelection(device.getDielectricDischargeEnabled() ? 0 : 1);

        spCapacitance.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if (position==0)
                    lineDielectric.setVisibility(View.VISIBLE);
                else
                    lineDielectric.setVisibility(View.GONE);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        if (firmware<FIRMWARE_DD)
        {
            lineCapacitance.setVisibility(View.GONE);
            lineDielectric.setVisibility(View.GONE);
        }

        //PASS-FAIL
        final Spinner passFail = helper.getSpinner(R.id.spPassFail, R.array.enable_disable, pageTestSetup);
        passFail.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if (position==0)
                    pageTestSetup.findViewById(R.id.lineMaxResistance).setVisibility(View.VISIBLE);
                else
                    pageTestSetup.findViewById(R.id.lineMaxResistance).setVisibility(View.GONE);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        // MAX RESISTANCE
        final TextView tvLimit = pageTestSetup.findViewById(R.id.tvLimit);
        helper.changeMaxResistanceSelector(device.getOperationMode(), tvLimit, passFail, pageTestSetup.findViewById(R.id.lineMaxResistance), device);
        ImageButton buttonSetLimite = pageTestSetup.findViewById(R.id.bt_set_limite);
        buttonSetLimite.setOnClickListener(v -> helper.pagePickNumberUnity( tvLimit ));

        // PI
        final TextView tvPiTimeA = pageTestSetup.findViewById(R.id.tvPiTimeA);
        final TextView tvPiTimeB = pageTestSetup.findViewById(R.id.tvPiTimeB);
        helper.changePISelector(device.getOperationMode(), tvPiTimeA, tvPiTimeB, pageTestSetup.findViewById(R.id.lineIndexes), device);
        ImageButton buttonSetPiA = pageTestSetup.findViewById(R.id.bt_set_pi_a);
        buttonSetPiA.setOnClickListener(v -> helper.pagePickNumberPiA( tvPiTimeA ));
        ImageButton buttonSetPiB = pageTestSetup.findViewById(R.id.bt_set_pi_b);
        buttonSetPiB.setOnClickListener(v -> helper.pagePickNumberPiB( tvPiTimeB ));

        // DAI
        final TextView tvDaiTimeA = pageTestSetup.findViewById(R.id.tvDaiTimeA);
        final TextView tvDaiTimeB = pageTestSetup.findViewById(R.id.tvDaiTimeB);
        helper.changeDAISelector(device.getOperationMode(), tvDaiTimeA, tvDaiTimeB, pageTestSetup.findViewById(R.id.lineIndexes), device);
        ImageButton buttonSetDaiA = pageTestSetup.findViewById(R.id.bt_set_dai_a);
        buttonSetDaiA.setOnClickListener(v -> helper.pagePickNumberDaiA( tvDaiTimeA ));
        ImageButton buttonSetDaiB = pageTestSetup.findViewById(R.id.bt_set_dai_b);
        buttonSetDaiB.setOnClickListener(v -> helper.pagePickNumberDaiB( tvDaiTimeB ));

        //SVT
        final LinearLayout lineStepVoltage   = pageTestSetup.findViewById(R.id.line_step_voltage);
        final LinearLayout lineStepTime      = pageTestSetup.findViewById(R.id.line_step_time);
        final TextView     tvStepVoltage     = pageTestSetup.findViewById(R.id.tv_step_voltage);
        final TextView     tvStepTime        = pageTestSetup.findViewById(R.id.tv_step_time);
        ImageButton        buttonStepVoltage = pageTestSetup.findViewById(R.id.ibt_step_voltage);
        buttonStepVoltage.setOnClickListener(v -> helper.pagePickNumberStepVoltage(tvStepVoltage));
        ImageButton buttonStepTime         = pageTestSetup.findViewById(R.id.ibt_step_time);
        buttonStepTime.setOnClickListener(v -> helper.pagePickNumberStepTime(tvStepTime));

        if ( device.getOperationMode()==Protocol.MODE_SVT )
        {
            lineStepVoltage.setVisibility(View.VISIBLE);
            lineStepTime.setVisibility(View.VISIBLE);
        }
        else
        {
            lineStepVoltage.setVisibility(View.GONE);
            lineStepTime.setVisibility(View.GONE);
        }
        tvStepVoltage.setText( device.getParamSVT().getSTEP_VOLTAGE() + " V" );
        tvStepTime.setText( device.getParamSVT().getSTEP_TIME() + " s" );

        // MODE SELECTION
        spMode.setSelection( device.getOperationMode() );
        spMode.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                helper.changeMaxVoltageSelector(position, tvManualVoltage, tvMaxVoltage, device);
                helper.changeMaxResistanceSelector(position, tvLimit, passFail, pageTestSetup.findViewById(R.id.lineMaxResistance), device);
                helper.changePISelector(position, tvPiTimeA, tvPiTimeB, pageTestSetup.findViewById(R.id.lineIndexes), device);
                helper.changeDAISelector(position, tvDaiTimeA, tvDaiTimeB, pageTestSetup.findViewById(R.id.lineIndexes), device);
                helper.changeTimerSelector(position, tvTimer, lineTimer, device);

                if (position==Protocol.MODE_MANUAL || position==Protocol.MODE_TIMER)
                    lineVoltage.setVisibility(View.VISIBLE);
                else
                    lineVoltage.setVisibility(View.GONE);

                if ( position==Protocol.MODE_SVT )
                {
                    lineStepVoltage.setVisibility(View.VISIBLE);
                    lineStepTime.setVisibility(View.VISIBLE);
                }
                else
                {
                    lineStepVoltage.setVisibility(View.GONE);
                    lineStepTime.setVisibility(View.GONE);
                }

                pageTestSetupMODE = position;
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        // ***** SAVE ***** //
        Button buttonSave = pageTestSetup.findViewById(R.id.buttonSave);
        buttonSave.setOnClickListener(v -> {
            // SAVE FILTER
            if ( spFilter.getSelectedItemPosition()!=filterSelection )
                sendMessage( modbus.write.filterOnOff() );

            // SAVE MODE
            if ( spMode.getSelectedItemPosition()!=device.getOperationMode() )
            {
                device.setOperationMode( spMode.getSelectedItemPosition() );
                sendMessage( modbus.write.mode( spMode.getSelectedItemPosition() ) );
            }

            // CAPACITANCE / DD
            boolean isCap = spCapacitance.getSelectedItemPosition()==0;
            boolean isDD  = spDielectric.getSelectedItemPosition()==0;
            if (isCap!=device.getCapacitanceEnabled() || isDD!=device.getDielectricDischargeEnabled()) {
                device.setCapacitanceEnabled(isCap);
                if(isCap)
                    device.setDielectricDischargeEnabled(isDD);
                else
                    device.setDielectricDischargeEnabled(false);
                sendMessage( modbus.write.setup(device) );
            }

            // SAVE MANUAL / TIMER VOLTAGE
            String voltage = tvManualVoltage.getText().toString().replace("V","").trim();
            device.setTestVoltage( Integer.parseInt(voltage) );
            sendMessage( modbus.write.voltage(device.getTestVoltage()) );

            // SAVE MAX VOLTAGE
            String maxVoltage = tvMaxVoltage.getText().toString().replace("V","").trim();
            switch (pageTestSetupMODE)
            {
                case Protocol.MODE_MANUAL:
                    device.getParamManual().setMAX_VOLTAGE(Integer.parseInt(maxVoltage));
                    if (passFail.getSelectedItemPosition()==0)
                        device.getParamManual().setLIMIT_RESISTANCE( formatData.parseMaxResistance(tvLimit.getText().toString()) );
                    else
                        device.getParamManual().setLIMIT_RESISTANCE( 0.0f );
                    device.getParamManual().setPI_TIME_A( Integer.parseInt(tvPiTimeA.getText().toString().replace("m","").trim()) );
                    device.getParamManual().setPI_TIME_B( Integer.parseInt(tvPiTimeB.getText().toString().replace("m","").trim()) );
                    device.getParamManual().setDAI_TIME_A( Integer.parseInt(tvDaiTimeA.getText().toString().replace("s","").trim()) );
                    device.getParamManual().setDAI_TIME_B( Integer.parseInt(tvDaiTimeB.getText().toString().replace("s","").trim()) );
                    sendMessage( modbus.write.testsParamsManual(device) );
                    break;
                case Protocol.MODE_TIMER:
                    device.getParamTimer().setMAX_VOLTAGE(Integer.parseInt(maxVoltage));
                    if (passFail.getSelectedItemPosition()==0)
                        device.getParamTimer().setLIMIT_RESISTANCE( formatData.parseMaxResistance(tvLimit.getText().toString()) );
                    else
                        device.getParamTimer().setLIMIT_RESISTANCE( 0.0f );
                    String[] parts = tvTimer.getText().toString().split(":");
                    int minutes = Integer.parseInt( parts[0] )*60;
                    int seconds = Integer.parseInt( parts[1] );
                    device.getParamTimer().setTEST_TIME(minutes+seconds);
                    device.getParamTimer().setPI_TIME_A( Integer.parseInt(tvPiTimeA.getText().toString().replace("m","").trim()) );
                    device.getParamTimer().setPI_TIME_B( Integer.parseInt(tvPiTimeB.getText().toString().replace("m","").trim()) );
                    device.getParamTimer().setDAI_TIME_A( Integer.parseInt(tvDaiTimeA.getText().toString().replace("s","").trim()) );
                    device.getParamTimer().setDAI_TIME_B( Integer.parseInt(tvDaiTimeB.getText().toString().replace("s","").trim()) );
                    sendMessage( modbus.write.testsParamsTimer(device) );
                    break;
                case Protocol.MODE_SVT:
                    device.getParamSVT().setFINAL_VOLTAGE(Integer.parseInt(maxVoltage));
                    if (passFail.getSelectedItemPosition()==0)
                        device.getParamSVT().setLIMIT_RESISTANCE( formatData.parseMaxResistance(tvLimit.getText().toString()) );
                    else
                        device.getParamSVT().setLIMIT_RESISTANCE( 0.0f );
                    device.getParamSVT().setSTEP_VOLTAGE( Integer.parseInt( tvStepVoltage.getText().toString().replace("V","").trim() ) );
                    device.getParamSVT().setSTEP_TIME( Integer.parseInt( tvStepTime.getText().toString().replace("s","").trim() ) );
                    sendMessage( modbus.write.testsParamsSVT(device) );
                    break;
                case Protocol.MODE_RAMP:
                    device.getParamRamp().setFINAL_VOLTAGE(Integer.parseInt(maxVoltage));
                    if (passFail.getSelectedItemPosition()==0)
                        device.getParamRamp().setLIMIT_RESISTANCE( formatData.parseMaxResistance(tvLimit.getText().toString()) );
                    else
                        device.getParamRamp().setLIMIT_RESISTANCE( 0.0f );
                    String[] partsRamp = tvTimer.getText().toString().split(":");
                    int minutesRamp   = Integer.parseInt( partsRamp[0] )*60;
                    int secondsRamp   = Integer.parseInt( partsRamp[1] );
                    device.getParamRamp().setTEST_TIME(minutesRamp+secondsRamp);
                    sendMessage( modbus.write.testsParamsRamp(device) );
                    break;
            }
            messageID.set(R.string.updating);
            helper.closeOverlay(overlay);
        });

        Button buttonCancel = pageTestSetup.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener(v -> {
            messageID.set(R.string.ready);
            helper.closeOverlay(overlay);
        });
    }

    private void pageGeneralSetup(){
        final FrameLayout
                overlay = MUtils.getOverlay( activity, OVERLAY_ID, pageGeneral );

        final TextView
                date   = pageGeneral.findViewById(R.id.device_date),
                time   = pageGeneral.findViewById(R.id.device_time),
                memory = pageGeneral.findViewById(R.id.device_memory);

        final Spinner
                spinnerConnect    = helper.getSpinner(R.id.spinnerAutoConnection, R.array.enable_disable,   pageGeneral),
                spinnerDateFormat = helper.getSpinner(R.id.spinnerDateFormat,     R.array.date_format,      pageGeneral),
                spinnerTimeFormat = helper.getSpinner(R.id.spinnerTimeFormat,     R.array.time_format,      pageGeneral),
                spinnerLanguage   = helper.getSpinner(R.id.spinnerLanguage,       R.array.language_options, pageGeneral);

        date.setText  ( formatData.date(device.getDateFormat(), device.getDate()) );
        time.setText  ( formatData.time(device.getTimeFormat(), device.getDate()) );
        String free = device.getFreeMemory() + " %";
        memory.setText( free );
        helper.getLL(R.id.line_gauge, pageGeneral).setVisibility(View.GONE);

        ImageButton btEraseMem = pageGeneral.findViewById(R.id.button_memory);
        btEraseMem.setOnClickListener(v -> {

            final Dialog dialog = new Dialog(activity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setContentView(R.layout.dialog_yes_no);
            if (dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.erase_memory);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.erase_memory_confirm);

            final Button btnNo = dialog.findViewById(R.id.btn_no);
            btnNo.setOnClickListener(v12 -> dialog.dismiss());

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener(v1 -> {
                sendMessage(modbus.write.memoryErase());
                messageID.set(R.string.erase_memory);
                memory.setText(activity.getString(R.string.wait));
                dialog.dismiss();
            });

            dialog.show();
        });

        if(device.getAutoConnection()){
            spinnerConnect.setSelection(0);
        } else {
            spinnerConnect.setSelection(1);
        }

        switch (device.getDateFormat()){
            case Modbus.DATE_DD_MM_YY:
                spinnerDateFormat.setSelection(0);
                break;
            case Modbus.DATE_MM_DD_YY:
                spinnerDateFormat.setSelection(1);
                break;
            default:
                spinnerDateFormat.setSelection(2);
                break;
        }

        switch (device.getTimeFormat()){
            case Modbus.TIME_24H:
                spinnerTimeFormat.setSelection(0);
                break;
            default:
                spinnerTimeFormat.setSelection(1);
                break;
        }

        switch (device.getLanguage()){
            case Modbus.LANG_ENGLISH:
                spinnerLanguage.setSelection(0);
                break;
            case Modbus.LANG_SPANISH:
                spinnerLanguage.setSelection(1);
                break;
            default:
                spinnerLanguage.setSelection(2);
                break;
        }

        spinnerDateFormat.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                switch (position){
                    case 0:
                        date.setText( formatData.date(Modbus.DATE_DD_MM_YY, device.getDate()) );
                        break;
                    case 1:
                        date.setText( formatData.date(Modbus.DATE_MM_DD_YY, device.getDate()) );
                        break;
                    default:
                        date.setText( formatData.date(Modbus.DATE_YYYY_MM_DD, device.getDate()) );
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        spinnerTimeFormat.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

                switch (position){
                    case 0:
                        time.setText( formatData.time(Modbus.TIME_24H, device.getDate()) );
                        timeFormat = Modbus.TIME_24H;
                        break;
                    default:
                        time.setText( formatData.time(Modbus.TIME_12H, device.getDate()) );
                        timeFormat = Modbus.TIME_12H;
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        ImageButton buttonDate = pageGeneral.findViewById(R.id.button_date);
        buttonDate.setOnClickListener(v -> pickDate());

        ImageButton buttonTime = pageGeneral.findViewById(R.id.button_time);
        buttonTime.setOnClickListener(v -> pickTime(timeFormat));

        Button buttonSave = pageGeneral.findViewById(R.id.buttonSave);
        buttonSave.setOnClickListener(v -> {
            switch (spinnerConnect.getSelectedItemPosition()) {
                case 0:
                    device.setAutoConnection(true);
                    break;
                case 1:
                    device.setAutoConnection(false);
                    break;
            }

            int dateFormat, timeFormat, language;
            // DATE
            switch (spinnerDateFormat.getSelectedItemPosition()) {
                case 0:
                    dateFormat = Modbus.DATE_DD_MM_YY;
                    break;
                case 1:
                    dateFormat = Modbus.DATE_MM_DD_YY;
                    break;
                default:
                    dateFormat = Modbus.DATE_YYYY_MM_DD;
                    break;
            }

            switch (spinnerTimeFormat.getSelectedItemPosition()) {
                case 0:
                    timeFormat = Modbus.TIME_24H;
                    break;
                default:
                    timeFormat = Modbus.TIME_12H;
                    break;
            }

            switch (spinnerLanguage.getSelectedItemPosition()) {
                case 0:
                    language = Modbus.LANG_ENGLISH;
                    break;
                case 1:
                    language = Modbus.LANG_SPANISH;
                    break;
                default:
                    language = Modbus.LANG_PORTUGUESE;
                    break;
            }

            device.setDateFormat(dateFormat);
            device.setTimeFormat(timeFormat);
            device.setLanguage(language);

            messageID.set(R.string.updating);
            sendMessage( modbus.write.date( device.getDate() ) );
            sendMessage( modbus.write.setup(device) );
            ((Main)activity).updateAutoConn(device.getAutoConnection());
            ((Main)activity).updateDateTimeFormat(dateFormat, timeFormat);
            helper.closeOverlay(overlay);
        });

        Button buttonCancel = pageGeneral.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener(v -> {
            messageID.set(R.string.ready);
            menuOptions.setEnabled(true);
            helper.closeOverlay(overlay);
        });
    }

    private void pageFilesManager(){
        final View view           = View.inflate(activity, R.layout.mpk257_files_manager, null);
        final FrameLayout overlay = MUtils.getOverlay(activity, OVERLAY_ID, view);
        final ListView list       = view.findViewById(R.id.list_savedfiles);

        DecimalFormat number = new DecimalFormat("00000");

        final Button
                btDelete = view.findViewById(R.id.bt_delete),
                btOpen   = view.findViewById(R.id.bt_open),
                btNew    = view.findViewById(R.id.bt_new),
                btRename = view.findViewById(R.id.bt_rename),
                btClose  = view.findViewById(R.id.bt_close),
                btExport = view.findViewById(R.id.bt_export);

        btNew.setVisibility(View.GONE);
        btRename.setVisibility(View.GONE);

        final List<FileDetails> fileList = database.getSavedFilesList(serial, device.getDateFormat(), device.getTimeFormat());
        int selectedListIndex = -1;

        final String[] nameList;
        if ( fileList!=null && fileList.size()>0 )
        {
            nameList = new String[fileList.size()];
            for(int i=0; i<fileList.size(); i++)
            {
                nameList[i] = activity.getString(R.string.test_number) + " " + number.format( Integer.parseInt(fileList.get(i).INDEX) ) + " - " + fileList.get(i).DATE;

                // Already opened
                if ( testData!=null && testData.getSerial().equals(fileList.get(i).SERIAL) && (testData.getTestNumber()+"").equals(fileList.get(i).INDEX) )
                {
                    selectedListIndex = i;
                    nameList[i] += "  \u2705"; // ✅⚐ ✓✅◄ ☚ \u2611 \u261C \uD83D\uDCC2";
                    btOpen.setEnabled( false );
                }
            }

            final ArrayAdapter<String> adapter = new ArrayAdapter<>(activity,
                    android.R.layout.simple_list_item_activated_1, android.R.id.text1, nameList);

            list.setAdapter    (adapter);
            list.setChoiceMode (ListView.CHOICE_MODE_SINGLE);
            list.setItemChecked(selectedListIndex, true);
            list.setSelection  (selectedListIndex);

            list.setOnItemClickListener((parent, view1, position, id) -> {
                btDelete.setEnabled(true);
                btExport.setEnabled(true);
                btOpen.setEnabled(true);
                btClose.setEnabled(false);
                if ( fileList.size()>0 )
                {
                    if ( testData!=null )
                    {
                        // Already opened
                        if ( (testData.getTestNumber()+"").equals( fileList.get(position).INDEX ) &&
                                formatData.getDateFormatted(testData.getDate(), device.getDateFormat(), device.getTimeFormat())
                                        .equals(fileList.get(position).DATE) )
                        {
                            btOpen.setEnabled(false);
                            btClose.setEnabled(true);
                        }
                    }
                }
            });
        }

        if((fileList!=null && fileList.size()<=0) || list.getCheckedItemPosition()<0)
        {
            btDelete.setEnabled(false);
            btOpen.setEnabled(false);
            btClose.setEnabled(false);
            btExport.setEnabled(false);
        }

        btDelete.setOnClickListener(v ->
        {
            final Dialog dialog = new Dialog(activity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setContentView(R.layout.dialog_yes_no);
            if(dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.erase);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.file_delete_all_confirm);

            dialog.findViewById(R.id.btn_no).setOnClickListener(v12 -> dialog.dismiss());

            dialog.findViewById(R.id.btn_yes).setOnClickListener(v1 -> {
                int selectedIndex = list.getCheckedItemPosition();
                if (selectedIndex > -1) {
                    if (fileList == null)
                        return;

                    database.deleteTable(fileList.get(selectedIndex).FILENAME);
                    if (testData != null && testData.getSerial().equals(fileList.get(selectedIndex).SERIAL) && (testData.getTestNumber() + "").equals(fileList.get(selectedIndex).INDEX)) {
                        testData = null;
                        enableToolBarWork(false);
                        pagerAdapter.resetAll(isConnected.get(), device);
                        if (isConnected.get())
                            mainScreen.setTestParameters(device);
                    }
                }
                dialog.dismiss();
                helper.closeOverlay(overlay);
                pageFilesManager();
            });

            dialog.show();
        });

        btOpen.setOnClickListener(v ->
        {
            int selectedIndex = list.getCheckedItemPosition();
            if ( selectedIndex > -1 )
            {
                testData=null;

                pagerAdapter.resetAll(isConnected.get(), device);

                if ( fileList==null )
                    return;

                testData = database.getResult( fileList.get(selectedIndex).FILENAME );

                if ( testData==null || testData.getDevice()==null )
                    return;

                testData.getDevice().setDateFormat(device.getDateFormat());
                testData.getDevice().setTimeFormat(device.getTimeFormat());
                pagerAdapter.showChartIDD(testData.getFirmware()>= FIRMWARE_IDD);
                viewPager.setAdapter(pagerAdapter);
                tabLayout.setViewPager(viewPager);
                pagerAdapter.resetAll(isConnected.get(), device);
                pagerAdapter.setAllValues(testData);

                enableToolBarWork(true);
                mCommonUI.enableButton(buttonSave, false);
            }
            helper.closeOverlay(overlay);
            viewPager.setCurrentItem(pagerAdapter.getCount(), true);
        });

        btClose.setOnClickListener(v ->
        {
            list.clearChoices();
            testData=null;
            pagerAdapter.resetAll(isConnected.get(), device);
            enableToolBarWork(false);
            btClose.setEnabled(false);
            btOpen.setEnabled(false);
            btDelete.setEnabled(false);

            helper.closeOverlay(overlay);
            pageFilesManager();
        });

        btExport.setOnClickListener(v ->
        {
          selectedIndex = list.getCheckedItemPosition();
          if(Main.permissionToMemoryAccepted)
              export(selectedIndex);
          else
              ((Main)activity).requestPermission(Main.REQUEST_MEMORY_REPORT_PERMISSION);
        });

        view.findViewById(R.id.bt_cancel).setOnClickListener(v -> helper.closeOverlay(overlay));
    }

    private void export(int index)
    {

        List<FileDetails> fileList = database.getSavedFilesList(serial, device.getDateFormat(), device.getTimeFormat());
        if (index > -1)
        {
            if (fileList == null)
                return;

            final TestData exportData = database.getResult(fileList.get(index).FILENAME);
            if (exportData == null)
                return;

            if (dlgProgress == null)
                creatDialogProgress();
            dlgProgress.show();

            exportData.getDevice().setDateFormat(device.getDateFormat());
            exportData.getDevice().setTimeFormat(device.getTimeFormat());
            chartResistance.reset();
            chartVoltage.reset();
            chartCurrent.reset();
            chartIDD.reset();
            chartResistance.setAllValues(exportData.getMeasurement());
            chartVoltage.setAllValues(exportData.getMeasurement());
            chartCurrent.setAllValues(exportData.getMeasurement());
            chartIDD.setAllIDD(exportData.getDielectricDischarge());

            new Thread(() ->
            {
                File[] chartImageArray;

                if (exportData.getDielectricDischarge().size()>0)
                {
                    chartImageArray    = new File[4];
                    chartImageArray[3] = helper.saveChartImage(chartIDD, database.getTemporaryFolder());
                }
                else
                {
                    chartImageArray    = new File[3];
                }
                chartImageArray[0] = helper.saveChartImage(chartResistance, database.getTemporaryFolder());
                chartImageArray[1] = helper.saveChartImage(chartVoltage   , database.getTemporaryFolder());
                chartImageArray[2] = helper.saveChartImage(chartCurrent   , database.getTemporaryFolder());

                reportPDF.generate(exportData,
                        database.getPictureFile(exportData.getSerial(), exportData.getDate().getTime() + ""),
                        database.getMapSnapshotFile(exportData.getSerial(), exportData.getDate().getTime() + ""),
                        chartImageArray
                );

//                report.saveDocReport(exportData,
//                        database.getPictureFile(exportData.getSerial(), exportData.getDate().getTime() + ""),
//                        database.getMapSnapshotFile(exportData.getSerial(), exportData.getDate().getTime() + ""),
//                        chartImageArray
//                );
            }).start();

            closeOverlay(activity);
        }
    }

    /***********************************************************************************************
     * REGION Report Handler
     ***********************************************************************************************/

    @SuppressWarnings("WeakerAccess")
    public static class mHandler extends Handler
    {
        private final WeakReference<Controller>
                mClass;

        public static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler(Controller c)
        {
            mClass = new WeakReference<>(c);
        }

        @Override
        public void handleMessage(Message msg)
        {
            final Controller c = mClass.get();

            switch(msg.what)
            {
                case mHandler.REPORT_UPDATE:
                    if (c.dlgProgress != null && msg.obj instanceof String)
                        ((TextView)c.dlgProgress.findViewById(R.id.status)).setText((String) msg.obj);
                    break;
                case mHandler.REPORT_FINISHED:
                    c.dlgProgress.dismiss();
                    c.chartResistance.reset();
                    c.chartVoltage.reset();
                    c.chartCurrent.reset();
                    c.chartIDD.reset();
                    if ( c.testData!=null )
                    {
                        c.chartResistance.setAllValues(c.testData.getMeasurement());
                        c.chartVoltage.setAllValues(c.testData.getMeasurement());
                        c.chartCurrent.setAllValues(c.testData.getMeasurement());
                        if (c.testData.getDielectricDischarge()!=null && c.testData.getDielectricDischarge().size()>0)
                            c.chartIDD.setAllIDD(c.testData.getDielectricDischarge());
                    }
                    break;
                case mHandler.REPORT_ERROR:
                    c.dlgProgress.dismiss();
                    c.chartResistance.reset();
                    c.chartVoltage.reset();
                    c.chartCurrent.reset();
                    c.chartIDD.reset();
                    if ( c.testData!=null )
                    {
                        c.chartResistance.setAllValues(c.testData.getMeasurement());
                        c.chartVoltage.setAllValues(c.testData.getMeasurement());
                        c.chartCurrent.setAllValues(c.testData.getMeasurement());
                        if (c.testData.getDielectricDischarge()!=null && c.testData.getDielectricDischarge().size()>0)
                            c.chartIDD.setAllIDD(c.testData.getDielectricDischarge());
                    }
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION Methods
     ***********************************************************************************************/

    public void createControlButtons(LinearLayout container){
        final View commandButtons = View.inflate(activity, controlButtonsID, null);

        commandButtons.setLayoutParams(
                new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        (int)activity.getResources().getDimension(R.dimen.md15kvr_controls_height)));
        container.removeAllViews();
        container.addView(commandButtons);

        buttonStart = commandButtons.findViewById(R.id.bt_start);
        new CountDown(3000, 100, buttonStart, new HandlerCountDown(this));

        buttonStop = commandButtons.findViewById(R.id.bt_stop);
        buttonStop.setOnClickListener(v -> {
            if ( !isTestRunning.get() )
                messageID.set(R.string.test_cancelled);
            sendPriorityMessage( modbus.write.stop() );
            mCommonUI.enableButtonBg(buttonStop, false);
        });

        buttonPrinter = commandButtons.findViewById(R.id.bt_printer);
        buttonPrinter.setOnClickListener(v -> {
            //buttonPrinter.setImageDrawable(activity.getResources().getDrawable(R.drawable.ic_wait));
            buttonPrinter.setImageDrawable( ResourcesCompat.getDrawable(activity.getResources(), R.drawable.ic_wait, null) );
            sendMessage( modbus.write.print() );
            sendMessage( modbus.read.status(Protocol.ST_BATTERY) );
        });

        buttonFilter = commandButtons.findViewById(R.id.bt_filter);
        buttonFilter.setOnClickListener(v -> {
            //buttonFilter.setImageDrawable(activity.getResources().getDrawable(R.drawable.ic_wait));
            buttonFilter.setImageDrawable( ResourcesCompat.getDrawable(activity.getResources(), R.drawable.ic_wait, null) );
            sendMessage( modbus.write.filterOnOff() );
            sendMessage( modbus.read.status(Protocol.ST_BATTERY) );
        });

        final Controller controller = this;
        buttonVoltage = commandButtons.findViewById(R.id.bt_voltage);
        buttonVoltage.setOnClickListener(v -> helper.pagePickNumberManualVoltage(device, controller, modbus));

        buttonSetup = commandButtons.findViewById(R.id.bt_setup);
        buttonSetup.setOnClickListener(v -> pageTestSetup());

        isTestRunning.set(false);
        isConnected.set(false);
        isPrinterEnabled.set(false);
        isFilterEnabled.set(false);
    }

    public void createSwipe()
    {
        pagerAdapter =
                new PagerAdapter(
                        activity.getSupportFragmentManager(),
                        activity.getBaseContext(),
                        OVERRANGE);

        viewPager = activity.findViewById(R.id.pager);

        if (viewPager == null)
            return;

        viewPager.setAdapter(pagerAdapter);
        viewPager.setOffscreenPageLimit(pagerAdapter.getCount());

        mainScreen      = pagerAdapter.getMainFragment();
        chartResistance = pagerAdapter.chartResistance();
        chartVoltage    = pagerAdapter.chartVoltage();
        chartCurrent    = pagerAdapter.chartCurrent();
        chartIDD        = pagerAdapter.chartIDD();
        table           = pagerAdapter.table();

        //Create customs Tabs
        tabLayout = activity.findViewById(R.id.sliding_tabs);
        tabLayout.setDistributeEvenly(true);
        tabLayout.setCustomTabView(R.layout.custom_tab, 0);
        tabLayout.setBackground(
                //activity.getResources().getDrawable(R.drawable.chamfer_black_top)
                ResourcesCompat.getDrawable(activity.getResources(), R.drawable.chamfer_black_top, null)
        );
        tabLayout.setCustomTabColorizer(new SlidingTabLayout.TabColorizer() {
            public int getIndicatorColor(int position) {
                return Color.GRAY;
            }

            public int getDividerColor(int position) {
                return Color.WHITE;
            }
        });
        tabLayout.setViewPager(viewPager);
    }

    public void sendMessage(com.megabras.modbus.Message message){
        ( (com.megabras.bluelogg.Main)activity ).addMessage(message);
    }

    private void sendPriorityMessage(com.megabras.modbus.Message message){
        ( (com.megabras.bluelogg.Main)activity ).addPriorityMessage(message);
    }

    @SuppressWarnings("SameParameterValue")
    private void userMessage(String message, int color, boolean busy){
        ( (com.megabras.bluelogg.Main)activity )
                .message(message, color, busy);
    }

    private void userMessage(int stringID, int color, boolean busy){
        String message = activity.getString(stringID);
        ( (com.megabras.bluelogg.Main)activity )
                .message(message, color, busy);
    }

    /***********************************************************************************************
     * REGION Permission
     ***********************************************************************************************/

    public void permissionCallback(int permission)
    {
        switch (permission)
        {
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
            case Main.REQUEST_MEMORY_REPORT_PERMISSION:
                export(selectedIndex);
                break;
            case Main.REQUEST_MEMORY_MAP_PERMISSION:
                map();
                break;
        }
    }

    /***********************************************************************************************
     * REGION AUDIO
     ***********************************************************************************************/

    private void recordAudio()
    {
        if ( testData!=null )
        {
            File location  = database.getAudioFile(testData.getSerial(), testData.getDate().getTime()+"");
            audio.setFile( location );
            audio.showDialog(testData.getTestNumber()+"");
        }
    }

    /***********************************************************************************************
     * REGION Picture
     ***********************************************************************************************/

    public void takePicture() {
        if(database!=null && testData!=null)
        {
            File file = database.getPictureFile( testData.getSerial(), testData.getDate().getTime()+"" );
            picture.showDialog(file);
        }
    }

    public void pictureStatus(final int status){
        activity.runOnUiThread(() -> {
            switch (status)
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_wait, false);
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_picture_small, true);
                    break;
            }
        });
    }

    /***********************************************************************************************
     * REGION GPS
     ***********************************************************************************************/

    private void getGPS()
    {
        if ( testData==null )
            return;

        if ( testData.getGpsAcuraccy()==0 )
            mGPS.getDataGPS();
        else
        {
            String[] gps = new String[]{
                    testData.getGpsLongitude()+"",
                    testData.getGpsLatitude()+"",
                    testData.getGpsAltitude()+"",
                    testData.getGpsAcuraccy()+""
            };
            mGPS.dialogConfirmReplace( gps );
        }
    }

    public void GpsData(int which, String[] data) {
        switch (which)
        {
            case GPS.RESULT:
                if ( data!=null && testData!=null)
                {
                    testData.setGpsLongitude( Double.parseDouble(data[0]) );
                    testData.setGpsLatitude ( Double.parseDouble(data[1]) );
                    testData.setGpsAltitude ( Double.parseDouble(data[2]) );
                    testData.setGpsAcuraccy ( Double.parseDouble(data[3]) );
                    database.updateGpsData( testData );
                    //table.update(currentObject);
                }
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_small, true);
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_small, false);
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_small, true);
                break;
        }
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void creatDialogProgress(){
        dlgProgress = new Dialog(activity);
        dlgProgress.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dlgProgress.setCanceledOnTouchOutside(false);
        dlgProgress.setContentView(R.layout.progress_bar);
        ((TextView) dlgProgress.findViewById(R.id.dlg_title)).setText(R.string.please_wait);
        ((TextView) dlgProgress.findViewById(R.id.message)).setText(R.string.saving_data_in_pdf);
        dlgProgress.findViewById(R.id.dlg_icon).setVisibility(View.GONE);
    }

    public com.megabras.modbus.Message statusMessage(){
        return modbus.read.status(2);
    }

    public com.megabras.modbus.Message stopMessage(){
        return modbus.write.stop();
    }

    public void readGeneralConfig(){
        messageID.set(R.string.read_general_params);
        if(menuOptions ==null) {
            menuOptions = menu.findItem(optionsID);
        }
        menuOptions.setEnabled(false);
        sendMessage( modbus.read.date() );
        sendMessage( modbus.read.memoryPointer() );
        sendMessage( modbus.read.setup() );
    }

    public void setMenuItemOptions(Menu menu, int optionsID){
        this.menu      = menu;
        this.optionsID = optionsID;
        if( menuOptions==null) {
            menuOptions = menu.findItem(optionsID);
        }
        if ( mainScreen.getView()==null )
            return;

        helper.calculateFontSize(
                mainScreen.getView().findViewById(R.id.container_resistance),
                mainScreen.getView().findViewById(R.id.lb_resistance)
        );
        mainScreen.reset();
    }

    /***********************************************************************************************
     * REGION Handlers
     ***********************************************************************************************/

    private static class HandlerCountDown extends Handler {
        private final WeakReference<Controller> mActivity;

        HandlerCountDown(Controller activity) {
            mActivity = new WeakReference<>(activity);
        }

        int buzzer = 0;
        @Override
        public void handleMessage(Message msg) {
            final Controller at = mActivity.get();
            String timer;

            switch(msg.what){
                case CountDown.STATE_CANCEL:
                    timer = (String) msg.obj;
                    at.userMessage(at.activity.getString(R.string.md_hold_start_by) + timer,
                            CommonUI.COLOR_YELLOW, false);
                    buzzer=0;
                    break;
                case CountDown.STATE_TICK:
                    timer = (String) msg.obj;
                    at.userMessage(at.activity.getString(R.string.md_start_in) + timer,
                            CommonUI.COLOR_YELLOW, true);
                    if ( buzzer%5==0 )
                    {
                        at.sendMessage( at.modbus.write.buzzerOnOff() );
                    }
                    buzzer++;
                    break;
                case CountDown.STATE_FINISH:
                    //int result = msg.arg2;
                    at.isTestStarting.set(true);
                    at.sendMessage( at.modbus.read.testNumber() );
                    buzzer=0;
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION CALLBACKs
     ***********************************************************************************************/

    private TimePickerDialog.OnTimeSetListener timeListener = new TimePickerDialog.OnTimeSetListener(){
        @Override
        public void onTimeSet(TimePicker view, int hourOfDay, int minute) {
            if(pageGeneral!=null){
                TextView time = pageGeneral.findViewById(R.id.device_time);
                Calendar c = Calendar.getInstance();
                c.setTime(device.getDate());
                c.set(Calendar.HOUR_OF_DAY, hourOfDay);
                c.set(Calendar.MINUTE, minute);
                c.set(Calendar.SECOND, 0);
                device.setDate( c.getTime() );
                time.setText(  formatData.time( device.getTimeFormat(), device.getDate() ) );
            }
        }
    };

    private void pickTime(int format)
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int hour     = calendar.get(Calendar.HOUR_OF_DAY);
        int minute   = calendar.get(Calendar.MINUTE);
        boolean is24 = format==Modbus.TIME_24H;
        new TimePickerDialog(activity, timeListener, hour, minute, is24).show();
    }

    private DatePickerDialog.OnDateSetListener dateListener = new DatePickerDialog.OnDateSetListener()
    {
        public void onDateSet(DatePicker arg0, int year, int month, int day)
        {
            if ( pageGeneral!=null )
            {
                TextView date  = pageGeneral.findViewById(R.id.device_date);
                TextView time  = pageGeneral.findViewById(R.id.device_time);
                String parts[] = time.getText().toString().split(":");
                int hour       = Integer.parseInt(parts[0]);
                int minutes    = Integer.parseInt(parts[1]);

                Calendar c     = Calendar.getInstance();
                c.set(year, month, day, hour, minutes);
                device.setDate( c.getTime() );
                date.setText( formatData.date(device.getDateFormat(), device.getDate()) );
            }
        }
    };

    private void pickDate()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int year  = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day   = calendar.get(Calendar.DAY_OF_MONTH);
        new DatePickerDialog(activity, dateListener, year, month, day).show();
    }

    private void dialogTestEndShow(boolean wait)
    {
        if (isTestStarting.get() || isTestRunning.get() || firmware<FIRMWARE_IDD || !showDialogEnd)
            return;

        if (!wait)
            showDialogEnd = false;

        if ( dialogFinished == null )
        {
            dialogFinished = new Dialog( activity );
            dialogFinished.requestWindowFeature( Window.FEATURE_NO_TITLE );
            dialogFinished.setCanceledOnTouchOutside( false );
            dialogFinished.setCancelable( false );
            dialogFinished.setContentView( R.layout.dialog_message_white );
            if ( dialogFinished.getWindow() != null )
            {
                dialogFinished.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            }
            ((ImageView) dialogFinished.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) dialogFinished.findViewById( R.id.dlg_title )).setText( R.string.md_test_finished );
            ((TextView) dialogFinished.findViewById( R.id.observation )).setText( "" );
            dialogFinished.findViewById( R.id.ui_iv_warning ).setVisibility( View.GONE );
            dialogFinished.findViewById( R.id.btn_close_dlg ).setVisibility( View.GONE );
            dialogFinished.findViewById( R.id.btn_yes ).setOnClickListener( v -> dialogFinished.dismiss() );
        }

        if (!dialogFinished.isShowing())
            dialogFinished.show();

        if (wait)
        {
            ((Button)dialogFinished.findViewById(R.id.btn_yes)).setText(R.string.wait);
            dialogFinished.findViewById(R.id.btn_yes).setEnabled(false);
            dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.VISIBLE);
        }
        else
        {
            ((Button)dialogFinished.findViewById(R.id.btn_yes)).setText(R.string.ok);
            dialogFinished.findViewById(R.id.btn_yes).setEnabled(true);
            dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.INVISIBLE);
        }

        String result = activity.getString(R.string.md_test_finished).toUpperCase() + "\r\n";

        // PASS / FAIL #############################################################################
        double resistance = 0.0f;
        switch (device.getOperationMode())
        {
            case Protocol.MODE_MANUAL:
                resistance = device.getParamManual().getLIMIT_RESISTANCE();
                break;
            case Protocol.MODE_TIMER:
                resistance = device.getParamTimer().getLIMIT_RESISTANCE();
                break;
            case Protocol.MODE_SVT:
                resistance = device.getParamSVT().getLIMIT_RESISTANCE();
                break;
            case Protocol.MODE_RAMP:
                resistance = device.getParamRamp().getLIMIT_RESISTANCE();
                break;
        }

        // Pass fail ENABLED
        if(resistance>0)
        {
            if (mainScreen.isResultApproved())
                result = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.pass) + "\r\n";
            else
                result = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.fail) + "\r\n";
        }

        if (device.getCapacitanceEnabled())
        {
            if (device.getStatus(Device.ST_CAPACITANCE) && testData.getCapacitance()>-1)
                result += "C= " + formatData.formatCapacitance(testData.getCapacitance()) + "\r\n";
            else
                result += activity.getString(R.string.md_calculating_capacitance) + "\r\n";
        }

        if (device.getDielectricDischargeEnabled())
        {
            if (device.getStatus(Device.ST_DD) && testData.getDD()>-1)
                result += "DD= " + formatData.formatIndexes(testData.getDD());
            else
                result += activity.getString(R.string.calc_dd);
        }

        ((TextView) dialogFinished.findViewById(R.id.message)    ).setText(result);
        ((TextView) dialogFinished.findViewById(R.id.observation)).setText("");
    }

    private void dialogTestEndSetObservation(String observation)
    {
        if (dialogFinished==null || !dialogFinished.isShowing())
            return;

        ((TextView)dialogFinished.findViewById(R.id.observation)).setText(observation);

        if (dialogFinished.findViewById(R.id.ui_iv_warning).getVisibility()!=View.VISIBLE)
            dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.VISIBLE);
        else
            dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.INVISIBLE);
    }

    private void enableCommandButtons(boolean enabled)
    {
        mCommonUI.enableButtonBg(buttonStart, enabled);
        mCommonUI.enableButtonBg(buttonStop, enabled);
        mCommonUI.enableButton(buttonFilter, enabled);
        mCommonUI.enableButton(buttonPrinter, enabled);
        mCommonUI.enableButton(buttonSetup  , enabled);
        enableButtonVoltage(enabled);
    }

    private void enableButtonVoltage(boolean enabled)
    {
        if ( device.getOperationMode()==Protocol.MODE_MANUAL || device.getOperationMode()==Protocol.MODE_TIMER )
            mCommonUI.enableButtonBg(buttonVoltage, enabled);
        else
            mCommonUI.enableButtonBg(buttonVoltage, false);
    }

    private void enableToolBar(boolean enable)
    {
        if (toolBar==null)
            return;

        mCommonUI.enableButton(buttonMap    , enable);
        mCommonUI.enableButton(buttonSave   , enable);
        mCommonUI.enableButton(buttonManage , enable);
        mCommonUI.enableButton(buttonGPS    , enable);
        mCommonUI.enableButton(buttonPicture, enable);
        mCommonUI.enableButton(buttonVoice  , enable);
    }

    private void enableToolBarWork(boolean enable)
    {
        if (toolBar==null)
            return;

        mCommonUI.enableButton(buttonMap    , enable);
        mCommonUI.enableButton(buttonGPS    , enable);
        mCommonUI.enableButton(buttonPicture, enable);
        mCommonUI.enableButton(buttonVoice  , enable);
    }

    private void printerEnabled(boolean enabled)
    {
        Drawable img;

        if (enabled) {
            //img = activity.getResources().getDrawable(R.drawable.ic_printer_on);
            img = ResourcesCompat.getDrawable(activity.getResources(), R.drawable.ic_printer_on_states, null);
        } else {
            //img = activity.getResources().getDrawable(R.drawable.ic_printer_off);
            img = ResourcesCompat.getDrawable(activity.getResources(), R.drawable.ic_printer_off_states, null);
        }
        if(img!=null){
            buttonPrinter.setImageDrawable(img);
        }
    }

    private void filterEnabled(boolean enabled)
    {
        Drawable img;

        if (enabled) {
            //img = activity.getResources().getDrawable(R.drawable.ic_filter_on);
            img = ResourcesCompat.getDrawable(activity.getResources(), R.drawable.ic_filter_on_states, null);
        } else {
            //img = activity.getResources().getDrawable(R.drawable.ic_filter_off);
            img = ResourcesCompat.getDrawable(activity.getResources(), R.drawable.ic_filter_off_states, null);
        }
        if(img!=null){
            buttonFilter.setImageDrawable(img);
        }
    }

    /***********************************************************************************************
     * REGION State machine
     ***********************************************************************************************/
    @Override
    public void update(Observable observable, Object arg1)
    {
        if (observable instanceof BooleanObservable)
        {
            BooleanObservable bol = (BooleanObservable) observable;
            if ( bol.ID == isTestStarting.ID )
            {
                if ( bol.get() )
                {
                    testData = null;

                    messageID.set(R.string.md_warning_test_start);

                    firmware = FIRMWARE_EQUIPMENT;
                    pagerAdapter.showChartIDD(firmware>=FIRMWARE_IDD);
                    viewPager.setAdapter(pagerAdapter);
                    tabLayout.setViewPager(viewPager);
                    pagerAdapter.resetAll(isConnected.get(), device);
                    viewPager.setCurrentItem(0);
                    if (menuOptions!=null)
                        menuOptions.setEnabled(false);
                    buttonStart.setVisibility(View.GONE);
                    buttonStop.setVisibility(View.VISIBLE);
                    mCommonUI.enableButtonBg(buttonStop, true);
                    mCommonUI.enableButton(buttonSetup, false);
                    enableToolBar(false);
                }
                else
                {
                    if (!isTestRunning.get())
                    {
                        buttonStart.setVisibility(View.VISIBLE);
                        buttonStop.setVisibility(View.GONE);
                        enableCommandButtons(true);
                    }
                }
            }
            else if ( bol.ID == isTestRunning.ID )
            {
                // TEST RUNNING
                if ( bol.get() )
                {
                    if (!isTestStarting.get()) // Test started in the device
                        isTestStarting.set(true);
                    isTestStarting.set(false);

                    sendMessage( modbus.read.testDate() );

                    testData = new TestData(Controller.model, Controller.serial);
                    testData.setFirmware(FIRMWARE_EQUIPMENT);
                    testData.setHardware(HARDWARE_EQUIPMENT);
                    testData.setDate(new Date()); // This is to ensure Date != null
                    testData.setDevice(device);
                    testData.setMode(device.getOperationMode());

                    if ( messageID.get() != R.string.md_test_running )
                    {
                        messageID.set( R.string.md_test_running );
                    }

                    if (device!=null)
                    {
                        if(device.getCapacitanceEnabled())
                            WAITING_CAP = true;
                        if(device.getDielectricDischargeEnabled())
                            WAITING_DD  = true;
                    }

                    showDialogEnd = true;
                }
                // TEST ENDED
                else
                {
                    if (menuOptions!=null)
                        menuOptions.setEnabled(true);
                    buttonStart.setVisibility(View.VISIBLE);
                    buttonStop.setVisibility(View.GONE);
                    mCommonUI.enableButton(buttonSetup, true);

                    if (isTestStarting.get()) // isTestStarting == true, test cancelled
                        enableCommandButtons(true);
                    else
                        enableCommandButtons(false);

                    // FIRMWARE 11 or higher
                    if (firmware>=FIRMWARE_IDD && device.getDielectricDischargeEnabled() && !isTestStarting.get()) // isTestStarting == true, test cancelled
                    {
                        WAITING_IDD = true;
                        dialogTestEndShow(true);
                        sendMessage( modbus.read.status(Protocol.ST_IDD) );
                    }
                    // FIRMWARE 0
                    if (firmware<FIRMWARE_DD && testData!=null && !isTestStarting.get())
                    {
                        messageID.set(R.string.md_calculating_capacitance);
                        dialogTestEndShow(true);
                    }
                    // FIRMWARE 1 or higher
                    else
                    {
                        if(device.getCapacitanceEnabled() && testData!=null && !isTestStarting.get())
                        {
                            messageID.set(R.string.md_calculating_capacitance);
                            dialogTestEndShow(true);
                        }
                        else
                        {
                            messageID.set(R.string.md_test_finished);
                            mCommonUI.enableButton(buttonManage, true);
                            enableCommandButtons(true);
                            if (testData != null && testData.getMeasurement().size() > 0)
                                mCommonUI.enableButton(buttonSave, true);
                            else
                                mCommonUI.enableButton(buttonSave, false);

                            if(testData!=null)
                            {
                                dialogTestEndShow(false);
                            }
                        }
                    }
                }
            }
            else if ( bol.ID == isConnected.ID )
            {
                enableCommandButtons(bol.get());
                //messageID.set(R.string.open_port);
                if ( !bol.get() )
                {
                    initialConfig = true;
                    device.setOperationMode(-1);
                    if ( mainScreen!=null )
                        mainScreen.resetTestParameters();
                }
            }
            else if ( bol.ID == isPrinterEnabled.ID )
            {
                printerEnabled( bol.get() );
            }
            else if ( bol.ID == isFilterEnabled.ID )
            {
                filterEnabled( bol.get() );
            }
            else if ( bol.ID == isAutoPilot.ID )
            {
                if ( bol.get() )
                {
                    helper.warnAboutLock(OVERLAY_ID);
                    enableCommandButtons(false);
                    mCommonUI.enableButtonBg(buttonStop , true);
                }
                else
                {
                    enableCommandButtons(true);
                }
            }
        }
        else if (observable instanceof com.megabras.modbus.IntegerObservable)
        {
            com.megabras.modbus.IntegerObservable ob = (com.megabras.modbus.IntegerObservable) observable;
            if ( ob.ID == Device.OBSERVABLE_VOLTAGE )
            {
                if ( pageTestSetup!=null )
                {
                    ((TextView) pageTestSetup.findViewById(R.id.tvManualVoltage)).setText( device.getTestVoltage()+"" );
                }
            }
        }
        else if (observable instanceof IntegerObservable)
        {
            IntegerObservable ob = (IntegerObservable) observable;
            if ( ob.ID == messageID.ID )
            {
                switch ( ob.get() )
                {
                    case R.string.read_general_params:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.buzzer_param_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.erase_memory:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.firmware_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.manual_parameters_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.memory_map_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.md_test_running:
                        userMessage(ob.get(), CommonUI.COLOR_RED, true );
                        break;
                    case R.string.md_warning_test_start:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.md_calculating_capacitance:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.md_configurations_changed:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.operation_mode_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.open_port:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false );
                        break;
                    case R.string.ready:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.md_test_finished:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.ramp_parameters_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.svt_parameters_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.timed_parameters_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.test_cancelled:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false );
                        break;
                    case R.string.test_voltage_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.updating:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.calc_dd:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                }
            }
        }
    }

    public void stateChanged(int state){
        switch (state){
            case BluetoothClassic.STATE_CONNECTED:
                isConnected.set(true);
                messageID.set( R.string.read_general_params  );
                sendMessage( modbus.read.memoryMap()         );
                sendMessage( modbus.read.firmware()          );
                sendMessage( modbus.read.hardware()          );
                sendMessage( modbus.read.timeout()           );
                sendMessage( modbus.read.setup()             );
                sendMessage( modbus.read.buzzer()            );
                sendMessage( modbus.read.mode()              );
                sendMessage( modbus.read.voltage()           );
                sendMessage( modbus.read.testsParamsManual() );
                sendMessage( modbus.read.testsParamsTimer()  );
                sendMessage( modbus.read.testsParamsSVT()    );
                sendMessage( modbus.read.testsParamsRamp()   );
                break;
            case BluetoothClassic.STATE_CONNECTING:
                break;
            case BluetoothClassic.STATE_NOT_RESPONDING:
                //isConnected.set(false);
                break;
            case BluetoothClassic.STATE_FAIL:
                isConnected.set(false);
                break;
            case BluetoothClassic.STATE_LOST:
                isConnected.set(false);
                isTestStarting.set(false);
                if ( isTestRunning.get() && testData!=null && testData.getMeasurement().size()>0 )
                    mCommonUI.enableButton(buttonSave, true);
                isTestRunning.set(false);
                mCommonUI.enableButton(buttonManage, true);
                break;
            case BluetoothClassic.STATE_CLOSED:
                isConnected.set(false);
                isTestStarting.set(false);
                if ( isTestRunning.get() && testData!=null && testData.getMeasurement().size()>0 )
                    mCommonUI.enableButton(buttonSave, true);
                isTestRunning.set(false);
                mCommonUI.enableButton(buttonManage, true);
                break;
        }
    }

    public void processResponse(int id, String response)
    {
        switch (id)
        {
            case Protocol.ID_REAL_TIME_READ:
                boolean[] status = modbus.parse.statusDevice(response);
                device.setStatus(status);
                int bat = modbus.parse.statusBatteryLevel(response);
                if (bat!=device.getBatteryLevel())
                    parent.updateBattery( bat );
                device.setBatteryLevel( bat );

                if ( device.getStatus( Device.ST_CONFIG_CHANGED ) )
                {
                    messageID.set( R.string.md_configurations_changed );
                    sendMessage( modbus.write.resetConfigChanged() );
                }

                if ( device.getStatus(Device.ST_RUNNING) )
                    messageID.set( R.string.md_test_running );

                // TEST RUNNING
                if ( isTestRunning.get() )
                {
                    if ( testData.getTestNumber()<=0 )
                    {
                        testData.setTestNumber(modbus.parse.statusTestNumber(response));
                        mainScreen.setTestNumber( testData.getTestNumber() );
                        table.setMode( testData.getMode() );
                        table.setTestNumber( testData.getTestNumber() );
                        table.setDate( formatData.getDateFormatted( testData.getDate(), device.getDateFormat(), device.getTimeFormat()) );

                        String titleDAI = "DAI";
                        String titlePI  = "PI";
                        switch ( device.getOperationMode() )
                        {
                            case Protocol.MODE_MANUAL:
                                titleDAI += " (" + device.getParamManual().getDAI_TIME_A() + "/" + device.getParamManual().getDAI_TIME_B() + ")";
                                titlePI  += " (" + device.getParamManual().getPI_TIME_A()  + "/" + device.getParamManual().getPI_TIME_B() + ")";
                                break;
                            case Protocol.MODE_TIMER:
                                titleDAI += " (" + device.getParamTimer().getDAI_TIME_A() + "/" + device.getParamTimer().getDAI_TIME_B() + ")";
                                titlePI  += " (" + device.getParamTimer().getPI_TIME_A()  + "/" + device.getParamTimer().getPI_TIME_B() + ")";
                                break;
                        }

                        table.setTitleDAI(titleDAI);
                        table.setTitlePI(titlePI);
                    }
                    if ( response.length()>40 )
                    {
                        Measurement m = modbus.parse.statusMeasurement(response);
                        pagerAdapter.updateChartsAndTables(m, device, true, false);

                        if (m.getElapsed() >= 15 && m.getElapsed()%5==0)
                        {
                            if(testData.getMeasurement().size()>0)
                            {
                                if ( testData.getMeasurementLast().getElapsed()==m.getElapsed() )
                                    testData.getMeasurement().remove( testData.getMeasurement().size()-1 );
                            }
                            testData.addMeasument(m);
                        }
                    }

                    if ( device.getStatus( Device.ST_DAI ) && testData.getDAI().getResult()<=0 )
                        sendMessage( modbus.read.dai() );
                    if ( device.getStatus( Device.ST_SVT_PI ) && testData.getPI().getResult()<=0 )
                        sendMessage( modbus.read.pi() );
                    if ( device.getStatus( Device.ST_BUZZER ) && testData.getSVT()<0 )
                        sendMessage( modbus.read.svt() );
                    sendMessage( modbus.read.status(Protocol.ST_CURRENT) );
                }
                // STAND BY
                else
                {
                    // Add the last measurement
                    if ( response.length()>40 )
                    {
                        if (firmware>=FIRMWARE_IDD && WAITING_IDD)
                        {
                            DielectricDischarge ddAmps = modbus.parse.statusDielectricDischarge(response);
                            testData.addDielectricDischarge(ddAmps);
                            dialogTestEndSetObservation("IDD= " + formatData.formatCurrent(ddAmps.getAmps()));

                            if (chartIDD!=null)
                                chartIDD.update(ddAmps);
                            sendMessage( modbus.read.status(Protocol.ST_IDD) );
                        }
                        else
                        {
                            Measurement m = modbus.parse.statusMeasurement(response);
                            pagerAdapter.updateChartsAndTables(m, device,true, true);
                            if ( m.getElapsed()>=15 && testData!=null && testData.getMeasurement()!=null)
                            {
                                if(testData.getMeasurement().size()>0)
                                {
                                    if ( testData.getMeasurementLast().getElapsed()==m.getElapsed() )
                                        testData.getMeasurement().remove( testData.getMeasurement().size()-1 );
                                }
                                testData.addMeasument(m);
                            }
                        }
                    }
                }
                if ( device.getStatus( Device.ST_CAPACITANCE ) && WAITING_CAP )
                {
                    sendMessage(modbus.read.capacitance());
                }
                if ( device.getStatus( Device.ST_DD ) && WAITING_DD )
                {
                    sendMessage(modbus.read.dd());
                    WAITING_IDD = false;
                }

                isPrinterEnabled .set( device.getStatus( Device.ST_PRINTER    ) );
                isFilterEnabled  .set( device.getStatus( Device.ST_FILTER     ) );
                isTestRunning    .set( device.getStatus( Device.ST_RUNNING    ) );
                isAutoPilot      .set( device.getStatus( Device.ST_AUTO_PILOT ) );
                break;
            case Protocol.ID_DAI_READ:
                testData.getDAI().setResult( modbus.parse.dai(response) );
                mainScreen.setDAI( testData.getDAI().getResult() );
                table.setDAI( formatData.formatIndexes(testData.getDAI().getResult()) );
                break;
            case Protocol.ID_PI_READ:
                testData.getPI().setResult( modbus.parse.pi(response) );
                mainScreen.setPI( testData.getPI().getResult() );
                table.setPI( formatData.formatIndexes(testData.getPI().getResult()) );
                break;
            case Protocol.ID_SVT_READ:
                testData.setSVT( modbus.parse.svt(response) );
                mainScreen.setSVT( testData.getSVT() );
                table.setSVT( formatData.formatIndexes(testData.getSVT()) );
                break;
            case Protocol.ID_CAPACITANCE_READ:
                WAITING_CAP = false;
                testData  .setCapacitance( modbus.parse.capacitance(response) );
                mainScreen.setCapacitance( testData.getCapacitance() );
                table     .setCAP        ( formatData.formatCapacitance( testData.getCapacitance() ));
                if (firmware<FIRMWARE_DD || !device.getDielectricDischargeEnabled())
                {
                    messageID.set(R.string.md_test_finished);
                    mCommonUI.enableButton(buttonManage, true);
                    enableCommandButtons(true);
                    if (testData != null && testData.getMeasurement().size() > 0)
                        mCommonUI.enableButton(buttonSave, true);
                    else
                        mCommonUI.enableButton(buttonSave, false);

                    dialogTestEndShow(false);
                }
                else
                {
                    if (device.getDielectricDischargeEnabled())
                    {
                        messageID.set(R.string.calc_dd);
                        dialogTestEndShow(true);
                    }
                }
                break;
            case Protocol.ID_DD_READ:
                WAITING_DD = false;
                testData   .setDD( modbus.parse.dd(response) );
                mainScreen .setDielectricDischarge( testData.getDD() );
                table      .setDD( formatData.formatIndexes( testData.getDD() ));
                messageID  .set( R.string.md_test_finished );
                mCommonUI  .enableButton(buttonManage, true);
                enableCommandButtons(true);
                if ( testData!=null && testData.getMeasurement().size()>0 )
                    mCommonUI.enableButton(buttonSave, true);
                else
                    mCommonUI.enableButton(buttonSave, false);

                dialogTestEndShow(false);
                break;
            case Protocol.ID_STOP_WRITE:
                String cmd = com.megabras.modbus.MUtils.cleanResponse(modbus.write.stop().getMessage());
                String msg = com.megabras.modbus.MUtils.cleanResponse(response);
                if ( !cmd.equals(msg) ) // Message STOP error
                    mCommonUI.enableButtonBg(buttonStop, true);
                break;
            case Protocol.ID_MEMORY_MAP_READ:
                int mapVersion = modbus.parse.memoryMap(response);
                Protocol.setRevisionMap(mapVersion);
                messageID.set(R.string.memory_map_was_read);
                break;
            case Protocol.ID_FIRMWARE_READ:
                firmware = modbus.parse.firmware(response);
                FIRMWARE_EQUIPMENT = firmware;
                messageID.set(R.string.firmware_was_read);
                pagerAdapter.showChartIDD(firmware>=FIRMWARE_IDD);
                viewPager.setAdapter(pagerAdapter);
                tabLayout.setViewPager(viewPager);
                pagerAdapter.resetAll(isConnected.get(), device);
                if (testData!=null)
                    pagerAdapter.setAllValues(testData);
                break;
            case Protocol.ID_HARDWARE_READ:
                HARDWARE_EQUIPMENT = modbus.parse.hardware(response);
                String version = "v1." + HARDWARE_EQUIPMENT + "." + FIRMWARE_EQUIPMENT;
                parent.setFirmwareVersion(version);
                break;
            case Protocol.ID_TIMEOUT_READ:
                int timeout = modbus.parse.timeout(response);
                if ( timeout>1000 )
                    sendMessage( modbus.write.timeout(1000) );
                break;
            case Protocol.ID_BUZZER_PARAM_READ:
                int buzzerTime = modbus.parse.buzzer(response);
                if ( buzzerTime!=buzzerParam )
                    sendMessage( modbus.write.buzzer( buzzerParam ) );
                messageID.set(R.string.buzzer_param_was_read);
                break;
            case Protocol.ID_MODE_READ:
                device.setOperationMode( modbus.parse.mode(response) );
                messageID.set(R.string.operation_mode_was_read);
                enableButtonVoltage(true);
                break;
            case Protocol.ID_TEST_VOLTAGE_READ:
                device.setTestVoltage( modbus.parse.voltage(response) );
                messageID.set(R.string.test_voltage_was_read);
                break;
            case Protocol.ID_TEST_VOLTAGE_WRITE:
                mainScreen.setTestParameters(device);
                break;
            case Protocol.ID_PARAMS_MANUAL_READ:
                messageID.set(R.string.manual_parameters_was_read);
                ParamManual paramManual = modbus.parse.testsParamsManual(response);
                device.setParamManual( paramManual );
                break;
            case Protocol.ID_PARAMS_MANUAL_WRITE:
                mainScreen.reset();
                mainScreen.setTestParameters(device);
                if (!isTestRunning.get())
                    messageID.set(R.string.ready);
                enableButtonVoltage(true);
                break;
            case Protocol.ID_PARAMS_TIMER_READ:
                messageID.set(R.string.timed_parameters_was_read);
                ParamTimer paramTimer = modbus.parse.testsParamsTimer(response);
                device.setParamTimer( paramTimer );
                break;
            case Protocol.ID_PARAMS_TIMER_WRITE:
                mainScreen.reset();
                mainScreen.setTestParameters(device);
                if (!isTestRunning.get())
                    messageID.set(R.string.ready);
                enableButtonVoltage(true);
                break;
            case Protocol.ID_PARAMS_SVT_READ:
                messageID.set(R.string.svt_parameters_was_read);
                ParamSVT paramSVT = modbus.parse.testsParamsSVT(response);
                device.setParamSVT( paramSVT );
                break;
            case Protocol.ID_PARAMS_SVT_WRITE:
                mainScreen.reset();
                mainScreen.setTestParameters(device);
                if (!isTestRunning.get())
                    messageID.set(R.string.ready);
                enableButtonVoltage(true);
                break;
            case Protocol.ID_PARAMS_RAMP_READ:
                messageID.set(R.string.ramp_parameters_was_read);
                ParamRamp paramRamp = modbus.parse.testsParamsRamp(response);
                device.setParamRamp( paramRamp );
                messageID.set(R.string.ready);
                mainScreen.reset();
                mainScreen.setTestParameters(device);
                break;
            case Protocol.ID_PARAMS_RAMP_WRITE:
                mainScreen.reset();
                mainScreen.setTestParameters(device);
                if (!isTestRunning.get())
                    messageID.set(R.string.ready);
                enableButtonVoltage(true);
                break;
            case Protocol.ID_GENERAL_SETUP_READ:
                modbus.parse.setup(device, response);
                if ( !initialConfig )
                    pageGeneralSetup();
                else
                    initialConfig = false;
                break;
            case Protocol.ID_GENERAL_SETUP_WRITE:
                messageID.set(R.string.ready);
                menuOptions.setEnabled(true);
                break;
            case Protocol.ID_DATE_READ:
                device.setDate( modbus.parse.date(response) );
                break;
            case Protocol.ID_MEM_POINTER_WRITE:
                sendMessage( modbus.read.memoryPointer() );
                break;
            case Protocol.ID_MEM_POINTER_READ:
                device.setFreeMemory( modbus.parse.memoryFree( response ) );
                if ( pageGeneral!=null )
                    ((TextView) pageGeneral.findViewById(R.id.device_memory)).setText( device.getFreeMemory()+"%" );
                break;
            case Protocol.ID_TEST_NUMBER_READ:
                sendMessage( modbus.write.start() );
                break;
            case Protocol.ID_TEST_DATE_READ:
                Date date = modbus.parse.date( response );
                if (date!=null && testData!=null)
                    testData.setDate(date);
                break;
            case Protocol.ID_START_WRITE:
                // Case the test was cancelled directly in the equipment
                handler.postDelayed(() -> {
                    if( !isTestRunning.get() && buttonStop.getVisibility()==View.VISIBLE )
                    {
                        isTestRunning.update(false);
                        isTestStarting.update(false);
                        messageID.set(R.string.test_cancelled);
                    }
                }, 4500);
                break;
            case Protocol.ID_CONFIG_CHANGE_WRITE:
                initialConfig = true;
                sendMessage( modbus.read.mode()              );
                sendMessage( modbus.read.voltage()           );
                sendMessage( modbus.read.setup()             );
                sendMessage( modbus.read.testsParamsManual() );
                sendMessage( modbus.read.testsParamsTimer()  );
                sendMessage( modbus.read.testsParamsSVT()    );
                sendMessage( modbus.read.testsParamsRamp()   );
                break;
        }
    }
}

