package com.megabras.bluelogg.md15kvr;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;

import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.R;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.DielectricDischarge;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.Protocol;
import com.megabras.modbus.MD15KVR.ResultDAI;
import com.megabras.modbus.MD15KVR.ResultPI;
import com.megabras.modbus.MD15KVR.TestData;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@SuppressWarnings("WeakerAccess")
public class Database extends SQLiteOpenHelper
{
    @SuppressWarnings("FieldCanBeLocal")
    private String
            TAG          = "Database";

    private static final String
            TEXT_TYPE    = " TEXT",
            COMMA_SEP    = ",";

    @SuppressWarnings("FieldCanBeLocal")
    private String
            root         = Environment.getExternalStorageDirectory().toString(),
            picSuffix    = ".jpg",
            audioSuffix  = "_annotation.3gp",
            mapSuffix    = "_map_snapshot.jpg";

    private File
            dir,
            tempDir,
            rootDir;

    protected com.megabras.modbus.MD15KVR.FormatData
            formatData;

    public Database(Context context, String model, String dbName)
    {
        super(context, dbName, null, Entry.DB_VERSION);
        createFolder(context, model);
        formatData = new com.megabras.modbus.MD15KVR.FormatData();
    }

    public void save(TestData testData)
    {
        String TABLE_NAME = testData.getModel().replaceAll("-", "") + "_" + testData.getSerial() + "_" + testData.getTestNumber() + "_" + testData.getDate().getTime();
        SQLiteDatabase db = this.getWritableDatabase();

        Cursor c = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + TABLE_NAME + "';", null);
        // TABLE ALREADY EXITS
        int len = c.getCount();
        c.close();
        if( len>0 )
            return;

        String FIELD_ENTRIES =
                Entry.DESCRIPTION   + TEXT_TYPE + COMMA_SEP +
                Entry.COL_A         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_B         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_C         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_D         + TEXT_TYPE;

        String SQL_CREATE_TABLE =
                "CREATE TABLE " + TABLE_NAME + " (" + FIELD_ENTRIES + " )";
        db.execSQL(SQL_CREATE_TABLE);

        db.beginTransaction();

        ContentValues values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.MODEL );
        values.put( Entry.COL_A      , testData.getModel() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SERIAL );
        values.put( Entry.COL_A      , testData.getSerial() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.FIRMWARE );
        values.put( Entry.COL_A      , testData.getFirmware() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.HARDWARE );
        values.put( Entry.COL_A      , testData.getHardware() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.TEST_NUMBER );
        values.put( Entry.COL_A      , testData.getTestNumber() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DATE );
        values.put( Entry.COL_A      , testData.getDate().getTime()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        Device device = testData.getDevice();

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.OP_MODE );
        values.put( Entry.COL_A      , device.getOperationMode() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        ResultDAI dai = new ResultDAI();
        ResultPI  pi  = new ResultPI();
        switch ( device.getOperationMode() )
        {
            case Protocol.MODE_MANUAL:
                dai.setTimeA( device.getParamManual().getDAI_TIME_A() );
                dai.setTimeB( device.getParamManual().getDAI_TIME_B() );
                pi.setTimeA( device.getParamManual().getPI_TIME_A() );
                pi.setTimeB( device.getParamManual().getPI_TIME_B() );
                break;
            case Protocol.MODE_TIMER:
                dai.setTimeA( device.getParamTimer().getDAI_TIME_A() );
                dai.setTimeB( device.getParamTimer().getDAI_TIME_B() );
                pi.setTimeA( device.getParamTimer().getPI_TIME_A() );
                pi.setTimeB( device.getParamTimer().getPI_TIME_B() );
                break;
        }

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DAI_CONFIG );
        values.put( Entry.COL_A      , dai.getTimeA() );
        values.put( Entry.COL_B      , dai.getTimeB() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.PI_CONFIG );
        values.put( Entry.COL_A      , pi.getTimeA() );
        values.put( Entry.COL_B      , pi.getTimeB() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DAI );
        values.put( Entry.COL_A      , testData.getDAI().getResult() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.PI );
        values.put( Entry.COL_A      , testData.getPI().getResult() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.CAPACITANCE );
        values.put( Entry.COL_A      , testData.getCapacitance() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        if (testData.getDD()>=0)
        {
            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.DD);
            values.put(Entry.COL_A, testData.getDD());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
        }

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SVT );
        values.put( Entry.COL_A      , testData.getSVT() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ACCURACY );
        values.put( Entry.COL_A      , testData.getGpsAcuraccy()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
        values.put( Entry.COL_A      , testData.getGpsAltitude()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LATITUDE );
        values.put( Entry.COL_A      , testData.getGpsLatitude()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
        values.put( Entry.COL_A      , testData.getGpsLongitude()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        for (Measurement m : testData.getMeasurement())
        {
            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.RESULT );
            values.put( Entry.COL_A      , m.getElapsed() );
            values.put( Entry.COL_B      , m.getResistance() );
            values.put( Entry.COL_C      , m.getVoltage() );
            values.put( Entry.COL_D      , m.getCurrent() );
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
        }

        if (testData.getDielectricDischarge()!=null)
        {
            for (DielectricDischarge dd : testData.getDielectricDischarge())
            {
                values = new ContentValues();
                values.put( Entry.DESCRIPTION, Entry.IDD       );
                values.put( Entry.COL_A      , dd.getElapsed() );
                values.put( Entry.COL_B      , dd.getAmps()    );
                db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
            }
        }

        db.setTransactionSuccessful();
        db.endTransaction();
    }

    public void updateGpsData(TestData testData)
    {
        try
        {
            String TABLE_NAME = testData.getModel().replaceAll("-", "") + "_" + testData.getSerial() + "_" + testData.getTestNumber() + "_" + testData.getDate().getTime();
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values;

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_LATITUDE );
            values.put(Entry.COL_A, testData.getGpsLatitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
            values.put(Entry.COL_A, testData.getGpsLongitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
            values.put(Entry.COL_A, testData.getGpsAltitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_ACCURACY );
            values.put(Entry.COL_A, testData.getGpsAcuraccy());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            db.setTransactionSuccessful();
            db.endTransaction();
        }
        catch (Exception e)
        {
            android.util.Log.e(TAG, e.getMessage());
        }
    }

    public List<FileDetails> getSavedFilesList(String serial, int dateFormat, int timeFormat)
    {
        try
        {
            List<FileDetails>
                    filesList = new ArrayList<>();
            SQLiteDatabase
                    db        = this.getReadableDatabase();
            Cursor c          = db.rawQuery("SELECT name FROM sqlite_master WHERE type='table' AND name LIKE '%"+serial+"%'", null);

            if (c.moveToFirst())
            {
                while ( !c.isAfterLast() )
                {
                    String tableName = c.getString(0);
                    if ( !tableName.contains("metadata") )
                    {
                        FileDetails file = new FileDetails();
                        String[] temp           = tableName.split("_");
                        file.FILENAME           = tableName;
                        file.MODEL              = temp[0];
                        file.SERIAL             = temp[1];
                        file.INDEX              = temp[2];
                        String seconds          = temp[3];
                        long time = Long.parseLong(seconds);
                        Date date = new Date(time);
                        file.DATE               = formatData.getDateFormatted(date, dateFormat, timeFormat);
                        filesList.add(file);
                    }
                    c.moveToNext();
                }
            }
            c.close();
            return filesList;
        }
        catch (Exception e)
        {
            return null;
        }
    }

    public TestData getResult(String TABLE_NAME)
    {
        try
        {
            TestData testData = new TestData();
            ResultDAI dai            = new ResultDAI();
            ResultPI  pi             = new ResultPI();
            Device    device         = new Device();

            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery("SELECT * FROM " + TABLE_NAME, null);

            if (c.moveToFirst())
            {
                while ( !c.isAfterLast() )
                {
                    String colA = c.getString(0);
                    String colB = c.getString(1);
                    String colC = c.getString(2);
                    String colD = c.getString(3);
                    String colE = c.getString(4);

                    switch (colA)
                    {
                        case Entry.MODEL:
                            testData.setModel(colB);
                            break;
                        case Entry.SERIAL:
                            testData.setSerial(colB);
                            break;
                        case Entry.FIRMWARE:
                            testData.setFirmware(Integer.parseInt(colB));
                            Controller.firmware = testData.getFirmware();
                            break;
                        case Entry.HARDWARE:
                            testData.setHardware(Integer.parseInt(colB));
                            break;
                        case Entry.TEST_NUMBER:
                            testData.setTestNumber((short) Integer.parseInt(colB));
                            break;
                        case Entry.DATE:
                            Date datef = new Date();
                            datef.setTime(Long.parseLong(colB));
                            testData.setDate(datef);
                            break;
                        case Entry.OP_MODE:
                            testData.setMode(Integer.parseInt(colB));
                            device.setOperationMode(testData.getMode());
                            break;
                        case Entry.DAI_CONFIG:
                            dai.setTimeA(Integer.parseInt(colB));
                            dai.setTimeB(Integer.parseInt(colC));
                            break;
                        case Entry.DAI:
                            dai.setResult(Double.parseDouble(colB));
                            testData.setDAI(dai);
                            break;
                        case Entry.PI_CONFIG:
                            pi.setTimeA(Integer.parseInt(colB));
                            pi.setTimeB(Integer.parseInt(colC));
                            break;
                        case Entry.PI:
                            pi.setResult(Double.parseDouble(colB));
                            testData.setPI(pi);
                            break;
                        case Entry.SVT:
                            testData.setSVT(Double.parseDouble(colB));
                            break;
                        case Entry.CAPACITANCE:
                            testData.setCapacitance(Integer.parseInt(colB));
                            break;
                        case Entry.DD:
                            testData.setDD(Double.parseDouble(colB));
                            if (testData.getFirmware()<=-1)
                            {
                                Controller.firmware = Controller.FIRMWARE_DD;
                                testData.setFirmware(Controller.firmware);
                            }
                            break;
                        case Entry.GPS_LATITUDE:
                            testData.setGpsLatitude(Double.parseDouble(colB));
                            break;
                        case Entry.GPS_LONGITUDE:
                            testData.setGpsLongitude(Double.parseDouble(colB));
                            break;
                        case Entry.GPS_ALTITUDE:
                            testData.setGpsAltitude(Double.parseDouble(colB));
                            break;
                        case Entry.GPS_ACCURACY:
                            testData.setGpsAcuraccy(Double.parseDouble(colB));
                            break;
                        case Entry.RESULT:
                            Measurement m = new Measurement();
                            m.setElapsed(Integer.parseInt(colB));
                            m.setResistance(Double.parseDouble(colC));
                            m.setVoltage(Integer.parseInt(colD));
                            m.setCurrent(Double.parseDouble(colE));
                            testData.addMeasument(m);
                            break;
                        case Entry.IDD:
                            DielectricDischarge dd = new DielectricDischarge();
                            dd.setElapsed(Integer.parseInt(colB));
                            dd.setAmps(Double.parseDouble(colC));
                            testData.addDielectricDischarge(dd);
                            break;
                    }
                    c.moveToNext();
                }
            }

            testData.setDevice(device);

            c.close();
            return testData;
        } catch (Exception e) {
            android.util.Log.e("TEST", e.getMessage());
            return null;
        }
    }

    public void deleteTable(String name){
        SQLiteDatabase db = this.getWritableDatabase();
        String SQL_DELETE_TABLE =
                "DROP TABLE IF EXISTS " + name;
        db.execSQL(SQL_DELETE_TABLE);

        //MD15KVR_MA4125B_72_1526065135873

        String[] parts = name.split("_");

        deleteFiles(parts[1], parts[3]);
    }

//    public void deleteFile(String model, String serial, int testNumber, Date date)
//    {
//        String TABLE_NAME = model + "_" + serial + "_" + testNumber + "_" + date.getTime();
//
//        File audioFile;
//        if(getAudioFile().getName().equals(".3gp")){
//            String[] pieces = fileName.split("_");
//            String audioName="";
//            for(int index = 2;index<pieces.length;index++){
//                audioName += pieces[index];
//            }
//            audioFile = new File(resultsFolder + "/" + audioName + ".3gp");
//        }
//        else{
//            audioFile = getAudioName();
//        }
//        if(audioFile.exists()){
//            audioFile.delete();
//        }
//
//        File picFile;
//        if(getPictureName().getName().equals(".jpg")){
//            String[] pieces = fileName.split("_");
//            String picName="";
//            for(int index = 2;index<pieces.length;index++){
//                picName += pieces[index];
//            }
//            picFile = new File(resultsFolder + "/" + picName + ".jpg");
//        }
//        else{
//            picFile = getPictureName();
//        }
//        if(picFile.exists()){
//            picFile.delete();
//        }
//
//        SQLiteDatabase db = mDbHelper.getWritableDatabase();
//        mDbHelper.deleteTable(db, fileName);
//
//        if(fileName.equals(TABLE_NAME)){
//            ((MD10KVx) activity).clearAll();
//        }
//    }

    //##########################################################################
    //# Folders
    //##########################################################################

    public boolean isExternalStorageAvaiable()
    {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals(state);
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public boolean createFolder(Context context, String model){
        if(!this.isExternalStorageAvaiable()){
            return false;
        }

        rootDir     = new File(root + "/" + context.getResources().getString(R.string.company));
        if(!rootDir.exists())
            rootDir.mkdir();

        dir     = new File(root + "/" + context.getResources().getString(R.string.company)+ "/" + model);
        tempDir = new File(dir + "/temp");

        if(!dir.exists())
            dir.mkdir();

        if(!tempDir.exists())
            tempDir.mkdir();

        return dir.exists();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getFolder()
    {
        if(!rootDir.exists())
            rootDir.mkdir();

        if(!dir.exists())
            dir.mkdir();

        return dir;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getTemporaryFolder()
    {
        if(!rootDir.exists())
            rootDir.mkdir();

        if(!dir.exists())
            dir.mkdir();

        if(!tempDir.exists())
            tempDir.mkdir();

        return tempDir;
    }

    public File getPictureFile(String serial, String timeStamp)
    {
        String img = serial + "_" + timeStamp + picSuffix;
        return new File(dir, img);
    }

    public File getAudioFile(String serial, String timeStamp)
    {
        String audio = serial + "_" + timeStamp + audioSuffix;
        return new File(dir, audio);
    }

    public File getMapSnapshotFile(String serial, String timeStamp)
    {
        String map = serial + "_" + timeStamp + mapSuffix;
        return new File(dir, map);
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public void deleteFiles(String serial, String timeStamp)
    {
        for ( File child : dir.listFiles() )
        {
            if( child.getName().contains( serial + "_" + timeStamp ) )
                child.delete();
        }
    }

    @Override
    public void onCreate(SQLiteDatabase db)
    {
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
    {
    }

    public static abstract class Entry
    {
        public static final int
                DB_VERSION = 1;

        public static String
                DB_NAME     = "MD15KVR_REGISTERS.db";

        public static final String
                DESCRIPTION = "DESCRIPTION",
                COL_A       = "COL_A",
                COL_B       = "COL_B",
                COL_C       = "COL_C",
                COL_D       = "COL_D";

        public static final String
                MODEL         = "MODEL",
                SERIAL        = "SERIAL",
                FIRMWARE      = "FIRMWARE",
                HARDWARE      = "HARDWARE",
                OP_MODE       = "OP_MODE",
                DAI_CONFIG    = "DAI_CONFIG",
                PI_CONFIG     = "PI_CONFIG",
                DATE          = "DATE",
                TEST_NUMBER   = "TEST_NUMBER",
                DAI           = "DAI",
                PI            = "PI",
                SVT           = "SVT",
                CAPACITANCE   = "CAPACITANCE",
                DD            = "DD",
                IDD           = "IDD",
                RESULT        = "RESULT",
                GPS_LATITUDE  = "GPS_LATITUDE",
                GPS_LONGITUDE = "GPS_LONGITUDE",
                GPS_ALTITUDE  = "GPS_ALTITUDE",
                GPS_ACCURACY  = "GPS_ACCURACY";
    }

}

