package com.megabras.bluelogg.md15kvr;

import android.app.Activity;
import android.graphics.Typeface;
import android.graphics.pdf.PdfDocument;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Paper;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.TestData;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Locale;

@SuppressWarnings("WeakerAccess")
public class ReportPDF extends ReportProvider
{

    @SuppressWarnings("FieldCanBeLocal")
    private final String
            TAG = "ReportPDF";

    protected com.megabras.modbus.MD15KVR.FormatData
            formatData = new com.megabras.modbus.MD15KVR.FormatData();

    public ReportPDF(Activity activity, File dir, File tempDir, Handler handler)
    {
        super(activity, dir, tempDir, handler);
    }

    public void generate(TestData testData, File picture, File map, File[] chartImageArray)
    {
        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();

        Device device        = testData.getDevice();
        PdfDocument document = new PdfDocument();
        PdfDocument.PageInfo pageInfo;

        if ( Locale.getDefault().getLanguage().equals( Locale.ENGLISH.getLanguage() ) )
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.Letter.getWidth(), Paper.Letter.getHeight(), 1).create();
        }
        else
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.A4.getWidth(), Paper.A4.getHeight(), 1).create();
        }

        CURSOR_Y     = 0;
        CURSOR_X     = 0;
        PAPER_WIDTH  = pageInfo.getPageWidth()  - (MARGIN * 2);
        PAPER_HEIGHT = pageInfo.getPageHeight() - (MARGIN * 2);

        PdfDocument.Page page = document.startPage(pageInfo);
        canvas = page.getCanvas();

        moveCursor(MARGIN, MARGIN);

        // BORDER
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(PAPER_HEIGHT);
        this.drawSquare();

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_general_info)).sendToTarget();

        // TITLE
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawTitle(activity.getString(R.string.md_insulation_test_report));

        ArrayList<String> titles = new ArrayList<>();
        ArrayList<String> values = new ArrayList<>();

        titles.add( activity.getString(R.string.md_model       ) );
        titles.add( activity.getString(R.string.md_serial      ) );
        titles.add( activity.getString(R.string.md_test_number ) );
        titles.add( activity.getString(R.string.md_test_date   ) );

        values.add( testData.getModel());
        values.add( testData.getSerial());
        values.add( testData.getTestNumber()+"");
        values.add( formatData.getDateFormatted(testData.getDate(), device.getDateFormat(), device.getTimeFormat()));

        if ( testData.isEquipmentVersionDefinied() )
        {
            titles.add(2, "FIRMWARE");
            values.add(2, testData.getEquipmentVersion());
        }

        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawLineItem(titles, values);

        if ( testData.getGpsAcuraccy()>0 )
        {
            titles = new ArrayList<>();
            values = new ArrayList<>();

            titles.add( activity.getString( R.string.md_latitude     ) );
            titles.add( activity.getString( R.string.md_longitude    ) );
            titles.add( activity.getString( R.string.md_altitude     ) );
            titles.add( activity.getString( R.string.md_gps_accuracy ) );

            values.add( testData.getGpsLatitude() + ""              );
            values.add( testData.getGpsLongitude() + ""             );
            values.add( testData.getGpsAltitude() + " m"            );
            values.add( "\u00B1" + testData.getGpsAcuraccy() + " m" );

            left  (0);
            top   (0);
            right (PAPER_WIDTH);
            bottom(20);
            this.drawLineItem(titles, values);
        }

        titles                = new ArrayList<>();
        ArrayList<File> files = new ArrayList<>();

        titles.add( activity.getString( R.string.picture ) );
        titles.add( activity.getString( R.string.map     ) );

        files.add(picture);
        files.add(map);

        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(200);

        if (picture.exists() && map.exists())
            this.drawLineImage(titles, files);
        else if (picture.exists())
            this.drawLineItem(titles.get(0), picture);
        else if (map.exists())
            this.drawLineItem(titles.get(1), map);


        saveCursor();

        int CHART_W_AREA = (int)(PAPER_WIDTH * 0.75);

        // TEST MODE
        left  (0);
        top   (0);
        right (CHART_W_AREA);
        bottom(20);

        titles = new ArrayList<>();
        values = new ArrayList<>();
        titles.add( activity.getString( R.string.md_mode ) );
        values.add( activity.getResources().getStringArray( R.array.op_modes )[testData.getMode()].toUpperCase() );
        this.drawLineItem(titles, values);

        String duration = formatData.formatElapsedTime( (short)testData.getMeasurementLast().getElapsed() );
        String lastOhm  = formatData.formatResistance ( testData.getMeasurementLast().getVoltage(), testData.getMeasurementLast().getResistance() );
        double OHM_MIN  = +Double.MAX_VALUE;
        double OHM_MAX  = -Double.MAX_VALUE;
        int V_MIN       = +Integer.MAX_VALUE;
        int V_MAX       = -Integer.MAX_VALUE;

        for (Measurement m : testData.getMeasurement())
        {
            if (m.getResistance() < OHM_MIN)
            {
                OHM_MIN = m.getResistance();
                V_MIN   = m.getVoltage();
            }
            if (m.getResistance() > OHM_MAX)
            {
                OHM_MAX = m.getResistance();
                V_MAX   = m.getVoltage();
            }
        }

        String minOhm = formatData.formatResistance( V_MIN, OHM_MIN );
        String maxOhm = formatData.formatResistance( V_MAX, OHM_MAX );

        titles = new ArrayList<>();
        values = new ArrayList<>();

        titles.add( activity.getString( R.string.test_duration_mmss ) );
        titles.add( activity.getString( R.string.resistance         ) );
        titles.add( activity.getString( R.string.minimum_resistance ) );
        titles.add( activity.getString( R.string.maximum_resistance ) );

        values.add( duration );
        values.add( lastOhm  );
        values.add( minOhm   );
        values.add( maxOhm   );

        this.drawLineItem(titles, values);

        titles = new ArrayList<>();
        values = new ArrayList<>();

        // DAI
        if (testData.getDAI().getResult()>-1)
        {
            String title = activity.getString(R.string.DAI);
            title += " (R" + testData.getDAI().getTimeA() + "s / ";
            title += " R" + testData.getDAI().getTimeB() + "s)";
            titles.add( title );
            values.add( formatData.formatIndexes( testData.getDAI().getResult() ) );
        }
        else
        {
            titles.add( activity.getString(R.string.DAI)    );
            values.add( activity.getString(R.string.dashes) );
        }

        // PI
        if (testData.getPI().getResult()>-1)
        {
            String title = activity.getString(R.string.PI);
            title += " (R" + testData.getPI().getTimeA() + "min / ";
            title += " R" + testData.getPI().getTimeB() + "min)";
            titles.add( title );
            values.add( formatData.formatIndexes( testData.getPI().getResult() ) );
        }
        else
        {
            titles.add( activity.getString(R.string.PI)     );
            values.add( activity.getString(R.string.dashes) );
        }

        // SVT
        if (testData.getSVT()>0)
        {
            String title = activity.getString(R.string.SVT);
            titles.add( title );
            values.add( formatData.formatIndexes( testData.getSVT() ) );
        }
        else
        {
            titles.add( activity.getString(R.string.SVT)    );
            values.add( activity.getString(R.string.dashes) );
        }

        // CAP
        if (testData.getCapacitance()>-1)
        {
            String title = activity.getString(R.string.md_capacitance).toUpperCase();
            titles.add( title );
            values.add( formatData.formatCapacitance( testData.getCapacitance() ) );
        }
        else
        {
            titles.add( activity.getString(R.string.md_capacitance) );
            values.add( activity.getString(R.string.dashes)         );
        }

        if (testData.getFirmware() >= Controller.FIRMWARE_DD)
        {
            // DD
            if (testData.getDD() > -1)
            {
                String title = activity.getString(R.string.dielectric_discharge).toUpperCase();
                titles.add(title);
                values.add(formatData.formatIndexes(testData.getDD()));
            }
            else
            {
                titles.add(activity.getString(R.string.dielectric_discharge));
                values.add(activity.getString(R.string.dashes));
            }
        }

        this.drawLineItem(titles, values);

        // CHARTS
        left  (0);
        top   (0);
        right (CHART_W_AREA);
        bottom((PAPER_HEIGHT - CURSOR_Y) / 2);

        files  = new ArrayList<>();
        if (chartImageArray.length>0)
            files.add(chartImageArray[0]);
        if (chartImageArray.length>1)
            files.add(chartImageArray[1]);

        this.drawLineImage(files);

        files  = new ArrayList<>();
        if (chartImageArray.length>2)
            files.add(chartImageArray[2]);
        else
            files.add(null);

        if (chartImageArray.length>3)
            files.add(chartImageArray[3]);
        else
            files.add(null);

        this.drawLineImage(files);

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_tables)).sendToTarget();

        // TABLE
        restoreCursor();

        moveCursor(CHART_W_AREA, 0);

        left  (0);
        top   (0);
        right (PAPER_WIDTH - CHART_W_AREA);
        bottom(10);

        values = new ArrayList<>();
        values.add( activity.getString( R.string.MMSS ) );
        values.add( activity.getString( R.string.V    ) );
        values.add( activity.getString( R.string.ohm  ) );
        values.add( activity.getString( R.string.A    ) );

        this.drawRow(Typeface.DEFAULT_BOLD, values);

        int PAPER_LEFT = PAPER_HEIGHT - CURSOR_Y - BRAND_HEIGHT;
        int ROWS_MAX   = PAPER_LEFT / getBounds().height();

        ArrayList<Measurement> data = testData.getMeasurement();

        if (data.size() > ROWS_MAX)
        {
            int FIRST_HALF = ROWS_MAX / 2;
            int LAST_HALF  = ROWS_MAX - FIRST_HALF;
            for ( int i = 0; i < FIRST_HALF; i++ )
            {
                int    baseSeconds = testData.getMeasurement().get(i).getElapsed();
                double resistance  = testData.getMeasurement().get(i).getResistance();
                int    voltage     = testData.getMeasurement().get(i).getVoltage();

                String formattedMilliseconds = formatData.formatElapsedTime((short)baseSeconds);
                String formattedResistance   = formatData.formatResistance(resistance);
                String formattedVoltage      = formatData.formatVoltage(voltage);
                String calculatedCurrent     = formatData.formatCurrent((double)voltage/resistance);

                values = new ArrayList<>();
                values.add(formattedMilliseconds);
                values.add(formattedVoltage);
                values.add(formattedResistance);
                values.add(calculatedCurrent);

                if (isHighlighted(testData.getDAI(), testData.getMeasurement().get(i)) || isHighlighted(testData.getPI(), testData.getMeasurement().get(i)))
                    this.drawHighlightedRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
                else
                    this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
            }
            for ( int i = (data.size()-1)-LAST_HALF; i < data.size(); i++ )
            {
                int    baseSeconds = testData.getMeasurement().get(i).getElapsed();
                double resistance  = testData.getMeasurement().get(i).getResistance();
                int    voltage     = testData.getMeasurement().get(i).getVoltage();

                String formattedMilliseconds = formatData.formatElapsedTime((short)baseSeconds);
                String formattedResistance   = formatData.formatResistance(resistance);
                String formattedVoltage      = formatData.formatVoltage(voltage);
                String calculatedCurrent     = formatData.formatCurrent((double)voltage/resistance);

                values = new ArrayList<>();
                values.add(formattedMilliseconds);
                values.add(formattedVoltage);
                values.add(formattedResistance);
                values.add(calculatedCurrent);
                if (isHighlighted(testData.getDAI(), testData.getMeasurement().get(i)) || isHighlighted(testData.getPI(), testData.getMeasurement().get(i)))
                    this.drawHighlightedRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
                else
                    this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
            }
        }
        else
        {
            for ( int i = 0; i < testData.getMeasurement().size(); i++ )
            {
                int    baseSeconds = testData.getMeasurement().get(i).getElapsed();
                double resistance  = testData.getMeasurement().get(i).getResistance();
                int    voltage     = testData.getMeasurement().get(i).getVoltage();

                String formattedMilliseconds = formatData.formatElapsedTime((short)baseSeconds);
                String formattedResistance   = formatData.formatResistance(resistance);
                String formattedVoltage      = formatData.formatVoltage(voltage);
                String calculatedCurrent     = formatData.formatCurrent((double)voltage/resistance);

                values = new ArrayList<>();
                values.add(formattedMilliseconds);
                values.add(formattedVoltage);
                values.add(formattedResistance);
                values.add(calculatedCurrent);
                if (isHighlighted(testData.getDAI(), testData.getMeasurement().get(i)) || isHighlighted(testData.getPI(), testData.getMeasurement().get(i)))
                    this.drawHighlightedRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
                else
                    this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), values);
            }
        }

        drawGeneratedBy();

        // finish the page
        document.finishPage(page);
        // draw text on the graphics object of the page

//        // Create Page 2
//        pageInfo = new PdfDocument.PageInfo.Builder(Paper.Letter.getWidth(), Paper.Letter.getHeight(), 2).create();
//        page = document.startPage(pageInfo);
//        canvas = page.getCanvas();
//        paint = new Paint();
//        paint.setColor(Color.BLUE);
//        canvas.drawCircle(100, 100, 100, paint);
//        document.finishPage(page);

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.saving_file)).sendToTarget();

        String docFile = testData.getModel() + "_" + testData.getSerial() + "_" + testData.getDate().getTime() + "_report.pdf";
        File tmpFile   = new File(dir + "/"+ docFile);

        try
        {
            document.writeTo(new FileOutputStream(tmpFile));
            // close the document
            document.close();

            handler.obtainMessage(Controller.mHandler.REPORT_FINISHED, 1, -1, activity.getString(R.string.finished)).sendToTarget();

            sendFile(tmpFile);
        }
        catch (IOException e)
        {
            Log.e(TAG, "error "+e.toString());
            handler.obtainMessage(Controller.mHandler.REPORT_ERROR, 1, -1, activity.getString(R.string.finished)).sendToTarget();
        }


    }

}

