package com.megabras.bluelogg.mpk257;

import android.animation.ValueAnimator;
import android.annotation.SuppressLint;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Handler;
import android.os.Message;
import android.text.InputFilter;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.TimePicker;
import android.widget.Toast;

import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.InputFilterMinMax;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.modbus.MPK257.Device;
import com.megabras.modbus.MPK257.FormatData;
import com.megabras.modbus.MPK257.Measurement;
import com.megabras.modbus.MPK257.Protocol;
import com.megabras.modbus.MPK257.TestData;
import com.megabras.modbus.Modbus;

import java.io.File;
import java.lang.ref.WeakReference;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;

//import android.support.v4.app.FragmentActivity;
//import android.support.v4.view.ViewPager;

@SuppressWarnings({"FieldCanBeLocal", "CodeBlock2Expr"})
public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Picture.PictureInterface
{

    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    private final String
            TAG = "mpk257.Controller";

    public static String
            model,
            serial;

//    static final String
//            fullModel = "MPK257";

    private DecimalFormat
            format02 = new DecimalFormat("00");

    static int
            firmware,
            minFirmareVersion  = 2;

    private int
            minFirmareVersion2 = Protocol.FIRMWARE_VOLTAGE,
            controlButtonsID   = R.layout.mpk257_control_buttons,
            generalSetupID     = R.layout.main_fgr_general_setup,
            testSetupID        = R.layout.mpk257_fgr_test_setup,
            saveOptionsID      = R.layout.mpk257_dialog_save,
            optionsID,
            currentTestDataID  = -1,
            memoryPointer,
            PWM                = 0;

    private FragmentActivity
            fragmentActivity;

    private Menu
            menu;

    private MenuItem
            optionsButton;

    private MainFragment
            mainFragment;

    private ChartFragment
            table;

    private ViewPager
            viewPager;

    private View
            viewSaveOptions,
            viewGeneral,
            viewTest;

    private LinearLayout
            toolBar;

    private com.megabras.modbus.MPK257.Protocol
            modbus = new com.megabras.modbus.MPK257.Protocol();

    private com.megabras.modbus.MPK257.Device
            device = new com.megabras.modbus.MPK257.Device();

//    private Space
//            bt_space;

    private ImageButton
            btSave,
            btFilter,
            btSetup,
            btPrinter,
            btCurrentSmallA,
            btCurrentSmallB,
            btCurrentMediumA,
            btCurrentMediumB,
            btCurrentLargeA,
            btCurrentLargeB;

    private Button
            btStart,
            btStop;

    private LinearLayout
            linePwmControls;

    private boolean
            testRunning     = false,
            showOverlay     = false,
            isSaving        = false,
            warnedLock      = false,
            configChanged   = false,
            showSaveDialog,
            saveInSameObject,
            waitingAutoSave = false,
            bAlreadySended  = false;

    private CommonUI
            mCommonUI;

    private ValueAnimator
            //finger,
            anim,
            anim2;

    private com.megabras.modbus.MPK257.TestData
            testData;

    private Database
            database;

    private Report
            report;

    private Picture
            picture;

    private GPS
            mGPS;

    private GoogleMaps
            map;

    private Database.ObjectData
            currentObject = null;

    private ImageButton
            buttonGPS,
            buttonPicture,
            buttonVoice,
            buttonMap;

    private mHandler
            handler;

    private Dialog
            dlgProgress,    
            dlgFinished;

    private SharedPreferences
            sharedPrefs;

    private SharedPreferences.Editor
            editors;

    private com.megabras.bluelogg.extended.Audio
            audio;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    @SuppressLint("CommitPrefEdits")
    public Controller( FragmentActivity fragmentActivity, CommonUI ui, String model, String serial )
    {
        this.fragmentActivity = fragmentActivity;
        //TEMP
        Protocol.setRevisionMap(4);//TEMP
        viewGeneral     = View.inflate(fragmentActivity, generalSetupID, null);
        viewTest        = View.inflate(fragmentActivity, testSetupID   , null);
        viewSaveOptions = View.inflate(fragmentActivity, saveOptionsID, null);
        mCommonUI       = ui;
        picture         = Picture.getInstance();
        picture.initialize(fragmentActivity, this);
        mGPS            = new GPS(this, fragmentActivity);
        audio           = new Audio(fragmentActivity);
        map             = new GoogleMaps(fragmentActivity);
        handler         = new mHandler(this);
        handler.postDelayed( new Runnable()
        {
            public void run()
            {
                stateMachineUI();
                handler.postDelayed( this, 500 );
            }
        }, 500 );
        Controller.model    = model;
        Controller.serial   = serial;

        sharedPrefs      = fragmentActivity.getSharedPreferences("BLUELOGG"+model, Context.MODE_PRIVATE);
        editors          = sharedPrefs.edit();
        showSaveDialog   = sharedPrefs.getBoolean("showDialogSaveOptions", true);
        saveInSameObject = sharedPrefs.getBoolean("saveInSameObject", true);

        testData         = new TestData(model, serial);
        database         = new Database(fragmentActivity, model);
        //report           = new Report(fragmentActivity, handler, database.getFolder(), database.getTemporaryFolder());
        report           = new Report(fragmentActivity, database.getFolder(), database.getTemporaryFolder(), handler);

        LogCat.LEVEL = LogCat.LEVEL_NONE;
    }

    /***********************************************************************************************
     * REGION On Destroy
     ***********************************************************************************************/

    public void onClose(){
        if (mGPS != null) {
            mGPS.cancel();
        }
        if (picture != null) {
            picture.close();
        }
//        if (shutter != null){
//            shutter.release();
//        }
    }

    public void setToolBarId(int toolBarId){
        toolBar = fragmentActivity.findViewById(toolBarId);
        toolBar.findViewById(R.id.bt_details).setVisibility(View.GONE);
        toolBar.findViewById(R.id.bt_save).setVisibility(View.GONE);
        toolBar.findViewById(R.id.bt_export).setVisibility(View.GONE);

        buttonMap = toolBar.findViewById(R.id.btn_map);
        buttonMap.setOnClickListener( v ->
        {
            if(Main.permissionToMemoryAccepted)
            {
                if (currentObject != null && !currentObject.getLongitude().equals("")) {
                    MapData mapData = new MapData(
                            currentObject.getName(),
                            Double.parseDouble(currentObject.getLongitude()),
                            Double.parseDouble(currentObject.getLatitude()),
                            currentObject.getAltitude(),
                            currentObject.getGpsAccuracy()
                    );
                    map.showMap(database.getMapSnapshotFile(currentObject.getId()), mapData);
                }
            }
            else
            {
                ((Main)fragmentActivity).requestPermission(Main.REQUEST_MEMORY_PERMISSION);
            }
        } );

        toolBar.findViewById(R.id.bt_manager).setOnClickListener( v ->
        {
            filesManagerPage();
        } );

        buttonPicture = toolBar.findViewById(R.id.bt_picture);
        buttonPicture.findViewById(R.id.bt_picture).setOnClickListener( v ->
        {
            if ( Main.permissionToPictureAccepted )
                takePicture();
            else
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_PICTURE_PERMISSION );
        } );

        buttonGPS = toolBar.findViewById(R.id.bt_gps);
        buttonGPS.setOnClickListener( v ->
        {
            if (Main.permissionToGPSAccepted)
                getGPS();
            else
                ((Main)fragmentActivity).requestPermission(Main.REQUEST_GPS_PERMISSION);
        } );

        buttonVoice = toolBar.findViewById(R.id.bt_record);
        buttonVoice.setOnClickListener( v ->
        {
            if (Main.permissionToRecordAccepted)
                recordAudio();
            else
                ((Main)fragmentActivity).requestPermission(Main.REQUEST_RECORD_AUDIO_PERMISSION);
        } );
    }

    public void setAutoConn(boolean enable){
        device.setAutoConnection(enable);
    }

    public void setDateTimeFormat(int dateFormat, int timeFormat){
        device.setDateFormat(dateFormat);
        device.setTimeFormat(timeFormat);
    }

    public void setTemperatureUnity(int unity){
        device.setTempUnity(unity);
    }

    public boolean getAutoConn(){
        return device.getAutoConnection();
    }

    /***********************************************************************************************
     * REGION Methods
     ***********************************************************************************************/

    public void createControlButtons(LinearLayout container){
        final View commandButtons = View.inflate(fragmentActivity, controlButtonsID, null);

        commandButtons.setLayoutParams(
                new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        LinearLayout.LayoutParams.WRAP_CONTENT));
        container.removeAllViews();
        container.addView(commandButtons);

        linePwmControls = commandButtons.findViewById(R.id.line_current);

        final int smallStep = 1;
        btCurrentSmallA = commandButtons.findViewById(R.id.bt_0);
        btCurrentSmallA.setOnClickListener( v ->
        {
            if(device.getCurrentControl()>0)
                PWM += -smallStep; //device.setCurrentControl(device.getCurrentControl()-smallStep);
        } );

        btCurrentSmallB = commandButtons.findViewById(R.id.bt_10);
        btCurrentSmallB.setOnClickListener( v ->
        {
            if(device.getCurrentControl()<1000)
                PWM += smallStep; //device.setCurrentControl(device.getCurrentControl()+smallStep);
        } );

        final int mediumStep = 25;
        btCurrentMediumA = commandButtons.findViewById(R.id.bt_20);
        btCurrentMediumA.setOnClickListener( v ->
        {
            if(device.getCurrentControl()>0)
                PWM += -mediumStep; // device.setCurrentControl(device.getCurrentControl()-mediumStep);
        } );

        btCurrentMediumB = commandButtons.findViewById(R.id.bt_30);
        btCurrentMediumB.setOnClickListener( v ->
        {
            if(device.getCurrentControl()<1000)
                PWM += mediumStep; // device.setCurrentControl(device.getCurrentControl()+mediumStep);
        } );

        final int bigStep = 50;
        btCurrentLargeA = commandButtons.findViewById(R.id.bt_40);
        btCurrentLargeA.setOnClickListener( v ->
        {
            if(device.getCurrentControl()>0)
                PWM += -bigStep; // device.setCurrentControl(device.getCurrentControl()-bigStep);
        } );

        btCurrentLargeB = commandButtons.findViewById(R.id.bt_50);
        btCurrentLargeB.setOnClickListener( v ->
        {
            if(device.getCurrentControl()<1000)
                PWM += bigStep; // device.setCurrentControl(device.getCurrentControl()+bigStep);
        } );

        btSetup = commandButtons.findViewById(R.id.bt_setup);
        btSetup.setOnClickListener( v ->
        {
            showOverlay = true;
            readTestConfig();
        } );

        btStart = commandButtons.findViewById( R.id.bt_start );
        btStart.setOnClickListener( v ->
        {
            if( currentObject != null && showSaveDialog )
            {
                saveOptionPage();
            }
            else
            {
                if( !saveInSameObject )
                {
                    currentObject = null;
                    createSwipe();
                }
                userMessage( fragmentActivity.getString(R.string.md_warning_test_start), CommonUI.COLOR_YELLOW, true );

                sendMessage( modbus.write.stop() );
                sendMessage( modbus.write.start() );
            }
        } );

        btStop = commandButtons.findViewById(R.id.bt_stop);
        btStop.setOnClickListener( v ->
        {
            sendPriorityMessage( modbus.write.stop() );
        } );

        btPrinter = commandButtons.findViewById(R.id.bt_printer);
        btPrinter.setOnClickListener( v ->
        {
            sendMessage( modbus.write.print() );
        } );

        btSave = commandButtons.findViewById(R.id.bt_save);
        btSave.setOnClickListener( v ->
        {
            if (testRunning)
            {
                isSaving = true;
                userMessage(fragmentActivity.getString(R.string.saving), CommonUI.COLOR_YELLOW, true);
                sendMessage(modbus.write.save());
            }
        } );

//        bt_space = commandButtons.findViewById(R.id.mpk_bt_space);

        btFilter = commandButtons.findViewById(R.id.bt_filter);
        btFilter.setOnClickListener( v ->
        {
            userMessage(fragmentActivity.getString(R.string.updating), CommonUI.COLOR_YELLOW, true );
            sendMessage( modbus.write.filterOnOff() );
        } );

        btStart.setEnabled(false);
        btStop.setEnabled(false);
        btStop.setVisibility(View.GONE);
        mCommonUI.enableButton(btSave,    false);
        mCommonUI.enableButton(btPrinter, false);
//        btStop.setVisibility(View.VISIBLE);
//        bt_space.setVisibility(View.GONE);
//        btSave.setVisibility(View.GONE);
//        btPrinter.setVisibility(View.GONE);

        mCommonUI.enableButton(btFilter,  false);
        mCommonUI.enableButton(btSetup,   false);

        linePwmControls.setEnabled(false);
        linePwmControls.setVisibility(View.GONE);
    }

    private void newObject(String name)
    {
        currentObject = database.getNewObjectData();

        if(name == null)
            name  = fragmentActivity.getResources().getString(R.string.object) + " " + currentObject.getId();
        database.addObject(name, testData.getModel(), testData.getSerial());
        currentObject.setName(name);
        table.update(currentObject);
    }

    private void saveInDatabase(TestData data)
    {
        if(currentObject==null)
            newObject(null);

        if(currentTestDataID==-1)
            currentTestDataID = database.addTestData(currentObject.getId(), data);

        database.addMeasurement(currentTestDataID, data.getMeasurementLast());
    }

    private void loadObjectData(int objectID)
    {
        createSwipe();
        currentObject = database.getObjectData(objectID);

        if(currentObject==null)
            return;

        table.update(currentObject);

        ArrayList<TestData> tests = database.getTestData(currentObject.getId());

        if(tests==null)
            return;

        for(TestData t : tests)
        {
            // CREATE THE HEADER
            table.setHeader(t.getTestNumber(), "", "", "");

            boolean isTempEnabled = false;
            ArrayList<Measurement> meas = database.getMeasurements(t.getID());

            if( meas != null )
            {
                for( Measurement m : meas )
                {
                    t.addMeasument(m);

                    //CHECK IF TEMPERATURE COMP IS ENABLED
                    if ( m.getResistanceCompensated() > 0 && m.getTemperature() > 0 )
                    {
                        isTempEnabled = true;
                    }

                    table.updateRows(m, device, false);
                }
            }

            Device d     = t.getSetup();
            String temp  = isTempEnabled ? FormatData.getStringTempRange( device.getTempUnity(), d.getTempRange() ) : "";
            String metal = isTempEnabled ? FormatData.getStringMetal( d.getMetalType() ) : "";
            String date  = FormatData.getDateFormatted( t.getDate(), device.getDateFormat(), device.getTimeFormat() );

            // UPDATE THE HEADER
            table.updateLastHeader(t.getTestNumber(), date, temp, metal);
            table.repaint();
        }
    }

    private void createObjectNameDialog()
    {
        final Dialog dialog = new Dialog( fragmentActivity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_object_new );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
        }

        ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.object_new );
        ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_new );

        final TextView objectName = dialog.findViewById( R.id.txt_object_name );

        dialog.findViewById( R.id.btn_object_cancel ).setOnClickListener( v -> dialog.dismiss() );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> dialog.dismiss() );

        dialog.findViewById(R.id.btn_object_save).setOnClickListener( v ->
        {
            createSwipe();
            String objName = objectName.getText().toString();
            if(!objName.equals("")){
                newObject(objName);
                dialog.dismiss();
            }
        } );

        dialog.show();
    }

    private void dialogFinished()
    {
        if ( dlgFinished == null )
        {
            dlgFinished = new Dialog( fragmentActivity );
            dlgFinished.requestWindowFeature( Window.FEATURE_NO_TITLE );
            dlgFinished.setCanceledOnTouchOutside( false );
            dlgFinished.setContentView( R.layout.dialog_yes_no_white );
            if ( dlgFinished.getWindow() != null )
                dlgFinished.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;


            dlgFinished.findViewById( R.id.title_bar ).setVisibility( View.GONE );
//            ((TextView) dlgFinished.findViewById( R.id.dlg_title )).setText( R.string.finished );
//            ((ImageView) dlgFinished.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

            ((TextView) dlgFinished.findViewById( R.id.message )).setText( R.string.md_test_finished );

            final Button ui_btn_dlg_no = dlgFinished.findViewById( R.id.btn_no );
            ui_btn_dlg_no.setVisibility( View.GONE );

            final Button ui_btn_dlg_yes = dlgFinished.findViewById( R.id.btn_yes );
            ui_btn_dlg_yes.setText( R.string.ok );
            ui_btn_dlg_yes.setOnClickListener( v ->
            {
                dlgFinished.dismiss();
            } );
        }

        dlgFinished.show();
    }

    private void createObjectRenameDialog( final int id, String name )
    {
        final Dialog dialog = new Dialog( fragmentActivity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_object_new );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
        }

        ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.object_new );
        ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_new );

        final TextView objectName = dialog.findViewById( R.id.txt_object_name );
        objectName.setText( name );
        objectName.setSelected( true );

        View.OnClickListener onClose = view ->
        {
            filesManagerPage();
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_object_cancel ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        dialog.findViewById(R.id.btn_object_save).setOnClickListener( v ->
        {
            String objName = objectName.getText().toString();
            if(!objName.equals("")){
                database.updateObjectName(id, objName);
                if(currentObject!=null && currentObject.getId()==id)
                    loadObjectData(id);
                filesManagerPage();
                dialog.dismiss();
            }
        } );

        dialog.show();
    }

    public void createSwipe() {
        PagerAdapter mPagerAdapter =
                new PagerAdapter(
                        fragmentActivity.getSupportFragmentManager(),
                        fragmentActivity.getBaseContext());

        viewPager = fragmentActivity.findViewById(R.id.pager);

        if (viewPager == null)
            return;

        viewPager.setAdapter(mPagerAdapter);
        viewPager.setOffscreenPageLimit(4);

        mainFragment    = mPagerAdapter.getMainFragment();
        table           = mPagerAdapter.table();

//        table.setButtonAction( v ->
//        {
//            currentObject=null;
//            createSwipe();
//        } );

        //Create customs Tabs
        SlidingTabLayout mSlidingTabLayout =
                fragmentActivity.findViewById(R.id.sliding_tabs);
        mSlidingTabLayout.setDistributeEvenly(true);
        mSlidingTabLayout.setCustomTabView(R.layout.custom_tab, 0);
        mSlidingTabLayout.setBackground(
                fragmentActivity.getResources().getDrawable(R.drawable.chamfer_black_top) );
        mSlidingTabLayout.setCustomTabColorizer(new SlidingTabLayout.TabColorizer() {
            public int getIndicatorColor(int position) {
                return Color.GRAY;
            }

            public int getDividerColor(int position) {
                return Color.WHITE;
            }
        });
        mSlidingTabLayout.setViewPager(viewPager);

        if(device!=null)
            changeGauge(device.getCurrentRange());
    }

    private void filesManagerPage(){
        final View view                = View.inflate(fragmentActivity, R.layout.mpk257_files_manager, null);
        final FrameLayout overlay      = MUtils.getOverlay(fragmentActivity, OVERLAY_ID, view);
        final ListView list            = view.findViewById(R.id.list_savedfiles);
        final ArrayList<Database.ObjectData> objects = database.getObjectsList();
        final String[] objectsNameList = new String[objects.size()];

        final Button
                btDelete = view.findViewById(R.id.bt_delete),
                btOpen   = view.findViewById(R.id.bt_open),
                btNew    = view.findViewById(R.id.bt_new),
                btRename = view.findViewById(R.id.bt_rename),
                btClose  = view.findViewById(R.id.bt_close),
                btExport = view.findViewById(R.id.bt_export);

        if(currentObject==null){
            btDelete.setEnabled(false);
            btOpen.setEnabled(false);
            btRename.setEnabled(false);
            btClose.setEnabled(false);
            btExport.setEnabled(false);
        }

        int selectedListIndex = -1;
        for(int i = 0; i<objects.size(); i++){
            objectsNameList[i] = objects.get(i).getName()+"";
            if(currentObject!=null && currentObject.getId()==objects.get(i).getId())
                selectedListIndex = i;
        }

        final ArrayAdapter<String> adapter = new ArrayAdapter<>(fragmentActivity,
                android.R.layout.simple_list_item_activated_1, android.R.id.text1, objectsNameList);

        list.setAdapter(adapter);
        list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        list.setItemChecked(selectedListIndex, true);

        list.setOnItemClickListener( ( parent, view1, position, id ) ->
        {
            btDelete.setEnabled(true);
            btOpen.setEnabled(true);
            btRename.setEnabled(true);
            btExport.setEnabled(true);
        } );

        btDelete.setOnClickListener( v ->
        {

            final Dialog dialog = new Dialog(fragmentActivity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setContentView(R.layout.dialog_yes_no);
            if (dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.erase);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.file_delete_confirm);

            dialog.findViewById( R.id.btn_no ).setOnClickListener( v1 ->
            {
                dialog.dismiss();
            } );

            dialog.findViewById(R.id.btn_yes).setOnClickListener( v12 ->
            {
                int selectedIndex = list.getCheckedItemPosition();
                if(selectedIndex > -1){
                    database.deleteObject( objects.get(selectedIndex).getId() );
                    if(currentObject!=null && currentObject.getId()==objects.get(selectedIndex).getId()){
                        currentObject=null;
                        createSwipe();
                    }
                }
                dialog.dismiss();
                closeOverlay(overlay);
                filesManagerPage();
            } );

            dialog.show();
        } );

        btOpen.setOnClickListener( v ->
        {
            int selectedIndex = list.getCheckedItemPosition();
            if(selectedIndex > -1){
                loadObjectData(objects.get(selectedIndex).getId());
            }
            closeOverlay(overlay);
            viewPager.setCurrentItem(1, true);
        } );

        btNew.setOnClickListener( v ->
        {
            //createSwipe();
            createObjectNameDialog();
            closeOverlay(overlay);
        } );

        btRename.setOnClickListener( v ->
        {
            int selectedIndex = list.getCheckedItemPosition();
            if(selectedIndex > -1){
                createObjectRenameDialog(objects.get(selectedIndex).getId(), objects.get(selectedIndex).getName());
                closeOverlay(overlay);
            }
        } );

        btClose.setOnClickListener( v ->
        {
            list.clearChoices();
            adapter.notifyDataSetChanged();
            //allTestData.clear();
            currentObject=null;
            createSwipe();
            btClose.setEnabled(false);
            btRename.setEnabled(false);
            btOpen.setEnabled(false);
            btDelete.setEnabled(false);
        } );

        btExport.setOnClickListener( v ->
        {
            if(Main.permissionToMemoryAccepted)
            {
                int selectedIndex = list.getCheckedItemPosition();
                if(selectedIndex > -1){
                    //String selectedItem           = adapter.getItem(selectedIndex);
                    final Database.ObjectData obj = database.getObjectData(objects.get(selectedIndex).getId());
                    if ( obj==null )
                        return;
                    final ArrayList<TestData> d   = database.getTestData(obj.getId());
                    if( d==null || d.size()==0 ) {
                        Toast.makeText(fragmentActivity, R.string.no_measurements, Toast.LENGTH_SHORT).show();
                        return;
                    }

                    for (TestData t : d){
                        ArrayList<Measurement> meas = database.getMeasurements(t.getID());
                        if ( meas==null )
                            return;
                        for(Measurement m : meas)
                            t.addMeasument(m);
                    }
                    if(dlgProgress==null)
                        creatDialogProgress();
                    dlgProgress.show();

                    new Thread( () ->
                    {
                        report.saveDocReport(
                            obj,
                            d,
                            device,
                            database.getPictureFile( obj.getId() ),
                            database.getMapSnapshotFile( obj.getId() ),
                            database.getReportFile( obj.getId() )
                        );
                    } ).start();
                    closeOverlay(overlay);
                }
            }
            else
            {
                ((Main)fragmentActivity).requestPermission(Main.REQUEST_MEMORY_PERMISSION);
            }
        } );

        view.findViewById( R.id.bt_cancel ).setOnClickListener( v ->
        {
            closeOverlay( overlay );
        } );
    }

    private void saveOptionPage()
    {
        final FrameLayout overlay = MUtils.getOverlay( fragmentActivity, OVERLAY_ID, viewSaveOptions );
        final CheckBox cbRemember = viewSaveOptions.findViewById( R.id.cb_remember );

        Button buttonSave = viewSaveOptions.findViewById( R.id.bt_same );
        buttonSave.setOnClickListener( v ->
        {
            showSaveDialog = !cbRemember.isChecked();
            editors.putBoolean( "showDialogSaveOptions", showSaveDialog );
            editors.commit();
            if( cbRemember.isChecked() )
            {
                saveInSameObject = true;
                editors.putBoolean( "saveInSameObject", true );
                editors.commit();
            }
            userMessage( fragmentActivity.getString(R.string.md_warning_test_start), CommonUI.COLOR_YELLOW, true );

            sendMessage( modbus.write.stop() );
            sendMessage( modbus.write.start() );

            closeOverlay( overlay );
        } );

        Button buttonNew = viewSaveOptions.findViewById( R.id.bt_new );
        buttonNew.setOnClickListener( v ->
        {
            showSaveDialog = !cbRemember.isChecked();
            editors.putBoolean( "showDialogSaveOptions", showSaveDialog );
            editors.commit();
            if( cbRemember.isChecked() )
            {
                saveInSameObject = false;
                editors.putBoolean( "saveInSameObject", false );
                editors.commit();
            }
            currentObject = null;
            createSwipe();
            userMessage( fragmentActivity.getString(R.string.md_warning_test_start), CommonUI.COLOR_YELLOW, true );

            sendMessage( modbus.write.stop() );
            sendMessage( modbus.write.start() );

            closeOverlay( overlay );
        } );

        Button buttonCancel = viewSaveOptions.findViewById( R.id.bt_cancel );
        buttonCancel.setOnClickListener( v ->
        {
            closeOverlay( overlay );
        } );
    }

    private void setupPage(){
        final FrameLayout
                overlay = MUtils.getOverlay(fragmentActivity, OVERLAY_ID, viewGeneral);

        final TextView
                date   = viewGeneral.findViewById(R.id.device_date),
                time   = viewGeneral.findViewById(R.id.device_time),
                memory = viewGeneral.findViewById(R.id.device_memory);

        final Spinner
                spinnerGauge      = getSpinner(R.id.spinnerGauge,          R.array.hp60kv_gauge,     viewGeneral),
                spinnerConnect    = getSpinner(R.id.spinnerAutoConnection, R.array.enable_disable,   viewGeneral),
                spinnerDateFormat = getSpinner(R.id.spinnerDateFormat,     R.array.date_format,      viewGeneral),
                spinnerTimeFormat = getSpinner(R.id.spinnerTimeFormat,     R.array.time_format,      viewGeneral),
                spinnerLanguage   = getSpinner(R.id.spinnerLanguage,       R.array.language_options, viewGeneral);

        date.setText  ( FormatData.date(device.getDateFormat(), device.getDate()) );
        time.setText  ( FormatData.time(device.getTimeFormat(), device.getDate()) );
        String free = device.getFreeMemory() + " %";
        memory.setText( free );

        getLL(R.id.line_gauge, viewGeneral).setVisibility(View.GONE);

        ImageButton btEraseMem = viewGeneral.findViewById(R.id.button_memory);
        btEraseMem.setOnClickListener( v ->
        {
            final Dialog dialog = new Dialog(fragmentActivity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setContentView(R.layout.dialog_yes_no);
            if (dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.erase_memory);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.erase_memory_confirm);

            final Button btnNo = dialog.findViewById(R.id.btn_no);
            btnNo.setOnClickListener( v1 ->
            {
                dialog.dismiss();
            } );

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener( v12 ->
            {
                sendMessage( modbus.write.memoryErase() );
                userMessage( fragmentActivity.getString(R.string.erase_memory), CommonUI.COLOR_YELLOW, true );
                memory.setText( fragmentActivity.getString(R.string.wait) );
                dialog.dismiss();
            } );

            dialog.show();
            //closeOverlay(overlay);
        } );

        if(device.getAutoConnection()){
            spinnerConnect.setSelection(0);
        } else {
            spinnerConnect.setSelection(1);
        }

        switch (device.getDateFormat()){
            case Modbus.DATE_DD_MM_YY:
                spinnerDateFormat.setSelection(0);
                break;
            case Modbus.DATE_MM_DD_YY:
                spinnerDateFormat.setSelection(1);
                break;
            default:
                spinnerDateFormat.setSelection(2);
                break;
        }

        switch (device.getTimeFormat()){
            case Modbus.TIME_24H:
                spinnerTimeFormat.setSelection(0);
                break;
            default:
                spinnerTimeFormat.setSelection(1);
                break;
        }

        switch (device.getLanguage()){
            case Modbus.LANG_ENGLISH:
                spinnerLanguage.setSelection(0);
                break;
            case Modbus.LANG_SPANISH:
                spinnerLanguage.setSelection(1);
                break;
            default:
                spinnerLanguage.setSelection(2);
                break;
        }

        switch (mainFragment.getGaugeType()){
            case MainFragment.TYPE_VOLTAGE:
                spinnerGauge.setSelection(0);
                break;
            case MainFragment.TYPE_CURRENT:
                spinnerGauge.setSelection(1);
                break;
        }

        ImageButton buttonDate = viewGeneral.findViewById(R.id.button_date);
        buttonDate.setOnClickListener( v ->
        {
            pickDate();
        } );

        ImageButton buttonTime = viewGeneral.findViewById(R.id.button_time);
        buttonTime.setOnClickListener( v ->
        {
            pickTime();
        } );

        Button buttonSave = viewGeneral.findViewById(R.id.buttonSave);
        buttonSave.setOnClickListener( v ->
        {
            switch ( spinnerConnect.getSelectedItemPosition() )
            {
                case 0:
                    device.setAutoConnection( true );
                    break;
                case 1:
                    device.setAutoConnection( false );
                    break;
            }

            int dateFormat, timeFormat, language;
            // DATE
            switch (spinnerDateFormat.getSelectedItemPosition()) {
                case 0:
                    dateFormat = Modbus.DATE_DD_MM_YY;
                    break;
                case 1:
                    dateFormat = Modbus.DATE_MM_DD_YY;
                    break;
                default:
                    dateFormat = Modbus.DATE_YYYY_MM_DD;
                    break;
            }

            switch (spinnerTimeFormat.getSelectedItemPosition()) {
                case 0:
                    timeFormat = Modbus.TIME_24H;
                    break;
                default:
                    timeFormat = Modbus.TIME_12H;
                    break;
            }

            switch (spinnerLanguage.getSelectedItemPosition()) {
                case 0:
                    language = Modbus.LANG_ENGLISH;
                    break;
                case 1:
                    language = Modbus.LANG_SPANISH;
                    break;
                default:
                    language = Modbus.LANG_PORTUGUESE;
                    break;
            }

            // Is reload necessary
            boolean reload = false;
            if( device.getDateFormat()!=dateFormat || device.getTimeFormat()!=timeFormat){
                reload = true;
            }

            device.setDateFormat(dateFormat);
            device.setTimeFormat(timeFormat);
            device.setLanguage(language);

            //sendMessage( modbus.write.generalConfig(dateFormat, timeFormat, language) );
            userMessage(fragmentActivity.getString(R.string.updating), CommonUI.COLOR_YELLOW, true );
            sendMessage( modbus.write.date( device.getDate() ) );
            sendMessage( modbus.write.setup(device) );
            ((Main)fragmentActivity).updateAutoConn(device.getAutoConnection());
            ((Main)fragmentActivity).updateDateTimeFormat(dateFormat, timeFormat);
            closeOverlay(overlay);

            // Reload table
            if( reload && currentObject!=null){
                loadObjectData(currentObject.getId());
            }
        } );

        Button buttonCancel = viewGeneral.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener( v ->
        {
            userMessage(fragmentActivity.getString(R.string.ready), CommonUI.COLOR_GREEN, false );
            optionsButton.setEnabled(true);
            closeOverlay(overlay);
        } );
    }

    @SuppressWarnings("SameParameterValue")
    private void pickTimer( final TextView tv, int minTime, int maxTime, int stepTime)
    {
        final Dialog dialog = new Dialog(fragmentActivity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_timer);
        if (dialog.getWindow()!=null) dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumberMM = dialog.findViewById(R.id.et_number_mm);
        final EditText etNumberSS = dialog.findViewById(R.id.et_number_ss);
        final TextView message    = dialog.findViewById(R.id.tv_message);
        final TextView warning    = dialog.findViewById(R.id.tv_warning);

        message.setText( fragmentActivity.getString(R.string.test_duration) );

        int mmMax = maxTime / 60;

        etNumberMM.setFilters(new InputFilter[]{new InputFilterMinMax(0, mmMax)});
        etNumberSS.setFilters(new InputFilter[]{ new InputFilterMinMax(0, 59)});

        String[] parts = tv.getText().toString().split(":");
        String num     = Integer.parseInt(parts[0])+"";
        etNumberMM.setText( num      );
        etNumberSS.setText( parts[1] );

        etNumberMM.setSelectAllOnFocus( true );
        etNumberSS.setSelectAllOnFocus( true );

        View.OnFocusChangeListener onFocus = new View.OnFocusChangeListener()
        {
            String temp = "";
            @Override
            public void onFocusChange(View v, boolean hasFocus)
            {
                EditText et = (EditText) v;
                if(hasFocus)
                {
                    temp = et.getText().toString();
                    et.setText("");
                }
                else
                {
                    if (et.getText().toString().equals(""))
                        et.setText(temp);
                }
            }
        };

        etNumberMM.setOnFocusChangeListener( onFocus );
        etNumberSS.setOnFocusChangeListener( onFocus );

        String warningMessage;
        if ( stepTime > 0 )
            warningMessage = fragmentActivity.getString(R.string.range) + ": " + minTime    + " s - " + maxTime    + " s ("+fragmentActivity.getString(R.string.step, stepTime + " s)");
        else
            warningMessage = fragmentActivity.getString(R.string.range) + ": " + minTime/60 + " m / " + maxTime/60 + " m";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( etNumberMM.getText().toString().equals("") || etNumberSS.getText().toString().equals("") )
                return;

            int minutes = Integer.parseInt(etNumberMM.getText().toString());
            int seconds = Integer.parseInt(etNumberSS.getText().toString());
            if ( stepTime > 0 )
                seconds = ( seconds / stepTime ) * stepTime;
            int time    = ( minutes * 60 ) + seconds;

            if ( stepTime > 0 )
            {
                if ( time >= minTime && time <= maxTime )
                {
                    String elapsed = format02.format(minutes) + ":" + format02.format(seconds);
                    tv.setText(elapsed);
                    dialog.dismiss();
                }
                else
                {
                    warning.setTextColor(fragmentActivity.getResources().getColor(R.color.common_signin_btn_default_background));
                }
            }
            else
            {
                if ( time == minTime || time == maxTime )
                {
                    String elapsed = format02.format(minutes) + ":" + format02.format(seconds);
                    tv.setText(elapsed);
                    dialog.dismiss();
                }
                else
                {
                    warning.setTextColor(fragmentActivity.getResources().getColor(R.color.common_signin_btn_default_background));
                }
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    @SuppressWarnings("SameParameterValue")
    private void pickNumber( final TextView tv, int title, final String unity, final int min, final int max, final int step)
    {
        final Dialog dialog = new Dialog(fragmentActivity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnity  = dialog.findViewById(R.id.tv_unit);

        message.setText( fragmentActivity.getString(title) );
        tvUnity.setText(unity);
        etNumber.setText( tv.getText().toString().replace(unity, "").trim() );

        etNumber.setSelection(0, etNumber.getText().length());
        etNumber.requestFocus();
        final InputMethodManager imm = (InputMethodManager) fragmentActivity.getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY);

        String warningMessage =
                fragmentActivity.getString(R.string.range)
                        + ": " + min + " " + unity + " - "
                        + max + " " + unity
                        + " ("+fragmentActivity.getString(R.string.step, step+" " + unity)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( etNumber.getText().toString().equals("") )
                return;

            int number  = Integer.parseInt( etNumber.getText().toString().replace(unity,"").trim() );
            number      = (number/step)*step;

            if( number >= min && number <= max )
            {
                String n = number + " " + unity;
                tv.setText( n );
                imm.hideSoftInputFromWindow(etNumber.getWindowToken(), 0);
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(fragmentActivity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            imm.hideSoftInputFromWindow(etNumber.getWindowToken(), 0);
            dialog.dismiss();
        });

        dialog.show();
    }

    @SuppressWarnings("SameParameterValue")
    private void pickSignedNumber( final TextView tv, int title, final String unity, final int min, final int max, final int step)
    {
        final Dialog dialog = new Dialog(fragmentActivity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_signed_number);
        if (dialog.getWindow()!=null)
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnity  = dialog.findViewById(R.id.tv_unit);
        final Spinner  spSign   = dialog.findViewById(R.id.spSign);

        message.setText( fragmentActivity.getString(title) );
        tvUnity.setText(unity);

        if ( tv.getText().toString().contains( "-" ) )
        {
            spSign.setSelection( 1 );
        }
        else
        {
            spSign.setSelection( 0 );
        }

        etNumber.setText( tv.getText().toString().replace(unity, "").replace( "-", "" ).trim() );

        etNumber.setSelection(0, etNumber.getText().length());
        etNumber.requestFocus();
        final InputMethodManager imm = (InputMethodManager) fragmentActivity.getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY);

        String warningMessage =
                fragmentActivity.getString(R.string.range)
                        + ": " + min + " " + unity + " - +"
                        + max + " " + unity
                        + " ("+fragmentActivity.getString(R.string.step, step+" " + unity)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( etNumber.getText().toString().equals("") )
                return;

            int number  = Integer.parseInt( etNumber.getText().toString().replace(unity,"").trim() );
            if (spSign.getSelectedItemPosition()==1)
            {
                number = number * -1;
            }
            number      = (number/step)*step;

            if( number >= min && number <= max )
            {
                String n = number + " " + unity;
                tv.setText( n );
                imm.hideSoftInputFromWindow(etNumber.getWindowToken(), 0);
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(fragmentActivity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            imm.hideSoftInputFromWindow(etNumber.getWindowToken(), 0);
            dialog.dismiss();
        });

        dialog.show();
    }

    private String[] opt;
    private void setupTestPage()
    {
        showOverlay = false;
        final FrameLayout overlay = MUtils.getOverlay(fragmentActivity, OVERLAY_ID, viewTest);

        final Spinner
                spAutoSave   = getSpinner(R.id.spAutoSave,     R.array.enable_disable, viewTest),
                spTempUnity  = getSpinner(R.id.spTempUnity,    new String[]{"Celcius","Fahrenheit"},    viewTest),
                spRange      = getSpinner(R.id.spCurrentRange, FormatData.getStringCurrentRange(), viewTest),
                spComp       = getSpinner(R.id.spComp,         R.array.enable_disable, viewTest),
                spSaveIn     = getSpinner(R.id.spSaveIn,       R.array.save_in_options, viewTest),
                spMode       = getSpinner(R.id.spMode,         R.array.mpk257_mode_manual_auto, viewTest),
                spMetal;


        String[] arrMetals = fragmentActivity.getResources().getStringArray( R.array.metals );

        if ( firmware < Protocol.FIRMWARE_AUTO_MODE )
        {
            viewTest.findViewById( R.id.line_test_mode ).setVisibility( View.GONE );
            viewTest.findViewById( R.id.lineTimer ).setVisibility( View.GONE );
            viewTest.findViewById( R.id.lineAutoCurrent ).setVisibility( View.GONE );
            viewTest.findViewById( R.id.lineCustomMetal ).setVisibility( View.GONE );
            String[] arrOldMetals = new String[]{ arrMetals[0], arrMetals[1] };
            spMetal = getSpinner(R.id.spMetal, arrOldMetals, viewTest);
        } // END IF ( firmware < Protocol.FIRMWARE_AUTO_MODE )
        else
        {
            spMetal = getSpinner(R.id.spMetal, arrMetals, viewTest);

            switch ( device.getTestMode() )
            {
                case Device.MODE_MANUAL:
                    spMode.setSelection( 0 );
                    break;
                case Device.MODE_AUTO:
                    spMode.setSelection( 1 );
                    break;
                case Device.MODE_PROD_LINE:
                    spMode.setSelection( 2 );
                    break;
            }

            spMode.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener()
            {
                @Override
                public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id)
                {
                    if ( position == 0 ) // MANUAL
                    {
                        viewTest.findViewById( R.id.lineTimer ).setVisibility( View.GONE );
                        viewTest.findViewById( R.id.lineAutoCurrent ).setVisibility( View.GONE );
                    }
                    else if ( position == 1 ) // AUTO
                    {
                        viewTest.findViewById( R.id.lineTimer ).setVisibility( View.VISIBLE );
                        viewTest.findViewById( R.id.lineAutoCurrent ).setVisibility( View.VISIBLE );
                    }
                    else // PRODUCTION LINE
                    {
                        viewTest.findViewById( R.id.lineTimer ).setVisibility( View.GONE );
                        viewTest.findViewById( R.id.lineAutoCurrent ).setVisibility( View.GONE );
                    }
                }

                @Override public void onNothingSelected(AdapterView<?> parentView) {}
            });

            // TEST DURATION
            ((TextView)viewTest.findViewById( R.id.ui_txw_timer )).setText( FormatData.formatElapsedTime( (short)device.getAutoDuration() ) );
            viewTest.findViewById( R.id.ui_btn_timer ).setOnClickListener( v ->
            {
                pickTimer( viewTest.findViewById( R.id.ui_txw_timer ), Protocol.AUTO_TIME_MIN, Protocol.AUTO_TIME_MAX, Protocol.AUTO_TIME_STEP );
            });

            // AUTO CONTROL
            String sAutoCurrent = device.getAutoCurrent() + " %";
            ((TextView)viewTest.findViewById( R.id.ui_txw_auto_amp )).setText( sAutoCurrent );
            viewTest.findViewById( R.id.ui_btn_auto_amp ).setOnClickListener( v ->
            {
                pickNumber( viewTest.findViewById( R.id.ui_txw_auto_amp ), R.string.auto_current_value, "%", Protocol.AUTO_VALUE_MIN, Protocol.AUTO_VALUE_MAX, Protocol.AUTO_VALUE_STEP);
            });

            // ALPHA METAL
            spMetal.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener()
            {
                @Override
                public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id)
                {
                    if ( position == Protocol.METAL_CUSTOM )
                    {
                        viewTest.findViewById( R.id.lineCustomMetal ).setVisibility( View.VISIBLE );
                    }
                    else
                    {
                        viewTest.findViewById( R.id.lineCustomMetal ).setVisibility( View.GONE );
                    }
                }

                @Override public void onNothingSelected(AdapterView<?> parentView) {}
            });
            String sAlpha = device.getAlphaMetal() + " ppm";
            ((TextView)viewTest.findViewById( R.id.ui_custom_metal )).setText( sAlpha );
            viewTest.findViewById( R.id.ui_btn_custom_metal ).setOnClickListener( v ->
            {
                pickSignedNumber( viewTest.findViewById( R.id.ui_custom_metal ), R.string.alpha_metal, "ppm", Protocol.AUTO_ALPHA_MIN, Protocol.AUTO_ALPHA_MAX, Protocol.AUTO_ALPHA_STEP);
            });
        } // END ELSE ( firmware < Protocol.FIRMWARE_AUTO_MODE )

        opt = FormatData.getArrayTempRange(device.getTempUnity());

        final Spinner spTempRange = viewTest.findViewById(R.id.spTempRange);
        final ArrayAdapter<CharSequence> filter = new ArrayAdapter<>( fragmentActivity,
                android.R.layout.simple_spinner_item,
                opt );
        filter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
        spTempRange.setAdapter(filter);


        spTempUnity.setEnabled(!testRunning);
        spTempRange.setEnabled(!testRunning);
        spMetal.setEnabled(!testRunning);
        spMode.setEnabled(!testRunning);
        viewTest.findViewById( R.id.ui_btn_timer ).setEnabled(!testRunning);
        viewTest.findViewById( R.id.ui_btn_auto_amp ).setEnabled(!testRunning);
        viewTest.findViewById( R.id.ui_btn_custom_metal ).setEnabled(!testRunning);

        if( Main.equipmentId != 257 )
        {
            getLL(R.id.line_temp_unity, viewTest).setVisibility(View.GONE);
            getLL(R.id.line_temp_ref  , viewTest).setVisibility(View.GONE);
            getLL(R.id.line_metal     , viewTest).setVisibility(View.GONE);
            getLL(R.id.line_temp_comp , viewTest).setVisibility(View.GONE);
        }

        if ( device.getAutoSave() )
            spAutoSave.setSelection( 0 );
        else
            spAutoSave.setSelection( 1 );

        switch ( device.getTempUnity() )
        {
            case Protocol.TEMP_CELCIUS:
                spTempUnity.setSelection(0);
                break;
            case Protocol.TEMP_FAHRENHEIT:
                spTempUnity.setSelection(1);
                break;
        }

        if ( device != null )
        {
            if ( device.statusTemperatureCompensation() )
                spComp.setSelection( 0 );
            else
                spComp.setSelection( 1 );
        }

        spTempRange.setSelection( device.getTempRange() );
        spMetal.setSelection( device.getMetalType() );
        spRange.setSelection( device.getCurrentRange() );

        spTempUnity.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener()
        {
            @Override
            public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id)
            {
                try
                {
                    opt[0] = FormatData.getArrayTempRange(position)[0];
                    opt[1] = FormatData.getArrayTempRange(position)[1];
                    filter.notifyDataSetChanged();
                }
                catch (Exception ignored){}
            }

            @Override public void onNothingSelected(AdapterView<?> parentView) {}
        });

        if ( saveInSameObject )
            spSaveIn.setSelection(0);
        else
            spSaveIn.setSelection(1);

        Button buttonSave = viewTest.findViewById(R.id.buttonSave);
        buttonSave.setOnClickListener( v ->
        {
            if ( spAutoSave.getSelectedItemPosition() == 0 )
                device.setAutoSave( true );
            else
                device.setAutoSave( false );

            // Is reload table necessary?
            int previusTempUnity = device.getTempUnity();
            if ( spTempUnity.getSelectedItemPosition() == 0 )
                device.setTempUnity( Protocol.TEMP_CELCIUS );
            else
                device.setTempUnity( Protocol.TEMP_FAHRENHEIT );

            device.setTempRange( spTempRange.getSelectedItemPosition() );
            device.setMetalType( spMetal.getSelectedItemPosition() );
            device.setCurrentRange( spRange.getSelectedItemPosition() );

            ((Main) fragmentActivity).updateTemperatureUnity( device.getTempUnity() );

            changeGauge( device.getCurrentRange() );
            device.setCurrentTarget(0);
            mainFragment.reset();

            int tempComp = device.statusTemperatureCompensation() ? 0 : 1;
            if( spComp.getSelectedItemPosition() != tempComp )
            {
                sendMessage( modbus.write.temperatureCompensationOnOff() );
            }

            saveInSameObject = spSaveIn.getSelectedItemPosition() == 0;
            editors.putBoolean("saveInSameObject", saveInSameObject);
            editors.commit();

            if ( firmware >= Protocol.FIRMWARE_AUTO_MODE )
            {
                switch ( spMode.getSelectedItemPosition() )
                {
                    case 0:
                        device.setTestMode( Device.MODE_MANUAL );
                        break;
                    case 1:
                        device.setTestMode( Device.MODE_AUTO );
                        break;
                    case 2:
                        device.setTestMode( Device.MODE_PROD_LINE );
                        break;
                }

                String[] parts = ((TextView)viewTest.findViewById( R.id.ui_txw_timer )).getText().toString().split(":");
                int minutes    = Integer.parseInt( parts[0] ) * 60;
                int seconds    = Integer.parseInt( parts[1] );
                int time       = minutes + seconds;
                device.setAutoDuration( time );

                String sAuto = ((TextView)viewTest.findViewById( R.id.ui_txw_auto_amp )).getText().toString().replace( "%", "" ).trim();
                int iAuto = Integer.parseInt( sAuto );
                device.setAutoCurrent( iAuto );

                String sAlpha = ((TextView)viewTest.findViewById( R.id.ui_custom_metal )).getText().toString().replace( "ppm", "" ).trim();
                int iAlpha = Integer.parseInt( sAlpha );
                device.setAlphaMetal( iAlpha );

                sendMessage( modbus.write.autoDetails( device ) );
            }

            userMessage( fragmentActivity.getString( R.string.updating ), CommonUI.COLOR_YELLOW, true );
            sendMessage( modbus.write.currentRange( device.getCurrentRange() ) );
            sendMessage( modbus.write.setup( device ) );
            closeOverlay( overlay );

            // Reload table
            if ( device.getTempUnity() != previusTempUnity && currentObject != null )
            {
                loadObjectData( currentObject.getId() );
            }
        } );

        Button buttonCancel = viewTest.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener( v ->
        {
            userMessage(fragmentActivity.getString(R.string.ready), CommonUI.COLOR_GREEN, false );
            optionsButton.setEnabled(true);
            closeOverlay(overlay);
        } );
    }

    public void sendMessage(com.megabras.modbus.Message message){
        ( (com.megabras.bluelogg.Main)fragmentActivity ).addMessage(message);
    }

    private void sendPriorityMessage(com.megabras.modbus.Message message){
        ( (com.megabras.bluelogg.Main)fragmentActivity ).addPriorityMessage(message);
    }

    private void userMessage(String message, int color, boolean busy){
        ( (com.megabras.bluelogg.Main)fragmentActivity )
                .message(message, color, busy);
    }

    /***********************************************************************************************
     * REGION Handler
     ***********************************************************************************************/

    public static class mHandler extends Handler {
        private final WeakReference<Controller> mClass;
        static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler(Controller c) {
            mClass = new WeakReference<>(c);
        }

        @Override
        public void handleMessage(Message msg) {
            final Controller c = mClass.get();

            switch(msg.what){
                case mHandler.REPORT_UPDATE:
                    c.update( (String) msg.obj );
                    break;
                case mHandler.REPORT_FINISHED:
                    c.dlgProgress.dismiss();
                    File file = (File) msg.obj;
                    c.report.sendFile(file);
                    break;
                case mHandler.REPORT_ERROR:
                    c.dlgProgress.dismiss();
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION Permission
     ***********************************************************************************************/

    public void permissionCallback(int permission)
    {
        switch (permission)
        {
            case Main.REQUEST_MEMORY_PERMISSION:
                database.createFolder(fragmentActivity, model);
                break;
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                database.createFolder(fragmentActivity, model);
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                database.createFolder(fragmentActivity, model);
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
        }
    }

    /***********************************************************************************************
     * REGION Audio
     ***********************************************************************************************/

    private void recordAudio()
    {
        if ( currentObject!=null )
        {
            File location  = database.getAudioFile(currentObject.getId());
            audio.setFile( location );
            audio.showDialog(currentObject.getName());
        }
    }

    /***********************************************************************************************
     * REGION Picture
     ***********************************************************************************************/

    public void takePicture() {
        if(database!=null && currentObject!=null){
            File file = database.getPictureFile(currentObject.getId());
            picture.showDialog(file);
        }
    }

    public void pictureStatus(final int status){
        fragmentActivity.runOnUiThread( () ->
        {
            switch (status)
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_wait, false);
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_picture_states, true);
                    break;
            }
        } );
    }

    /***********************************************************************************************
     * REGION GPS
     ***********************************************************************************************/

    private void getGPS(){
        if (currentObject==null) {
            return;
        }
        if ( currentObject.getLatitude().equals("") ) {
            mGPS.getDataGPS();
        } else {
            String[] gps = new String[]{
                    currentObject.getLongitude(),
                    currentObject.getLatitude(),
                    currentObject.getAltitude(),
                    currentObject.getGpsAccuracy()
            };
            mGPS.dialogConfirmReplace( gps );
        }
    }

    public void GpsData(int which, String[] data) {
        switch (which) {
            case GPS.RESULT:
                if (data != null && currentObject != null) {
                    currentObject.setLongitude  ( data[0] );
                    currentObject.setLatitude   ( data[1] );
                    currentObject.setAltitude   ( data[2] );
                    currentObject.setGpsAccuracy( data[3] );
                    database.updateObjectGPS(
                            currentObject.getId(),
                            currentObject.getLongitude(),
                            currentObject.getLatitude(),
                            currentObject.getAltitude(),
                            currentObject.getGpsAccuracy()
                    );
                    table.update(currentObject);
                }
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_states, true);
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_wait, false);
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_states, true);
                break;
        }
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void creatDialogProgress(){
        dlgProgress = new Dialog(fragmentActivity);
        dlgProgress.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dlgProgress.setCanceledOnTouchOutside(false);
        dlgProgress.setContentView(R.layout.progress_bar);
        ((TextView) dlgProgress.findViewById(R.id.dlg_title)).setText(R.string.please_wait);
        ((TextView) dlgProgress.findViewById(R.id.message)).setText(R.string.saving_data_in_pdf);
        dlgProgress.findViewById(R.id.dlg_icon).setVisibility(View.GONE);
    }

    public void update(String progress){
        ((TextView)dlgProgress.findViewById(R.id.status)).setText(progress);
    }

    /***********************************************************************************************
     * REGION Utilities
     ***********************************************************************************************/

    private LinearLayout getLL(int layoutID, View view) {
        return (LinearLayout) view.findViewById(layoutID);
    }

    private Spinner getSpinner(int spinnerID, int arrayID, View inflatedView) {
        ArrayAdapter<CharSequence> filter =
                ArrayAdapter.createFromResource(fragmentActivity, arrayID, android.R.layout.simple_spinner_item);
        filter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);

        final Spinner spinner = inflatedView.findViewById(spinnerID);
        spinner.setAdapter(filter);
        return spinner;
    }

    private Spinner getSpinner(int spinnerID, String[] values, View inflatedView){
        final Spinner spinner = inflatedView.findViewById(spinnerID);
        ArrayAdapter<CharSequence> filter = new ArrayAdapter<>( fragmentActivity,
                android.R.layout.simple_spinner_item,
                values );
        filter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(filter);
        return spinner;
    }

    private void closeOverlay(FrameLayout v)
    {
        //FrameLayout overlay = (FrameLayout) fragmentActivity.findViewById(overlayID);
        v.removeAllViews();
        v.setVisibility(View.GONE);
    }

    private void closeOverlay()
    {
        FrameLayout overlay = fragmentActivity.findViewById(OVERLAY_ID);
        overlay.removeAllViews();
        overlay.setVisibility(View.GONE);
    }

    public com.megabras.modbus.Message statusMessage(){
        return modbus.read.status(2);
    }

    public com.megabras.modbus.Message stopMessage(){
        return modbus.write.stop();
    }

    public void readGeneralConfig(){
        userMessage(fragmentActivity.getString(R.string.read_general_params), CommonUI.COLOR_YELLOW, true);
        if(optionsButton==null) {
            optionsButton = menu.findItem(optionsID);
        }
        optionsButton.setEnabled(false);
        sendMessage(modbus.read.setup());
        sendMessage(modbus.read.date());
        sendMessage(modbus.read.memoryPointer());
    }

    private void readTestConfig(){
        userMessage(fragmentActivity.getString(R.string.read_general_params), CommonUI.COLOR_YELLOW, true);
        if(optionsButton==null) {
            optionsButton = menu.findItem(optionsID);
        }
        optionsButton.setEnabled(false);
        if ( firmware >= Protocol.FIRMWARE_AUTO_MODE )
            sendMessage( modbus.read.autoDetails() );
        sendMessage(modbus.read.setup());
        sendMessage(modbus.read.currentRange());
    }

    public void setMenuItemOptions(Menu menu, int optionsID){
        this.menu = menu;
        this.optionsID = optionsID;

//        final ImageView fin = mainFragment.getFinger();
//        finger = ValueAnimator.ofInt(100, 0);
//        finger.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
//            @Override
//            public void onAnimationUpdate(ValueAnimator valueAnimator) {
//                int val = (Integer) valueAnimator.getAnimatedValue();
//                fin.setX(fin.getX()-(100-val));
//            }
//        });
//        finger.setDuration(10000);


        anim = ValueAnimator.ofInt(toolBar.getMeasuredHeight(), 0);
        anim.addUpdateListener( valueAnimator ->
        {
            int val = (Integer) valueAnimator.getAnimatedValue();
            ViewGroup.LayoutParams layoutParams = toolBar.getLayoutParams();
            layoutParams.height = val;
            toolBar.setLayoutParams(layoutParams);
        } );
        anim.setDuration(1000);

        int widthSpec  = View.MeasureSpec.makeMeasureSpec(toolBar.getWidth(), View.MeasureSpec.EXACTLY);
        int heightSpec = View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED);
        linePwmControls.measure(widthSpec, heightSpec);

        anim2 = ValueAnimator.ofInt( linePwmControls.getMeasuredHeight(), 0);
        anim2.addUpdateListener( valueAnimator ->
        {
            int val = (Integer) valueAnimator.getAnimatedValue();
            ViewGroup.LayoutParams layoutParams = linePwmControls.getLayoutParams();
            layoutParams.height = val;
            linePwmControls.setLayoutParams(layoutParams);
        } );
        anim2.setDuration(1000);
    }

    /***********************************************************************************************
     * REGION CALLBACKs
     ***********************************************************************************************/

    private TimePickerDialog.OnTimeSetListener timeListener = new TimePickerDialog.OnTimeSetListener(){
        @Override
        public void onTimeSet(TimePicker view, int hourOfDay, int minute) {
            if(viewGeneral!=null){
                TextView time = viewGeneral.findViewById(R.id.device_time);
                Calendar c = Calendar.getInstance();
                c.setTime(device.getDate());
                c.set(Calendar.HOUR_OF_DAY, hourOfDay);
                c.set(Calendar.MINUTE, minute);
                c.set(Calendar.SECOND, 0);
                device.setDate( c.getTime() );
                time.setText(  FormatData.time( device.getTimeFormat(), device.getDate() ) );
            }
        }
    };

    private void pickTime(){
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int hour     = calendar.get(Calendar.HOUR_OF_DAY);
        int minute   = calendar.get(Calendar.MINUTE);
        boolean is24 = device.getTimeFormat()==Modbus.TIME_24H;
        new TimePickerDialog(fragmentActivity, timeListener, hour, minute, is24).show();
    }

    private DatePickerDialog.OnDateSetListener dateListener = new DatePickerDialog.OnDateSetListener() {

        public void onDateSet(DatePicker arg0, int year, int month, int day) {
            if(viewGeneral!=null){
                TextView date  = viewGeneral.findViewById(R.id.device_date);
                TextView time  = viewGeneral.findViewById(R.id.device_time);
                String parts[] = time.getText().toString().split(":");
                int hour       = Integer.parseInt(parts[0]);
                int minutes    = Integer.parseInt(parts[1]);

                Calendar c     = Calendar.getInstance();
                c.set(year, month, day, hour, minutes);
                device.setDate( c.getTime() );
                date.setText( FormatData.date(device.getDateFormat(), device.getDate()) );
            }
        }
    };

    private void pickDate(){
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int year  = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day   = calendar.get(Calendar.DAY_OF_MONTH);
        new DatePickerDialog(fragmentActivity, dateListener, year, month, day).show();
    }

    private void changeGauge(int range){
        switch (range){
            case Protocol.RANGE_1mA:
                // MAX RANGE, Minor tick space, number of Major ticks, Legend
                mainFragment.setCurrentGauge(1.1, 0.01, 11, "1 mA\n2000 \u03A9");
                break;
            case Protocol.RANGE_10mA:
                mainFragment.setCurrentGauge(11, 0.1, 11, "10 mA\n20 \u03A9");
                break;
            case Protocol.RANGE_100mA:
                mainFragment.setCurrentGauge(110, 1.1, 11, "100 mA\n2000 m\u03A9");
                break;
            case Protocol.RANGE_1A:
                mainFragment.setCurrentGauge(1.1, 0.01, 11, "1 A\n200 m\u03A9");
                break;
            case Protocol.RANGE_5A:
                mainFragment.setCurrentGauge(6, 0.1, 6, "5 A\n1000 m\u03A9");
                break;
            case Protocol.RANGE_10A:
                mainFragment.setCurrentGauge(11, 0.1, 11, "10 A\n200 m\u03A9");
                break;
        }
    }

    private void enableButtons(boolean enable){
        btStart.setEnabled(enable);
        btStop.setEnabled(enable);
        mCommonUI.enableButton(btFilter, enable);
        mCommonUI.enableButton(btSetup, enable);
        linePwmControls.setEnabled(false);

        if(testRunning){
            if(btStart.getVisibility()!=View.GONE) {
                btStop.setVisibility(View.VISIBLE);
                btStart.setVisibility(View.GONE);
//                bt_space.setVisibility(View.VISIBLE);
//                btSave.setVisibility(View.VISIBLE);
//                btPrinter.setVisibility(View.VISIBLE);
                mCommonUI.enableButton(btSave, true);
                mCommonUI.enableButton(btPrinter, true);
                if ( firmware < Protocol.FIRMWARE_AUTO_MODE || device.getTestMode() != Device.MODE_AUTO )
                {
                    linePwmControls.setEnabled(true);
                    linePwmControls.setVisibility(View.VISIBLE);
                    anim.start();
                    anim2.reverse();
                }
            }
        } else {
            if(btStart.getVisibility()!=View.VISIBLE){
                btStart.setVisibility(View.VISIBLE);
                btStop.setVisibility(View.GONE);
//                bt_space.setVisibility(View.GONE);
//                btSave.setVisibility(View.GONE);
//                btPrinter.setVisibility(View.GONE);
                mCommonUI.enableButton(btSave, false);
                mCommonUI.enableButton(btPrinter, false);
                if ( firmware < Protocol.FIRMWARE_AUTO_MODE || device.getTestMode() != Device.MODE_AUTO )
                {
                    linePwmControls.setEnabled(false);
                    anim.reverse();
                    anim2.start();
                }
                dialogFinished();
            }
        }
    }

    public void stateChanged(int state){
        testRunning = false;
        switch (state){
            case BluetoothClassic.STATE_CONNECTED:
                sendMessage( modbus.read.memoryMap() );
                sendMessage( modbus.read.firmware() );
                sendMessage( modbus.read.setup() );
                sendMessage( modbus.read.currentRange() );
                enableButtons(true);
                break;
            case BluetoothClassic.STATE_CONNECTING:
                break;
            case BluetoothClassic.STATE_NOT_RESPONDING:
                break;
            case BluetoothClassic.STATE_FAIL:
                enableButtons(false);
                break;
            case BluetoothClassic.STATE_LOST:
                userMessage(fragmentActivity.getString(R.string.connection_lost), CommonUI.COLOR_BLACK, false );
                closeOverlay();
                enableButtons(false);
                if ( mainFragment != null )
                {
                    mainFragment.disconnected();
                }
                break;
            case BluetoothClassic.STATE_CLOSED:
                userMessage(fragmentActivity.getString(R.string.disconnected), CommonUI.COLOR_BLACK, false );
                closeOverlay();
                enableButtons(false);
                if ( mainFragment != null )
                {
                    mainFragment.disconnected();
                }
                break;
        }
    }

    @SuppressLint("ClickableViewAccessibility")
    private void warnAboutLock(){
        final FrameLayout
                overlay = MUtils.getOverlay(fragmentActivity, OVERLAY_ID, viewGeneral);

        final View inflatedView = View.inflate(fragmentActivity, R.layout.md10kvx_fgr_msg, null);

        overlay.removeAllViews();
        overlay.addView(inflatedView);
        overlay.setVisibility(View.VISIBLE);

        //This is to prevent the ontouch propagate to others views
        RelativeLayout overlayBase = inflatedView.findViewById(R.id.overlay_base);
        overlayBase.requestDisallowInterceptTouchEvent(true);
        overlayBase.setOnTouchListener( ( v, event ) -> true );

        TextView title = inflatedView.findViewById(R.id.tx_title);
        title.setText(fragmentActivity.getResources().getString(R.string.md_remote_control_locked));

        TextView message = inflatedView.findViewById(R.id.tx_message);
        message.setText(fragmentActivity.getResources().getString(R.string.md_remote_control_locked_explained));

        Button btClose = inflatedView.findViewById(R.id.bt_confirm);
        btClose.setVisibility(View.GONE);

        Button btCancel = inflatedView.findViewById(R.id.bt_cancel);
        btCancel.setText(fragmentActivity.getResources().getString(R.string.md_ok));
        btCancel.setOnClickListener( v ->
        {
            closeOverlay( overlay );
        } );
    }

    private void buttonsInAutoPilot(boolean enabled)
    {
        if (btFilter.isEnabled()!=enabled)
        {
            mCommonUI.enableButton( btFilter        , enabled );
            mCommonUI.enableButton( btSetup         , enabled );
            mCommonUI.enableButton( btPrinter       , enabled );
            mCommonUI.enableButton( btSave          , enabled );
        }
        if (btCurrentSmallA.isEnabled()!=enabled)
        {
            mCommonUI.enableButton( btCurrentSmallA , enabled );
            mCommonUI.enableButton( btCurrentSmallB , enabled );
            mCommonUI.enableButton( btCurrentMediumA, enabled );
            mCommonUI.enableButton( btCurrentMediumB, enabled );
            mCommonUI.enableButton( btCurrentLargeA , enabled );
            mCommonUI.enableButton( btCurrentLargeB , enabled );
        }
    }

    public void processResponse( int id, String response )
    {
        switch (id)
        {
            case Protocol.ID_MEMORY_MAP_READ:
                int mapVersion = modbus.parse.memoryMap( response );
                Protocol.setRevisionMap( mapVersion );
                userMessage( fragmentActivity.getString(R.string.memory_map_was_read), CommonUI.COLOR_YELLOW, true );
                break;
//            case Protocol.ID_STOP_WRITE:
//                sendMessage( modbus.read.pwm() );
//                break;
//            case Protocol.ID_PWM_READ:
//                break;
            case Protocol.ID_FIRMWARE_READ:
                firmware = modbus.parse.firmware(response);
                if ( firmware >= Protocol.FIRMWARE_AUTO_MODE )
                    sendMessage( modbus.read.autoDetails() );
                userMessage(fragmentActivity.getString(R.string.firmware_was_read), CommonUI.COLOR_YELLOW, true);
                break;
            case Protocol.ID_GENERAL_SETUP_READ:
                modbus.parse.setup(device, response);
                changeGauge(device.getCurrentRange());
                ((com.megabras.bluelogg.Main)fragmentActivity).updateTemperatureUnity(device.getTempUnity());
                mainFragment.updateParameters(device);
                if ( configChanged )
                {
                    userMessage( fragmentActivity.getString( R.string.ready ), CommonUI.COLOR_GREEN, false );
                    configChanged = false;
                }
                break;
            case Protocol.ID_AUTO_DETAILS_READ:
                modbus.parse.autoDetails( device, response );
                break;
            case Protocol.ID_GENERAL_SETUP_WRITE:
                userMessage(fragmentActivity.getString(R.string.ready), CommonUI.COLOR_GREEN, false);
                optionsButton.setEnabled(true);
                break;
            case Protocol.ID_CURRENT_RANGE_READ:
                device.setCurrentRange( modbus.parse.currentRange(response) );
                changeGauge( device.getCurrentRange() );
                mainFragment.updateParameters( device );
                userMessage(fragmentActivity.getString(R.string.connected), CommonUI.COLOR_GREEN, false);
                if( showOverlay )
                    setupTestPage();
                break;
            case Protocol.ID_MEM_POINTER_READ:
                memoryPointer = modbus.parse.memoryPointer(response);
                device.setFreeMemory( modbus.parse.memoryFree(response) );
                if ( !device.statusTestRunning() && !waitingAutoSave )
                {
                    setupPage();
                }
                else
                {
                    if (optionsButton!=null)
                        optionsButton.setEnabled(true);
                    int blocksLenght = 2;
                    if (device.statusTemperatureCompensation())
                        blocksLenght = 3;
                    sendMessage( modbus.read.memory(memoryPointer-(blocksLenght-1), blocksLenght) );
                }
                break;
            case Protocol.ID_MEMORY_READ:
                TestData savedTestData = new TestData(testData.getModel(), testData.getSerial());
                savedTestData.setID        ( currentTestDataID        );
                savedTestData.setTestNumber( testData.getTestNumber() );
                savedTestData.setDate      ( testData.getDate()       );
                savedTestData.setSetup     ( device                   );
                Measurement mea        = new Measurement();
                String[] blocks        = modbus.parse.memorySplitBlocks(response);

                for ( String block : blocks )
                {
                    Log.e(TAG, "block: " + block);
                    int blockType = modbus.parse.memoryGetBlockType(block);
                    switch (blockType){
                        case Protocol.MEM_BLOCK_RESULT:
                            modbus.parse.memoryBlockResult(block, mea);
                            break;
                        case Protocol.MEM_BLOCK_CURRENT:
                            modbus.parse.memoryBlockCurrent(block, device, mea);
                            break;
                        case Protocol.MEM_BLOCK_TEMPERATURE:
                            modbus.parse.memoryBlockTemperature(block, mea);
                            break;
                    }
                }
                savedTestData.addMeasument(mea);
                saveInDatabase(savedTestData);

                String
                        temp  = (device.statusTemperatureCompensation()) ? FormatData.getStringTempRange(device.getTempUnity(), device.getTempRange()) : "",
                        metal = (device.statusTemperatureCompensation()) ? FormatData.getStringMetal(device.getMetalType()) : "",
                        date  = FormatData.getDateFormatted(savedTestData.getDate(), device.getDateFormat(), device.getTimeFormat());
                table.setHeader(savedTestData.getTestNumber(), date, temp, metal);
                table.updateRows(savedTestData.getMeasurementLast(), device, true);

                isSaving = false;

                if (!testRunning)
                {
                    waitingAutoSave   = false;
                    currentTestDataID = -1;
                }
                break;
            case Protocol.ID_DATE_READ:
                device.setDate( modbus.parse.date(response) );
                break;
            case Protocol.ID_CURRENT_CONTROL_WRITE:
                sendMessage( modbus.read.currentControl() );
                break;
            case Protocol.ID_FILTER_WRITE:
                if(!device.statusTestRunning())
                    userMessage(fragmentActivity.getString(R.string.ready), CommonUI.COLOR_GREEN, false );
                mainFragment.updateParameters(device);
                break;
            case Protocol.ID_SAVE_WRITE:
                if(!device.statusTestRunning())
                    userMessage(fragmentActivity.getString(R.string.ready), CommonUI.COLOR_GREEN, false );
                sendMessage( modbus.read.memoryPointer() );
                break;
            case Protocol.ID_MEM_POINTER_WRITE:
                sendMessage( modbus.read.memoryPointer() );
                break;
            case Protocol.ID_START_WRITE:
                btStop.setVisibility( View.VISIBLE );
                btStop.setEnabled( true );
                btStart.setVisibility( View.GONE );
                mCommonUI.enableButton( btSave, true );
                mCommonUI.enableButton( btPrinter, true );
                bAlreadySended = false;
//                bt_space.setVisibility(View.VISIBLE);
//                btSave.setVisibility(View.VISIBLE);
//                btPrinter.setVisibility(View.VISIBLE);

                if ( firmware < Protocol.FIRMWARE_AUTO_MODE || device.getTestMode() != Device.MODE_AUTO )
                {
                    linePwmControls.setEnabled( true );
                    linePwmControls.setVisibility( View.VISIBLE );
                    anim.start();
                    anim2.reverse();
                }

                handler.postDelayed( () ->
                {
                    if ( !testRunning && btStop.getVisibility() == View.VISIBLE )
                    {
                        enableButtons( true );
                        userMessage( fragmentActivity.getString( R.string.test_cancelled ), CommonUI.COLOR_YELLOW, false );
                    }
                }, 4500 );
                break;
            case Protocol.ID_REAL_TIME_READ:
                boolean[] status = modbus.parse.statusDevice( response );
                device.setStatus( status );
                device.setBatteryLevel( modbus.parse.statusBatteryLevel( response ) );



                if ( device.statusConfigurationChanged() )
                {
                    userMessage( fragmentActivity.getString( R.string.md_configurations_changed ), CommonUI.COLOR_YELLOW, true );
                    sendMessage( modbus.write.resetConfigChanged() );
                    configChanged = true;
                }

                if ( device.statusAutoPilot() )
                {
                    if( !warnedLock )
                    {
                        warnAboutLock();
                        warnedLock = true;
                    }
                    userMessage(fragmentActivity.getString(R.string.md_remote_control_locked), CommonUI.COLOR_YELLOW, false );
                    buttonsInAutoPilot(false);
                }
                else
                {
                    warnedLock = false;
                    buttonsInAutoPilot(true);
                }

                if ( device.statusTestRunning() )
                {
                    sendMessage( modbus.read.status(17) );

                    if ( Modbus.FIRMWARE_VERSION >= minFirmareVersion2 )
                    {
                        device.setCurrentControl( modbus.parse.statusPWM( response ) );
                        if ( device.getTestMode() == Device.MODE_PROD_LINE && device.getLastPWM() > 0 && device.getCurrentControl() == 0 && !bAlreadySended )
                        {
                            sendMessage( modbus.write.currentControl( device.getLastPWM() ) );
                            bAlreadySended = true;
                        }
                    }

                    // Update PWM
                    if ( PWM != 0 )
                    {
                        sendMessage( modbus.write.currentControl( device.getCurrentControl()+PWM ) );
                        if ( Modbus.FIRMWARE_VERSION < minFirmareVersion2 )
                            device.setCurrentControl(device.getCurrentControl()+PWM);
                        PWM = 0;
                    }

                    if ( !testRunning )
                    {
                        testData    = new TestData(testData.getModel(), testData.getSerial());
                        testRunning = true;
                        device.setCurrentControl(0);
                        enableButtons(true);
                    }

                    Measurement measure = new Measurement();
                    if ( response.length()>Protocol.ST_DATE_TIME*4 ) // 4 is the package size
                    {
                        testData.setTestNumber( modbus.parse.statusTestNumber(response) );
                        mainFragment.updateTestNumber( testData.getTestNumber() );
                        testData.setDate( modbus.parse.statusTestDate(response) );
                        measure = modbus.parse.statusMeasurement(response);
                        measure.setCurrentRange( (short)device.getCurrentRange() );
                        measure.setCompensationEnabled( device.statusTemperatureCompensation() );

                        if ( status[5] )
                            measure.setStatus(1);
                        if ( status[6] )
                            measure.setStatus(2);
                        if ( status[7] )
                            measure.setStatus(4);

                        if( !device.statusTemperatureCompensation() )
                        {
                            measure.setResistanceCompensated(0);
                            measure.setTemperature(0);
                        }

                        testData.updateLastMeasument(measure);
                    }

                    mainFragment.update(device, measure, modbus.parse.statusVoltage(response));

                    if( !isSaving && !device.statusAutoPilot() )
                    {
                        if ( Modbus.FIRMWARE_VERSION < minFirmareVersion2 )
                        {
                            if ( measure.getCurrent() < device.getMinCurrent() )
                                userMessage( fragmentActivity.getString( R.string.low_current ), CommonUI.COLOR_RED, true );
                            else
                                userMessage( fragmentActivity.getString( R.string.running ), CommonUI.COLOR_RED, true );
                        }
                        else
                        {
                            if ( status[5] )
                                userMessage( fragmentActivity.getString( R.string.wait ), CommonUI.COLOR_RED, true );
                            else if ( status[6] )
                                userMessage( fragmentActivity.getString( R.string.overrange ), CommonUI.COLOR_RED, true );
                            else if ( status[7] )
                                userMessage( fragmentActivity.getString( R.string.low_current ), CommonUI.COLOR_RED, true );
                            else
                                userMessage( fragmentActivity.getString( R.string.running ), CommonUI.COLOR_RED, true );
                        }
                    }
                }
                else
                {
                    if ( testRunning )
                    {
                        testRunning = false;
                        isSaving    = false;
                        userMessage(fragmentActivity.getString(R.string.finished), CommonUI.COLOR_GREEN, false );
                        enableButtons(true);
                        if (device.getAutoSave())
                        {
                            waitingAutoSave = true;
                            sendMessage( modbus.read.memoryPointer() );
                        }
                        else
                        {
                            currentTestDataID = -1;
                        }
                        device.setLastPWM( device.getCurrentControl() );
                        device.setCurrentControl(0);
                    }
                    mainFragment.updateParameters( device );
                }

                ( (com.megabras.bluelogg.Main)fragmentActivity ).updateBattery( device.getBatteryLevel() );
                break;
            case Protocol.ID_CURRENT_CONTROL_READ:
//                int currentControl = modbus.parse.currentControl( response );
//                device.setLastPWM( currentControl );
                break;
            case Protocol.ID_CONFIG_CHANGE_WRITE:
                //sendMessage( modbus.read.currentRange() );
                sendMessage( modbus.read.autoDetails() );
                sendMessage( modbus.read.setup() );
                break;
        }
    }

    private void stateMachineUI(){
        //Object Data
        if(currentObject==null){
            table.showTable(false);
            mCommonUI.enableButton(buttonGPS    , false);
            mCommonUI.enableButton(buttonPicture, false);
            mCommonUI.enableButton(buttonMap    , false);
            mCommonUI.enableButton(buttonVoice  , false);
        } else {
            table.showTable(true);
            mCommonUI.enableButton(buttonGPS    , true);
            mCommonUI.enableButton(buttonPicture, true);
            mCommonUI.enableButton(buttonVoice  , true);
            mCommonUI.enableButton(buttonMap    , !currentObject.getLongitude().equals(""));
        }
    }
}
