package com.megabras.bluelogg.mpk257;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.modbus.MPK257.Device;
import com.megabras.modbus.MPK257.FormatData;
import com.megabras.modbus.MPK257.Protocol;

import org.achartengine.GraphicalView;
import org.achartengine.model.CategorySeries;
import org.achartengine.renderer.DialRenderer;
import org.achartengine.renderer.SimpleSeriesRenderer;

import androidx.fragment.app.Fragment;

//import android.support.v4.app.Fragment;

public class MainFragment extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG             = "MainFragment",
            CATEGORY_NAME   = "VOLTAGE";

    static final int
            TYPE_VOLTAGE    = 0,
            TYPE_CURRENT    = 1;

    @SuppressWarnings("FieldCanBeLocal")
    private int
            GAUGE_TYPE      = 0,
            layoutID        =  R.layout.mpk257_fgr_main;

    private Context
            context;

    private LinearLayout
            layout;

    private GraphicalView
            graphicalView;

    private DialRenderer
            renderer;

    private CategorySeries
            category;

    private View
            view;

    @SuppressWarnings("NullableProblems")
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        context = inflater.getContext();
        view = inflater.inflate(layoutID, container, false);
        view.setBackgroundColor(context.getResources().getColor(R.color.transparent));
        return view;
    }

    @Override
    public void onStart()
    {
        super.onStart();
        prepareGauge();
        setCurrentGauge( 11, 0.11, 11, "A" );
        this.updateGauge( 0 );
        getLN( R.id.icon_temp_ref ).setVisibility( View.INVISIBLE );
        getTV( R.id.icon_metal ).setVisibility( View.INVISIBLE );
        getIV( R.id.icon_filter ).setVisibility( View.INVISIBLE );
        getIV( R.id.icon_autosave ).setVisibility( View.INVISIBLE );
        getLN( R.id.ln_resistance_comp ).setVisibility( View.GONE );
        //getTV(R.id.icon_range).setVisibility(View.GONE);
        getLN( R.id.ll_voltage ).setVisibility( View.GONE );
        getTV( R.id.icon_auto_current ).setVisibility( View.GONE );
        getTV( R.id.icon_auto_timet ).setVisibility( View.GONE );
        getTV( R.id.icon_prod_line ).setVisibility( View.GONE );
    }

    public void disconnected()
    {
        setCurrentGauge( 11, 0.11, 11, "A" );
        this.updateGauge( 0 );
        getLN( R.id.icon_temp_ref ).setVisibility( View.INVISIBLE );
        getTV( R.id.icon_metal ).setVisibility( View.INVISIBLE );
        getIV( R.id.icon_filter ).setVisibility( View.INVISIBLE );
        getIV( R.id.icon_autosave ).setVisibility( View.INVISIBLE );
        getLN( R.id.ln_resistance_comp ).setVisibility( View.GONE );
        getLN( R.id.ll_voltage ).setVisibility( View.GONE );
        getTV( R.id.icon_auto_current ).setVisibility( View.GONE );
        getTV( R.id.icon_auto_timet ).setVisibility( View.GONE );
        getTV( R.id.icon_prod_line ).setVisibility( View.GONE );
        getTV(R.id.tv_voltage).setText( "" );
        getTV(R.id.lb_elapsed).setText( "" );
        getTV(R.id.lb_current).setText( "" );
        getTV(R.id.lb_resistance).setText( "" );
        getTV(R.id.lb_resistance_comp).setText( "" );
        getTV(R.id.lb_temperature).setText( "" );
        getTV(R.id.lb_test_number).setText( "" );
    }

    private void prepareGauge()
    {
        if ( this.getView() != null )
        {
            layout = this.getView().findViewById( R.id.gauge );
        }
        else
        {
            return;
        }

        if ( layout.getChildCount() > 0 )
        {
            layout.removeAllViews();
        }

        category = new CategorySeries(CATEGORY_NAME);
        category.add(CATEGORY_NAME, 0);

        renderer = new DialRenderer();

        DisplayMetrics metrics = context.getResources().getDisplayMetrics();
        int fSize = (int) context.getResources().getDimension(R.dimen.mpk257_dial_title_font);
        float val = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, fSize, metrics);
        renderer.setChartTitleTextSize(val);

        fSize = (int) context.getResources().getDimension(R.dimen.mpk257_dial_label_font);
        val = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, fSize, metrics);
        renderer.setLabelsTextSize(val);
        //@param margins an array containing the margin size values, in this order:
        renderer.setMargins(new int[] {0, 0, 0, 0});// top, left, bottom, right
        renderer.setInScroll(true);

        SimpleSeriesRenderer r = new SimpleSeriesRenderer();
        r.setColor(Color.RED);
        renderer.addSeriesRenderer(r);
        renderer.setLabelsColor(Color.WHITE);
        renderer.setAngleMin(270);
        renderer.setAngleMax(72);
        renderer.setShowLabels(true);
        renderer.setLegendHeight(0);
        renderer.setShowLegend(false);
        renderer.setVisualTypes(new DialRenderer.Type[] {DialRenderer.Type.NEEDLE});
        renderer.setMinValue(0);
        renderer.setPanEnabled(false);
        renderer.setZoomEnabled(false);
    }

    private GraphicalView getDialChartView( Context context, CategorySeries dataset,
                                            DialRenderer renderer, int majorTicksRatio ) {
        CustomDialChart chart = new CustomDialChart(dataset, renderer, majorTicksRatio);
        chart.setContext(context);
        return new GraphicalView(context, chart);
    }

    private void updateGauge(double voltage){
        category.set(0, CATEGORY_NAME, voltage);
        graphicalView.repaint();
    }

    public void update(com.megabras.modbus.MPK257.Device device, com.megabras.modbus.MPK257.Measurement measure, float voltage){
        if( device.statusTestRunning() ){
            String resistance, resistanceC;

            if(Main.equipmentId==257)
            {
                resistance = com.megabras.modbus.MPK257
                        .FormatData.formatResistance(measure.getCurrentRange(), measure.getResistance());
                resistanceC = com.megabras.modbus.MPK257
                        .FormatData.formatResistance  ( measure.getCurrentRange(), measure.getResistanceCompensated());
            }
            else
            {
                resistance = com.megabras.modbus.MPK256
                        .FormatData.formatResistance(measure.getCurrentRange(), measure.getResistance());
                resistanceC = com.megabras.modbus.MPK256
                        .FormatData.formatResistance  ( measure.getCurrentRange(), measure.getResistanceCompensated());
            }

            String
                    elapsed     = FormatData.formatElapsedTime ( measure.getElapsedTime()),
                    current     = FormatData.formatCurrent     ( measure.getCurrentRange(), measure.getCurrent()),
                    temperature = FormatData.formatTemperature ( device.getTempUnity(), measure.getTemperature());

            if ( measure.getTemperature() <= 0 || measure.getTemperature() > Protocol.TEMP_MAX )
            {
                resistanceC = "";
                temperature = "";
            }

            if ( Controller.firmware > Controller.minFirmareVersion )
            {
                if ( measure.getStatus()>0 )
                {
                    resistance  = "- - -";
                    resistanceC = "- - -";
                }
            }
            else
            {
                if( measure.getCurrent() < device.getMinCurrent() )
                {
                    resistance  = "- - -";
                    resistanceC = "- - -";
                }
            }

            getTV(R.id.tv_voltage).setText        ( FormatData.voltage(voltage) );
            getTV(R.id.lb_elapsed).setText        ( elapsed );
            getTV(R.id.lb_current).setText        ( current );
            getTV(R.id.lb_resistance).setText     ( resistance );
            getTV(R.id.lb_resistance_comp).setText( resistanceC );
            getTV(R.id.lb_temperature).setText    ( temperature );
        }

        if(renderer.getChartTitle().contains("mA"))
            updateGauge(measure.getCurrent()*1000);
        else
            updateGauge(measure.getCurrent());

        updateParameters(device);
    }

    void updateTestNumber( int testNumber )
    {
        String sNum = testNumber +"";
        getTV(R.id.lb_test_number).setText( sNum );
    }

    void updateParameters( com.megabras.modbus.MPK257.Device device )
    {
        if ( Controller.firmware >= Protocol.FIRMWARE_AUTO_MODE )
        {
            String[] modes = getResources().getStringArray( R.array.mpk257_mode_manual_auto );

            if ( device.getTestMode() == Device.MODE_AUTO )
            {
                setTextViewText( R.id.icon_mode, modes[Device.MODE_AUTO] );
                setVisible( getTV(R.id.icon_auto_current), View.VISIBLE );
                setVisible( getTV(R.id.icon_auto_timet)  , View.VISIBLE );
                setVisible( getTV(R.id.icon_prod_line)   , View.GONE    );

                String sAuto = device.getAutoCurrent() + " %";
                setTextViewText( R.id.icon_auto_current, sAuto );
                setTextViewText( R.id.icon_auto_timet  , FormatData.formatElapsedTime( (short)device.getAutoDuration() ) );
            }
            else if ( device.getTestMode() == Device.MODE_PROD_LINE )
            {
                setTextViewText( R.id.icon_mode, modes[Device.MODE_PROD_LINE] );
                setVisible( getTV(R.id.icon_auto_current), View.GONE    );
                setVisible( getTV(R.id.icon_auto_timet)  , View.GONE    );
                setVisible( getTV(R.id.icon_prod_line)   , View.VISIBLE );
                if ( device.getLastPWM() > 0 )
                {
                    setTextViewText( R.id.icon_prod_line, R.string.play );
                }
                else
                {
                    setTextViewText( R.id.icon_prod_line, R.string.record );
                }
            }
            else
            {
                setTextViewText( R.id.icon_mode, modes[Device.MODE_MANUAL] );
                setVisible( getTV(R.id.icon_auto_current), View.GONE );
                setVisible( getTV(R.id.icon_auto_timet)  , View.GONE );
                setVisible( getTV(R.id.icon_prod_line)   , View.GONE );
            }
        } // END IF ( Controller.firmware >= Protocol.FIRMWARE_AUTO_MODE )
        else
        {
            setVisible( getTV(R.id.icon_auto_current), View.GONE );
            setVisible( getTV(R.id.icon_auto_timet)  , View.GONE );
            setVisible( getTV(R.id.icon_prod_line)   , View.GONE );
        } // END ELSE ( Controller.firmware >= Protocol.FIRMWARE_AUTO_MODE )

        // TEMPERATURE COMPENSATION
        if ( device.statusTemperatureCompensation() )
        {
            setVisible( getLN( R.id.icon_temp_ref )     , View.VISIBLE );
            setVisible( getTV( R.id.icon_metal )        , View.VISIBLE );
            setVisible( getLN( R.id.ln_resistance_comp ), View.VISIBLE );
        }
        else
        {
            setVisible( getLN( R.id.icon_temp_ref )     , View.INVISIBLE );
            setVisible( getTV( R.id.icon_metal )        , View.INVISIBLE );
            setVisible( getLN( R.id.ln_resistance_comp ), View.GONE      );
        }

        // TEMPERATURE RANGE
        String tempRange = "";
        if ( device.statusTemperatureCompensation() )
        {
            switch ( device.getTempRange() )
            {
                case Protocol.TEMP_RANGE_20C:
                    tempRange = (device.getTempUnity() == Protocol.TEMP_CELCIUS) ? "(20 \u2103)" : "(68 \u2109)";
                    break;
                case Protocol.TEMP_RANGE_25C:
                    tempRange = (device.getTempUnity() == Protocol.TEMP_CELCIUS) ? "(25 \u2103)" : "(77 \u2109)";
                    break;
            }
        }
        setTextViewText( R.id.icon_temp_text, tempRange );

        // AUTOSAVE
        if ( device.getAutoSave() )
        {
            setVisible( getIV( R.id.icon_autosave ), View.VISIBLE );
        }
        else
        {
            setVisible( getIV( R.id.icon_autosave ), View.INVISIBLE );
        }

        // METAL
        String[] metals = context.getResources().getStringArray(R.array.metals_short);
        setTextViewText( R.id.icon_metal, metals[device.getMetalType()] );

        // FILTER
        if(device.statusFilter())
        {
            setVisible( getIV( R.id.icon_filter ), View.VISIBLE );
        }
        else
        {
            setVisible( getIV( R.id.icon_filter ), View.INVISIBLE );
        }
    }

    private void setTextViewText( int id, int text)
    {
        TextView tv = view.findViewById( id );
        String tx   = getString( text );
        if ( tv != null && !tv.getText().toString().equals( tx ) )
        {
            tv.setText( tx );
        }
    }

    private void setTextViewText( int id, String text)
    {
        TextView tv = view.findViewById( id );
        if ( tv != null && !tv.getText().toString().equals( text ) )
        {
            tv.setText( text );
        }
    }

    private void setVisible(TextView view, int visibility)
    {
        if ( view != null && view.getVisibility() != visibility )
        {
            view.setVisibility( visibility );
        }
    }

    private void setVisible(LinearLayout linearLayout, int visibility)
    {
        if ( linearLayout != null && linearLayout.getVisibility() != visibility )
        {
            linearLayout.setVisibility( visibility );
        }
    }

    private void setVisible(ImageView imageView, int visibility)
    {
        if ( imageView != null && imageView.getVisibility() != visibility )
        {
            imageView.setVisibility( visibility );
        }
    }

    void setCurrentGauge( double range, double minorTickSpacing, int majorTickRatio, String unit ){
        if(layout.getChildCount()>0){
            layout.removeAllViews();
        }

        category.clear();
        category = new CategorySeries(CATEGORY_NAME);
        category.add(CATEGORY_NAME, 0);
        renderer.setMinorTicksSpacing(minorTickSpacing);
        renderer.setChartTitle(unit);
        renderer.setMaxValue(range);

        if ( view != null )
        {
            graphicalView = this.getDialChartView(
                    context,
                    category,
                    renderer,
                    majorTickRatio );

            layout.addView( graphicalView, new TableRow.LayoutParams( TableRow.LayoutParams.WRAP_CONTENT, TableRow.LayoutParams.WRAP_CONTENT ) );

            GAUGE_TYPE = TYPE_CURRENT;
        }
    }

    public void reset()
    {
        getTV(R.id.lb_test_number).setText( "" );
        getTV(R.id.lb_elapsed).setText( "" );
        getTV(R.id.lb_current).setText( "" );
        getTV(R.id.lb_resistance).setText( "" );
        getTV(R.id.lb_resistance_comp).setText( "" );
        getTV(R.id.lb_temperature).setText( "" );
    }

    int getGaugeType()
    {
        return GAUGE_TYPE;
    }

    private TextView getTV( int id )
    {
        return ((TextView) view.findViewById( id ));
    }

    private ImageView getIV( int id )
    {
        return ((ImageView) view.findViewById( id ));
    }

    private LinearLayout getLN( int id )
    {
        return ((LinearLayout) view.findViewById( id ));
    }
}

