package com.megabras.bluelogg.tm25r;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.media.ThumbnailUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.AbsListView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TabHost;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.BooleanObservable;
import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.extended.StringObservable;
import com.megabras.modbus.Modbus;
import com.megabras.modbus.TM25R.Device;
import com.megabras.modbus.TM25R.FormatData;
import com.megabras.modbus.TM25R.Measurement;
import com.megabras.modbus.TM25R.Objects;
import com.megabras.modbus.TM25R.TestData;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;
import java.util.Observable;
import java.util.Observer;

public class Dialogs implements Observer
{

    @SuppressWarnings("unused")
    private final String
            TAG = "Dialogs";

    private Activity
            activity;

    private Dialog
            dialogManager,
            dialogOptions;

    private Database
            database;

    private DialogsInterface
            parent;

    private CommonUI
            mCommonUI;

    private StringObservable
            selectedTestSite,
            loadedTestSite,
            selectedOperator;

    private BooleanObservable
            fileLoaded;

    Dialogs( DialogsInterface parent, Activity activity, Database database )
    {
        this.parent = parent;
        this.activity = activity;
        this.database = database;
        selectedTestSite = new StringObservable( 1, " " );
        loadedTestSite = new StringObservable( 2, " " );
        selectedOperator = new StringObservable( 3, " " );
        fileLoaded = new BooleanObservable( 1, false );
        selectedTestSite.addObserver( this );
        loadedTestSite.addObserver( this );
        selectedOperator.addObserver( this );
        fileLoaded.addObserver( this );
        mCommonUI = new CommonUI( activity );
    }

    void showManager( int index, final TestData testData )
    {
        dialogManager = new Dialog( activity );
        dialogManager.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialogManager.setCanceledOnTouchOutside( false );
        dialogManager.setContentView( R.layout.dialog_tab );
        if ( dialogManager.getWindow() != null )
        {
            dialogManager.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            dialogManager.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialogManager.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }

        final TabHost tabs = dialogManager.findViewById( R.id.tabhost );
        tabs.setup();

        setTabs(activity, R.id.dlg_savedfiles, R.string.files_manager, R.drawable.ic_manager_small , tabs);
        setTabs(activity, R.id.dlg_sites     , R.string.clients      , R.drawable.ic_sites   , tabs);
        setTabs(activity, R.id.dlg_operator  , R.string.operators    , R.drawable.ic_operator, tabs);
        setTabs(activity, R.id.dlg_comp      , R.string.firm         , R.drawable.ic_company , tabs);

        tabs.setOnTabChangedListener( tabId ->
        {
            int i = tabs.getCurrentTab();
            switch ( i )
            {
                case 0:
                    tabSavedFiles( testData );
                    break;
                case 1:
                    tabSites( testData );
                    break;
                case 2:
                    tabOperators( testData );
                    break;
                case 3:
                    tabCompany();
                    break;
            }
        } );

        final Button btnCancel = dialogManager.findViewById( R.id.btn_close_manager );
        btnCancel.setOnClickListener( v -> dialogManager.dismiss() );

        if ( index == 0 )
        {
            tabSavedFiles( testData );
        }

        tabs.setCurrentTab( index );
        dialogManager.show();
    }


    private void tabSavedFiles(final TestData testData){
        final ListView
                list     = dialogManager.findViewById(R.id.list_savedfiles);
        final Button
                btLoad   = dialogManager.findViewById(R.id.btn_load),
                btClose  = dialogManager.findViewById(R.id.btn_close_dlg),
                btDelete = dialogManager.findViewById(R.id.btn_delete),
                btNew    = dialogManager.findViewById(R.id.btn_new),
                btEmail  = dialogManager.findViewById(R.id.btn_email);

        selectedTestSite.update("");
        fileLoaded.update(false);

        final ListSitesDetails
                adapterListView = new ListSitesDetails( activity, database.getSavedFiles() );
        list.setChoiceMode( ListView.CHOICE_MODE_SINGLE );
        list.setAdapter( adapterListView );

        list.setOnItemClickListener( ( parent, view, position, id ) ->
        {
            String item = ((ListSites) parent.getItemAtPosition( position )).getUrl();
            adapterListView.setSelectedFileDetails( database.getDataFileDetails( item ) );
            adapterListView.setSelectedIndex( position );
            adapterListView.notifyDataSetChanged();
            selectedTestSite.set( ((ListSites) parent.getItemAtPosition( position )).getUrl() );
        } );

        if (testData.getFolder() != null) {
            String fileName = testData.getFolder().getName();
            int selected = -1;
            for (int i = 0; i < adapterListView.getCount(); i++) {
                if (adapterListView.getItem(i).getUrl().equals(fileName)) {
                    selected = i;
                    break;
                }
            }
            adapterListView.setSelectedFileDetails(database.getDataFileDetails(fileName));
            adapterListView.setSelectedIndex(selected);
            list.setItemChecked(selected, true);
            list.smoothScrollToPosition(selected);
            adapterListView.notifyDataSetChanged();
            selectedTestSite.set(((ListSites) list.getItemAtPosition(selected)).getUrl());
            loadedTestSite.set(selectedTestSite.get());
        }

        btLoad.setOnClickListener( v ->
        {
            if ( !selectedTestSite.get().equals( "" ) )
            {
                parent.loadedFile( adapterListView.getSelectedItem().getUrl() );
                loadedTestSite.set( adapterListView.getSelectedItem().getUrl() );
                dialogManager.dismiss();
            }
        } );

        btClose.setOnClickListener( v ->
        {
            parent.closeFile();
            list.setItemChecked( -1, true );
            adapterListView.setSelectedIndex( -1 );
            adapterListView.notifyDataSetChanged();
            selectedTestSite.set( "" );
            fileLoaded.set( false );
            loadedTestSite.set( "" );
        } );

        btEmail.setOnClickListener( v ->
        {
            if ( !selectedTestSite.get().equals( "" ) )
            {
                parent.export( selectedTestSite.get() );
            }
        } );

        btDelete.setOnClickListener( v ->
        {
            final Dialog
                    dialog  = uGetDialogYesNo(activity, R.string.file_delete, R.drawable.ic_warning, R.string.file_delete_confirm);

            final Button
                    btnNo   = dialog.findViewById(R.id.btn_no),
                    btnYes  = dialog.findViewById(R.id.btn_yes);

            btnNo.setOnClickListener( v1 -> dialog.dismiss() );

            btnYes.setOnClickListener( v12 ->
            {
                String item = adapterListView.getSelectedItem().getUrl();
                database.deleteDataFile(item);
                int index   = adapterListView.getSelectedIndex();
                list.setItemChecked(-1, true);
                adapterListView.setSelectedIndex(-1);
                adapterListView.remove(index);
                adapterListView.notifyDataSetChanged();
                parent.deletedFile(item);
                if (loadedTestSite.get().equals(item)) {
                    loadedTestSite.set("");
                    selectedTestSite.set("");
                }
                else {
                    int selected = -1;
                    for (int i = 0; i < adapterListView.getCount(); i++) {
                        if (adapterListView.getItem(i).getUrl().equals(loadedTestSite.get())) {
                            selected = i;
                            break;
                        }
                    }
                    adapterListView.setSelectedFileDetails(database.getDataFileDetails(loadedTestSite.get()));
                    adapterListView.setSelectedIndex(selected);
                    list.setItemChecked(selected, true);
                    list.smoothScrollToPosition(selected);
                    adapterListView.notifyDataSetChanged();
                    selectedTestSite.set(loadedTestSite.get());
                }

                dialog.dismiss();
            } );

            dialog.show();
        } );

        btNew.setOnClickListener( v ->
        {
            testData.reset();
            if (database.getSites().size() > 0)
                testSetup(true, testData);
            else
                addTestSite(1, testData);
            dialogManager.dismiss();
        } );
    }

    private void tabSites(final TestData testData) {
        final ListView list     = dialogManager.findViewById(R.id.list_sites);
        final Button newSite    = dialogManager.findViewById(R.id.btn_sites_new);
        final Button deleteSite = dialogManager.findViewById(R.id.btn_site_delete);
        final Button editSite   = dialogManager.findViewById(R.id.btn_sites_edit);
        selectedTestSite.update("");

        final List<String> sites;
        if (database.getSites()!=null) {
            sites = database.getSites();
            Collections.sort(sites);
        }
        else {
            sites = new ArrayList<>();
        }
        final ArrayAdapter<String> modeAdapter = new ArrayAdapter<>(activity, R.layout.simple_list_item, R.id.text1, sites);
        list.setChoiceMode(AbsListView.CHOICE_MODE_SINGLE);
        list.setAdapter(modeAdapter);

        list.setOnItemClickListener( ( parent, view, position, id ) ->
        {
            String item = ((TextView) view).getText().toString();
            selectedTestSite.set(item);
        } );

        newSite.setOnClickListener( v ->
        {
            selectedTestSite.set("");
            dialogManager.dismiss();
            addTestSite(0, testData);
        } );

        editSite.setOnClickListener( v ->
        {
            createSiteDetailsDialog(selectedTestSite.get(), testData);
            dialogManager.dismiss();
        } );

        deleteSite.setOnClickListener( v ->
        {
            if ( selectedTestSite.get().equals( "" ) )
            {
                return;
            }
            final Dialog dialog = uGetDialogYesNo(activity, R.string.site_delete, R.drawable.ic_warning, R.string.site_delete_confirm);

            final Button btnNo = dialog.findViewById(R.id.btn_no);
            btnNo.setOnClickListener( v1 -> dialog.dismiss() );

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener( v12 ->
            {
                database.removeSite(selectedTestSite.get());
                sites.remove(list.getCheckedItemPosition());
                list.setItemChecked(-1, true);
                modeAdapter.notifyDataSetChanged();
                selectedTestSite.set("");
                dialog.dismiss();
            } );

            dialog.show();
        } );
    }

    private void tabOperators(final TestData testData) {
        final ListView list         = dialogManager.findViewById(R.id.list_operator);
        final Button newOperator    = dialogManager.findViewById(R.id.btn_operator_new);
        final Button editOperator   = dialogManager.findViewById(R.id.btn_operator_edit);
        selectedOperator.update("");

        String[] operators;
        if (database.getOperators() != null) {
            operators = database.getOperators();
            Arrays.sort(operators);
        } else {
            operators = new String[]{""};
        }

        ArrayAdapter<String> modeAdapter = new ArrayAdapter<>(activity, R.layout.simple_list_item, R.id.text1, operators);
        list.setChoiceMode(1);
        list.setAdapter(modeAdapter);

        list.setOnItemClickListener( ( parent, view, position, id ) ->
        {
            String item = ((TextView) view).getText().toString();
            item = item.replace(" " + activity.getResources().getString(R.string.defaults), "");
            selectedOperator.set(item);
        } );


        newOperator.setOnClickListener( v ->
        {
            createOperatorAddDialog(testData);
            dialogManager.dismiss();
        } );

        editOperator.setOnClickListener( v ->
        {
            createOperatorDetailsDialog(selectedOperator.get(), testData);
            selectedOperator.set("");
            dialogManager.dismiss();
        } );
    }

    void objectsList( final TestData testData ) {
        final Dialog
                dialog        = uGetDialogFull(activity, R.layout.dialog_test_details, R.string.tests_details, R.drawable.ic_details_50);

        final TextView
                tvTestsite    = dialog.findViewById(R.id.txt_testsite),
                tvContact     = dialog.findViewById(R.id.txt_contact),
                tvAddress     = dialog.findViewById(R.id.txt_address),
                tvCity        = dialog.findViewById(R.id.txt_city),
                //tvState       = dialog.findViewById(R.id.txt_state),
                //tvZipcode     = dialog.findViewById(R.id.txt_zipcode),
                //tvPhone       = dialog.findViewById(R.id.txt_phone),
                //tvFax         = dialog.findViewById(R.id.txt_fax),
                tvOperator    = dialog.findViewById(R.id.txt_operator);
                //tvMobile      = dialog.findViewById(R.id.txt_mobile),
                //tvEmail       = dialog.findViewById(R.id.txt_email);

        final Button
                btEdit      = dialog.findViewById(R.id.btn_edit),
                btDeleteObj = dialog.findViewById(R.id.btn_object_delete),
                btNewObj    = dialog.findViewById(R.id.btn_object_new),
                btEditObj   = dialog.findViewById(R.id.btn_object_edit),
                btReady     = dialog.findViewById(R.id.btn_object_cancel),
                btComments  = dialog.findViewById(R.id.btn_comments);

        tvTestsite  .setText(testData.getSiteName());
        tvContact   .setText(testData.getSiteContact());
        tvAddress   .setText(testData.getSiteAddress());
        tvCity      .setText(testData.getSiteCity());
        //tvState     .setText(testData.getSiteState());
        //tvZipcode   .setText(testData.getSiteZipCode());
        //tvPhone     .setText(testData.getSitePhone());
        //tvFax       .setText(testData.getSiteFax());
        tvOperator  .setText(testData.getOperatorName());
        //tvMobile    .setText(testData.getOperatorExtension());
        //tvEmail     .setText(testData.getOperatorEmail());

        btReady.setText(R.string.close);

        final ListView list = dialog.findViewById(R.id.list_object);
        final List<String> obj = database.getObjectsList(testData.getFile());
        final ArrayAdapter<String> modeAdapter = new ArrayAdapter<>(activity, R.layout.simple_list_item, R.id.text1, obj);
        list.setChoiceMode(AbsListView.CHOICE_MODE_SINGLE);
        list.setAdapter(modeAdapter);

        final TextView lblObj = dialog.findViewById(R.id.lbl_obj_list);
        lblObj.append(" (" + obj.size() + ")");

        final TextView lblObs = dialog.findViewById(R.id.lbl_obs);

        if (testData.getObject()!=null) {
            for (int i = 0; i < obj.size(); i++) {
                if (testData.getObjectName().equals(obj.get(i))) {
                    lblObs.setVisibility(View.GONE);
                    btReady.setText(R.string.ready_measure);
                    list.setItemChecked(i, true);
                    list.smoothScrollToPosition(i);
                    break;
                }
            }
        }
        else {
            btDeleteObj.setEnabled(false);
            btEditObj.setEnabled(false);
        }

        list.setOnItemClickListener( ( parent, view, position, id ) ->
        {
            String objName = ((TextView) view).getText().toString();
            testData.addObject(objName);
            lblObs.setVisibility(View.GONE);
            btReady.setText(R.string.ready_measure);
            btDeleteObj.setEnabled(true);
            btEditObj.setEnabled(true);
        } );

        btDeleteObj.setOnClickListener( v ->
        {
            final Dialog
                    dialog1 = uGetDialogYesNo(activity, R.string.object_delete, R.drawable.ic_warning, R.string.object_delete_confirm);

            final Button
                    btnNo   = dialog1.findViewById(R.id.btn_no),
                    btnYes  = dialog1.findViewById(R.id.btn_yes);

            btnNo.setOnClickListener( v1 -> dialog1.dismiss() );

            btnYes.setOnClickListener( v12 ->
            {
                if (list.getCheckedItemPosition()>-1) {
                    String name = testData.getObjectName();
                    database.removeObject(testData.getFile(), name);
                    testData.clearObject(name);
                    obj.remove(list.getCheckedItemPosition());
                    list.setItemChecked(-1, true);
                    modeAdapter.notifyDataSetChanged();
                    lblObs.setVisibility(View.VISIBLE);
                    btReady.setText(R.string.close);
                    btDeleteObj.setEnabled(false);
                    btEditObj.setEnabled(false);
                }
                dialog1.dismiss();
            } );

            dialog1.show();
        } );

        btEdit.setOnClickListener( v ->
        {
            updateTestsiteDetails(testData);
            dialog.dismiss();
        } );

        btNewObj.setOnClickListener( v ->
        {
            createObjectAddDialog(testData);
            dialog.dismiss();
        } );

        btEditObj.setOnClickListener( v ->
        {
            if (testData.getObject()!=null) {
                testData.addObject(
                        database.getObject(testData, testData.getObjectName())
                );
                objectDetails(testData, null);
            }
        } );

        btReady.setOnClickListener( v ->
        {
            if(testData.getObject()!=null) {
                Objects objs = database.getObject(testData, testData.getObjectName());
                testData.addObject(objs);
                parent.objectSelected();
            }
            dialog.dismiss();
        } );

        btComments.setOnClickListener( v ->
        {
            createCommentsDialog(testData);
            dialog.dismiss();
        } );

        dialog.show();
    }

    private void createObjectAddDialog( final TestData testData ) {
        final Dialog
                dialog      = uGetDialog(activity, R.layout.dialog_object_new, R.string.object_new, R.drawable.ic_new);

        final TextView
                objectName  = dialog.findViewById(R.id.txt_object_name);

        View.OnClickListener onClose = view ->
        {
            objectsList( testData );
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_object_cancel ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        final Button btnSave = dialog.findViewById(R.id.btn_object_save);
        btnSave.setOnClickListener( v ->
        {
            String objName = objectName.getText().toString();
            if (!objName.equals("")) {
                if (database.objectExists(testData.getFile(), objName)) {
                    alert(R.string.object_name_alredy_exists);
                } else {
                    testData.addObject(objName);
                    database.addTestObject(testData);
                    objectsList(testData);
                    dialog.dismiss();
                }
            }
        } );

        dialog.show();
    }

    void newObjectName() {
        final Dialog
                dialog      = uGetDialog(activity, R.layout.dialog_object_new, R.string.object_new, R.drawable.ic_new);

        final TextView
                objectName  = dialog.findViewById(R.id.txt_object_name);

        dialog.findViewById( R.id.btn_object_cancel ).setOnClickListener( v -> dialog.dismiss() );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> dialog.dismiss() );

        final Button btnSave = dialog.findViewById(R.id.btn_object_save);
        btnSave.setOnClickListener( v ->
        {
            String name = objectName.getText().toString();
            if (!name.equals("")) {
                parent.newObjectName(name);
                dialog.dismiss();
            }
        } );

        dialog.show();
    }

    @SuppressLint("SetTextI18n")
    public void options( final Device device) {
        dialogOptions       = uGetDialog(activity, R.layout.dialog_equipment_details2, R.string.equipment_details, R.drawable.ic_device);

        final TextView
                tvDate      = dialogOptions.findViewById(R.id.tv_date),
                tvMem       = dialogOptions.findViewById(R.id.tv_memory),
                tvTime      = dialogOptions.findViewById(R.id.tv_time);

        final Button
                btSave      = dialogOptions.findViewById(R.id.bt_save);

        final ImageButton
                ibtDate     = dialogOptions.findViewById(R.id.bt_date),
                ibtTime     = dialogOptions.findViewById(R.id.bt_time),
                ibtMem      = dialogOptions.findViewById(R.id.bt_memory);

        final Spinner
                spConnect   = dialogOptions.findViewById(R.id.sp_connection);

        ArrayAdapter<CharSequence>
                filter      = ArrayAdapter.createFromResource(
                                  activity,
                                  R.array.enable_disable,
                                  android.R.layout.simple_list_item_activated_1
                              );
        filter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);
        spConnect.setAdapter(filter);

        final Date
                currentDate = device.getDate();

        tvMem.setText(device.getFreeMemory()+" %");
        tvDate.setText(FormatData.date(device.getDateFormat(), device.getDate()));
        tvTime.setText(FormatData.time(device.getTimeFormat(), device.getDate()));

        if(device.getAutoConnection()){
            spConnect.setSelection(0);
        } else {
            spConnect.setSelection(1);
        }

        ibtDate.setOnClickListener( v -> pickDate(device) );

        ibtTime.setOnClickListener( v -> pickTime(device) );

        ibtMem.setOnClickListener( v ->
        {
            final Dialog dialog = uGetDialogYesNo(activity, R.string.erase_memory, R.drawable.ic_warning, R.string.erase_memory_confirm);

            final Button btnNo = dialog.findViewById(R.id.btn_no);
            btnNo.setOnClickListener( v1 -> dialog.dismiss() );

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener( v12 ->
            {
                parent.eraseMemory();
                dialogOptions.dismiss();
                dialog.dismiss();
            } );

            dialog.show();
        } );

        View.OnClickListener onClose = view ->
        {
            device.setDate( currentDate );
            dialogOptions.dismiss();
        };

        dialogOptions.findViewById( R.id.bt_cancel ).setOnClickListener( onClose );
        dialogOptions.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        btSave.setOnClickListener( v ->
        {
            switch (spConnect.getSelectedItemPosition()) {
                case 0:
                    device.setAutoConnection(true);
                    break;
                case 1:
                    device.setAutoConnection(false);
                    break;
            }
            parent.updateDevice(device);
            dialogOptions.dismiss();
        } );

        dialogOptions.show();
    }

    public void pickDate(final Device device)
    {
        final DatePickerDialog.OnDateSetListener dateListener = ( arg0, year, month, day ) ->
        {
            if ( dialogOptions!=null )
            {
                TextView date  = dialogOptions.findViewById(R.id.tv_date);
                TextView time  = dialogOptions.findViewById(R.id.tv_time);
                String parts[] = time.getText().toString().split(":");
                int hour       = Integer.parseInt(parts[0]);
                int minutes    = Integer.parseInt(parts[1]);

                Calendar c     = Calendar.getInstance();
                c.set(year, month, day, hour, minutes);

                device.setDate(c.getTime());
                date.setText( FormatData.date(device.getDateFormat(), device.getDate()) );
            }
        };

        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int year  = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day   = calendar.get(Calendar.DAY_OF_MONTH);
        new DatePickerDialog(activity, dateListener, year, month, day).show();
    }

    public void pickTime(final Device device)
    {
        TimePickerDialog.OnTimeSetListener timeListener = ( view, hourOfDay, minute ) ->
        {
        if( dialogOptions!=null ){
            TextView time = dialogOptions.findViewById(R.id.tv_time);
            Calendar c    = Calendar.getInstance();
            c.setTime(device.getDate());
            c.set(Calendar.HOUR_OF_DAY, hourOfDay);
            c.set(Calendar.MINUTE, minute);
            c.set(Calendar.SECOND, 0);
            device.setDate( c.getTime() );
            time.setText(  FormatData.time( device.getTimeFormat(), device.getDate() ) );
        }
        };

        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int hour     = calendar.get(Calendar.HOUR_OF_DAY);
        int minute   = calendar.get(Calendar.MINUTE);
        boolean is24 = device.getTimeFormat()== Modbus.TIME_24H;
        new TimePickerDialog(activity, timeListener, hour, minute, is24).show();
    }

    @SuppressWarnings("ManualArrayCopy")
    private void updateTestsiteDetails( final TestData testData) {
        final Dialog
                dialog      = uGetDialogFull(activity, R.layout.dialog_site, R.string.site_edit, R.drawable.ic_edit);

        final TextView
                lblOperator = dialog.findViewById(R.id.lbl_operator_sel),
                required    = dialog.findViewById(R.id.lbl_required),
                site        = dialog.findViewById(R.id.txt_site),
                contact     = dialog.findViewById(R.id.txt_site_contact),
                address     = dialog.findViewById(R.id.txt_site_address),
                city        = dialog.findViewById(R.id.txt_site_city),
                state       = dialog.findViewById(R.id.txt_site_state),
                zipcode     = dialog.findViewById(R.id.txt_site_zip),
                phone       = dialog.findViewById(R.id.txt_site_phone),
                fax         = dialog.findViewById(R.id.txt_site_fax);

        final Spinner
                spinner     = dialog.findViewById(R.id.spinner_operator);

        final String[]
                operatorsList = database.getOperators();

        int index           = operatorsList.length+1;

        final String[]
                operatorsListWithNone;

        if (operatorsList.length==0 && !testData.getOperatorName().equals("")) {
            operatorsListWithNone = new String[2];
            operatorsListWithNone[1] = testData.getOperatorName();
        }
        else
        {
            operatorsListWithNone = new String[index];
            for ( int i = 0; i < operatorsList.length; i++ )
            {
                operatorsListWithNone[i + 1] = operatorsList[i];
            }
        }

        operatorsListWithNone[0] = "";

        ArrayAdapter<String> dataAdapter = new ArrayAdapter<>(activity, android.R.layout.simple_spinner_item, operatorsListWithNone);
        dataAdapter.setDropDownViewResource(android.R.layout.simple_list_item_activated_1);
        spinner.setAdapter(dataAdapter);

        spinner.setSelection(0);

        for (int i=0; i<operatorsListWithNone.length;i++){
            if (testData.getOperatorName().equals(operatorsListWithNone[i])){
                spinner.setSelection(i);
            }
        }

        lblOperator.setVisibility(View.VISIBLE);
        spinner.setVisibility(View.VISIBLE);
        required.setVisibility(View.GONE);
        site.setKeyListener(null);
        site.setClickable(false);
        site.setEnabled(false);
        site.setTextColor(0x66FFFFFF);

        site.setText(testData.getSiteName());
        contact.setText(testData.getSiteContact());
        address.setText(testData.getSiteAddress());
        city.setText(testData.getSiteCity());
        state.setText(testData.getSiteState());
        zipcode.setText(testData.getSiteZipCode());
        phone.setText(testData.getSitePhone());
        fax.setText(testData.getSiteFax());

        View.OnClickListener onClose = view ->
        {
            objectsList(testData);
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_site_cancel ).setOnClickListener( onClose );

        final Button save = dialog.findViewById(R.id.btn_site_save);
        save.setOnClickListener( v ->
        {
            String selOp = String.valueOf(spinner.getSelectedItem());

            if (!selOp.equals(testData.getOperatorName())) {
                String[] temp = database.getOperatorDetails(selOp);
                testData.setOperatorName(selOp);
                testData.setOperatorExtension(temp[0]);
                testData.setOperatorEmail(temp[1]);
            }
            testData.setSiteContact(contact.getText().toString());
            testData.setSiteAddress(address.getText().toString());
            testData.setSiteCity(city.getText().toString());
            testData.setSiteState(state.getText().toString());
            testData.setSiteZipCode(zipcode.getText().toString());
            testData.setSitePhone(phone.getText().toString());
            testData.setSiteFax(fax.getText().toString());

            database.updateSiteInFile(testData);
            objectsList(testData);
            dialog.dismiss();
        } );

        dialog.show();
    }

    private void createCommentsDialog( final TestData testData )
    {
        final Dialog dialog = uGetDialogFull( activity, R.layout.dialog_comments, R.string.comments, R.drawable.ic_edit );

        final EditText comments = dialog.findViewById(R.id.txt_comments);
        String currentSite = database.getComments(testData.getFile());
        comments.setText(currentSite);

        View.OnClickListener onClose = v ->
        {
            objectsList(testData);
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_cancel ).setOnClickListener( onClose );

        final Button save = dialog.findViewById(R.id.btn_save);
        save.setOnClickListener( v ->
        {
            database.addComments(testData.getFile(), comments.getText().toString());
            objectsList(testData);
            dialog.dismiss();
        } );

        dialog.show();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    @SuppressLint("SetTextI18n")
    void objectDetails( final TestData testData, final File file )
    {
        final Dialog
                dialog          = uGetDialogFull(activity, R.layout.dialog_object_details, R.string.tests_list, R.drawable.ic_details_50);

        final TextView
                lblName         = dialog.findViewById(R.id.lbl_object_name),
                lblNumTests     = dialog.findViewById(R.id.lbl_num_tests),
                txtLongitude    = dialog.findViewById(R.id.txt_longitude),
                txtLatitude     = dialog.findViewById(R.id.txt_latitude),
                txtAltitude     = dialog.findViewById(R.id.txt_altitude),
                txtAccuracy     = dialog.findViewById(R.id.txt_accuracy);

        final Button
                btVoice         = dialog.findViewById(R.id.btn_object_play),
                btDeleteTest    = dialog.findViewById(R.id.btn_delete),
                btDeleteAudio   = dialog.findViewById(R.id.btn_del_audio);

        final File
                imgFile         = database.getPictureFile(testData.getFolder(), testData.getObjectName());

        mCommonUI.enableButton(btVoice, false);
        mCommonUI.enableButton(btDeleteTest, false);
        mCommonUI.enableButton(btDeleteAudio, false);

        if (imgFile.exists()) {
            ImageView imgThumb  = dialog.findViewById(R.id.img_thumb);
            Bitmap img          = BitmapFactory.decodeFile(imgFile.getAbsolutePath());
            Bitmap thumb        = ThumbnailUtils.extractThumbnail(img, 200, 200);
            imgThumb.setImageBitmap(thumb);

            imgThumb.setOnClickListener( v ->
            {
                DialogInterface.OnDismissListener onDismissListener = dialog1 -> objectDetails(testData, file);
                parent.pictureShow(imgFile, onDismissListener);
                dialog.dismiss();
            } );
        }

        lblName.setText(testData.getObjectName());
        txtLongitude.setText(testData.getObject().getLongitude());
        txtLatitude.setText(testData.getObject().getLatitude());
        txtAltitude.setText(testData.getObject().getAltitude());
        txtAccuracy.setText(testData.getObject().getAccuracy());

        final ListView
                list            = dialog.findViewById(R.id.list_object_details);

        final ListDetails
                adapterListView = new ListDetails(activity, testData.getObject().getMeasurements());

        list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        list.setAdapter(adapterListView);

        list.setOnItemClickListener( ( parent, view, position, id ) ->
        {
            Measurement m = adapterListView.getItem(position);

            if (m.getAudio().exists()) {
                mCommonUI.enableButton(btDeleteAudio, true);
            } else {
                mCommonUI.enableButton(btDeleteAudio, false);
            }
            mCommonUI.enableButton(btVoice, true);
            mCommonUI.enableButton(btDeleteTest, true);
        } );

        if (file!=null) {
            for (int i = 0; i < adapterListView.getCount(); i++) {
                if (adapterListView.getItem(i).getAudio() == file) {
                    list.setItemChecked(i, true);
                    mCommonUI.enableButton(btDeleteAudio, true);
                    mCommonUI.enableButton(btVoice, true);
                    mCommonUI.enableButton(btDeleteTest, true);
                    break;
                }
            }
        }

        lblNumTests.setText(adapterListView.getCount() + "");

        btVoice.setOnClickListener( v ->
        {
            int index = list.getCheckedItemPosition();
            if (index>-1) {
                parent.playAudio(adapterListView.getItem(index).getAudio(), adapterListView.getItem(index).getTestNumber());
                dialog.dismiss();
            }
        } );

       // final Button cancel = dialog.findViewById( R.id.btn_object_cancel );
        dialog.findViewById( R.id.btn_object_cancel ).setOnClickListener( v -> dialog.dismiss() );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> dialog.dismiss() );

        btDeleteTest.setOnClickListener( v ->
        {
            final Dialog
                    dialog12 = uGetDialogYesNo(activity, R.string.test_delete, R.drawable.ic_warning, R.string.test_delete_confirm);

            final Button
                    btNo    = dialog12.findViewById(R.id.btn_no),
                    btYes   = dialog12.findViewById(R.id.btn_yes);

            btNo.setOnClickListener( v1 -> dialog12.dismiss() );

            btYes.setOnClickListener( v12 ->
            {
                int index = list.getCheckedItemPosition();
                if (index>-1){
                    String testNumber = adapterListView.getItem(index).getTestNumber();
                    if ( database.removeMeasurement(testData.getFile(), testData.getObjectName(), testNumber) ){
                        testData.getObject().removeMeasurement(testNumber);
                        list.setItemChecked(-1, true);
                        adapterListView.notifyDataSetChanged();
                    }
                }

                dialog12.dismiss();
            } );

            dialog12.show();
        } );

        btDeleteAudio.setOnClickListener( v ->
        {
            final Dialog
                    dialog13 = uGetDialogYesNo(activity, R.string.audio_delete, R.drawable.ic_warning, R.string.audio_delete_confirm);

            final Button
                    btNo    = dialog13.findViewById(R.id.btn_no),
                    btYes   = dialog13.findViewById(R.id.btn_yes);

            btNo.setOnClickListener( v13 -> dialog13.dismiss() );

            btYes.setOnClickListener( v14 ->
            {
                if (list.getCheckedItemPosition() > -1) {
                    adapterListView.getItem(list.getCheckedItemPosition()).getAudio().delete();
                    adapterListView.notifyDataSetChanged();
                    mCommonUI.enableButton(btDeleteAudio, false);
                    mCommonUI.enableButton(btVoice, true);
                    mCommonUI.enableButton(btDeleteTest, true);
                }
                dialog13.dismiss();
            } );

            dialog13.show();
        } );

        dialog.show();
    }

    private void createOperatorAddDialog(final TestData testData) {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.dialog_operator_details);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
        }
        ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.operator);
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_operator);

        final TextView operator     = dialog.findViewById(R.id.txt_operator);
        final TextView extension    = dialog.findViewById(R.id.txt_extension);
        final TextView email        = dialog.findViewById(R.id.txt_email);

        View.OnClickListener onClose = view ->
        {
            showManager(2, testData);
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        final Button setDefault = dialog.findViewById(R.id.btn_operator_default);
        setDefault.setVisibility(View.GONE);
        final Button delete = dialog.findViewById(R.id.btn_operator_delete);
        delete.setVisibility(View.GONE);

        final Button save = dialog.findViewById(R.id.btn_operator_save);
        save.setOnClickListener( v ->
        {
            String opName = operator.getText().toString();
            if (!"".equals(opName)) {
                if (database.operatorExists(opName)) {
                    alert(R.string.operator_name_alredy_exists);
                } else {
                    database.addOperator(opName, extension.getText().toString(), email.getText().toString());
                    showManager(2, testData);
                    dialog.dismiss();
                }
            }
        } );
        dialog.show();
    }

    private void createOperatorDetailsDialog(String selectedOperator, final TestData testData) {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.dialog_operator_details);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
        }
        ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.operator);
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_operator);

        final String operat = selectedOperator;
        final TextView operator = dialog.findViewById(R.id.txt_operator);
        final TextView extension = dialog.findViewById(R.id.txt_extension);
        final TextView email = dialog.findViewById(R.id.txt_email);

        String[] details = database.getOperatorDetails(selectedOperator);

        operator.setText(selectedOperator);
        extension.setText(details[0]);
        email.setText(details[1]);

        View.OnClickListener onClose = view ->
        {
            showManager(2, testData);
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        final Button delete = dialog.findViewById(R.id.btn_operator_delete);
        //final Dialog dlg = dialog;
        delete.setOnClickListener( v ->
        {
            final Dialog dialog1 = new Dialog(activity);
            dialog1.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog1.setCanceledOnTouchOutside(false);
            dialog1.setContentView(R.layout.dialog_yes_no);
            if ( dialog1.getWindow() != null )
            {
                dialog1.getWindow().getAttributes().width = WindowManager.LayoutParams.WRAP_CONTENT;
                dialog1.getWindow().getAttributes().height = WindowManager.LayoutParams.WRAP_CONTENT;
            }

            ((TextView) dialog1.findViewById(R.id.dlg_title)).setText(R.string.operator_delete);
            ((ImageView) dialog1.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog1.findViewById(R.id.message);
            message.setText(R.string.operator_delete_confirm);

            final Button btnNo = dialog1.findViewById(R.id.btn_no);
            btnNo.setOnClickListener( v1 -> dialog1.dismiss() );

            final Button btnYes = dialog1.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener( v12 ->
            {
                if (database.getDefaultOperator().equals(operat)) {
                    database.setDefaultOperator("");
                }
                database.removeOperator(operat);
                showManager(2, testData);
                dialog.dismiss();
                dialog1.dismiss();
            } );

            dialog1.show();
        } );

        final Button setDefault = dialog.findViewById(R.id.btn_operator_default);
        setDefault.setOnClickListener( v ->
        {
            database.removeOperator(operat);
            database.updateOperator(operator.getText().toString(), extension.getText().toString(), email.getText().toString());
            database.setDefaultOperator(operator.getText().toString());
            showManager(2, testData);
            dialog.dismiss();
        } );

        final Button save = dialog.findViewById(R.id.btn_operator_save);
        save.setOnClickListener( v ->
        {
            if (!operator.getText().toString().equals("")) {
                database.removeOperator(operat);
                database.updateOperator(operator.getText().toString(), extension.getText().toString(), email.getText().toString());
                showManager(2, testData);
                dialog.dismiss();
            }
        } );

        dialog.show();
    }

    void addTestSite( final int option, final TestData testData ) {
        final Dialog dialog = new Dialog(activity, R.style.MyThemeCompat);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.dialog_site);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }
        ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.site_new);
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_new);

        final TextView site     = dialog.findViewById(R.id.txt_site);
        final TextView contact  = dialog.findViewById(R.id.txt_site_contact);
        final TextView address  = dialog.findViewById(R.id.txt_site_address);
        final TextView city     = dialog.findViewById(R.id.txt_site_city);
        final TextView state    = dialog.findViewById(R.id.txt_site_state);
        final TextView zipcode  = dialog.findViewById(R.id.txt_site_zip);
        final TextView phone    = dialog.findViewById(R.id.txt_site_phone);
        final TextView fax      = dialog.findViewById(R.id.txt_site_fax);


        View.OnClickListener onClose = view ->
        {
            if ( option == 0 )
            {
                showManager( 1, testData );
            }
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_site_cancel ).setOnClickListener( onClose );

        final Button save = dialog.findViewById(R.id.btn_site_save);
        save.setOnClickListener( v ->
        {
            String siteName = site.getText().toString();
            if (!"".equals(siteName)) {
                if (database.testsiteExists(siteName)) {
                    alert(R.string.site_name_alredy_exists);
                } else {
                    database.addSite(
                            site.getText().toString(),
                            contact.getText().toString(),
                            address.getText().toString(),
                            city.getText().toString(),
                            state.getText().toString(),
                            zipcode.getText().toString(),
                            phone.getText().toString(),
                            fax.getText().toString()
                    );
                    if (option == 0) {
                        showManager(1, testData);
                    } else {
                        testSetup(true, testData);
                    }
                    dialog.dismiss();
                }
            }
        } );

        site.requestFocus();

        dialog.show();
    }

    private void tabCompany() {
        String[] data = database.getCompany();

        final TextView company  = dialogManager.findViewById(R.id.txt_company);
        final TextView address  = dialogManager.findViewById(R.id.txt_address);
        final TextView city     = dialogManager.findViewById(R.id.txt_city);
        final TextView state    = dialogManager.findViewById(R.id.txt_state);
        final TextView zipcode  = dialogManager.findViewById(R.id.txt_zip);
        final TextView phone    = dialogManager.findViewById(R.id.txt_phone);
        final TextView fax      = dialogManager.findViewById(R.id.txt_fax);
        final Button saveData   = dialogManager.findViewById(R.id.btn_firm_save);
        final Button clear      = dialogManager.findViewById(R.id.btn_firm_clean);

        if (data != null) {
            company.setText(data[0]);
            address.setText(data[1]);
            city.setText(data[2]);
            state.setText(data[3]);
            zipcode.setText(data[4]);
            phone.setText(data[5]);
            fax.setText(data[6]);
        }

        saveData.setOnClickListener( v ->
        {
            database.updateCompany(
                    company.getText().toString(),
                    address.getText().toString(),
                    city.getText().toString(),
                    state.getText().toString(),
                    zipcode.getText().toString(),
                    phone.getText().toString(),
                    fax.getText().toString()
            );
            dialogManager.dismiss();
        } );

        clear.setOnClickListener( v ->
        {
            company.setText("");
            address.setText("");
            city.setText("");
            state.setText("");
            zipcode.setText("");
            phone.setText("");
            fax.setText("");
        } );
    }

    void testSetup( final boolean openManager, final TestData testData ) {
        final Dialog
                dialog          = uGetDialog(activity, R.layout.dialog_test_setup, R.string.test_setup, R.drawable.ic_new);

        final Spinner
                spinnerOperator = dialog.findViewById(R.id.spinner_operator),
                spinnerSite     = dialog.findViewById(R.id.spinner_sites);

        final TextView
                txtObject       = dialog.findViewById(R.id.txt_object_name);

        String[]
                operators       = database.getOperatorsOrderByDefault();

        ArrayAdapter<String>
                dataAdapter     = new ArrayAdapter<>(activity, android.R.layout.simple_spinner_item, operators);
        dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinnerOperator.setAdapter(dataAdapter);

        List<String>
                sites           = database.getSites();

        ArrayAdapter<String>
                dataAdapterSite = new ArrayAdapter<>(activity, android.R.layout.simple_spinner_item, sites);
        dataAdapterSite.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinnerSite.setAdapter(dataAdapterSite);

        final Button btnCancel = dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener( v ->
        {
            dialog.dismiss();
            if(openManager)
                showManager(0, testData);
        } );

        final Button btnSave = dialog.findViewById(R.id.btn_save);
        btnSave.setOnClickListener( v ->
        {
            String site         = String.valueOf(spinnerSite.getSelectedItem());
            String operator     = String.valueOf(spinnerOperator.getSelectedItem());
            String objectName   = txtObject.getText().toString();
            if (!site.equals("") && !objectName.equals("")) {
                parent.savedFile(site, objectName, operator);
                dialog.dismiss();
            }
        } );

        dialog.findViewById(R.id.dlg_newtest).requestFocus();
        dialog.show();
    }

    private void createSiteDetailsDialog(String selectedSite, final TestData testData) {
        final Dialog dialog = new Dialog(activity, R.style.MyThemeCompat);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.dialog_site);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }
        ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.site_edit);
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_edit);

        final String selSite    = selectedSite;
        final TextView required = dialog.findViewById(R.id.lbl_required);
        final TextView site     = dialog.findViewById(R.id.txt_site);
        final TextView contact  = dialog.findViewById(R.id.txt_site_contact);
        final TextView address  = dialog.findViewById(R.id.txt_site_address);
        final TextView city     = dialog.findViewById(R.id.txt_site_city);
        final TextView state    = dialog.findViewById(R.id.txt_site_state);
        final TextView zipcode  = dialog.findViewById(R.id.txt_site_zip);
        final TextView phone    = dialog.findViewById(R.id.txt_site_phone);
        final TextView fax      = dialog.findViewById(R.id.txt_site_fax);

        required.setVisibility(View.GONE);
        site.setKeyListener(null);
        site.setClickable(false);
        site.setEnabled(false);
        site.setTextColor(0x66FFFFFF);

        String[] details = database.getSiteDetails(selectedSite);
        site.setText(selectedSite);
        contact.setText(details[0]);
        address.setText(details[1]);
        city.setText(details[2]);
        state.setText(details[3]);
        zipcode.setText(details[4]);
        phone.setText(details[5]);
        fax.setText(details[6]);

        View.OnClickListener onClose = view ->
        {
            showManager(1, testData);
            dialog.dismiss();
        };

        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_site_cancel ).setOnClickListener( onClose );


        final Button save = dialog.findViewById(R.id.btn_site_save);
        save.setOnClickListener( v ->
        {
            if (!"".equals(site.getText().toString())) {
                database.removeSite(selSite);
                database.addSite(
                        site.getText().toString(),
                        contact.getText().toString(),
                        address.getText().toString(),
                        city.getText().toString(),
                        state.getText().toString(),
                        zipcode.getText().toString(),
                        phone.getText().toString(),
                        fax.getText().toString()
                );
                showManager(1, testData);
                dialog.dismiss();
            }
        } );

        dialog.show();
    }


    private void setTabs(Activity activity, int dialog,int message, int image, TabHost tabs) {
        String description = activity.getResources().getString(message);
        View tabIndicator  = LayoutInflater.from(activity).inflate(R.layout.tab_indicator, tabs.getTabWidget(), false);
        ((TextView) tabIndicator.findViewById(R.id.tab_title)).setText(description.toUpperCase(Locale.getDefault()));
        ((ImageView) tabIndicator.findViewById(R.id.tab_icon)).setImageResource(image);

        TabHost.TabSpec spec = tabs.newTabSpec("tab_" + dialog);
        spec.setIndicator(tabIndicator);
        spec.setContent(dialog);
        tabs.addTab(spec);
    }

    private void alert(int msg) {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.dialog_yes_no);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.WRAP_CONTENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.WRAP_CONTENT;
        }

        ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.warning);
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

        final TextView message = dialog.findViewById(R.id.message);
        message.setText(msg);

        final Button btnNo = dialog.findViewById(R.id.btn_no);
        btnNo.setText(R.string.close);
        btnNo.setOnClickListener( v -> dialog.dismiss() );

        final Button btnYes = dialog.findViewById(R.id.btn_yes);
        btnYes.setVisibility(View.GONE);

        dialog.show();
    }

    private Dialog uGetDialogFull(Activity activity, int layout, int title, int icon){
        final Dialog dialog = new Dialog(activity, R.style.MyThemeCompat);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(layout);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }
        ((TextView)  dialog.findViewById(R.id.dlg_title)).setText(title);
        ((ImageView) dialog.findViewById(R.id.dlg_icon) ).setImageResource(icon);
        return dialog;
    }

    private Dialog uGetDialog(Activity activity, int layout, int title, int icon){
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(layout);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            //dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            //dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }
        ((TextView)  dialog.findViewById(R.id.dlg_title)).setText(title);
        ((ImageView) dialog.findViewById(R.id.dlg_icon) ).setImageResource(icon);
        return dialog;
    }

    @SuppressWarnings("SameParameterValue")
    private Dialog uGetDialogYesNo( Activity activity, int title, int icon, int message){
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.dialog_yes_no);
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.WRAP_CONTENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.WRAP_CONTENT;
        }

        ((TextView) dialog.findViewById(R.id.dlg_title)).setText(title);
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(icon);

        final TextView tvMessage = dialog.findViewById(R.id.message);
        tvMessage.setText(message);
        return dialog;
    }


    @Override
    public void update(Observable o, Object arg) {
        if (o instanceof StringObservable){
            if (dialogManager==null)
                return;
            StringObservable bol = (StringObservable) o;
            if ( bol.ID == selectedTestSite.ID ){
                if ( bol.get().equals("") ){
                    // FILES
                    dialogManager.findViewById(R.id.btn_delete).setEnabled(false);
                    dialogManager.findViewById(R.id.btn_load).setEnabled(false);
                    dialogManager.findViewById(R.id.btn_email).setEnabled(false);
                    dialogManager.findViewById(R.id.btn_new).setEnabled(true);
                    // TEST SITE
                    dialogManager.findViewById(R.id.btn_site_delete).setEnabled(false);
                    dialogManager.findViewById(R.id.btn_sites_edit).setEnabled(false);
                }
                else{
                    // FILES
                    dialogManager.findViewById(R.id.btn_delete).setEnabled(true);
                    dialogManager.findViewById(R.id.btn_load).setEnabled(true);
                    dialogManager.findViewById(R.id.btn_email).setEnabled(true);
                    dialogManager.findViewById(R.id.btn_new).setEnabled(true);
                    if ( bol.get().equals(loadedTestSite.get()) ){
                        dialogManager.findViewById(R.id.btn_load).setVisibility(View.GONE);
                        dialogManager.findViewById(R.id.btn_close_dlg).setVisibility(View.VISIBLE);
                    }
                    else {
                        dialogManager.findViewById(R.id.btn_load).setVisibility(View.VISIBLE);
                        dialogManager.findViewById(R.id.btn_close_dlg).setVisibility(View.GONE);
                    }
                    // TEST SITE
                    dialogManager.findViewById(R.id.btn_site_delete).setEnabled(true);
                    dialogManager.findViewById(R.id.btn_sites_edit).setEnabled(true);
                }
            }
            else if ( bol.ID == loadedTestSite.ID ){
                if ( bol.get().equals("") ){
                    dialogManager.findViewById(R.id.btn_load).setVisibility(View.VISIBLE);
                    dialogManager.findViewById(R.id.btn_close_dlg).setVisibility(View.GONE);
                }
                else {
                    dialogManager.findViewById(R.id.btn_load).setVisibility(View.GONE);
                    dialogManager.findViewById(R.id.btn_close_dlg).setVisibility(View.VISIBLE);
                }
            }
            else if ( bol.ID == selectedOperator.ID ){
                if ( bol.get().equals("") ){
                    if(dialogManager!=null) {
                        dialogManager.findViewById(R.id.btn_operator_edit).setEnabled(false);
                    }
                }
                else{
                    if(dialogManager!=null) {
                        dialogManager.findViewById(R.id.btn_operator_edit).setEnabled(true);
                    }
                }
            }
        }
//        else if (o instanceof BooleanObservable) {
//            BooleanObservable bol = (BooleanObservable) o;
//            if (bol.ID == fileLoaded.ID){
//                if (bol.get()){
//                    dialogManager.findViewById(R.id.btn_load).setVisibility(View.GONE);
//                    dialogManager.findViewById(R.id.btn_close).setVisibility(View.VISIBLE);
//                }
//                else{
//                    dialogManager.findViewById(R.id.btn_load).setVisibility(View.VISIBLE);
//                    dialogManager.findViewById(R.id.btn_close).setVisibility(View.GONE);
//                }
//            }
//        }

    }

//    public class TheSpecialListener implements DatePickerDialog.OnDateSetListener {
//
//        private int id;
//
//        public int getDateFormat(){
//            return  id;
//        }
//
//        public TheSpecialListener(int id) {
//            this.id = id;
//        }
//
//
//
//        @Override
//        public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {
//
//        }
//    }
}
