package com.megabras.bluelogg.tm25r;

import android.app.Activity;
import android.graphics.Typeface;
import android.graphics.pdf.PdfDocument;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Paper;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.modbus.TM25R.Objects;
import com.megabras.modbus.TM25R.TestData;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

@SuppressWarnings("WeakerAccess")
public class ReportPDF extends ReportProvider
{

    @SuppressWarnings("FieldCanBeLocal")
    private final String
            TAG = "ReportPDF";

    private TestData
            testData;

    private ArrayList<Objects>
            objList;

    private Database
            database;

    public ReportPDF(Activity activity, Handler handler)
    {
        super(activity);
        this.handler = handler;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public void generate(Database database, TestData testData, ArrayList<Objects> objList)
    {
        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();
        //handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();

        PdfDocument document = new PdfDocument();
        PdfDocument.PageInfo pageInfo;

        if ( Locale.getDefault().getLanguage().equals( Locale.ENGLISH.getLanguage() ) )
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.Letter.getWidth(), Paper.Letter.getHeight(), 1).create();
        }
        else
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.A4.getWidth(), Paper.A4.getHeight(), 1).create();
        }

        CURSOR_Y     = 0;
        CURSOR_X     = 0;
        PAPER_WIDTH  = pageInfo.getPageWidth()  - (MARGIN * 2);
        PAPER_HEIGHT = pageInfo.getPageHeight() - (MARGIN * 2);

        PdfDocument.Page page = document.startPage( pageInfo );
        canvas = page.getCanvas();

        moveCursor(MARGIN, MARGIN);

        // BORDER
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(PAPER_HEIGHT);
        this.drawSquare();

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_general_info)).sendToTarget();

        // TITLE
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawTitle( activity.getString(R.string.ground_tester_report).toUpperCase() );

        drawLineCompanyTestSite(database, testData);

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
        ArrayList<String> titles = new ArrayList<>();
        ArrayList<String> values = new ArrayList<>();

        // A
        titles.add( activity.getString( R.string.date      ).toUpperCase() );
        titles.add( activity.getString( R.string.equipment ).toUpperCase() );

        values.add( dateFormat.format(new Date()) );
        values.add( testData.getModel() + " - " + testData.getSerial());

        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawLineItem(titles, values, ALIGN_LEFT);

        // B
        titles.clear();
        titles.add( activity.getString( R.string.operator  ).toUpperCase() );

        values.clear();
        values.add( testData.getOperatorName() + " / " + testData.getOperatorExtension() + " / " + testData.getOperatorEmail() );

        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawLineItem(titles, values, ALIGN_LEFT );

        // SNAPSHOT
        File snapshot = database.getMapSnapshotFile(testData.getFolder());
        if ( snapshot != null && snapshot.exists() )
        {
            left(0);
            top(0);
            right(PAPER_WIDTH);
            bottom(300);

            this.drawLineItem(activity.getString(R.string.map).toUpperCase(), snapshot);
        }

        // COMMENTS
        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(20);
        drawMultilineText( activity.getString( R.string.comments).toUpperCase(), database.getComments(testData.getFile()), false );

        drawGeneratedBy();

        drawPageNumber(1, objList.size()+1, ALIGN_LEFT);


        document.finishPage(page);

        int curPage = 2;
        for ( Objects obj : objList )
        {
            PdfDocument.Page p = document.startPage( pageInfo );
            canvas = p.getCanvas();

            resetCursor();

            moveCursor( MARGIN, MARGIN );

            saveCursor();

            // BORDER
            top   (0);
            left  (0);
            right (PAPER_WIDTH);
            bottom(PAPER_HEIGHT);
            this.drawSquare();

            // TITLE
            top   (0);
            left  (0);
            right (PAPER_WIDTH);
            bottom(20);
            this.drawTitle( activity.getString(R.string.ground_tester_report).toUpperCase() );

            top   (0);
            left  (0);
            right (PAPER_WIDTH);
            bottom(20);
            drawLineItem( activity.getString(R.string.object_name).toUpperCase(), obj.getObjectName(), ALIGN_LEFT );

            // GPS
            String[] gps = database.getGpsData( testData.getFile(), obj.getObjectName() );

            titles.clear();
            titles.add( activity.getString( R.string.latitude  ).toUpperCase() );
            titles.add( activity.getString( R.string.longitude ).toUpperCase() );
            titles.add( activity.getString( R.string.altitude  ).toUpperCase() );
            titles.add( activity.getString( R.string.accuracy  ).toUpperCase() );

            values.clear();
            values.add( gps[1] );
            values.add( gps[0] );
            values.add( gps[2] );
            values.add( gps[3] );

            left  (0);
            top   (0);
            right (PAPER_WIDTH);
            bottom(20);
            this.drawLineItem(titles, values, ALIGN_LEFT);

            // BLANK LINE
            moveCursor(0, 20);

            // MEASUREMENTS

            int[] widths = new int[]
            {
                35,
                60,
                45,
                45,
                45,
                45,
            };

            values.clear();
            values.add( "#" );
            values.add( activity.getString(R.string.date).toUpperCase() );
            values.add( "R" );
            values.add( "Rc" );
            values.add( "C" );
            values.add( "L" );

            left  (0);
            top   (0);
            right (PAPER_WIDTH);
            bottom(10);
            this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), widths, values);

            for ( com.megabras.modbus.TM25R.Measurement mea : obj.getMeasurements() )
            {
                values.clear();
                values.add( mea.getTestNumber()            );
                values.add( mea.getDate()                  );
                values.add( mea.getResistance()            );
                values.add( mea.getResistanceCompensated() );
                values.add( mea.getCapacitance()           );
                values.add( mea.getInductance()            );

                left  (0);
                top   (0);
                right (PAPER_WIDTH);
                bottom(10);
                this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), widths, values);
            }

            // PICTURE
            File picture = database.getPictureFile( testData.getFolder(), obj.getObjectName() );

            if ( picture != null && picture.exists() )
            {
                restoreCursor();
                saveCursor();

                int imgHeight = 300;
                moveCursor(0, PAPER_HEIGHT - imgHeight);

                left(0);
                top(0);
                right(PAPER_WIDTH);
                bottom(imgHeight);

                this.drawLineItem(activity.getString(R.string.picture).toUpperCase(), picture);

                restoreCursor();
            }

            drawPageNumber(curPage++, objList.size()+1, ALIGN_LEFT);

            document.finishPage( p );
        }

        handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.saving_file)).sendToTarget();

        String fileName = getFileName( testData.getFolder(), testData.getFolder().getName() );
        File tmpFile    = new File( fileName );
        if( tmpFile.exists() )
            tmpFile.delete();

        try
        {
            document.writeTo(new FileOutputStream(tmpFile));
            // close the document
            document.close();

            handler.obtainMessage(Controller.mHandler.REPORT_FINISHED, 1, -1, activity.getString(R.string.finished)).sendToTarget();

            sendFile(tmpFile);

        }
        catch (IOException e)
        {
            Log.e(TAG, "error "+e.toString());
            handler.obtainMessage(Controller.mHandler.REPORT_ERROR, 1, -1, activity.getString(R.string.finished)).sendToTarget();
        }
    }

    private String getFileName(File folder, String site)
    {
        site      = MUtils.removeSpecialChars( site );
        File file = new File(folder, site + ".pdf");
        return file.toString();
    }
}

