package com.megabras.bluelogg.tm25r;

import android.app.Activity;

import com.megabras.bluelogg.em4058.EM4058test;
import com.megabras.bluelogg.R;
import com.megabras.modbus.TM25R.Measurement;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map.Entry;

public class ReportTable {

	private final
			String TAG = "ReportTable";

	private Document
			doc;

	private Element
			table;

	private String
			tableName,
			unity_r_not_compensated = "",
			unity_r_compensated = "";

	private Activity
			activity;

	public ReportTable(Activity activity, Document doc){
		this.activity = activity;
		unity_r_not_compensated = activity.getResources().getString(R.string.unity_resistance_not_compensated);
		unity_r_compensated     = activity.getResources().getString(R.string.unity_resistance_compensated);
		this.doc = doc;
	}
	
	public void createTable(String name){
		tableName = name;
		table     = doc.createElement("table:table");
		table.setAttribute("table:name", tableName);
		table.setAttribute("table:style-name", tableName);
	}
	
	public void setColumns(int numOfColumns) /**throws TableNullException*/{		
		char a = 'A';
		int intOfA = (int) a;
		
		for (int i = 0; i < numOfColumns; i++){
			char letter = (char) (intOfA + i);
			Element column = doc.createElement("table:table-column");
			column.setAttribute("table:style-name", tableName + "." + letter);
			table.appendChild(column);
		}
	}

	public void setRowsNew(ArrayList<Measurement> results){
		Element row = doc.createElement("table:table-row");
		LinkedHashMap<String,String> header = new LinkedHashMap<String,String>();
		header.put("#", "A1");
		header.put(activity.getString(R.string.date).toUpperCase(), "A1");
		header.put(unity_r_not_compensated, "A1");
		header.put(unity_r_compensated, "A1");
		header.put("C", "A1");
		header.put("L", "F1");

		for(Entry<String,String> e : header.entrySet()){
			Element cell = doc.createElement("table:table-cell");
			cell.setAttribute("table:style-name", tableName + "." +e.getValue() );
			cell.setAttribute("office:value-type", "string");
			Element line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P13");
			line.setTextContent(e.getKey());
			cell.appendChild(line);
			row.appendChild(cell);
		}
		table.appendChild(row);

		for (Measurement mea : results){
			row = doc.createElement("table:table-row");
			String[] data = new String[]{mea.getTestNumber(), mea.getDate(), mea.getResistance(), mea.getResistanceCompensated(), mea.getCapacitance(), mea.getInductance() };
			for (String value : data) {
				Element cell = doc.createElement("table:table-cell");
				cell.setAttribute("table:style-name", tableName + ".A2");
				cell.setAttribute("office:value-type", "string");
				Element line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P14");
				line.setTextContent(value);
				cell.appendChild(line);
				row.appendChild(cell);
			}

			table.appendChild(row);

		}
	}
	
	public void setRows(ArrayList<ListTest> results){
		Element row = doc.createElement("table:table-row");		
		LinkedHashMap<String,String> header = new LinkedHashMap<String,String>();		
		header.put("#", "A1");
		header.put("DATE", "A1");
		header.put(unity_r_not_compensated+"=", "A1");
		header.put(unity_r_compensated+"=", "A1");
		header.put("C", "A1");
		header.put("L", "F1");
		
		for(Entry<String,String> e : header.entrySet()){
			Element cell = doc.createElement("table:table-cell");
			cell.setAttribute("table:style-name", tableName + "." +e.getValue() );
			cell.setAttribute("office:value-type", "string");
			Element line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P13");
			line.setTextContent(e.getKey());
			cell.appendChild(line);
			row.appendChild(cell);
		}
		table.appendChild(row);
		
		for (ListTest result : results){
			row = doc.createElement("table:table-row");						
			String[] data = result.getAll();
			for (String value : data) {
				Element cell = doc.createElement("table:table-cell");
				cell.setAttribute("table:style-name", tableName + ".A2");
				cell.setAttribute("office:value-type", "string");
				Element line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P14");
				line.setTextContent(value);
				cell.appendChild(line);
				row.appendChild(cell);
			}
			
			table.appendChild(row);

		}		
	}

	public void setRowsEM4058(ArrayList<EM4058test.EM4058values> results){
		Element row = doc.createElement("table:table-row");
		LinkedHashMap<String,String> header = new LinkedHashMap<String,String>();
		header.put("#", "A1");
		header.put("DATE", "A1");
		header.put("TYPE", "A1");
		header.put("DISTANCE", "A1");
		header.put("FREQUENCY", "A1");
		header.put("RESULT", "A1");
		header.put("270 Hz", "A1");
		header.put("570 Hz", "A1");
		header.put("870 Hz", "A1");
		header.put("1170 Hz", "A1");
		header.put("1470 Hz", "F1");

		for(Entry<String,String> e : header.entrySet()){
			Element cell = doc.createElement("table:table-cell");
			cell.setAttribute("table:style-name", tableName + "." +e.getValue() );
			cell.setAttribute("office:value-type", "string");
			Element line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P21");
			line.setTextContent(e.getKey());
			cell.appendChild(line);
			row.appendChild(cell);
		}
		table.appendChild(row);

		for (EM4058test.EM4058values result : results){
			row = doc.createElement("table:table-row");
			String[] data = result.getAll();
			for (String value : data) {
				Element cell = doc.createElement("table:table-cell");
				cell.setAttribute("table:style-name", tableName + ".A2");
				cell.setAttribute("office:value-type", "string");
				Element line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P20");
				line.setTextContent(value);
				cell.appendChild(line);
				row.appendChild(cell);
			}

			table.appendChild(row);
		}
	}
	
	public Element getTable(){
		return table;
	}
	

}

